
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
% The program code were used for generating the results 
% publushed in the manuscript: 
% Konovalov Y.V.:Ice-shelf vibrations modeled by a full 3-D elastic model, 
% Annals of Glaciology, 60(79), 68-74, 2019. doi:10.1017/aog.2019.9 
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

clear

dens=910; % ice dencity

dens_w=1000; % water dencity

g=9.81; 



LXSho=16000; % ice shelf extent in X-direction

LYSho=800;  % ice shelf extent in Y-derection

HGrl=100; % ice thickness at the grounding line

HShTerm=100; % ice thickness at the terminus


                              
E_Yung=9.0*10^9; % Young's modulus 

P_Puasson=0.33; % Poisson's ratio



Kp1=2*(1-P_Puasson)/(1-2*P_Puasson);

Kp2=2*P_Puasson/(1-2*P_Puasson);

KpE=2*(1+P_Puasson)/E_Yung;

KpE0=E_Yung/(1+P_Puasson);



Kpb1=(1-P_Puasson)/(1-2*P_Puasson);

Kpb2=P_Puasson/(1-2*P_Puasson);


alfa1=1.0; alfa2=0.0; % the parameters of the model (see the manuscript)

alfa12=alfa1+alfa2;


%----------------------;

PT0=10; % initial value of the periodicity of the vibrations

D_PT=0.025; % the periodicity resolution  

AmSLC=1.0; % the amplitude of the incident wave

NFreq=200; % the number of the main cycle repeats


%-------------------;

Nx=241;  % grid points number in X-direction (horizontal direction)

Ny=13; % grid points number in Y-derection (horizontal direction)

Nz=6; % grid points number in Z-derection (vertical direction)


dx=LXSho/(Nx-1); % the resolution in the X-direction

dy=1/(Ny-1); % the resolution in the Y-direction (non-dimensional)

dz=1/(Nz-1); % the resolution in the Z-direction (non-dimensional)

%++++++++++++++++++++++++ L_X, L_Y, L_Z +++++++++++++++++++++++;


L_X=zeros(1,Nx);  

for ii=1:1:Nx, L_X(ii)=dx*(ii-1); end


L_Y=zeros(1,Ny);  

for jj=1:1:Ny, L_Y(jj)=dy*(jj-1); end


L_Z=zeros(1,Nz); 

for kk=1:1:Nz, L_Z(kk)=dz*(kk-1); end



%+++++ Ice Width in Y-direction (B), Y1(x), Y2(x), dB/dx ++++++++++++;

dfW=0.0;

Wd1=LYSho; Wd2=LYSho+dfW;


Width=zeros(1,Nx); Y1=zeros(1,Nx); Y2=zeros(1,Nx);

for ii=1:1:Nx, Width(ii)=Wd1+((Wd2-Wd1)/LXSho)*L_X(ii); end

for ii=1:1:Nx, Y1(ii)=-0.5*(Wd1+((Wd2-Wd1)/LXSho)*L_X(ii)); end

for ii=1:1:Nx, Y2(ii)=0.5*(Wd1+((Wd2-Wd1)/LXSho)*L_X(ii)); end



%++++++ Ice Surface, Ice Base, Ice Thickness, Sea Bottom +++++++++++;

HIC=zeros(Nx,Ny); 

for ii=1:1:Nx 
    for jj=1:1:Ny 
        HIC(ii,jj)=HGrl+((HShTerm-HGrl)/LXSho)*L_X(ii);
    end 
end


%-----------------------------------------------;

HSF=zeros(Nx,Ny);

for ii=1:1:Nx 
    for jj=1:1:Ny 
        HSF(ii,jj)=HIC(ii,jj)*(1-dens/dens_w); 
    end 
end


%------------------------------------------------;

HBF=zeros(Nx,Ny);

for ii=1:1:Nx 
    for jj=1:1:Ny 
        HBF(ii,jj)=-HIC(ii,jj)*(dens/dens_w); 
    end 
end


%---------------------------------------------------;

HBSEA=zeros(Nx,Ny); 


for ii=1:1:Nx
    for jj=1:1:Ny 
        HBSEA(ii,jj)=HBF(1,jj)-100 + (((HBF(Nx,jj)-100)-(HBF(1,jj)-100))/LXSho)*L_X(ii); 
    end 
end



WDepth_ij=zeros(Nx,Ny);

for ii=1:1:Nx
    for jj=1:1:Ny 
        WDepth_ij(ii,jj)=HBF(ii,jj)-HBSEA(ii,jj); 
    end 
end


%------------------------------------------------;

figure(1);

%surf(L_X./LXSho,L_Y,HBF'), title('Ice Shelf Base'), xlabel('X, m'), ylabel('Y, m'), zlabel('Elevation, m')

surf(L_X,L_Y.*LYSho,HBF'), title('Ice Shelf Base'), xlabel('X, m'), ylabel('Y, m'), zlabel('Elevation, m')

%contourf(L_X./Lo,L_Y,HBF), title('Ice Sheet Bed'), xlabel('X, m'), ylabel('Y, m')

figure(2);

%surf(L_X./LXSho,L_Y,HSF'), title('Ice Shelf Surface'), xlabel('X, m'), ylabel('Y, m'), zlabel('Elevation, m')

surf(L_X,L_Y.*LYSho,HSF'), title('Ice Shelf Surface'), xlabel('X, m'), ylabel('Y, m'), zlabel('Elevation, m')

%contourf(L_X./Lo,L_Y,HSF), title('Ice Sheet Surface'), xlabel('X, m'), ylabel('Y, m')

figure(3);

%surf(L_X./LXSho,L_Y,HBSEA'), title('Sea Bottom'), xlabel('X, m'), ylabel('Y, m'), zlabel('Elevation, m')

surf(L_X,L_Y.*LYSho,HBSEA'), title('Sea Bottom'), xlabel('X, m'), ylabel('Y, m'), zlabel('Elevation, m')

%contourf(L_X./Lo,L_Y,HSF), title('Ice Sheet Surface'), xlabel('X, m'), ylabel('Y, m')



LP_HSF=zeros(1,Nx); LP_HBF=zeros(1,Nx); LP_HBSEA=zeros(1,Nx); LP_HIC=zeros(1,Nx); LP_WDepth=zeros(1,Nx);

Jy=round(Ny/2)+1;

for ii=1:1:Nx, LP_HSF(ii)=HSF(ii,Jy); end

for ii=1:1:Nx, LP_HBF(ii)=HBF(ii,Jy); end

for ii=1:1:Nx, LP_HBSEA(ii)=HBSEA(ii,Jy); end

for ii=1:1:Nx, LP_HIC(ii)=HIC(ii,Jy); end

for ii=1:1:Nx, LP_WDepth(ii)=WDepth_ij(ii,Jy); end


figure(4);

plot(L_X,LP_HSF,'-b.',L_X,LP_HBF,'-rd',L_X,LP_HBSEA,'-k.'), title('Ice Shelf Centre Line Cross-section'), legend('Ice Shelf Surface','Ice Shelf Base','Sea Bottom'), xlabel('X, m'), ylabel('Elevation, m'), grid


figure(5);

plot(L_X,LP_HIC,'-b.',L_X,LP_WDepth,'-rd'), title('Ice Shelf Centre Line Thickness'), legend('Ice Shelf Thickness','Water Channel Depth'), xlabel('X, m'), ylabel('Thickness, m'), grid



%-------------- dB/dx, dY1/dx, dY2/dx -----------------;

difWd=zeros(1,Nx);

difWd(1)=(Width(2)-Width(1))/dx;
for ii=2:1:Nx-1, difWd(ii)=(Width(ii+1)-Width(ii-1))/(2*dx); end
difWd(Nx)=(Width(Nx)-Width(Nx-1))/dx;


difY1=zeros(1,Nx);

difY1(1)=(Y1(2)-Y1(1))/dx;
for ii=2:1:Nx-1, difY1(ii)=(Y1(ii+1)-Y1(ii-1))/(2*dx); end
difY1(Nx)=(Y1(Nx)-Y1(Nx-1))/dx;


difY2=zeros(1,Nx);

difY2(1)=(Y2(2)-Y2(1))/dx;
for ii=2:1:Nx-1, difY2(ii)=(Y2(ii+1)-Y2(ii-1))/(2*dx); end
difY2(Nx)=(Y2(Nx)-Y2(Nx-1))/dx;


%+++++++++++++++++++++++++++ dQ/dx, dQ/dy +++++++++++++++++++++++++++++++;

dQx=zeros(Nx,Ny);

for ii=1:1:Nx 
    for jj=1:1:Ny 
        dQx(ii,jj)=-(1/Width(ii))*(difY1(ii)+L_Y(jj)*difWd(ii)); 
    end 
end


dQy=zeros(Nx,1);

for ii=1:1:Nx, dQy(ii)=1/Width(ii); end



%++++++++++++++++++++++++ dHs/dx, dHs/dy +++++++++++++++++++++++++++++;

dHsx=zeros(Nx,Ny); dHsy=zeros(Nx,Ny);


%--------------------- dHs/dx -----------------------;

dHsx(1,1)=(-HSF(3,1)+4*HSF(2,1)-3*HSF(1,1))/(2*dx)+((-HSF(1,3)+4*HSF(1,2)-3*HSF(1,1))/(2*dy))*dQx(1,1);

for jj=2:1:Ny-1 
    dHsx(1,jj)=(-HSF(3,jj)+4*HSF(2,jj)-3*HSF(1,jj))/(2*dx)+((HSF(1,jj+1)-HSF(1,jj-1))/(2*dy))*dQx(1,jj); 
end

dHsx(1,Ny)=(-HSF(3,Ny)+4*HSF(2,Ny)-3*HSF(1,Ny))/(2*dx)+((3*HSF(1,Ny)-4*HSF(1,Ny-1)+HSF(1,Ny-2))/(2*dy))*dQx(1,Ny);

for ii=2:1:Nx-1 
    dHsx(ii,1)=(HSF(ii+1,1)-HSF(ii-1,1))/(2*dx)+((-HSF(ii,3)+4*HSF(ii,2)-3*HSF(ii,1))/(2*dy))*dQx(ii,1);
end 

for ii=2:1:Nx-1 
    for jj=2:1:Ny-1 
        dHsx(ii,jj)=(HSF(ii+1,jj)-HSF(ii-1,jj))/(2*dx)+((HSF(ii,jj+1)-HSF(ii,jj-1))/(2*dy))*dQx(ii,jj); 
    end 
end

for ii=2:1:Nx-1 
    dHsx(ii,Ny)=(HSF(ii+1,Ny)-HSF(ii-1,Ny))/(2*dx)+((3*HSF(ii,Ny)-4*HSF(ii,Ny-1)+HSF(ii,Ny-2))/(2*dy))*dQx(ii,Ny); 
end

dHsx(Nx,1)=(3*HSF(Nx,1)-4*HSF(Nx-1,1)+HSF(Nx-2,1))/(2*dx)+((-HSF(Nx,3)+4*HSF(Nx,2)-3*HSF(Nx,1))/(2*dy))*dQx(Nx,1);

for jj=2:1:Ny-1 
    dHsx(Nx,jj)=(3*HSF(Nx,jj)-4*HSF(Nx-1,jj)+HSF(Nx-2,jj))/(2*dx)+((HSF(Nx,jj+1)-HSF(Nx,jj-1))/(2*dy))*dQx(Nx,jj); 
end

dHsx(Nx,Ny)=(3*HSF(Nx,Ny)-4*HSF(Nx-1,Ny)+HSF(Nx-2,Ny))/(2*dx)+((3*HSF(Nx,Ny)-4*HSF(Nx,Ny-1)+HSF(Nx,Ny-2))/(2*dy))*dQx(Nx,Ny);


%-------------------- dHs/dy ----------------------;

for ii=1:1:Nx, dHsy(ii,1)=((-HSF(ii,3)+4*HSF(ii,2)-3*HSF(ii,1))/(2*dy))*dQy(ii); end

for ii=1:1:Nx 
    for jj=2:1:Ny-1 
        dHsy(ii,jj)=((HSF(ii,jj+1)-HSF(ii,jj-1))/(2*dy))*dQy(ii); 
    end 
end

for ii=1:1:Nx, dHsy(ii,Ny)=((3*HSF(ii,Ny)-4*HSF(ii,Ny-1)+HSF(ii,Ny-2))/(2*dy))*dQy(ii); end



%++++++++++++++++++++++++ dHb/dx, dHb/dy +++++++++++++++++++++++++++++;

dHbx=zeros(Nx,Ny); dHby=zeros(Nx,Ny);


%--------------------------- dHb/dx ----------------------------;

dHbx(1,1)=(-HBF(3,1)+4*HBF(2,1)-3*HBF(1,1))/(2*dx)+((-HBF(1,3)+4*HBF(1,2)-3*HBF(1,1))/(2*dy))*dQx(1,1);

for jj=2:1:Ny-1 
    dHbx(1,jj)=(-HBF(3,jj)+4*HBF(2,jj)-3*HBF(1,jj))/(2*dx)+((HBF(1,jj+1)-HBF(1,jj-1))/(2*dy))*dQx(1,jj); 
end

dHbx(1,Ny)=(-HBF(3,Ny)+4*HBF(2,Ny)-3*HBF(1,Ny))/(2*dx)+((3*HBF(1,Ny)-4*HBF(1,Ny-1)+HBF(1,Ny-2))/(2*dy))*dQx(1,Ny);

for ii=2:1:Nx-1 
    dHbx(ii,1)=(HBF(ii+1,1)-HBF(ii-1,1))/(2*dx)+((-HBF(ii,3)+4*HBF(ii,2)-3*HBF(ii,1))/(2*dy))*dQx(ii,1);
end 

for ii=2:1:Nx-1 
    for jj=2:1:Ny-1 
        dHbx(ii,jj)=(HBF(ii+1,jj)-HBF(ii-1,jj))/(2*dx)+((HBF(ii,jj+1)-HBF(ii,jj-1))/(2*dy))*dQx(ii,jj); 
    end 
end

for ii=2:1:Nx-1 
    dHbx(ii,Ny)=(HBF(ii+1,Ny)-HBF(ii-1,Ny))/(2*dx)+((3*HBF(ii,Ny)-4*HBF(ii,Ny-1)+HBF(ii,Ny-2))/(2*dy))*dQx(ii,Ny); 
end

dHbx(Nx,1)=(3*HBF(Nx,1)-4*HBF(Nx-1,1)+HBF(Nx-2,1))/(2*dx)+((-HBF(Nx,3)+4*HBF(Nx,2)-3*HBF(Nx,1))/(2*dy))*dQx(Nx,1);

for jj=2:1:Ny-1 
    dHbx(Nx,jj)=(3*HBF(Nx,jj)-4*HBF(Nx-1,jj)+HBF(Nx-2,jj))/(2*dx)+((HBF(Nx,jj+1)-HBF(Nx,jj-1))/(2*dy))*dQx(Nx,jj); 
end

dHbx(Nx,Ny)=(3*HBF(Nx,Ny)-4*HBF(Nx-1,Ny)+HBF(Nx-2,Ny))/(2*dx)+((3*HBF(Nx,Ny)-4*HBF(Nx,Ny-1)+HBF(Nx,Ny-2))/(2*dy))*dQx(Nx,Ny);

%-------------------- dHb/dy ----------------------;

for ii=1:1:Nx, dHby(ii,1)=((-HBF(ii,3)+4*HBF(ii,2)-3*HBF(ii,1))/(2*dy))*dQy(ii); end

for ii=1:1:Nx 
    for jj=2:1:Ny-1 
        dHby(ii,jj)=((HBF(ii,jj+1)-HBF(ii,jj-1))/(2*dy))*dQy(ii); 
    end 
end

for ii=1:1:Nx, dHby(ii,Ny)=((3*HBF(ii,Ny)-4*HBF(ii,Ny-1)+HBF(ii,Ny-2))/(2*dy))*dQy(ii); end


% for ii=1:1:Nx 
%     for jj=1:1:Ny 
%         dHby(ii,jj)=0.; 
%     end 
% end


%++++++++++++++++++++++++ dH/dx, dH/dy +++++++++++++++++++++++++++++;

dHx=zeros(Nx,Ny); dHy=zeros(Nx,Ny);


%------------------ dH/dx --------------------;

dHx(1,1)=(-HIC(3,1)+4*HIC(2,1)-3*HIC(1,1))/(2*dx)+((-HIC(1,3)+4*HIC(1,2)-3*HIC(1,1))/(2*dy))*dQx(1,1);

for jj=2:1:Ny-1 
    dHx(1,jj)=(-HIC(3,jj)+4*HIC(2,jj)-3*HIC(1,jj))/(2*dx)+((HIC(1,jj+1)-HIC(1,jj-1))/(2*dy))*dQx(1,jj); 
end

dHx(1,Ny)=(-HIC(3,Ny)+4*HIC(2,Ny)-3*HIC(1,Ny))/(2*dx)+((3*HIC(1,Ny)-4*HIC(1,Ny-1)+HIC(1,Ny-2))/(2*dy))*dQx(1,Ny);

for ii=2:1:Nx-1 
    dHx(ii,1)=(HIC(ii+1,1)-HIC(ii-1,1))/(2*dx)+((-HIC(ii,3)+4*HIC(ii,2)-3*HIC(ii,1))/(2*dy))*dQx(ii,1);
end 

for ii=2:1:Nx-1 
    for jj=2:1:Ny-1 
        dHx(ii,jj)=(HIC(ii+1,jj)-HIC(ii-1,jj))/(2*dx)+((HIC(ii,jj+1)-HIC(ii,jj-1))/(2*dy))*dQx(ii,jj); 
    end 
end

for ii=2:1:Nx-1 
    dHx(ii,Ny)=(HIC(ii+1,Ny)-HIC(ii-1,Ny))/(2*dx)+((3*HIC(ii,Ny)-4*HIC(ii,Ny-1)+HIC(ii,Ny-2))/(2*dy))*dQx(ii,Ny); 
end

dHx(Nx,1)=(3*HIC(Nx,1)-4*HIC(Nx-1,1)+HIC(Nx-2,1))/(2*dx)+((-HIC(Nx,3)+4*HIC(Nx,2)-3*HIC(Nx,1))/(2*dy))*dQx(Nx,1);

for jj=2:1:Ny-1 
    dHx(Nx,jj)=(3*HIC(Nx,jj)-4*HIC(Nx-1,jj)+HIC(Nx-2,jj))/(2*dx)+((HIC(Nx,jj+1)-HIC(Nx,jj-1))/(2*dy))*dQx(Nx,jj); 
end

dHx(Nx,Ny)=(3*HIC(Nx,Ny)-4*HIC(Nx-1,Ny)+HIC(Nx-2,Ny))/(2*dx)+((3*HIC(Nx,Ny)-4*HIC(Nx,Ny-1)+HIC(Nx,Ny-2))/(2*dy))*dQx(Nx,Ny);


%-------------------- dH/dy ----------------------;

for ii=1:1:Nx, dHy(ii,1)=((-HIC(ii,3)+4*HIC(ii,2)-3*HIC(ii,1))/(2*dy))*dQy(ii); end

for ii=1:1:Nx 
    for jj=2:1:Ny-1 
        dHy(ii,jj)=((HIC(ii,jj+1)-HIC(ii,jj-1))/(2*dy))*dQy(ii); 
    end 
end

for ii=1:1:Nx, dHy(ii,Ny)=((3*HIC(ii,Ny)-4*HIC(ii,Ny-1)+HIC(ii,Ny-2))/(2*dy))*dQy(ii); end



%++++++++++++++++++++++ dZ/dx, dZ/dy, dZ/dz +++++++++++++++++++++++++;

dZx=zeros(Nx,Ny,Nz);

for ii=1:1:Nx 
    for jj=1:1:Ny 
        for kk=1:1:Nz 
            dZx(ii,jj,kk)=dHsx(ii,jj)/HIC(ii,jj)-dHx(ii,jj)*L_Z(kk)/HIC(ii,jj);
        end 
    end
end 


dZy=zeros(Nx,Ny,Nz);

for ii=1:1:Nx 
    for jj=1:1:Ny 
        for kk=1:1:Nz 
            dZy(ii,jj,kk)=dHsy(ii,jj)/HIC(ii,jj)-dHy(ii,jj)*L_Z(kk)/HIC(ii,jj); 
        end 
    end
end


dZz=zeros(Nx,Ny);

for ii=1:1:Nx 
    for jj=1:1:Ny 
        dZz(ii,jj)=-1/HIC(ii,jj); 
    end 
end



%+++++++++++++++++++++++++ Arrays +++++++++++++++++++++++++++++++;


data_U0=zeros(Ny,Nz);
data_ULE1=zeros(Nx,Nz);
data_ULE2=zeros(Nx,Nz);
data_UN=zeros(Ny,Nz);

data_V0=zeros(Ny,Nz);
data_VLE1=zeros(Nx,Nz);
data_VLE2=zeros(Nx,Nz);
data_VN=zeros(Ny,Nz);

data_W0=zeros(Ny,Nz);
data_WLE1=zeros(Nx,Nz);
data_WLE2=zeros(Nx,Nz);
data_WN=zeros(Ny,Nz);

masU=zeros(Nx,Ny,Nz);
masV=zeros(Nx,Ny,Nz);
masW=zeros(Nx,Ny,Nz);


Delta_H=zeros(Nx,Ny,NFreq);
Delta_P=zeros(Nx,Ny,NFreq);
Delta_Ws=zeros(Nx,Ny,NFreq);

Delta_HX=zeros(Nx,1);


L_X_node_max=zeros(Nx,1);

L_X_node_min=zeros(Nx,1);


%-----------------------------;


M_X=zeros(Nx*Ny,1);
M_Y=zeros(Nx*Ny,1);

mVx=zeros(Nx*Ny,1);
mVy=zeros(Nx*Ny,1);
mVz=zeros(Nx*Ny,1);
mFxz=zeros(Nx*Ny,1);
mFyz=zeros(Nx*Ny,1);
mdP=zeros(Nx*Ny,1);

%---------------------------;

Gpy1_ijk=zeros(Nx,Ny);
Gpz1_ijk=zeros(Nx,Ny,Nz);
Gpz2_ijk=zeros(Nx,Ny,Nz);

%---------------------------;

d2Qxx=zeros(Nx,Ny);
d2Qxq=zeros(Nx,Ny);
d2Qyx=zeros(Nx,1);

d2Zxx=zeros(Nx,Ny,Nz);
d2Zxq=zeros(Nx,Ny,Nz);
d2Zxz=zeros(Nx,Ny,Nz);

d2Zyx=zeros(Nx,Ny,Nz);
d2Zyq=zeros(Nx,Ny,Nz);
d2Zyz=zeros(Nx,Ny,Nz);

d2Zzx=zeros(Nx,Ny);
d2Zzq=zeros(Nx,Ny);


WDx_ij=zeros(Nx,Ny);
WDy_ij=zeros(Nx,Ny);
WDQxx_ij=zeros(Nx,Ny);
WDQxy_ij=zeros(Nx,Ny);
WDQx2y_ij=zeros(Nx,Ny);
WWx_i=zeros(Nx,1);

%----------------------------;

max_Delta_H=zeros(NFreq,1);


%------------------------------------------------------------------;

MB=zeros(3*Nx*Ny*Nz+2*Nx*Ny);

VB=zeros(3*Nx*Ny*Nz+2*Nx*Ny,1);


%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

for ii=1:1:Nx 
for jj=2:1:Ny 
Gpy1_ijk(ii,jj)=(dQx(ii,jj-1)+dQx(ii,jj))/2;
end
end


for ii=1:1:Nx 
for jj=1:1:Ny 
for kk=2:1:Nz
Gpz1_ijk(ii,jj,kk)=(dZx(ii,jj,kk-1)+dZx(ii,jj,kk))/2;
end 
end
end


for ii=1:1:Nx 
for jj=1:1:Ny 
for kk=2:1:Nz
Gpz2_ijk(ii,jj,kk)=(dZy(ii,jj,kk-1)+dZy(ii,jj,kk))/2;
end 
end
end



for ii=2:1:Nx 
for jj=1:1:Ny
WDx_ij(ii,jj)=(WDepth_ij(ii-1,jj)+WDepth_ij(ii,jj))/2;
end
end

for ii=1:1:Nx 
for jj=2:1:Ny
WDy_ij(ii,jj)=(WDepth_ij(ii,jj-1)+WDepth_ij(ii,jj))/2;
end
end


for ii=2:1:Nx 
for jj=1:1:Ny
WDQxx_ij(ii,jj)=(WDepth_ij(ii-1,jj)*dQx(ii-1,jj)+WDepth_ij(ii,jj)*dQx(ii,jj))/2;
end
end


for ii=1:1:Nx 
for jj=2:1:Ny
WDQxy_ij(ii,jj)=(WDepth_ij(ii,jj-1)*dQx(ii,jj-1)+WDepth_ij(ii,jj)*dQx(ii,jj))/2;
end
end


for ii=1:1:Nx 
for jj=2:1:Ny
WDQx2y_ij(ii,jj)=(WDepth_ij(ii,jj-1)*dQx(ii,jj-1)^2+WDepth_ij(ii,jj)*dQx(ii,jj)^2)/2;
end
end


for ii=2:1:Nx 
WWx_i(ii)=(Width(ii-1)+Width(ii))/2;
end

%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

for jj=1:1:Ny, d2Qxx(1,jj)=(-3*dQx(1,jj)+4*dQx(2,jj)-dQx(3,jj))/(2*dx); end 

for ii=2:1:Nx-1
for jj=1:1:Ny
    d2Qxx(ii,jj)=(dQx(ii+1,jj)-dQx(ii-1,jj))/(2*dx);
end
end

for jj=1:1:Ny, d2Qxx(Nx,jj)=(3*dQx(Nx,jj)-4*dQx(Nx-1,jj)+dQx(Nx-2,jj))/(2*dx); end 


%-----------------------------------------------------------------------;

for ii=1:1:Nx, d2Qxq(ii,1)=(-3*dQx(ii,1)+4*dQx(ii,2)-dQx(ii,3))/(2*dy); end 

for ii=1:1:Nx
for jj=2:1:Ny-1
    d2Qxq(ii,jj)=(dQx(ii,jj+1)-dQx(ii,jj-1))/(2*dy);
end
end

for ii=1:1:Nx, d2Qxq(ii,Ny)=(3*dQx(ii,Ny)-4*dQx(ii,Ny-1)+dQx(ii,Ny-2))/(2*dy); end 



%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

d2Qyx(1)=(-3*dQy(1)+4*dQy(2)-dQy(3))/(2*dx); 

for ii=2:1:Nx-1
    d2Qyx(ii)=(dQy(ii+1)-dQy(ii-1))/(2*dx);
end

d2Qyx(Nx)=(3*dQy(Nx)-4*dQy(Nx-1)+dQy(Nx-2))/(2*dx);


%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

for jj=1:1:Ny
for kk=1:1:Nz
    d2Zxx(1,jj,kk)=(-3*dZx(1,jj,kk)+4*dZx(2,jj,kk)-dZx(3,jj,kk))/(2*dx);
end
end 

for ii=2:1:Nx-1
for jj=1:1:Ny
for kk=1:1:Nz
    d2Zxx(ii,jj,kk)=(dZx(ii+1,jj,kk)-dZx(ii-1,jj,kk))/(2*dx);
end
end
end

for jj=1:1:Ny
for kk=1:1:Nz
    d2Zxx(Nx,jj,kk)=(3*dZx(Nx,jj,kk)-4*dZx(Nx-1,jj,kk)+dZx(Nx-2,jj,kk))/(2*dx);
end
end 


%---------------------------------------------------------------------;

for ii=1:1:Nx
for kk=1:1:Nz
    d2Zxq(ii,1,kk)=(-3*dZx(ii,1,kk)+4*dZx(ii,2,kk)-dZx(ii,3,kk))/(2*dy);
end
end 

for ii=1:1:Nx
for jj=2:1:Ny-1
for kk=1:1:Nz
    d2Zxq(ii,jj,kk)=(dZx(ii,jj+1,kk)-dZx(ii,jj-1,kk))/(2*dy);
end
end
end

for ii=1:1:Nx
for kk=1:1:Nz
    d2Zxq(ii,Ny,kk)=(3*dZx(ii,Ny,kk)-4*dZx(ii,Ny-1,kk)+dZx(ii,Ny-2,kk))/(2*dy);
end
end


%---------------------------------------------------------------------;

for ii=1:1:Nx
for jj=1:1:Ny
for kk=1:1:Nz
    d2Zxz(ii,jj,kk)=-dHx(ii,jj)/HIC(ii,jj);
end
end
end



%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

for jj=1:1:Ny
for kk=1:1:Nz
    d2Zyx(1,jj,kk)=(-3*dZy(1,jj,kk)+4*dZy(2,jj,kk)-dZy(3,jj,kk))/(2*dx);
end
end 

for ii=2:1:Nx-1
for jj=1:1:Ny
for kk=1:1:Nz
    d2Zyx(ii,jj,kk)=(dZy(ii+1,jj,kk)-dZy(ii-1,jj,kk))/(2*dx);
end
end
end

for jj=1:1:Ny
for kk=1:1:Nz
    d2Zyx(Nx,jj,kk)=(3*dZy(Nx,jj,kk)-4*dZy(Nx-1,jj,kk)+dZy(Nx-2,jj,kk))/(2*dx);
end
end 


%--------------------------------------------------------------------;

for ii=1:1:Nx
for kk=1:1:Nz
    d2Zyq(ii,1,kk)=(-3*dZy(ii,1,kk)+4*dZy(ii,2,kk)-dZy(ii,3,kk))/(2*dy);
end
end 

for ii=1:1:Nx
for jj=2:1:Ny-1
for kk=1:1:Nz
    d2Zyq(ii,jj,kk)=(dZy(ii,jj+1,kk)-dZy(ii,jj-1,kk))/(2*dy);
end
end
end

for ii=1:1:Nx
for kk=1:1:Nz
    d2Zyq(ii,Ny,kk)=(3*dZy(ii,Ny,kk)-4*dZy(ii,Ny-1,kk)+dZy(ii,Ny-2,kk))/(2*dy);
end
end


%---------------------------------------------------------------------;

for ii=1:1:Nx
for jj=1:1:Ny
for kk=1:1:Nz
    d2Zyz(ii,jj,kk)=-dHy(ii,jj)/HIC(ii,jj);
end
end
end


%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

for jj=1:1:Ny, d2Zzx(1,jj)=(-3*dZz(1,jj)+4*dZz(2,jj)-dZz(3,jj))/(2*dx); end 

for ii=2:1:Nx-1
for jj=1:1:Ny
    d2Zzx(ii,jj)=(dZz(ii+1,jj)-dZz(ii-1,jj))/(2*dx);
end
end

for jj=1:1:Ny, d2Zzx(Nx,jj)=(3*dZz(Nx,jj)-4*dZz(Nx-1,jj)+dZz(Nx-2,jj))/(2*dx); end 

%---------------------------------------------------------------------;

for ii=1:1:Nx, d2Zzq(ii,1)=(-3*dZz(ii,1)+4*dZz(ii,2)-dZz(ii,3))/(2*dy); end 

for ii=1:1:Nx
for jj=2:1:Ny-1
    d2Zzq(ii,jj)=(dZz(ii,jj+1)-dZz(ii,jj-1))/(2*dy);
end
end

for ii=1:1:Nx, d2Zzq(ii,Ny)=(3*dZz(ii,Ny)-4*dZz(ii,Ny-1)+dZz(ii,Ny-2))/(2*dy); end 


%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++++++++ THE MAIN CYCLE START +++++++++++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


for JFreq=1:1:NFreq

%-------------------------------;

MB=zeros(3*Nx*Ny*Nz+2*Nx*Ny);

VB=zeros(3*Nx*Ny*Nz+2*Nx*Ny,1);

%------------------------------;

Freq=2*pi/(PT0+JFreq*D_PT); % frequency of the vibrations


%++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++ BOUNDARY CONDITIONS ++++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++++++++++ Ix=1 (X=0) +++++++++++++++++++++++++;
%++++++++++ deformations at the grounding line +++++++++++++;
% In the model the boundary is considered as fixed boundary ;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

%------------- (U=0) ----------------;

for jj=1:1:Ny
for kk=1:1:Nz 
MB((jj-1)*Nz+kk,(jj-1)*Nz+kk)=1; 
end 
end

% for jj=1:1:Ny
% for kk=1:1:Nz 
% VB((jj-1)*Nz+kk)=0; 
% end 
% end

%-------------- (V=0) ----------------;

for jj=1:1:Ny
for kk=1:1:Nz 
MB(Nx*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(jj-1)*Nz+kk)=1; 
end 
end

% for jj=1:1:Ny
% for kk=1:1:Nz 
% VB(Nx*Ny*Nz+(jj-1)*Nz+kk)=0; 
% end 
% end

%-------------- (W=0) ----------------;

for jj=1:1:Ny
for kk=1:1:Nz 
MB(2*Nx*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(jj-1)*Nz+kk)=1; 
end 
end

% for jj=1:1:Ny
% for kk=1:1:Nz, 
% VB(2*Nx*Ny*Nz+(jj-1)*Nz+kk)=0; 
% end 
% end



%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++ Ice Shelf Terminus: Ix=Nx (X=LXSho) +++++++++++++;
%+++++++++++++++++++++ (free edge) +++++++++++++++++++++++++;


%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++++++ The first equation +++++++++++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++ KpE 1/(2*dx) (Fxx)^Nx-2 +++++++++++++++++;

%-------------- 1/(2*dx) Kp1 dU/dx -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-4)*Ny*Nz+(jj-1)*Nz+kk)=alfa2*(1/(2*dx))*Kp1*(-1/(2*dx));            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk)=alfa2*(1/(2*dx))*Kp1*(1/(2*dx));                                                      
end 
end


%-------------- 1/(2*dx) Kp1 dQx dU/dQ -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-2)*Nz+kk)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-2)*Nz+kk)+alfa2*(1/(2*dx))*(-1/(2*dy))*dQx(Nx-2,jj)*Kp1;            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+jj*Nz+kk)=    MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+jj*Nz+kk)+     alfa2*(1/(2*dx))*(1/(2*dy))*dQx(Nx-2,jj)*Kp1;                                                      
end 
end



%-------------- 1/(2*dx) Kp1 dZx dU/dz -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*(1/(2*dx))*(-1/(2*dz))*dZx(Nx-2,jj,kk)*Kp1;            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*(1/(2*dx))*(1/(2*dz))*dZx(Nx-2,jj,kk)*Kp1;                                                      
end 
end


%-------------- 1/(2*dx) Kp2 dQy dV/dQ -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+kk)=alfa2*(1/(2*dx))*(-1/(2*dy))*dQy(Nx-2)*Kp2;            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+kk)=     alfa2*(1/(2*dx))*(1/(2*dy))*dQy(Nx-2)*Kp2;                                                      
end 
end


%-------------- 1/(2*dx) Kp2 dZy dV/dz -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*(1/(2*dx))*(-1/(2*dz))*dZy(Nx-2,jj,kk)*Kp2;            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*(1/(2*dx))*(1/(2*dz))*dZy(Nx-2,jj,kk)*Kp2;                                                      
end 
end


%-------------- 1/(2*dx) Kp2 dZz dW/dz -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)=alfa2*(1/(2*dx))*(-1/(2*dz))*dZz(Nx-2,jj)*Kp2;            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)= alfa2*(1/(2*dx))*(1/(2*dz))*dZz(Nx-2,jj)*Kp2;                                                      
end 
end



%+++++++++++++++ KpE (-4/(2*dx)) Fxx^N-1 +++++++++++++++++;

%-------------- (-4/(2*dx)) Kp1 dU/dx -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk)+alfa2*(-4/(2*dx))*Kp1*(-1/(2*dx));            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa2*(-4/(2*dx))*Kp1*(1/(2*dx));                                                      
end 
end


%-------------- (-4/(2*dx)) Kp1 dQx dU/dQ -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-2)*Nz+kk)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-2)*Nz+kk)+alfa2*(-4/(2*dx))*(-1/(2*dy))*dQx(Nx-1,jj)*Kp1;            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+jj*Nz+kk)=    MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+jj*Nz+kk)+     alfa2*(-4/(2*dx))*(1/(2*dy))*dQx(Nx-1,jj)*Kp1;                                                      
end 
end


%-------------- (-4/(2*dx)) Kp1 dZx dU/dz -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*(-4/(2*dx))*(-1/(2*dz))*dZx(Nx-1,jj,kk)*Kp1;            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*(-4/(2*dx))*(1/(2*dz))*dZx(Nx-1,jj,kk)*Kp1;                                                      
end 
end


%-------------- (-4/(2*dx)) Kp2 dQy dV/dQ -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+kk)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+kk)+alfa2*(-4/(2*dx))*(-1/(2*dy))*dQy(Nx-1)*Kp2;            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+kk)=    MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+kk)+     alfa2*(-4/(2*dx))*(1/(2*dy))*dQy(Nx-1)*Kp2;                                                      
end 
end



%-------------- (-4/(2*dx)) Kp2 dZy dV/dz -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*(-4/(2*dx))*(-1/(2*dz))*dZy(Nx-1,jj,kk)*Kp2;            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*(-4/(2*dx))*(1/(2*dz))*dZy(Nx-1,jj,kk)*Kp2;                                                      
end 
end


%-------------- (-4/(2*dx)) Kp2 dZz dW/dz -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*(-4/(2*dx))*(-1/(2*dz))*dZz(Nx-1,jj)*Kp2;            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*(-4/(2*dx))*(1/(2*dz))*dZz(Nx-1,jj)*Kp2;                                                      
end 
end



%+++++++++++++++++++ + KpE dens Freq^2 U ++++++++++++++++++++++;

for jj=2:1:Ny-1 
for kk=2:1:Nz-1 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk) + alfa2*KpE*dens*Freq^2;
end 
end



%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++++ The second equation +++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++ KpE (1/(2*dx)) Fyx^Nx-2 +++++++++++++++++;

%---------------- (1/(2*dx)) dQy dU/dQ --------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-2)*Nz+kk)=alfa2*(1/(2*dx))*(-1/(2*dy))*dQy(Nx-2);            
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+jj*Nz+kk)=     alfa2*(1/(2*dx))*(1/(2*dy))*dQy(Nx-2);                                                      
end 
end


%-------------- (1/(2*dx)) dZy dU/dz -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*(1/(2*dx))*(-1/(2*dz))*dZy(Nx-2,jj,kk);            
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*(1/(2*dx))*(1/(2*dz))*dZy(Nx-2,jj,kk);                                                      
end 
end

%-------------- (1/(2*dx)) dV/dx ---------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-4)*Ny*Nz+(jj-1)*Nz+kk)=alfa2*(1/(2*dx))*(-1/(2*dx));            
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk)= alfa2*(1/(2*dx))*(1/(2*dx));                                        
end 
end

%------------ (1/(2*dx)) dQx dV/dQ -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+kk)+alfa2*(1/(2*dx))*(-1/(2*dy))*dQx(Nx-2,jj);            
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+kk)+     alfa2*(1/(2*dx))*(1/(2*dy))*dQx(Nx-2,jj);                                                      
end 
end


%------------ (1/(2*dx)) dZx dV/dz -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*(1/(2*dx))*(-1/(2*dz))*dZx(Nx-2,jj,kk);            
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*(1/(2*dx))*(1/(2*dz))*dZx(Nx-2,jj,kk);                                                      
end 
end



%+++++++++++++++ KpE (-4/(2*dx)) Fyx^Nx-1 +++++++++++++++++;

%---------------- (-4/(2*dx)) dQy dU/dQ --------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-2)*Nz+kk)+alfa2*(-4/(2*dx))*(-1/(2*dy))*dQy(Nx-1);            
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+jj*Nz+kk)+     alfa2*(-4/(2*dx))*(1/(2*dy))*dQy(Nx-1);                                                      
end 
end


%-------------- (-4/(2*dx)) dZy dU/dz -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*(-4/(2*dx))*(-1/(2*dz))*dZy(Nx-1,jj,kk);            
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*(-4/(2*dx))*(1/(2*dz))*dZy(Nx-1,jj,kk);                                                      
end 
end

%-------------- (-4/(2*dx)) dV/dx ---------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk)+alfa2*(-4/(2*dx))*(-1/(2*dx));            
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+ alfa2*(-4/(2*dx))*(1/(2*dx));                                        
end 
end

%------------ (-4/(2*dx)) dQx dV/dQ -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+kk)+alfa2*(-4/(2*dx))*(-1/(2*dy))*dQx(Nx-1,jj);            
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+kk)+     alfa2*(-4/(2*dx))*(1/(2*dy))*dQx(Nx-1,jj);                                                      
end 
end


%------------ (-4/(2*dx)) dZx dV/dz -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*(-4/(2*dx))*(-1/(2*dz))*dZx(Nx-1,jj,kk);            
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*(-4/(2*dx))*(1/(2*dz))*dZx(Nx-1,jj,kk);                                                      
end 
end



%+++++++ KpE dQy dFyy/dy + KpE dZy dFyy/dz + KpE dZz dFyz/dz +++++++++;


%-------------- Kp1 dQ/dy d/dQ ( dQ/dy dV/dQ ) -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)+ alfa2*Kp1*(1/dy^2)*dQy(Nx)^2; 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa2*Kp1*(-2/dy^2)*dQy(Nx)^2; 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk)+     alfa2*Kp1*(1/dy^2)*dQy(Nx)^2; 
end 
end


%-------------- Kp1 dQ/dy d/dQ ( dZ/dy dV/dz ) -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa2*Kp1*(dZy(Nx,jj-1,kk)/(4*dy*dz))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa2*Kp1*(-dZy(Nx,jj-1,kk)/(4*dy*dz))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)+    alfa2*Kp1*(-dZy(Nx,jj+1,kk)/(4*dy*dz))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)+     alfa2*Kp1*(dZy(Nx,jj+1,kk)/(4*dy*dz))*dQy(Nx); 
end 
end


%-------------- Kp1 dZ/dy d/dz ( dQ/dy dV/dQ ) -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa2*Kp1*dQy(Nx)*(1/(4*dz*dy))*dZy(Nx,jj,kk); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)+    alfa2*Kp1*dQy(Nx)*(-1/(4*dz*dy))*dZy(Nx,jj,kk); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa2*Kp1*dQy(Nx)*(-1/(4*dz*dy))*dZy(Nx,jj,kk); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)+     alfa2*Kp1*dQy(Nx)*(1/(4*dz*dy))*dZy(Nx,jj,kk); 
end 
end


%------------- Kp1 dZ/dy d/dz ( dZ/dy dV/dz ) -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*Kp1*(Gpz2_ijk(Nx,jj,kk)/dz^2)*dZy(Nx,jj,kk); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa2*Kp1*(-(Gpz2_ijk(Nx,jj,kk)+Gpz2_ijk(Nx,jj,kk+1))/dz^2)*dZy(Nx,jj,kk); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa2*Kp1*(Gpz2_ijk(Nx,jj,kk+1)/dz^2)*dZy(Nx,jj,kk); 
end 
end


%------------------ Kp2 dQ/dy d/dQ ( dU/dx ) -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-2)*Nz+kk)+alfa2*Kp2*(-1/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+jj*Nz+kk)+     alfa2*Kp2*(1/(4*dy*dx))*dQy(Nx);
    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-2)*Nz+kk)+ alfa2*Kp2*(4/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+jj*Nz+kk)+    alfa2*Kp2*(-4/(4*dy*dx))*dQy(Nx); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa2*Kp2*(-3/(4*dy*dx))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk)+     alfa2*Kp2*(3/(4*dy*dx))*dQy(Nx);  
end 
end


%----------------- Kp2 dQ/dy d/dQ ( dQ/dx dU/dQ ) -----------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa2*Kp2*(Gpy1_ijk(Nx,jj)/dy^2)*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa2*Kp2*(-(Gpy1_ijk(Nx,jj)+Gpy1_ijk(Nx,jj+1))/dy^2)*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk)+    alfa2*Kp2*(Gpy1_ijk(Nx,jj+1)/dy^2)*dQy(Nx); 
end 
end


%----------------- Kp2 dQ/dy d/dQ ( dZ/dx dU/dz ) -----------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa2*Kp2*(dZx(Nx,jj-1,kk)/(4*dy*dz))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa2*Kp2*(-dZx(Nx,jj-1,kk)/(4*dy*dz))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk-1)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk-1)+    alfa2*Kp2*(-dZx(Nx,jj+1,kk)/(4*dy*dz))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk+1)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk+1)+     alfa2*Kp2*(dZx(Nx,jj+1,kk)/(4*dy*dz))*dQy(Nx); 
end 
end


%------------------- Kp2 dZ/dy d/dz ( dU/dx ) -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*Kp2*(-1/(4*dz*dx))*dZy(Nx,jj,kk);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*Kp2*(1/(4*dz*dx))*dZy(Nx,jj,kk);     

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa2*Kp2*(4/(4*dz*dx))*dZy(Nx,jj,kk);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa2*Kp2*(-4/(4*dz*dx))*dZy(Nx,jj,kk); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*Kp2*(-3/(4*dz*dx))*dZy(Nx,jj,kk); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*Kp2*(3/(4*dz*dx))*dZy(Nx,jj,kk);
end 
end


%----------------- Kp2 dZ/dy d/dz ( dQ/dx dU/dQ ) ------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa2*Kp2*(dQx(Nx,jj)/(4*dz*dy))*dZy(Nx,jj,kk); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk-1)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk-1)+    alfa2*Kp2*(-dQx(Nx,jj)/(4*dz*dy))*dZy(Nx,jj,kk); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa2*Kp2*(-dQx(Nx,jj)/(4*dz*dy))*dZy(Nx,jj,kk); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk+1)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk+1)+     alfa2*Kp2*(dQx(Nx,jj)/(4*dz*dy))*dZy(Nx,jj,kk); 
end 
end


%---------------- Kp2 dZ/dy d/dz ( dZ/dx dU/dz ) ------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*Kp2*(Gpz1_ijk(Nx,jj,kk)/dz^2)*dZy(Nx,jj,kk); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa2*Kp2*(-(Gpz1_ijk(Nx,jj,kk)+Gpz1_ijk(Nx,jj,kk+1))/dz^2)*dZy(Nx,jj,kk); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa2*Kp2*(Gpz1_ijk(Nx,jj,kk+1)/dz^2)*dZy(Nx,jj,kk); 
end 
end


%---------------- Kp2 dQ/dy d/dQ ( dZ/dz dW/dz ) ------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa2*Kp2*(dZz(Nx,jj-1)/(4*dy*dz))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa2*Kp2*(-dZz(Nx,jj-1)/(4*dy*dz))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)+    alfa2*Kp2*(-dZz(Nx,jj+1)/(4*dy*dz))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)+     alfa2*Kp2*(dZz(Nx,jj+1)/(4*dy*dz))*dQy(Nx);                                                                                          
end 
end



%---------------- Kp2 dZ/dy d/dz ( dZ/dz dW/dz ) ------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*Kp2*dZz(Nx,jj)*(1/dz^2)*dZy(Nx,jj,kk); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+ alfa2*Kp2*dZz(Nx,jj)*(-2/dz^2)*dZy(Nx,jj,kk); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa2*Kp2*dZz(Nx,jj)*(1/dz^2)*dZy(Nx,jj,kk);                                                                              
end 
end



%---------------- dZ/dz d/dz ( dZ/dz dV/dz ) ------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*dZz(Nx,jj)*(1/dz^2)*dZz(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+ alfa2*dZz(Nx,jj)*(-2/dz^2)*dZz(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa2*dZz(Nx,jj)*(1/dz^2)*dZz(Nx,jj);                                                                              
end 
end


%---------------- dZ/dz d/dz ( dQ/dy dW/dQ ) -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa2*dQy(Nx)*(1/(4*dz*dy))*dZz(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)+    alfa2*dQy(Nx)*(-1/(4*dz*dy))*dZz(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa2*dQy(Nx)*(-1/(4*dz*dy))*dZz(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)+     alfa2*dQy(Nx)*(1/(4*dz*dy))*dZz(Nx,jj); 
end 
end


%--------------- dZ/dz d/dz ( dZ/dy dW/dz ) -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*(Gpz2_ijk(Nx,jj,kk)/dz^2)*dZz(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa2*(-(Gpz2_ijk(Nx,jj,kk)+Gpz2_ijk(Nx,jj,kk+1))/dz^2)*dZz(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa2*(Gpz2_ijk(Nx,jj,kk+1)/dz^2)*dZz(Nx,jj); 
end 
end


%+++++++++++++++++++ + KpE dens Freq^2 V ++++++++++++++++++++++;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk) + alfa2*KpE*dens*Freq^2;
end 
end



%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++++++++ The third equation ++++++++++++++++++++++++; 
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++ KpE (1/(2*dx)) Fzx^Nx-2 +++++++++++++++++;

%------------- (1/(2*dx)) dZz dU/dz --------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)=alfa2*(1/(2*dx))*(-1/(2*dz))*dZz(Nx-2,jj);            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)= alfa2*(1/(2*dx))*(1/(2*dz))*dZz(Nx-2,jj);                                                      
end 
end

%-------------- (1/(2*dx)) dW/dx ----------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-4)*Ny*Nz+(jj-1)*Nz+kk)=alfa2*(1/(2*dx))*(-1/(2*dx));            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk)= alfa2*(1/(2*dx))*(1/(2*dx));                                
end 
end 

%------------ (1/(2*dx)) dQx dW/dQ -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+kk)+alfa2*(1/(2*dx))*(-1/(2*dy))*dQx(Nx-2,jj);            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+kk)=    MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+kk) +    alfa2*(1/(2*dx))*(1/(2*dy))*dQx(Nx-2,jj);                                                      
end 
end


%-------------- (1/(2*dx)) dZx dW/dz -------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*(1/(2*dx))*(-1/(2*dz))*dZx(Nx-2,jj,kk);            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*(1/(2*dx))*(1/(2*dz))*dZx(Nx-2,jj,kk);                                                      
end 
end



%+++++++++++++++ KpE (-4/(2*dx)) Fzx^Nx-1 +++++++++++++++++;

%------------- (-4/(2*dx)) dZz dU/dz --------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*(-4/(2*dx))*(-1/(2*dz))*dZz(Nx-1,jj);            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*(-4/(2*dx))*(1/(2*dz))*dZz(Nx-1,jj);                                                      
end 
end

%-------------- (-4/(2*dx)) dW/dx ----------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk)+alfa2*(-4/(2*dx))*(-1/(2*dx));            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+ alfa2*(-4/(2*dx))*(1/(2*dx));                                
end 
end 

%------------ (-4/(2*dx)) dQx dW/dQ -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+kk)+alfa2*(-4/(2*dx))*(-1/(2*dy))*dQx(Nx-1,jj);            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+kk)=    MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+kk) +    alfa2*(-4/(2*dx))*(1/(2*dy))*dQx(Nx-1,jj);                                                      
end 
end


%-------------- (-4/(2*dx)) dZx dW/dz -------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*(-4/(2*dx))*(-1/(2*dz))*dZx(Nx-1,jj,kk);            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*(-4/(2*dx))*(1/(2*dz))*dZx(Nx-1,jj,kk);                                                      
end 
end


%++++++++ KpE dQy dFzy/dy + KpE dZy dFzy/dz + KpE dZz dFzz/dz ++++++++++;


%------------------ dQ/dy d/dQ ( dZ/dz dV/dz ) -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1   
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa2*(dZz(Nx,jj-1)/(4*dy*dz))*dQy(Nx); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa2*(-dZz(Nx,jj-1)/(4*dy*dz))*dQy(Nx); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)+    alfa2*(-dZz(Nx,jj+1)/(4*dy*dz))*dQy(Nx); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)+     alfa2*(dZz(Nx,jj+1)/(4*dy*dz))*dQy(Nx);                                                                                          
end 
end


%----------------- dZ/dy d/dz ( dZ/dz dV/dz ) ------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1   
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*alfa2*dZz(Nx,jj)*(1/dz^2)*dZy(Nx,jj,kk); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+ alfa2*dZz(Nx,jj)*(-2/dz^2)*dZy(Nx,jj,kk); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa2*dZz(Nx,jj)*(1/dz^2)*dZy(Nx,jj,kk);                                                                              
end 
end


%---------------- dQ/dy d/dQ ( dQ/dy dW/dQ ) -----------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1   
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)+ alfa2*dQy(Nx)*(1/dy^2)*dQy(Nx); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa2*dQy(Nx)*(-2/dy^2)*dQy(Nx); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk)=    MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk)+     alfa2*dQy(Nx)*(1/dy^2)*dQy(Nx); 
end 
end


%-------------- dQ/dy d/dQ ( dZ/dy dW/dz ) -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1   
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa2*(dZy(Nx,jj-1,kk)/(4*dy*dz))*dQy(Nx); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa2*(-dZy(Nx,jj-1,kk)/(4*dy*dz))*dQy(Nx); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)+    alfa2*(-dZy(Nx,jj+1,kk)/(4*dy*dz))*dQy(Nx); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)+     alfa2*(dZy(Nx,jj+1,kk)/(4*dy*dz))*dQy(Nx); 
end 
end



%--------------- dZ/dy d/dz ( dQ/dy dW/dQ ) -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1   
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa2*dQy(Nx)*(1/(4*dz*dy))*dZy(Nx,jj,kk); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)+    alfa2*dQy(Nx)*(-1/(4*dz*dy))*dZy(Nx,jj,kk); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa2*dQy(Nx)*(-1/(4*dz*dy))*dZy(Nx,jj,kk); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)+     alfa2*dQy(Nx)*(1/(4*dz*dy))*dZy(Nx,jj,kk); 
end 
end



%--------------- dZ/dy d/dz ( dZ/dy dW/dz ) -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1   
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*(Gpz2_ijk(Nx,jj,kk)/dz^2)*dZy(Nx,jj,kk); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa2*(-(Gpz2_ijk(Nx,jj,kk)+Gpz2_ijk(Nx,jj,kk+1))/dz^2)*dZy(Nx,jj,kk); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa2*(Gpz2_ijk(Nx,jj,kk+1)/dz^2)*dZy(Nx,jj,kk); 
end 
end


%------------ Kp1 dZ/dz d/dz ( dZ/dz dW/dz ) -------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*Kp1*dZz(Nx,jj)*(1/dz^2)*dZz(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+ alfa2*Kp1*dZz(Nx,jj)*(-2/dz^2)*dZz(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa2*Kp1*dZz(Nx,jj)*(1/dz^2)*dZz(Nx,jj);                                                                              
end 
end


%------------------- Kp2 dZ/dz d/dz ( dU/dx ) -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*Kp2*(-1/(4*dz*dx))*dZz(Nx,jj);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*Kp2*(1/(4*dz*dx))*dZz(Nx,jj); 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa2*Kp2*(4/(4*dz*dx))*dZz(Nx,jj);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa2*Kp2*(-4/(4*dz*dx))*dZz(Nx,jj); 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*Kp2*(-3/(4*dz*dx))*dZz(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa2*Kp2*(3/(4*dz*dx))*dZz(Nx,jj); 
end 
end


%----------------- Kp2 dZ/dz d/dz ( dQ/dx dU/dQ ) ------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa2*Kp2*dQx(Nx,jj)*(1/(4*dz*dy))*dZz(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk-1)+    alfa2*Kp2*dQx(Nx,jj)*(-1/(4*dz*dy))*dZz(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa2*Kp2*dQx(Nx,jj)*(-1/(4*dz*dy))*dZz(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk+1)+     alfa2*Kp2*dQx(Nx,jj)*(1/(4*dz*dy))*dZz(Nx,jj); 
end 
end


%--------------- Kp2 dZ/dz d/dz ( dZ/dx dU/dz ) ------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*Kp2*(Gpz1_ijk(Nx,jj,kk)/dz^2)*dZz(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa2*Kp2*(-(Gpz1_ijk(Nx,jj,kk)+Gpz1_ijk(Nx,jj,kk+1))/dz^2)*dZz(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa2*Kp2*(Gpz1_ijk(Nx,jj,kk+1)/dz^2)*dZz(Nx,jj); 
end 
end


%-------------- Kp2 dZ/dz d/dz ( dQ/dy dV/dQ ) -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa2*Kp2*dQy(Nx)*(1/(4*dz*dy))*dZz(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk-1)+    alfa2*Kp2*dQy(Nx)*(-1/(4*dz*dy))*dZz(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa2*Kp2*dQy(Nx)*(-1/(4*dz*dy))*dZz(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk+1)+     alfa2*Kp2*dQy(Nx)*(1/(4*dz*dy))*dZz(Nx,jj); 
end 
end


%------------- Kp2 dZ/dz d/dz ( dZ/dy dV/dz ) -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa2*Kp2*(Gpz2_ijk(Nx,jj,kk)/dz^2)*dZz(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa2*Kp2*(-(Gpz2_ijk(Nx,jj,kk)+Gpz2_ijk(Nx,jj,kk+1))/dz^2)*dZz(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa2*Kp2*(Gpz2_ijk(Nx,jj,kk+1)/dz^2)*dZz(Nx,jj); 
end 
end


%+++++++++++++++++++ + KpE dens Freq^2 W ++++++++++++++++++++++;

for jj=2:1:Ny-1
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk) + alfa2*KpE*dens*Freq^2;
end 
end



%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++ Basic Boundary Conditions at the terminus +++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++ The first equation ( basic equation #1) ++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++ KpE Fxx +++++++++++++++++;

%---------------- Kp1 dU/dx --------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-3)*Ny*Nz+(jj-1)*Nz+kk)+ alfa1*Kp1*(1/(2*dx));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-2)*Ny*Nz+(jj-1)*Nz+kk)+alfa1*Kp1*(-4/(2*dx));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+ alfa1*Kp1*(3/(2*dx));                                                      
end 
end


%-------------- Kp1 dQx dU/dQ -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa1*Kp1*(-1/(2*dy))*dQx(Nx,jj);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk)=    MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk)+     alfa1*Kp1*(1/(2*dy))*dQx(Nx,jj);                                                      
end 
end


%-------------- Kp1 dZx dU/dz -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa1*Kp1*(-1/(2*dz))*dZx(Nx,jj,kk);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa1*Kp1*(1/(2*dz))*dZx(Nx,jj,kk);                                                      
end 
end



%-------------- Kp2 dQy dV/dQ -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa1*Kp2*(-1/(2*dy))*dQy(Nx);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk)=    MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk)+     alfa1*Kp2*(1/(2*dy))*dQy(Nx);                                                      
end 
end


%-------------- Kp2 dZy dV/dz -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa1*Kp2*(-1/(2*dz))*dZy(Nx,jj,kk);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa1*Kp2*(1/(2*dz))*dZy(Nx,jj,kk);                                                      
end 
end


%-------------- Kp2 dZz dW/dz -----------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa1*Kp2*(-1/(2*dz))*dZz(Nx,jj);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa1*Kp2*(1/(2*dz))*dZz(Nx,jj);                                                      
end 
end



%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++ The second equation (basic equation #2) +++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++ KpE Fyx ++++++++++++++++++++;

%------------------ dQy dU/dQ -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa1*(-1/(2*dy))*dQy(Nx);            
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+jj*Nz+kk)+     alfa1*(1/(2*dy))*dQy(Nx);                                                      
end 
end


%------------------ dZy dU/dz -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa1*(-1/(2*dz))*dZy(Nx,jj,kk);            
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa1*(1/(2*dz))*dZy(Nx,jj,kk);                                                      
end 
end

%-------------------- dV/dx ---------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk)+ alfa1*(1/(2*dx));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk)+alfa1*(-4/(2*dx));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+ alfa1*(3/(2*dx));                                        
end 
end

%----------------- dQx dV/dQ -------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa1*(-1/(2*dy))*dQx(Nx,jj);            
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk)+     alfa1*(1/(2*dy))*dQx(Nx,jj);                                                      
end 
end


%----------------- dZx dV/dz ------------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa1*(-1/(2*dz))*dZx(Nx,jj,kk);            
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa1*(1/(2*dz))*dZx(Nx,jj,kk);                                                      
end 
end



%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++ The third equation (basic equation) ++++++++++++; 
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%++++++++++++++ KpE Fzx +++++++++++++++;

%-------------- dZz dU/dz --------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa1*(-1/(2*dz))*dZz(Nx,jj);            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa1*(1/(2*dz))*dZz(Nx,jj);                                                      
end 
end

%---------------- dW/dx ----------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+kk)+ alfa1*(1/(2*dx));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+kk)+alfa1*(-4/(2*dx));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk)+ alfa1*(3/(2*dx));                                
end
end

%-------------- dQx dW/dQ -------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa1*(-1/(2*dy))*dQx(Nx,jj);            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk)=    MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+kk) +    alfa1*(1/(2*dy))*dQx(Nx,jj);                                                      
end 
end


%-------------- dZx dW/dz -------------;

for jj=2:1:Ny-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa1*(-1/(2*dz))*dZx(Nx,jj,kk);            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa1*(1/(2*dz))*dZx(Nx,jj,kk);                                                      
end 
end




%++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++ The lateral edge Y=Y1 (Jy=1) +++++++++++;
%++++++++++++++ (free lateral edge) +++++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++ The first equation +++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++;

%+++++++++++++++++++ KpE dFxx/dx +++++++++++++++++++++;

%------------------- Kp1 d^2U/dx^2 ----------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk)= alfa2*Kp1*(1/dx^2); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=-alfa2*Kp1*(2/dx^2); 
MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk)=     alfa2*Kp1*(1/dx^2); 
end 
end

%-------- i=Nx ------;

for kk=2:1:Nz-1 
MB((Nx-1)*Ny*Nz+kk,(Nx-4)*Ny*Nz+kk)=alfa2*Kp1*(-1/dx^2); 
MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk)= alfa2*Kp1*(4/dx^2); 
MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk)=alfa2*Kp1*(-5/dx^2); 
MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk)= alfa2*Kp1*(2/dx^2); 
end


%----------------- Kp1 d/dx (dQ/dx dU/dQ) --------------;

%--------------- 1: Kp1 dQ/dx d/dx ( dU/dQ ) ------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk)=     MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk)+     alfa2*Kp1*dQx(ii,1)*(3/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+Nz+kk)=  MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+Nz+kk)+  alfa2*Kp1*dQx(ii,1)*(-4/(4*dx*dy));
MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+2*Nz+kk)+alfa2*Kp1*dQx(ii,1)*(1/(4*dx*dy));

MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk)=         MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk)+         alfa2*Kp1*dQx(ii,1)*(-3/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+Nz+kk)=      MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+Nz+kk)+      alfa2*Kp1*dQx(ii,1)*(4/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+2*Nz+kk)=    MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+2*Nz+kk)+    alfa2*Kp1*dQx(ii,1)*(-1/(4*dx*dy));  
end 
end

%-------- i=Nx ------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk)=     MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk)+     alfa2*Kp1*dQx(Nx,1)*(-3/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+Nz+kk)=  MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+Nz+kk)+   alfa2*Kp1*dQx(Nx,1)*(4/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+2*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+2*Nz+kk)+alfa2*Kp1*dQx(Nx,1)*(-1/(4*dx*dy)); 

MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk)=     MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk)+    alfa2*Kp1*dQx(Nx,1)*(12/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+Nz+kk)=  MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+Nz+kk)+alfa2*Kp1*dQx(Nx,1)*(-16/(4*dx*dy));
MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+2*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+2*Nz+kk)+alfa2*Kp1*dQx(Nx,1)*(4/(4*dx*dy));

MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk)=     MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk)+      alfa2*Kp1*dQx(Nx,1)*(-9/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+Nz+kk)=  MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+Nz+kk)+   alfa2*Kp1*dQx(Nx,1)*(12/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+2*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+2*Nz+kk)+ alfa2*Kp1*dQx(Nx,1)*(-3/(4*dx*dy)); 
end

%----------------- 2: Kp1 d^2Q/dx^2 dU/dQ --------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=     MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+     alfa2*Kp1*d2Qxx(ii,1)*(-3/(2*dy)); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)+   alfa2*Kp1*d2Qxx(ii,1)*(4/(2*dy)); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*Kp1*d2Qxx(ii,1)*(-1/(2*dy)); 
end 
end


%--------------------- Kp1 d/dx ( dZ/dx dU/dz ) --------------;

%------------------- 1: Kp1 dZ/dx d/dx ( dU/dz )  ------------;


for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk-1)+ alfa2*Kp1*dZx(ii,1,kk)*(1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk+1)+alfa2*Kp1*dZx(ii,1,kk)*(-1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk-1)=    MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk-1)+    alfa2*Kp1*dZx(ii,1,kk)*(-1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk+1)=    MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk+1)+     alfa2*Kp1*dZx(ii,1,kk)*(1/(4*dx*dz));                                  
end 
end

%-------- i=Nx ------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk-1)=MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk-1)+alfa2*Kp1*dZx(Nx,1,kk)*(-1/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk+1)=MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk+1)+ alfa2*Kp1*dZx(Nx,1,kk)*(1/(4*dx*dz));

MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk-1)=MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk-1)+ alfa2*Kp1*dZx(Nx,1,kk)*(4/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk+1)=MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk+1)+alfa2*Kp1*dZx(Nx,1,kk)*(-4/(4*dx*dz));

MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk-1)=MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk-1)+alfa2*Kp1*dZx(Nx,1,kk)*(-3/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk+1)=MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk+1)+ alfa2*Kp1*dZx(Nx,1,kk)*(3/(4*dx*dz));                                  
end

%------------------- 2: Kp1 d^2Z/dx^2  dU/dz  ------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*Kp1*d2Zxx(ii,1,kk)*(-1/(2*dz)); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+ alfa2*Kp1*d2Zxx(ii,1,kk)*(1/(2*dz)); 
end 
end


%-------------------- Kp1 dQ/dx d/dQ ( dU/dx ) -----------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk)=     MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk)+     alfa2*Kp1*(3/(4*dy*dx))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+Nz+kk)=  MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+Nz+kk)+ alfa2*Kp1*(-4/(4*dy*dx))*dQx(ii,1);
MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+2*Nz+kk)+alfa2*Kp1*(1/(4*dy*dx))*dQx(ii,1);

MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk)=      MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk)+     alfa2*Kp1*(-3/(4*dy*dx))*dQx(ii,1);
MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+Nz+kk)=   MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+Nz+kk)+   alfa2*Kp1*(4/(4*dy*dx))*dQx(ii,1);
MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+2*Nz+kk)= MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+2*Nz+kk)+alfa2*Kp1*(-1/(4*dy*dx))*dQx(ii,1);
end 
end

%-------- i=Nx ------;

for kk=2:1:Nz-1 
MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk)=     MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk)+     alfa2*Kp1*(-3/(4*dy*dx))*dQx(Nx,1);
MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+Nz+kk)=  MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+Nz+kk)+   alfa2*Kp1*(4/(4*dy*dx))*dQx(Nx,1);
MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+2*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+2*Nz+kk)+alfa2*Kp1*(-1/(4*dy*dx))*dQx(Nx,1);

MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk)=     MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk)+     alfa2*Kp1*(12/(4*dy*dx))*dQx(Nx,1);
MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+Nz+kk)=  MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+Nz+kk)+ alfa2*Kp1*(-16/(4*dy*dx))*dQx(Nx,1);
MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+2*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+2*Nz+kk)+ alfa2*Kp1*(4/(4*dy*dx))*dQx(Nx,1);

MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk)=     MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk)+     alfa2*Kp1*(-9/(4*dy*dx))*dQx(Nx,1);
MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+Nz+kk)=  MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+Nz+kk)+  alfa2*Kp1*(12/(4*dy*dx))*dQx(Nx,1); 
MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+2*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+2*Nz+kk)+alfa2*Kp1*(-3/(4*dy*dx))*dQx(Nx,1); 
end 


%---------------- Kp1 dQ/dx d/dQ ( dQ/dx dU/dQ ) -----------------;

%-------------- 1: Kp1 (dQ/dx)^2 d/dQ ( dU/dQ ) -----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=     MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+      alfa2*Kp1*(2/dy^2)*dQx(ii,1)^2;
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)+  alfa2*Kp1*(-5/dy^2)*dQx(ii,1)^2; 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)+ alfa2*Kp1*(4/dy^2)*dQx(ii,1)^2; 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+3*Nz+kk)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+3*Nz+kk)+alfa2*Kp1*(-1/dy^2)*dQx(ii,1)^2; 
end 
end

%----------------- 2: Kp1 dQ/dx d^2Q/dxdq dU/dQ -----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=     MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+     alfa2*Kp1*dQx(ii,1)*d2Qxq(ii,1)*(-3/(2*dy));
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)+   alfa2*Kp1*dQx(ii,1)*d2Qxq(ii,1)*(4/(2*dy)); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*Kp1*dQx(ii,1)*d2Qxq(ii,1)*(-1/(2*dy)); 
end 
end


%-------------- Kp1 dQ/dx d/dQ ( dZ/dx dU/dz ) ------------------;

%------------ 1: Kp1 dQ/dx dZ/dx d/dQ ( dU/dz ) ------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=     MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+      alfa2*Kp1*dZx(ii,1,kk)*(3/(4*dy*dz))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)+  alfa2*Kp1*dZx(ii,1,kk)*(-4/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)+ alfa2*Kp1*dZx(ii,1,kk)*(1/(4*dy*dz))*dQx(ii,1);

MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=     MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+     alfa2*Kp1*dZx(ii,1,kk)*(-3/(4*dy*dz))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*Kp1*dZx(ii,1,kk)*(4/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*Kp1*dZx(ii,1,kk)*(-1/(4*dy*dz))*dQx(ii,1);
end 
end

%----------------- 2: Kp1 dQ/dx d^2Z/dxdq dU/dz  -----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*Kp1*dQx(ii,1)*d2Zxq(ii,1,kk)*(-1/(2*dz)); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+ alfa2*Kp1*dQx(ii,1)*d2Zxq(ii,1,kk)*(1/(2*dz)); 
end 
end


%------------------- Kp1 dZ/dx d/dz ( dU/dx ) -------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk-1)+ alfa2*Kp1*(1/(4*dz*dx))*dZx(ii,1,kk); 
MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk-1)=    MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk-1)+    alfa2*Kp1*(-1/(4*dz*dx))*dZx(ii,1,kk); 
MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk+1)+alfa2*Kp1*(-1/(4*dz*dx))*dZx(ii,1,kk); 
MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk+1)=    MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk+1)+     alfa2*Kp1*(1/(4*dz*dx))*dZx(ii,1,kk); 
end 
end

%-------- i=Nx ------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk-1)=MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk-1)+alfa2*Kp1*(-1/(4*dz*dx))*dZx(Nx,1,kk); 
MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk-1)=MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk-1)+ alfa2*Kp1*(4/(4*dz*dx))*dZx(Nx,1,kk); 
MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk-1)=MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk-1)+alfa2*Kp1*(-3/(4*dz*dx))*dZx(Nx,1,kk);

MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk+1)=MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk+1)+ alfa2*Kp1*(1/(4*dz*dx))*dZx(Nx,1,kk); 
MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk+1)=MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk+1)+alfa2*Kp1*(-4/(4*dz*dx))*dZx(Nx,1,kk); 
MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk+1)=MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk+1)+ alfa2*Kp1*(3/(4*dz*dx))*dZx(Nx,1,kk); 
end 


%------------------- Kp1 dZ/dx d/dz ( dQ/dx dU/dQ ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=     MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+     alfa2*Kp1*dQx(ii,1)*(3/(4*dz*dy))*dZx(ii,1,kk); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*Kp1*dQx(ii,1)*(-4/(4*dz*dy))*dZx(ii,1,kk);
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*Kp1*dQx(ii,1)*(1/(4*dz*dy))*dZx(ii,1,kk);


MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=     MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+     alfa2*Kp1*dQx(ii,1)*(-3/(4*dz*dy))*dZx(ii,1,kk); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*Kp1*dQx(ii,1)*(4/(4*dz*dy))*dZx(ii,1,kk); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*Kp1*dQx(ii,1)*(-1/(4*dz*dy))*dZx(ii,1,kk); 
end 
end


%------------------- Kp1 dZ/dx d/dz ( dZ/dx dU/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*Kp1*(Gpz1_ijk(ii,1,kk)/dz^2)*dZx(ii,1,kk); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+  alfa2*Kp1*(-(Gpz1_ijk(ii,1,kk)+Gpz1_ijk(ii,1,kk+1))/dz^2)*dZx(ii,1,kk); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+alfa2*Kp1*(Gpz1_ijk(ii,1,kk+1)/dz^2)*dZx(ii,1,kk); 
end  
end


%------------------- Kp2 d/dx ( dQ/dy dV/dQ ) -------------------;

%----------------- 1: Kp2 dQ/dy d/dx ( dV/dQ ) -------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)=     alfa2*Kp2*dQy(ii)*(3/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+kk)= alfa2*Kp2*dQy(ii)*(-4/(4*dx*dy));
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+kk)=alfa2*Kp2*dQy(ii)*(1/(4*dx*dy));

MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk)=     alfa2*Kp2*dQy(ii)*(-3/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+Nz+kk)=   alfa2*Kp2*dQy(ii)*(4/(4*dx*dy));   
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+kk)=alfa2*Kp2*dQy(ii)*(-1/(4*dx*dy));  
end 
end

%-------- i=Nx ------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)=     alfa2*Kp2*dQy(Nx)*(-3/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+kk)=   alfa2*Kp2*dQy(Nx)*(4/(4*dx*dy));
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+kk)=alfa2*Kp2*dQy(Nx)*(-1/(4*dx*dy));

MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)=    alfa2*Kp2*dQy(Nx)*(12/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+kk)=alfa2*Kp2*dQy(Nx)*(-16/(4*dx*dy));
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+kk)=alfa2*Kp2*dQy(Nx)*(4/(4*dx*dy));

MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)=     alfa2*Kp2*dQy(Nx)*(-9/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+kk)=  alfa2*Kp2*dQy(Nx)*(12/(4*dx*dy));
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+kk)=alfa2*Kp2*dQy(Nx)*(-3/(4*dx*dy));
end

%----------------- 2: Kp2 d^2Q/dydx dV/dQ  -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+     alfa2*Kp2*d2Qyx(ii)*(-3/(2*dy)); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)+   alfa2*Kp2*d2Qyx(ii)*(4/(2*dy));
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*Kp2*d2Qyx(ii)*(-1/(2*dy));
end 
end


%------------------- Kp2 d/dx ( dZ/dy dV/dz ) -------------------;

%----------------- 1: Kp2 dZ/dy d/dx ( dV/dz ) -------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)+ alfa2*Kp2*dZy(ii,1,kk)*(1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)+alfa2*Kp2*dZy(ii,1,kk)*(-1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk-1)=    MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk-1)+    alfa2*Kp2*dZy(ii,1,kk)*(-1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk+1)=    MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk+1)+     alfa2*Kp2*dZy(ii,1,kk)*(1/(4*dx*dz)); 
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)=MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)+alfa2*Kp2*dZy(Nx,1,kk)*(-1/(4*dx*dz));
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)=MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)+ alfa2*Kp2*dZy(Nx,1,kk)*(4/(4*dx*dz));
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)=MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)+alfa2*Kp2*dZy(Nx,1,kk)*(-3/(4*dx*dz)); 

MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)=MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)+ alfa2*Kp2*dZy(Nx,1,kk)*(1/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)=MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)+alfa2*Kp2*dZy(Nx,1,kk)*(-4/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)=MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)+ alfa2*Kp2*dZy(Nx,1,kk)*(3/(4*dx*dz)); 
end 


%----------------- 2: Kp2 d^2Z/dydx  dV/dz  -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*Kp2*d2Zyx(ii,1,kk)*(-1/(2*dz)); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa2*Kp2*d2Zyx(ii,1,kk)*(1/(2*dz)); 
end 
end


%--------------- Kp2 dQ/dx d/dQ ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+      alfa2*Kp2*dQy(ii)*(2/dy^2)*dQx(ii,1); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)+  alfa2*Kp2*dQy(ii)*(-5/dy^2)*dQx(ii,1); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+ alfa2*Kp2*dQy(ii)*(4/dy^2)*dQx(ii,1);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+kk)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+kk)+alfa2*Kp2*dQy(ii)*(-1/dy^2)*dQx(ii,1);
end 
end


%--------------- Kp2 dQ/dx d/dQ ( dZ/dy dV/dz ) ------------------;

%------------- 1: Kp2 dQ/dx dZ/dy d/dQ ( dV/dz ) ------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=     MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+     alfa2*Kp2*dZy(ii,1,kk)*(3/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=  MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*Kp2*dZy(ii,1,kk)*(-4/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*Kp2*dZy(ii,1,kk)*(1/(4*dy*dz))*dQx(ii,1);

MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=     MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+     alfa2*Kp2*dZy(ii,1,kk)*(-3/(4*dy*dz))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=  MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*Kp2*dZy(ii,1,kk)*(4/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*Kp2*dZy(ii,1,kk)*(-1/(4*dy*dz))*dQx(ii,1);
end 
end

%------------- 2: Kp2 dQ/dx d^2Z/dydq  dV/dz  ------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*Kp2*d2Zyq(ii,1,kk)*(-1/(2*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa2*Kp2*d2Zyq(ii,1,kk)*(1/(2*dz))*dQx(ii,1); 
end 
end


%-------------- Kp2 dZ/dx d/dz ( dQ/dy dV/dQ ) -----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=     MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+     alfa2*Kp2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,1,kk); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=  MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*Kp2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,1,kk);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*Kp2*dQy(ii)*(1/(4*dz*dy))*dZx(ii,1,kk);

MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=     MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+     alfa2*Kp2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,1,kk); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=  MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*Kp2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,1,kk);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*Kp2*dQy(ii)*(-1/(4*dz*dy))*dZx(ii,1,kk);
end 
end


%------------- Kp2 dZ/dx d/dz ( dZ/dy dV/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*Kp2*(Gpz2_ijk(ii,1,kk)/dz^2)*dZx(ii,1,kk); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=  MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+  alfa2*Kp2*(-(Gpz2_ijk(ii,1,kk)+Gpz2_ijk(ii,1,kk+1))/dz^2)*dZx(ii,1,kk); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+alfa2*Kp2*(Gpz2_ijk(ii,1,kk+1)/dz^2)*dZx(ii,1,kk); 
end  
end


%----------------- Kp2 d/dx ( dZ/dz dW/dz ) -----------------;

%--------------- 1: Kp2 dZ/dz d/dx ( dW/dz ) -----------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)= alfa2*Kp2*dZz(ii,1)*(1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)=alfa2*Kp2*dZz(ii,1)*(-1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk-1)=    alfa2*Kp2*dZz(ii,1)*(-1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk+1)=     alfa2*Kp2*dZz(ii,1)*(1/(4*dx*dz));                                  
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)= alfa2*Kp2*dZz(Nx,1)*(-1/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)=  alfa2*Kp2*dZz(Nx,1)*(1/(4*dx*dz)); 

MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)= alfa2*Kp2*dZz(Nx,1)*(4/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)=alfa2*Kp2*dZz(Nx,1)*(-4/(4*dx*dz));

MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)=alfa2*Kp2*dZz(Nx,1)*(-3/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)= alfa2*Kp2*dZz(Nx,1)*(3/(4*dx*dz));                                  
end

%--------------- 2: Kp2 d^2Z/dzdx dW/dz -----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*Kp2*d2Zzx(ii,1)*(-1/(2*dz)); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa2*Kp2*d2Zzx(ii,1)*(1/(2*dz)); 
end 
end


%--------------- Kp2 dQ/dx d/dQ ( dZ/dz dW/dz ) ---------------------;

%------------- 1: Kp2 dQ/dx dZ/dz d/dQ ( dW/dz ) ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=     MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+     alfa2*Kp2*dZz(ii,1)*(3/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=  MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*Kp2*dZz(ii,1)*(-4/(4*dy*dz))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*Kp2*dZz(ii,1)*(1/(4*dy*dz))*dQx(ii,1); 


MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=     MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+     alfa2*Kp2*dZz(ii,1)*(-3/(4*dy*dz))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=  MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*Kp2*dZz(ii,1)*(4/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*Kp2*dZz(ii,1)*(-1/(4*dy*dz))*dQx(ii,1);
end 
end

%------------- 2: Kp2 dQ/dx d^2Z/dzdq dW/dz ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*Kp2*d2Zzq(ii,1)*(-1/(2*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa2*Kp2*d2Zzq(ii,1)*(1/(2*dz))*dQx(ii,1); 
end 
end



%---------------- Kp2 dZ/dx d/dz ( dZ/dz dW/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*Kp2*dZz(ii,1)*(1/dz^2)*dZx(ii,1,kk); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=  MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+ alfa2*Kp2*dZz(ii,1)*(-2/dz^2)*dZx(ii,1,kk); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+alfa2*Kp2*dZz(ii,1)*(1/dz^2)*dZx(ii,1,kk); 
end  
end



%++++++++++++++++ KpE dQy (-1/(2*dy)) (Fxy)^3 +++++++++++++++;

%----------------- dQy (-1/(2*dy)) dQy dU/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk) + alfa2*dQy(ii)*(-1/(2*dy))*(-1/(2*dy))*dQy(ii);
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+3*Nz+kk)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+3*Nz+kk)+alfa2*dQy(ii)*(-1/(2*dy))*(1/(2*dy))*dQy(ii);                              
end 
end

%---------------- dQy (-1/(2*dy)) dZy dU/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dQy(ii)*(-1/(2*dy))*(-1/(2*dz))*dZy(ii,3,kk);
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*dQy(ii)*(-1/(2*dy))*(1/(2*dz))*dZy(ii,3,kk);                                                          
end 
end

%----------------- dQy (-1/(2*dy)) dV/dx -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+kk)+alfa2*dQy(ii)*(-1/(2*dy))*(-1/(2*dx));
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+kk)=    MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+kk) +   alfa2*dQy(ii)*(-1/(2*dy))*(1/(2*dx));                                          
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+kk)+ alfa2*dQy(Nx)*(-1/(2*dy))*(1/(2*dx));    
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+kk)+alfa2*dQy(Nx)*(-1/(2*dy))*(-4/(2*dx));
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+kk)+ alfa2*dQy(Nx)*(-1/(2*dy))*(3/(2*dx));                                          
end


%--------------- dQy (-1/(2*dy)) dQx dV/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)+ alfa2*dQy(ii)*(-1/(2*dy))*(-1/(2*dy))*dQx(ii,3);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+kk)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+kk)+alfa2*dQy(ii)*(-1/(2*dy))*(1/(2*dy))*dQx(ii,3);                              
end 
end

%--------------- dQy (-1/(2*dy)) dZx dV/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dQy(ii)*(-1/(2*dy))*(-1/(2*dz))*dZx(ii,3,kk);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*dQy(ii)*(-1/(2*dy))*(1/(2*dz))*dZx(ii,3,kk);                                                          
end 
end


%++++++++++++++ KpE dQy (4/(2*dy)) (Fxy)^2 +++++++++++++++++;

%----------------- dQy (4/(2*dy)) dQy dU/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=     MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk) +   alfa2*dQy(ii)*(4/(2*dy))*(-1/(2*dy))*dQy(ii);
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*dQy(ii)*(4/(2*dy))*(1/(2*dy))*dQy(ii);                              
end 
end

%---------------- dQy (4/(2*dy)) dZy dU/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)+alfa2*dQy(ii)*(4/(2*dy))*(-1/(2*dz))*dZy(ii,2,kk);
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)+ alfa2*dQy(ii)*(4/(2*dy))*(1/(2*dz))*dZy(ii,2,kk);                                                          
end 
end

%----------------- dQy (4/(2*dy)) dV/dx -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+kk)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+kk)+alfa2*dQy(ii)*(4/(2*dy))*(-1/(2*dx));
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+Nz+kk)=    MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+Nz+kk) +    alfa2*dQy(ii)*(4/(2*dy))*(1/(2*dx));                                          
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+kk)=MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+kk)+ alfa2*dQy(Nx)*(4/(2*dy))*(1/(2*dx));    
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+kk)=MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+kk)+alfa2*dQy(Nx)*(4/(2*dy))*(-4/(2*dx));
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+kk)=MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+kk)+ alfa2*dQy(Nx)*(4/(2*dy))*(3/(2*dx));                                          
end


%--------------- dQy (4/(2*dy)) dQx dV/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+    alfa2*dQy(ii)*(4/(2*dy))*(-1/(2*dy))*dQx(ii,2);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*dQy(ii)*(4/(2*dy))*(1/(2*dy))*dQx(ii,2);                              
end 
end

%--------------- dQy (4/(2*dy)) dZx dV/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+alfa2*dQy(ii)*(4/(2*dy))*(-1/(2*dz))*dZx(ii,2,kk);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+ alfa2*dQy(ii)*(4/(2*dy))*(1/(2*dz))*dZx(ii,2,kk);                                                          
end 
end


%++++++++++++++ KpE dQy (-3/(2*dy)) (Fxx)^1 nx  ++++++++++++++;

%-------------- dQy (-3/(2*dy)) Kp1 dU/dx Y1x ---------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk)=MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk)+alfa2*Kp1*dQy(ii)*(-3/(2*dy))*(-1/(2*dx))*difY1(ii);
MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk)=    MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk)    + alfa2*Kp1*dQy(ii)*(-3/(2*dy))*(1/(2*dx))*difY1(ii);                                          
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk)+ alfa2*Kp1*dQy(Nx)*(-3/(2*dy))*(1/(2*dx))*difY1(Nx);
MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk)+alfa2*Kp1*dQy(Nx)*(-3/(2*dy))*(-4/(2*dx))*difY1(Nx);
MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk)+ alfa2*Kp1*dQy(Nx)*(-3/(2*dy))*(3/(2*dx))*difY1(Nx);                                          
end 


%---------- dQy (-3/(2*dy)) Kp1 dQx dU/dQ Y1x -------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=     MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk) +    alfa2*Kp1*dQy(ii)*(-3/(2*dy))*(-3/(2*dy))*dQx(ii,1)*difY1(ii);
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk) +  alfa2*Kp1*dQy(ii)*(-3/(2*dy))*(4/(2*dy))*dQx(ii,1)*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*Kp1*dQy(ii)*(-3/(2*dy))*(-1/(2*dy))*dQx(ii,1)*difY1(ii);
end 
end

%---------- dQy (-3/(2*dy)) Kp1 dZx dU/dz Y1x ---------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*Kp1*dQy(ii)*(-3/(2*dy))*(-1/(2*dz))*dZx(ii,1,kk)*difY1(ii);
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+ alfa2*Kp1*dQy(ii)*(-3/(2*dy))*(1/(2*dz))*dZx(ii,1,kk)*difY1(ii);                                                          
end 
end


%---------- dQy (-3/(2*dy)) Kp2 dQy dV/dQ Y1x ---------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk) +    alfa2*Kp2*dQy(ii)*(-3/(2*dy))*(-3/(2*dy))*dQy(ii)*difY1(ii);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk) +  alfa2*Kp2*dQy(ii)*(-3/(2*dy))*(4/(2*dy))*dQy(ii)*difY1(ii);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*Kp2*dQy(ii)*(-3/(2*dy))*(-1/(2*dy))*dQy(ii)*difY1(ii);
end 
end

%---------- dQy (-3/(2*dy)) Kp2 dZy dV/dz Y1x --------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*Kp2*dQy(ii)*(-3/(2*dy))*(-1/(2*dz))*dZy(ii,1,kk)*difY1(ii);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa2*Kp2*dQy(ii)*(-3/(2*dy))*(1/(2*dz))*dZy(ii,1,kk)*difY1(ii);                                                          
end 
end

%----------- dQy (-3/(2*dy)) Kp2 dZz dW/dz Y1x -------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*Kp2*dQy(ii)*(-3/(2*dy))*(-1/(2*dz))*dZz(ii,1)*difY1(ii);
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa2*Kp2*dQy(ii)*(-3/(2*dy))*(1/(2*dz))*dZz(ii,1)*difY1(ii);                                                          
end 
end



%++++++++++++++++++++++++ KpE dQy (dFxx/dz) nx  +++++++++++++++++++++;


%------------------- dZ/dy Kp1 d/dz ( dU/dx ) Y1x -------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk-1)+ alfa2*Kp1*dZy(ii,1,kk)*(1/(4*dz*dx))*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk-1)=    MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk-1)+    alfa2*Kp1*dZy(ii,1,kk)*(-1/(4*dz*dx))*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk+1)+alfa2*Kp1*dZy(ii,1,kk)*(-1/(4*dz*dx))*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk+1)=    MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk+1)+     alfa2*Kp1*dZy(ii,1,kk)*(1/(4*dz*dx))*difY1(ii); 
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1 
MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk-1)=MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk-1)+alfa2*Kp1*dZy(Nx,1,kk)*(-1/(4*dz*dx))*difY1(Nx);
MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk+1)=MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk+1)+ alfa2*Kp1*dZy(Nx,1,kk)*(1/(4*dz*dx))*difY1(Nx); 

MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk-1)=MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk-1)+ alfa2*Kp1*dZy(Nx,1,kk)*(4/(4*dz*dx))*difY1(Nx);
MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk+1)=MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk+1)+alfa2*Kp1*dZy(Nx,1,kk)*(-4/(4*dz*dx))*difY1(Nx); 

MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk-1)=MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk-1)+alfa2*Kp1*dZy(Nx,1,kk)*(-3/(4*dz*dx))*difY1(Nx); 
MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk+1)=MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk+1)+ alfa2*Kp1*dZy(Nx,1,kk)*(3/(4*dz*dx))*difY1(Nx); 
end 


%--------------- dZ/dy Kp1 d/dz ( dQ/dx dU/dQ ) Y1x ------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=     MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+     alfa2*Kp1*dZy(ii,1,kk)*dQx(ii,1)*(3/(4*dz*dy))*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*Kp1*dZy(ii,1,kk)*dQx(ii,1)*(-4/(4*dz*dy))*difY1(ii);
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*Kp1*dZy(ii,1,kk)*dQx(ii,1)*(1/(4*dz*dy))*difY1(ii);

MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=     MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+     alfa2*Kp1*dZy(ii,1,kk)*dQx(ii,1)*(-3/(4*dz*dy))*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*Kp1*dZy(ii,1,kk)*dQx(ii,1)*(4/(4*dz*dy))*difY1(ii);
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*Kp1*dZy(ii,1,kk)*dQx(ii,1)*(-1/(4*dz*dy))*difY1(ii);
end 
end


%-------------- dZ/dy Kp1 d/dz ( dZ/dx dU/dz ) Y1x -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*Kp1*dZy(ii,1,kk)*(Gpz1_ijk(ii,1,kk)/dz^2)*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+  alfa2*Kp1*dZy(ii,1,kk)*(-(Gpz1_ijk(ii,1,kk)+Gpz1_ijk(ii,1,kk+1))/dz^2)*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+alfa2*Kp1*dZy(ii,1,kk)*(Gpz1_ijk(ii,1,kk+1)/dz^2)*difY1(ii); 
end  
end


%--------------- dZ/dy Kp2 d/dz ( dQ/dy dV/dQ ) Y1x ---------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=     MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+     alfa2*Kp2*dZy(ii,1,kk)*dQy(ii)*(3/(4*dz*dy))*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=  MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*Kp2*dZy(ii,1,kk)*dQy(ii)*(-4/(4*dz*dy))*difY1(ii);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*Kp2*dZy(ii,1,kk)*dQy(ii)*(1/(4*dz*dy))*difY1(ii);

MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=     MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+     alfa2*Kp2*dZy(ii,1,kk)*dQy(ii)*(-3/(4*dz*dy))*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=  MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*Kp2*dZy(ii,1,kk)*dQy(ii)*(4/(4*dz*dy))*difY1(ii);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*Kp2*dZy(ii,1,kk)*dQy(ii)*(-1/(4*dz*dy))*difY1(ii);
end 
end


%---------------- dZ/dy Kp2 d/dz ( dZ/dy dV/dz ) Y1x -----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*Kp2*dZy(ii,1,kk)*(Gpz2_ijk(ii,1,kk)/dz^2)*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=  MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+  alfa2*Kp2*dZy(ii,1,kk)*(-(Gpz2_ijk(ii,1,kk)+Gpz2_ijk(ii,1,kk+1))/dz^2)*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+alfa2*Kp2*dZy(ii,1,kk)*(Gpz2_ijk(ii,1,kk+1)/dz^2)*difY1(ii); 
end  
end

%---------------- dZ/dy Kp2 d/dz ( dZ/dz dW/dz ) Y1x -----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+ alfa2*Kp2*dZy(ii,1,kk)*dZz(ii,1)*(1/dz^2)*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=  MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+  alfa2*Kp2*dZy(ii,1,kk)*dZz(ii,1)*(-2/dz^2)*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa2*Kp2*dZy(ii,1,kk)*dZz(ii,1)*(1/dz^2)*difY1(ii); 
end  
end


%++++++++++++++++++++++++ KpE dFxz/dz +++++++++++++++++++++++++;

%---------------- dZ/dz d/dz ( dZ/dz dU/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*dZz(ii,1)*(1/dz^2)*dZz(ii,1); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+ alfa2*dZz(ii,1)*(-2/dz^2)*dZz(ii,1); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+alfa2*dZz(ii,1)*(1/dz^2)*dZz(ii,1); 
end  
end


%------------------- dZ/dz d/dz ( dW/dx ) -------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)+ alfa2*(1/(4*dz*dx))*dZz(ii,1); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk-1)=    MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk-1)+    alfa2*(-1/(4*dz*dx))*dZz(ii,1); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)+alfa2*(-1/(4*dz*dx))*dZz(ii,1); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk+1)=    MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk+1)+     alfa2*(1/(4*dz*dx))*dZz(ii,1); 
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)=MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)+alfa2*(-1/(4*dz*dx))*dZz(Nx,1);
MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)=MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)+ alfa2*(1/(4*dz*dx))*dZz(Nx,1); 

MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)=MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)+ alfa2*(4/(4*dz*dx))*dZz(Nx,1);
MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)=MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)+alfa2*(-4/(4*dz*dx))*dZz(Nx,1); 

MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)=MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)+alfa2*(-3/(4*dz*dx))*dZz(Nx,1); 
MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)=MB((Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)+ alfa2*(3/(4*dz*dx))*dZz(Nx,1); 
end 


%------------------- dZ/dz d/dz ( dQ/dx dW/dQ ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=     MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+     alfa2*dQx(ii,1)*(3/(4*dz*dy))*dZz(ii,1); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=  MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*dQx(ii,1)*(-4/(4*dz*dy))*dZz(ii,1);
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dQx(ii,1)*(1/(4*dz*dy))*dZz(ii,1);

MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=     MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+     alfa2*dQx(ii,1)*(-3/(4*dz*dy))*dZz(ii,1); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=  MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*dQx(ii,1)*(4/(4*dz*dy))*dZz(ii,1);
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*dQx(ii,1)*(-1/(4*dz*dy))*dZz(ii,1);
end 
end


%------------------- dZ/dz d/dz ( dZ/dx dW/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*(Gpz1_ijk(ii,1,kk)/dz^2)*dZz(ii,1); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=  MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+  alfa2*(-(Gpz1_ijk(ii,1,kk)+Gpz1_ijk(ii,1,kk+1))/dz^2)*dZz(ii,1); 
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+alfa2*(Gpz1_ijk(ii,1,kk+1)/dz^2)*dZz(ii,1); 
end  
end



%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++ + KpE dens Freq^2 Ui +++++++++++++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk) + alfa2*KpE*dens*Freq^2;
end  
end






%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++++ The second equation +++++++++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++++++++ KpE dFyx/dx ++++++++++++++++++++++++++;

%--------------------- d/dx ( dQ/dy dU/dQ ) ---------------------;

%------------------- 1: dQ/dy d/dx ( dU/dQ ) ---------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk)=     alfa2*dQy(ii)*(3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+Nz+kk)= alfa2*dQy(ii)*(-4/(4*dx*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+2*Nz+kk)=alfa2*dQy(ii)*(1/(4*dx*dy));

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk)=     alfa2*dQy(ii)*(-3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+Nz+kk)=   alfa2*dQy(ii)*(4/(4*dx*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+2*Nz+kk)=alfa2*dQy(ii)*(-1/(4*dx*dy));
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk)=     alfa2*dQy(Nx)*(-3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+Nz+kk)=   alfa2*dQy(Nx)*(4/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+2*Nz+kk)=alfa2*dQy(Nx)*(-1/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk)=    alfa2*dQy(Nx)*(12/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+Nz+kk)=alfa2*dQy(Nx)*(-16/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+2*Nz+kk)=alfa2*dQy(Nx)*(4/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk)=     alfa2*dQy(Nx)*(-9/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+Nz+kk)=  alfa2*dQy(Nx)*(12/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+2*Nz+kk)=alfa2*dQy(Nx)*(-3/(4*dx*dy));
end 

%------------------- 2: d^2Q/dydx  dU/dQ  ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+     alfa2*d2Qyx(ii)*(-3/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)+   alfa2*d2Qyx(ii)*(4/(2*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*d2Qyx(ii)*(-1/(2*dy));
end 
end



%--------------------- d/dx ( dZ/dy dU/dz ) ---------------------;

%------------------- 1: dZ/dy d/dx ( dU/dz ) ---------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk-1)+ alfa2*dZy(ii,1,kk)*(1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk+1)+alfa2*dZy(ii,1,kk)*(-1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk-1)+    alfa2*dZy(ii,1,kk)*(-1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk+1)+     alfa2*dZy(ii,1,kk)*(1/(4*dx*dz));                                  
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk-1)+alfa2*dZy(Nx,1,kk)*(-1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk+1)+ alfa2*dZy(Nx,1,kk)*(1/(4*dx*dz));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk-1)+ alfa2*dZy(Nx,1,kk)*(4/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk+1)+alfa2*dZy(Nx,1,kk)*(-4/(4*dx*dz));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk-1)+ alfa2*dZy(Nx,1,kk)*(-3/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk+1)+  alfa2*dZy(Nx,1,kk)*(3/(4*dx*dz));                                  
end

%------------------- 2: d^2Z/dydx dU/dz  ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*d2Zyx(ii,1,kk)*(-1/(2*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+ alfa2*d2Zyx(ii,1,kk)*(1/(2*dz)); 
end 
end


%----------------- dQ/dx d/dQ ( dQ/dy dU/dQ ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+      alfa2*dQy(ii)*(2/dy^2)*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)+  alfa2*dQy(ii)*(-5/dy^2)*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)+ alfa2*dQy(ii)*(4/dy^2)*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+3*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+3*Nz+kk)+alfa2*dQy(ii)*(-1/dy^2)*dQx(ii,1); 
end 
end

%----------------- dQ/dx d/dQ ( dZ/dy dU/dz ) -------------------;

%--------------- 1: dQ/dx dZ/dy d/dQ ( dU/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+     alfa2*dZy(ii,1,kk)*(3/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*dZy(ii,1,kk)*(-4/(4*dy*dz))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dZy(ii,1,kk)*(1/(4*dy*dz))*dQx(ii,1); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+     alfa2*dZy(ii,1,kk)*(-3/(4*dy*dz))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*dZy(ii,1,kk)*(4/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*dZy(ii,1,kk)*(-1/(4*dy*dz))*dQx(ii,1);
end 
end

%----------------- 2: dQ/dx d^2Z/dydq dU/dz --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*d2Zyq(ii,1,kk)*(-1/(2*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+ alfa2*d2Zyq(ii,1,kk)*(1/(2*dz))*dQx(ii,1); 
end 
end


%----------------- dZ/dx d/dz ( dQ/dy dU/dQ ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+     alfa2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,1,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,1,kk);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dQy(ii)*(1/(4*dz*dy))*dZx(ii,1,kk);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+     alfa2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,1,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,1,kk);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZx(ii,1,kk);
end 
end


%------------- dZ/dx d/dz ( dZ/dy dU/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*(Gpz2_ijk(ii,1,kk)/dz^2)*dZx(ii,1,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+alfa2*(-(Gpz2_ijk(ii,1,kk)+Gpz2_ijk(ii,1,kk+1))/dz^2)*dZx(ii,1,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+alfa2*(Gpz2_ijk(ii,1,kk+1)/dz^2)*dZx(ii,1,kk); 
end  
end

%--------------------- d/dx ( dV/dx ) -----------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)= alfa2*(1/dx^2); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=alfa2*(-2/dx^2); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk)=     alfa2*(1/dx^2); 
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-4)*Ny*Nz+kk)=alfa2*(-1/dx^2);     
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)= alfa2*(4/dx^2); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)=alfa2*(-5/dx^2); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)= alfa2*(2/dx^2); 
end 


%--------------------- d/dx ( dQ/dx dV/dQ ) ---------------------;

%------------------- 1: dQ/dx d/dx ( dV/dQ ) ---------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)+     alfa2*dQx(ii,1)*(3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+kk)+ alfa2*dQx(ii,1)*(-4/(4*dx*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+kk)+alfa2*dQx(ii,1)*(1/(4*dx*dy));

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk)+     alfa2*dQx(ii,1)*(-3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+Nz+kk)+   alfa2*dQx(ii,1)*(4/(4*dx*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+kk)+alfa2*dQx(ii,1)*(-1/(4*dx*dy));
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)+     alfa2*dQx(Nx,1)*(-3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+kk)+   alfa2*dQx(Nx,1)*(4/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+kk)+alfa2*dQx(Nx,1)*(-1/(4*dx*dy));    
    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)+    alfa2*dQx(Nx,1)*(12/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+kk)+alfa2*dQx(Nx,1)*(-16/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+kk)+alfa2*dQx(Nx,1)*(4/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)+     alfa2*dQx(Nx,1)*(-9/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+kk)+  alfa2*dQx(Nx,1)*(12/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+kk)+alfa2*dQx(Nx,1)*(-3/(4*dx*dy));
end 

%------------------- 2: d^2Q/dx^2  dV/dQ  ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+     alfa2*d2Qxx(ii,1)*(-3/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)+   alfa2*d2Qxx(ii,1)*(4/(2*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*d2Qxx(ii,1)*(-1/(2*dy));
end 
end


%--------------------- d/dx ( dZ/dx dV/dz ) ---------------------;

%------------------- 1: dZ/dx d/dx ( dV/dz ) --------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)+ alfa2*dZx(ii,1,kk)*(1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)+alfa2*dZx(ii,1,kk)*(-1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk-1)+    alfa2*dZx(ii,1,kk)*(-1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk+1)+     alfa2*dZx(ii,1,kk)*(1/(4*dx*dz)); 
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)+alfa2*dZx(Nx,1,kk)*(-1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)+ alfa2*dZx(Nx,1,kk)*(1/(4*dx*dz)); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)+ alfa2*dZx(Nx,1,kk)*(4/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)+alfa2*dZx(Nx,1,kk)*(-4/(4*dx*dz));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)+alfa2*dZx(Nx,1,kk)*(-3/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)+ alfa2*dZx(Nx,1,kk)*(3/(4*dx*dz)); 
end

%-------------------- 2: d^2Z/dxdx dV/dz ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*d2Zxx(ii,1,kk)*(-1/(2*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa2*d2Zxx(ii,1,kk)*(1/(2*dz)); 
end 
end


%-------------------- dQ/dx d/dQ ( dV/dx ) ---------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)+     alfa2*(3/(4*dy*dx))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+kk)+ alfa2*(-4/(4*dy*dx))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+kk)+alfa2*(1/(4*dy*dx))*dQx(ii,1); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk)+     alfa2*(-3/(4*dy*dx))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+Nz+kk)+   alfa2*(4/(4*dy*dx))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+kk)+alfa2*(-1/(4*dy*dx))*dQx(ii,1);
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)+     alfa2*(-3/(4*dy*dx))*dQx(Nx,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+kk)+   alfa2*(4/(4*dy*dx))*dQx(Nx,1); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+kk)+alfa2*(-1/(4*dy*dx))*dQx(Nx,1); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)+    alfa2*(12/(4*dy*dx))*dQx(Nx,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+kk)+alfa2*(-16/(4*dy*dx))*dQx(Nx,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+kk)+alfa2*(4/(4*dy*dx))*dQx(Nx,1);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)+     alfa2*(-9/(4*dy*dx))*dQx(Nx,1); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+kk)+  alfa2*(12/(4*dy*dx))*dQx(Nx,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+kk)+alfa2*(-3/(4*dy*dx))*dQx(Nx,1);
end 


%------------------ dQ/dx d/dQ ( dQ/dx dV/dQ ) --------------------;

%---------------- 1: dQ/dx dQ/dx d/dQ ( dV/dQ ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+      alfa2*(2/dy^2)*dQx(ii,1)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)+  alfa2*(-5/dy^2)*dQx(ii,1)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+ alfa2*(4/dy^2)*dQx(ii,1)^2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+kk)+alfa2*(-1/dy^2)*dQx(ii,1)^2;
end 
end

%---------------- 2: dQ/dx d^2Q/dxdq  dV/dQ --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+     alfa2*d2Qxq(ii,1)*(-3/(2*dy))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)+   alfa2*d2Qxq(ii,1)*(4/(2*dy))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*d2Qxq(ii,1)*(-1/(2*dy))*dQx(ii,1);
end 
end


%-------------------- dQ/dx d/dQ ( dZ/dx dV/dz ) --------------------;

%------------------ 1: dQ/dx dZ/dx d/dQ ( dV/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+     alfa2*dZx(ii,1,kk)*(3/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*dZx(ii,1,kk)*(-4/(4*dy*dz))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dZx(ii,1,kk)*(1/(4*dy*dz))*dQx(ii,1); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+     alfa2*dZx(ii,1,kk)*(-3/(4*dy*dz))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*dZx(ii,1,kk)*(4/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*dZx(ii,1,kk)*(-1/(4*dy*dz))*dQx(ii,1);
end 
end

%----------------- 2: dQ/dx d^2Z/dxdq dV/dz -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*d2Zxq(ii,1,kk)*(-1/(2*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa2*d2Zxq(ii,1,kk)*(1/(2*dz))*dQx(ii,1); 
end 
end


%-------------------- dZ/dx d/dz ( dV/dx ) --------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)+ alfa2*(1/(4*dz*dx))*dZx(ii,1,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk-1)+    alfa2*(-1/(4*dz*dx))*dZx(ii,1,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)+alfa2*(-1/(4*dz*dx))*dZx(ii,1,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk+1)+     alfa2*(1/(4*dz*dx))*dZx(ii,1,kk); 
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)+alfa2*(-1/(4*dz*dx))*dZx(Nx,1,kk);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)+ alfa2*(1/(4*dz*dx))*dZx(Nx,1,kk); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)+ alfa2*(4/(4*dz*dx))*dZx(Nx,1,kk);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)+alfa2*(-4/(4*dz*dx))*dZx(Nx,1,kk); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)+alfa2*(-3/(4*dz*dx))*dZx(Nx,1,kk); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)+ alfa2*(3/(4*dz*dx))*dZx(Nx,1,kk); 
end 


%-------------------- dZ/dx d/dz ( dQ/dx dV/dQ ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+     alfa2*dQx(ii,1)*(3/(4*dz*dy))*dZx(ii,1,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*dQx(ii,1)*(-4/(4*dz*dy))*dZx(ii,1,kk);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dQx(ii,1)*(1/(4*dz*dy))*dZx(ii,1,kk);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+     alfa2*dQx(ii,1)*(-3/(4*dz*dy))*dZx(ii,1,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*dQx(ii,1)*(4/(4*dz*dy))*dZx(ii,1,kk);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*dQx(ii,1)*(-1/(4*dz*dy))*dZx(ii,1,kk);
end 
end


%-------------------- dZ/dx d/dz ( dZ/dx dV/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*(Gpz1_ijk(ii,1,kk)/dz^2)*dZx(ii,1,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+alfa2*(-(Gpz1_ijk(ii,1,kk)+Gpz1_ijk(ii,1,kk+1))/dz^2)*dZx(ii,1,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+alfa2*(Gpz1_ijk(ii,1,kk+1)/dz^2)*dZx(ii,1,kk);                                                                            
end 
end


%+++++++++++++++ KpE dQy (-1/(2*dy)) (Fyy)^3 +++++++++++++++++;

%---------------- dQy (-1/(2*dy)) Kp1 dQy dV/dQ --------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk) + alfa2*dQy(ii)*(-1/(2*dy))*(-1/(2*dy))*dQy(ii)*Kp1;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+kk)+alfa2*dQy(ii)*(-1/(2*dy))*(1/(2*dy))*dQy(ii)*Kp1;                              
end 
end

%--------------  dQy (-1/(2*dy)) Kp1 dZy dV/dz ---------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dQy(ii)*(-1/(2*dy))*(-1/(2*dz))*dZy(ii,3,kk)*Kp1;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+ alfa2*dQy(ii)*(-1/(2*dy))*(1/(2*dz))*dZy(ii,3,kk)*Kp1;                                                          
end 
end

%---------------- dQy (-1/(2*dy)) Kp2 dU/dx -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+2*Nz+kk)+alfa2*dQy(ii)*(-1/(2*dy))*(-1/(2*dx))*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+2*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+2*Nz+kk)    + alfa2*dQy(ii)*(-1/(2*dy))*(1/(2*dx))*Kp2;                                          
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+2*Nz+kk)+ alfa2*dQy(Nx)*(-1/(2*dy))*(1/(2*dx))*Kp2;
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+2*Nz+kk)+alfa2*dQy(Nx)*(-1/(2*dy))*(-4/(2*dx))*Kp2;
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+2*Nz+kk)+ alfa2*dQy(Nx)*(-1/(2*dy))*(3/(2*dx))*Kp2;                                          
end 


%-------------- dQy (-1/(2*dy)) Kp2 dQx dU/dQ ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)+ alfa2*dQy(ii)*(-1/(2*dy))*(-1/(2*dy))*dQx(ii,3)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+3*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+3*Nz+kk)+alfa2*dQy(ii)*(-1/(2*dy))*(1/(2*dy))*dQx(ii,3)*Kp2;                              
end 
end

%-------------  dQy (-1/(2*dy)) Kp2 dZx dU/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dQy(ii)*(-1/(2*dy))*(-1/(2*dz))*dZx(ii,3,kk)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)+ alfa2*dQy(ii)*(-1/(2*dy))*(1/(2*dz))*dZx(ii,3,kk)*Kp2;                                                          
end 
end

%--------------- dQy (-1/(2*dy)) Kp2 dZz dW/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dQy(ii)*(-1/(2*dy))*(-1/(2*dz))*dZz(ii,3)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+ alfa2*dQy(ii)*(-1/(2*dy))*(1/(2*dz))*dZz(ii,3)*Kp2;                                                          
end 
end


%+++++++++++++++ KpE dQy (4/(2*dy)) (Fyy)^2 +++++++++++++++++;

%---------------- dQy 4/(2*dy) Kp1 dQy dV/dQ --------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk) +   alfa2*dQy(ii)*(4/(2*dy))*(-1/(2*dy))*dQy(ii)*Kp1;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*dQy(ii)*(4/(2*dy))*(1/(2*dy))*dQy(ii)*Kp1;                              
end 
end

%--------------  dQy 4/(2*dy) Kp1 dZy dV/dz ---------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+alfa2*dQy(ii)*(4/(2*dy))*(-1/(2*dz))*dZy(ii,2,kk)*Kp1;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+ alfa2*dQy(ii)*(4/(2*dy))*(1/(2*dz))*dZy(ii,2,kk)*Kp1;                                                          
end 
end

%---------------- dQy 4/(2*dy) Kp2 dU/dx -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+Nz+kk)+alfa2*dQy(ii)*(4/(2*dy))*(-1/(2*dx))*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+Nz+kk)    + alfa2*dQy(ii)*(4/(2*dy))*(1/(2*dx))*Kp2;                                          
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+Nz+kk)+ alfa2*dQy(Nx)*(4/(2*dy))*(1/(2*dx))*Kp2;
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+Nz+kk)+alfa2*dQy(Nx)*(4/(2*dy))*(-4/(2*dx))*Kp2;
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+Nz+kk)+ alfa2*dQy(Nx)*(4/(2*dy))*(3/(2*dx))*Kp2;                                          
end 


%-------------- dQy 4/(2*dy) Kp2 dQx dU/dQ ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+    alfa2*dQy(ii)*(4/(2*dy))*(-1/(2*dy))*dQx(ii,2)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*dQy(ii)*(4/(2*dy))*(1/(2*dy))*dQx(ii,2)*Kp2;                              
end 
end

%-------------  dQy 4/(2*dy) Kp2 dZx dU/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)+alfa2*dQy(ii)*(4/(2*dy))*(-1/(2*dz))*dZx(ii,2,kk)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)+ alfa2*dQy(ii)*(4/(2*dy))*(1/(2*dz))*dZx(ii,2,kk)*Kp2;                                                          
end 
end

%--------------- dQy 4/(2*dy) Kp2 dZz dW/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+alfa2*dQy(ii)*(4/(2*dy))*(-1/(2*dz))*dZz(ii,2)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+ alfa2*dQy(ii)*(4/(2*dy))*(1/(2*dz))*dZz(ii,2)*Kp2;                                                          
end 
end


%+++++++++++++ KpE dQy (-3/(2*dy)) (Fyx)^1 nx ++++++++++++++;

%------------- dQy (-3/(2*dy)) dQy dU/dQ Y1x ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+     alfa2*dQy(ii)*(-3/(2*dy))*(-3/(2*dy))*dQy(ii)*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)+   alfa2*dQy(ii)*(-3/(2*dy))*(4/(2*dy))*dQy(ii)*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*dQy(ii)*(-3/(2*dy))*(-1/(2*dy))*dQy(ii)*difY1(ii);
end 
end

%------------- dQy (-3/(2*dy)) dZy dU/dz Y1x ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*dQy(ii)*(-3/(2*dy))*(-1/(2*dz))*dZy(ii,1,kk)*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+ alfa2*dQy(ii)*(-3/(2*dy))*(1/(2*dz))*dZy(ii,1,kk)*difY1(ii);                                                          
end 
end

%--------------- dQy (-3/(2*dy)) dV/dx Y1x ------------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)+alfa2*dQy(ii)*(-3/(2*dy))*(-1/(2*dx))*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk)+     alfa2*dQy(ii)*(-3/(2*dy))*(1/(2*dx))*difY1(ii);                                          
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)+ alfa2*dQy(Nx)*(-3/(2*dy))*(1/(2*dx))*difY1(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)+alfa2*dQy(Nx)*(-3/(2*dy))*(-4/(2*dx))*difY1(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)+ alfa2*dQy(Nx)*(-3/(2*dy))*(3/(2*dx))*difY1(Nx);                                          
end 


%-------------- dQy (-3/(2*dy)) dQx dV/dQ Y1x ---------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+     alfa2*dQy(ii)*(-3/(2*dy))*(-3/(2*dy))*dQx(ii,1)*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)+   alfa2*dQy(ii)*(-3/(2*dy))*(4/(2*dy))*dQx(ii,1)*difY1(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*dQy(ii)*(-3/(2*dy))*(-1/(2*dy))*dQx(ii,1)*difY1(ii); 
end 
end

%-------------- dQy (-3/(2*dy)) dZx dV/dz Y1x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*dQy(ii)*(-3/(2*dy))*(-1/(2*dz))*dZx(ii,1,kk)*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa2*dQy(ii)*(-3/(2*dy))*(1/(2*dz))*dZx(ii,1,kk)*difY1(ii);                                                          
end 
end


%++++++++++++++++++++ KpE dZy (dFyx/dz) nx  +++++++++++++++++++++;

%----------------- dZ/dy d/dz ( dQ/dy dU/dQ ) Y1x ----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+     alfa2*dZy(ii,1,kk)*dQy(ii)*(3/(4*dz*dy))*difY1(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*dZy(ii,1,kk)*dQy(ii)*(-4/(4*dz*dy))*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dZy(ii,1,kk)*dQy(ii)*(1/(4*dz*dy))*difY1(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+     alfa2*dZy(ii,1,kk)*dQy(ii)*(-3/(4*dz*dy))*difY1(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*dZy(ii,1,kk)*dQy(ii)*(4/(4*dz*dy))*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*dZy(ii,1,kk)*dQy(ii)*(-1/(4*dz*dy))*difY1(ii);
end 
end

%----------------- dZ/dy d/dz ( dZ/dy dU/dz ) Y1x -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*dZy(ii,1,kk)*(Gpz2_ijk(ii,1,kk)/dz^2)*difY1(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+  alfa2*dZy(ii,1,kk)*(-(Gpz2_ijk(ii,1,kk)+Gpz2_ijk(ii,1,kk+1))/dz^2)*difY1(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+alfa2*dZy(ii,1,kk)*(Gpz2_ijk(ii,1,kk+1)/dz^2)*difY1(ii); 
end  
end


%-------------------- dZ/dy d/dz ( dV/dx ) Y1x --------------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)+ alfa2*dZy(ii,1,kk)*(1/(4*dz*dx))*difY1(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk-1)+    alfa2*dZy(ii,1,kk)*(-1/(4*dz*dx))*difY1(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)+alfa2*dZy(ii,1,kk)*(-1/(4*dz*dx))*difY1(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk+1)+     alfa2*dZy(ii,1,kk)*(1/(4*dz*dx))*difY1(ii); 
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)+alfa2*dZy(Nx,1,kk)*(-1/(4*dz*dx))*difY1(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)+ alfa2*dZy(Nx,1,kk)*(1/(4*dz*dx))*difY1(Nx); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)+ alfa2*dZy(Nx,1,kk)*(4/(4*dz*dx))*difY1(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)+alfa2*dZy(Nx,1,kk)*(-4/(4*dz*dx))*difY1(Nx); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)+alfa2*dZy(Nx,1,kk)*(-3/(4*dz*dx))*difY1(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)+ alfa2*dZy(Nx,1,kk)*(3/(4*dz*dx))*difY1(Nx); 
end 


%----------------- dZ/dy d/dz ( dQ/dx dV/dQ ) Y1x -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+     alfa2*dZy(ii,1,kk)*dQx(ii,1)*(3/(4*dz*dy))*difY1(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*dZy(ii,1,kk)*dQx(ii,1)*(-4/(4*dz*dy))*difY1(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dZy(ii,1,kk)*dQx(ii,1)*(1/(4*dz*dy))*difY1(ii); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+     alfa2*dZy(ii,1,kk)*dQx(ii,1)*(-3/(4*dz*dy))*difY1(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*dZy(ii,1,kk)*dQx(ii,1)*(4/(4*dz*dy))*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*dZy(ii,1,kk)*dQx(ii,1)*(-1/(4*dz*dy))*difY1(ii);
end 
end

%----------------- dZ/dy d/dz ( dZ/dx dV/dz ) Y1x -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*dZy(ii,1,kk)*(Gpz1_ijk(ii,1,kk)/dz^2)*difY1(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+alfa2*dZy(ii,1,kk)*(-(Gpz1_ijk(ii,1,kk)+Gpz1_ijk(ii,1,kk+1))/dz^2)*difY1(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+alfa2*dZy(ii,1,kk)*(Gpz1_ijk(ii,1,kk+1)/dz^2)*difY1(ii);                                                                            
end 
end


%+++++++++++++++++++++ KpE dZz (dFyz/dz) +++++++++++++++++++++++;

%---------------- dZ/dz d/dz ( dZ/dz dV/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*dZz(ii,1)*(1/dz^2)*dZz(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+ alfa2*dZz(ii,1)*(-2/dz^2)*dZz(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+alfa2*dZz(ii,1)*(1/dz^2)*dZz(ii,1); 
end  
end

%----------------- dZ/dz d/dz ( dQ/dy dW/dQ ) ------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+     alfa2*dQy(ii)*(3/(4*dz*dy))*dZz(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*dQy(ii)*(-4/(4*dz*dy))*dZz(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dQy(ii)*(1/(4*dz*dy))*dZz(ii,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+     alfa2*dQy(ii)*(-3/(4*dz*dy))*dZz(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*dQy(ii)*(4/(4*dz*dy))*dZz(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZz(ii,1);
end 
end

%----------------- dZ/dz d/dz ( dZ/dy dW/dz ) ------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*(Gpz2_ijk(ii,1,kk)/dz^2)*dZz(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+alfa2*(-(Gpz2_ijk(ii,1,kk)+Gpz2_ijk(ii,1,kk+1))/dz^2)*dZz(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+alfa2*(Gpz2_ijk(ii,1,kk+1)/dz^2)*dZz(ii,1); 
end 
end



%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++++ + KpE dens Freq^2 V ++++++++++++++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk) + alfa2*KpE*dens*Freq^2;
end  
end




%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++++++ The third equation ++++++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++++++++ KpE dFzx/dx ++++++++++++++++++++++++++;

%--------------------- d/dx ( dZ/dz dU/dz ) ---------------------;

%------------------- 1: dZ/dz d/dx ( dU/dz ) ---------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk-1)= alfa2*dZz(ii,1)*(1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk+1)=alfa2*dZz(ii,1)*(-1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk-1)=    alfa2*dZz(ii,1)*(-1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk+1)=     alfa2*dZz(ii,1)*(1/(4*dx*dz));                                  
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk-1)=alfa2*dZz(Nx,1)*(-1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk+1)= alfa2*dZz(Nx,1)*(1/(4*dx*dz));
    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk-1)= alfa2*dZz(Nx,1)*(4/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk+1)=alfa2*dZz(Nx,1)*(-4/(4*dx*dz));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk-1)=alfa2*dZz(Nx,1)*(-3/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk+1)= alfa2*dZz(Nx,1)*(3/(4*dx*dz));                                  
end

%------------------- 2: d^2Z/dzdx  dU/dz ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*d2Zzx(ii,1)*(-1/(2*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+ alfa2*d2Zzx(ii,1)*(1/(2*dz)); 
end 
end


%-------------------- dQ/dx d/dQ ( dZ/dz dU/dz ) --------------------;

%------------------ 1: dQ/dx dZ/dz d/dQ ( dU/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+     alfa2*dZz(ii,1)*(3/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*dZz(ii,1)*(-4/(4*dy*dz))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dZz(ii,1)*(1/(4*dy*dz))*dQx(ii,1); 

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+     alfa2*dZz(ii,1)*(-3/(4*dy*dz))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*dZz(ii,1)*(4/(4*dy*dz))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*dZz(ii,1)*(-1/(4*dy*dz))*dQx(ii,1); 
end 
end

%------------------- 2: dQ/dx d^2Z/dzdq  dU/dz --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*d2Zzq(ii,1)*(-1/(2*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+ alfa2*d2Zzq(ii,1)*(1/(2*dz))*dQx(ii,1); 
end 
end



%------------------- dZ/dx d/dz ( dZ/dz dU/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*dZz(ii,1)*(1/dz^2)*dZx(ii,1,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+ alfa2*dZz(ii,1)*(-2/dz^2)*dZx(ii,1,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+alfa2*dZz(ii,1)*(1/dz^2)*dZx(ii,1,kk); 
end  
end

%------------------------ d/dx ( dW/dx ) -----------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)= alfa2*(1/dx^2); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=alfa2*(-2/dx^2); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk)=     alfa2*(1/dx^2); 
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-4)*Ny*Nz+kk)=alfa2*(-1/dx^2);     
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)= alfa2*(4/dx^2); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)=alfa2*(-5/dx^2); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)= alfa2*(2/dx^2); 
end 


%--------------------- d/dx ( dQ/dx dW/dQ ) ---------------------;

%------------------- 1: dQ/dx d/dx ( dW/dQ ) --------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)+     alfa2*dQx(ii,1)*(3/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+kk)+ alfa2*dQx(ii,1)*(-4/(4*dx*dy));
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+kk)+alfa2*dQx(ii,1)*(1/(4*dx*dy));

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk)+     alfa2*dQx(ii,1)*(-3/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+Nz+kk)+   alfa2*dQx(ii,1)*(4/(4*dx*dy));
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+2*Nz+kk)+alfa2*dQx(ii,1)*(-1/(4*dx*dy));
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)+     alfa2*dQx(Nx,1)*(-3/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+kk)+   alfa2*dQx(Nx,1)*(4/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+kk)+alfa2*dQx(Nx,1)*(-1/(4*dx*dy));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)+    alfa2*dQx(Nx,1)*(12/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+kk)+alfa2*dQx(Nx,1)*(-16/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+kk)+alfa2*dQx(Nx,1)*(4/(4*dx*dy));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)+     alfa2*dQx(Nx,1)*(-9/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+kk)+  alfa2*dQx(Nx,1)*(12/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+kk)+alfa2*dQx(Nx,1)*(-3/(4*dx*dy));
end 

%--------------------- 2: d^2Q/dxdx dW/dQ ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+     alfa2*d2Qxx(ii,1)*(-3/(2*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)+   alfa2*d2Qxx(ii,1)*(4/(2*dy));
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*d2Qxx(ii,1)*(-1/(2*dy));
end 
end


%--------------------- d/dx ( dZ/dx dW/dz ) ---------------------;

%------------------- 1: dZ/dx d/dx ( dW/dz ) ---------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)+ alfa2*dZx(ii,1,kk)*(1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)+alfa2*dZx(ii,1,kk)*(-1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk-1)+    alfa2*dZx(ii,1,kk)*(-1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk+1)+     alfa2*dZx(ii,1,kk)*(1/(4*dx*dz)); 
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)+alfa2*dZx(Nx,1,kk)*(-1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)+ alfa2*dZx(Nx,1,kk)*(1/(4*dx*dz));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)+ alfa2*dZx(Nx,1,kk)*(4/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)+alfa2*dZx(Nx,1,kk)*(-4/(4*dx*dz));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)+alfa2*dZx(Nx,1,kk)*(-3/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)+ alfa2*dZx(Nx,1,kk)*(3/(4*dx*dz)); 
end

%-------------------- 2: d^2Z/dxdx  dW/dz ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*d2Zxx(ii,1,kk)*(-1/(2*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa2*d2Zxx(ii,1,kk)*(1/(2*dz)); 
end 
end


%-------------------- dQ/dx d/dQ ( dW/dx ) ---------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)+     alfa2*(3/(4*dy*dx))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+kk)+ alfa2*(-4/(4*dy*dx))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+kk)+alfa2*(1/(4*dy*dx))*dQx(ii,1); 

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk)+     alfa2*(-3/(4*dy*dx))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+Nz+kk)+   alfa2*(4/(4*dy*dx))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+2*Nz+kk)+alfa2*(-1/(4*dy*dx))*dQx(ii,1);
end 
end

%-------- i=Nx -------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)+     alfa2*(-3/(4*dy*dx))*dQx(Nx,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+kk)+   alfa2*(4/(4*dy*dx))*dQx(Nx,1); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+kk)+alfa2*(-1/(4*dy*dx))*dQx(Nx,1); 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)+    alfa2*(12/(4*dy*dx))*dQx(Nx,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+kk)+alfa2*(-16/(4*dy*dx))*dQx(Nx,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+kk)+alfa2*(4/(4*dy*dx))*dQx(Nx,1);

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)+     alfa2*(-9/(4*dy*dx))*dQx(Nx,1); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+kk)+  alfa2*(12/(4*dy*dx))*dQx(Nx,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+kk)+alfa2*(-3/(4*dy*dx))*dQx(Nx,1);
end 


%------------------- dQ/dx d/dQ ( dQ/dx dW/dQ ) --------------------;

%----------------- 1: dQ/dx dQ/dx d/dQ ( dW/dQ ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+      alfa2*(2/dy^2)*dQx(ii,1)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)+  alfa2*(-5/dy^2)*dQx(ii,1)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+ alfa2*(4/dy^2)*dQx(ii,1)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+kk)+alfa2*(-1/dy^2)*dQx(ii,1)^2;
end 
end

%----------------- 2: dQ/dx d^2Q/dxdq  dW/dQ  --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+     alfa2*d2Qxq(ii,1)*(-3/(2*dy))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)+   alfa2*d2Qxq(ii,1)*(4/(2*dy))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*d2Qxq(ii,1)*(-1/(2*dy))*dQx(ii,1); 
end 
end


%-------------------- dQ/dx d/dQ ( dZ/dx dW/dz ) --------------------;

%----------------- 1: dQ/dx dZ/dx d/dQ ( dW/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+     alfa2*dZx(ii,1,kk)*(3/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*dZx(ii,1,kk)*(-4/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dZx(ii,1,kk)*(1/(4*dy*dz))*dQx(ii,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+     alfa2*dZx(ii,1,kk)*(-3/(4*dy*dz))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*dZx(ii,1,kk)*(4/(4*dy*dz))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*dZx(ii,1,kk)*(-1/(4*dy*dz))*dQx(ii,1); 
end 
end

%------------------- 2: dQ/dx d^2Z/dxdq dW/dz --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*d2Zxq(ii,1,kk)*(-1/(2*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa2*d2Zxq(ii,1,kk)*(1/(2*dz))*dQx(ii,1); 
end 
end


%-------------------- dZ/dx d/dz ( dW/dx ) --------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)+ alfa2*(1/(4*dz*dx))*dZx(ii,1,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk-1)+    alfa2*(-1/(4*dz*dx))*dZx(ii,1,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)+alfa2*(-1/(4*dz*dx))*dZx(ii,1,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk+1)+     alfa2*(1/(4*dz*dx))*dZx(ii,1,kk); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)+alfa2*(-1/(4*dz*dx))*dZx(Nx,1,kk);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)+ alfa2*(1/(4*dz*dx))*dZx(Nx,1,kk); 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)+ alfa2*(4/(4*dz*dx))*dZx(Nx,1,kk);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)+alfa2*(-4/(4*dz*dx))*dZx(Nx,1,kk); 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)+alfa2*(-3/(4*dz*dx))*dZx(Nx,1,kk); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)+ alfa2*(3/(4*dz*dx))*dZx(Nx,1,kk); 
end 


%-------------------- dZ/dx d/dz ( dQ/dx dW/dQ ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+     alfa2*dQx(ii,1)*(3/(4*dz*dy))*dZx(ii,1,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*dQx(ii,1)*(-4/(4*dz*dy))*dZx(ii,1,kk);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dQx(ii,1)*(1/(4*dz*dy))*dZx(ii,1,kk);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+     alfa2*dQx(ii,1)*(-3/(4*dz*dy))*dZx(ii,1,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*dQx(ii,1)*(4/(4*dz*dy))*dZx(ii,1,kk);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*dQx(ii,1)*(-1/(4*dz*dy))*dZx(ii,1,kk);
end 
end

%-------------------- dZ/dx d/dz ( dZ/dx dW/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*(Gpz1_ijk(ii,1,kk)/dz^2)*dZx(ii,1,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+alfa2*(-(Gpz1_ijk(ii,1,kk)+Gpz1_ijk(ii,1,kk+1))/dz^2)*dZx(ii,1,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+alfa2*(Gpz1_ijk(ii,1,kk+1)/dz^2)*dZx(ii,1,kk);                                                                            
end  
end


%++++++++++++++ KpE dQy (-1/(2*dy)) (Fzy)^3 ++++++++++++++++;

%--------------- dQy (-1/(2*dy)) dZz dV/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dQy(ii)*(-1/(2*dy))*(-1/(2*dz))*dZz(ii,3);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+ alfa2*dQy(ii)*(-1/(2*dy))*(1/(2*dz))*dZz(ii,3);                                                          
end 
end

%--------------- dQy (-1/(2*dy)) dQy dW/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)+ alfa2*dQy(ii)*(-1/(2*dy))*(-1/(2*dy))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+kk)+alfa2*dQy(ii)*(-1/(2*dy))*(1/(2*dy))*dQy(ii);                              
end 
end

%--------------- dQy (-1/(2*dy)) dZy dW/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dQy(ii)*(-1/(2*dy))*(-1/(2*dz))*dZy(ii,3,kk);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+ alfa2*dQy(ii)*(-1/(2*dy))*(1/(2*dz))*dZy(ii,3,kk);                                                          
end 
end


%++++++++++++++ KpE dQy 4/(2*dy) (Fzy)^2 ++++++++++++++++;

%--------------- dQy (4/(2*dy)) dZz dV/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+alfa2*dQy(ii)*(4/(2*dy))*(-1/(2*dz))*dZz(ii,2);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+ alfa2*dQy(ii)*(4/(2*dy))*(1/(2*dz))*dZz(ii,2);                                                          
end 
end

%--------------- dQy (4/(2*dy)) dQy dW/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+    alfa2*dQy(ii)*(4/(2*dy))*(-1/(2*dy))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*dQy(ii)*(4/(2*dy))*(1/(2*dy))*dQy(ii);                              
end 
end

%--------------- dQy (4/(2*dy)) dZy dW/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+alfa2*dQy(ii)*(4/(2*dy))*(-1/(2*dz))*dZy(ii,2,kk);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+ alfa2*dQy(ii)*(4/(2*dy))*(1/(2*dz))*dZy(ii,2,kk);                                                          
end 
end


%+++++++++++++++ KpE dQy (-3/(2*dy)) (Fzx)^1 nx +++++++++++++++;

%--------------- dQy (-3/(2*dy)) dZz dU/dz Y1x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*dQy(ii)*(-3/(2*dy))*(-1/(2*dz))*dZz(ii,1)*difY1(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+ alfa2*dQy(ii)*(-3/(2*dy))*(1/(2*dz))*dZz(ii,1)*difY1(ii);                                                          
end 
end

%----------------- dQy (-3/(2*dy)) dW/dx Y1x -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)+alfa2*dQy(ii)*(-3/(2*dy))*(-1/(2*dx))*difY1(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk)+     alfa2*dQy(ii)*(-3/(2*dy))*(1/(2*dx))*difY1(ii);                                          
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)+ alfa2*dQy(Nx)*(-3/(2*dy))*(1/(2*dx))*difY1(Nx);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)+alfa2*dQy(Nx)*(-3/(2*dy))*(-4/(2*dx))*difY1(Nx);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)+ alfa2*dQy(Nx)*(-3/(2*dy))*(3/(2*dx))*difY1(Nx);                                          
end 


%---------------- dQy (-3/(2*dy)) dQx dW/dQ Y1x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk) +    alfa2*dQy(ii)*(-3/(2*dy))*(-3/(2*dy))*dQx(ii,1)*difY1(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk) +  alfa2*dQy(ii)*(-3/(2*dy))*(4/(2*dy))*dQx(ii,1)*difY1(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+alfa2*dQy(ii)*(-3/(2*dy))*(-1/(2*dy))*dQx(ii,1)*difY1(ii);
end 
end

%----------------- dQy (-3/(2*dy)) dZx dW/dz Y1x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*dQy(ii)*(-3/(2*dy))*(-1/(2*dz))*dZx(ii,1,kk)*difY1(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa2*dQy(ii)*(-3/(2*dy))*(1/(2*dz))*dZx(ii,1,kk)*difY1(ii);                                                          
end 
end


%++++++++++++++++++ KpE dZy (dFzx/dz) nx  +++++++++++++++++++++;

%------------- dZ/dy d/dz ( dZ/dz dU/dz ) Y1x ------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*dZy(ii,1,kk)*dZz(ii,1)*(1/dz^2)*difY1(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+ alfa2*dZy(ii,1,kk)*dZz(ii,1)*(-2/dz^2)*difY1(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+alfa2*dZy(ii,1,kk)*dZz(ii,1)*(1/dz^2)*difY1(ii); 
end  
end

%----------------- dZ/dy d/dz ( dW/dx ) Y1x --------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk-1)+ alfa2*dZy(ii,1,kk)*(1/(4*dz*dx))*difY1(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk-1)+    alfa2*dZy(ii,1,kk)*(-1/(4*dz*dx))*difY1(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk+1)+alfa2*dZy(ii,1,kk)*(-1/(4*dz*dx))*difY1(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk+1)+     alfa2*dZy(ii,1,kk)*(1/(4*dz*dx))*difY1(ii); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk-1)+alfa2*dZy(Nx,1,kk)*(-1/(4*dz*dx))*difY1(Nx);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk+1)+ alfa2*dZy(Nx,1,kk)*(1/(4*dz*dx))*difY1(Nx);     

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk-1)+ alfa2*dZy(Nx,1,kk)*(4/(4*dz*dx))*difY1(Nx);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk+1)+alfa2*dZy(Nx,1,kk)*(-4/(4*dz*dx))*difY1(Nx); 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk-1)+alfa2*dZy(Nx,1,kk)*(-3/(4*dz*dx))*difY1(Nx); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk+1)+ alfa2*dZy(Nx,1,kk)*(3/(4*dz*dx))*difY1(Nx); 
end 


%-------------- dZ/dy d/dz ( dQ/dx dW/dQ ) Y1x -----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+     alfa2*dZy(ii,1,kk)*dQx(ii,1)*(3/(4*dz*dy))*difY1(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*dZy(ii,1,kk)*dQx(ii,1)*(-4/(4*dz*dy))*difY1(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*dZy(ii,1,kk)*dQx(ii,1)*(1/(4*dz*dy))*difY1(ii);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+     alfa2*dZy(ii,1,kk)*dQx(ii,1)*(-3/(4*dz*dy))*difY1(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*dZy(ii,1,kk)*dQx(ii,1)*(4/(4*dz*dy))*difY1(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*dZy(ii,1,kk)*dQx(ii,1)*(-1/(4*dz*dy))*difY1(ii);
end 
end

%--------------- dZ/dy d/dz ( dZ/dx dW/dz ) Y1x -----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*dZy(ii,1,kk)*(Gpz1_ijk(ii,1,kk)/dz^2)*difY1(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+alfa2*dZy(ii,1,kk)*(-(Gpz1_ijk(ii,1,kk)+Gpz1_ijk(ii,1,kk+1))/dz^2)*difY1(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+alfa2*dZy(ii,1,kk)*(Gpz1_ijk(ii,1,kk+1)/dz^2)*difY1(ii);                                                                            
end  
end


%+++++++++++++++++++++++++ KpE dZz (dFzz/dz) +++++++++++++++++++++++++;

%------------------- Kp1 dZ/dz d/dz ( dZ/dz dW/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*Kp1*dZz(ii,1)*(1/dz^2)*dZz(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+ alfa2*Kp1*dZz(ii,1)*(-2/dz^2)*dZz(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+alfa2*Kp1*dZz(ii,1)*(1/dz^2)*dZz(ii,1); 
end  
end

%-------------------- Kp2 dZ/dz d/dz ( dU/dx ) ----------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk-1)+ alfa2*Kp2*(1/(4*dz*dx))*dZz(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk-1)+    alfa2*Kp2*(-1/(4*dz*dx))*dZz(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk+1)+alfa2*Kp2*(-1/(4*dz*dx))*dZz(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk+1)+     alfa2*Kp2*(1/(4*dz*dx))*dZz(ii,1); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk-1)+alfa2*Kp2*(-1/(4*dz*dx))*dZz(Nx,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk+1)+ alfa2*Kp2*(1/(4*dz*dx))*dZz(Nx,1); 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk-1)+ alfa2*Kp2*(4/(4*dz*dx))*dZz(Nx,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk+1)+alfa2*Kp2*(-4/(4*dz*dx))*dZz(Nx,1); 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk-1)+alfa2*Kp2*(-3/(4*dz*dx))*dZz(Nx,1); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk+1)+ alfa2*Kp2*(3/(4*dz*dx))*dZz(Nx,1); 
end 


%----------------- Kp2 dZ/dz d/dz ( dQ/dx dU/dQ ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+     alfa2*Kp2*dQx(ii,1)*(3/(4*dz*dy))*dZz(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*Kp2*dQx(ii,1)*(-4/(4*dz*dy))*dZz(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*Kp2*dQx(ii,1)*(1/(4*dz*dy))*dZz(ii,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+     alfa2*Kp2*dQx(ii,1)*(-3/(4*dz*dy))*dZz(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*Kp2*dQx(ii,1)*(4/(4*dz*dy))*dZz(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*Kp2*dQx(ii,1)*(-1/(4*dz*dy))*dZz(ii,1);
end 
end

%---------------- Kp2 dZ/dz d/dz ( dZ/dx dU/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa2*Kp2*(Gpz1_ijk(ii,1,kk)/dz^2)*dZz(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+  alfa2*Kp2*(-(Gpz1_ijk(ii,1,kk)+Gpz1_ijk(ii,1,kk+1))/dz^2)*dZz(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+alfa2*Kp2*(Gpz1_ijk(ii,1,kk+1)/dz^2)*dZz(ii,1);                                                                            
end  
end


%--------------- Kp2 dZ/dz d/dz ( dQ/dy dV/dQ ) ---------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+     alfa2*Kp2*dQy(ii)*(3/(4*dz*dy))*dZz(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk-1)+ alfa2*Kp2*dQy(ii)*(-4/(4*dz*dy))*dZz(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk-1)+alfa2*Kp2*dQy(ii)*(1/(4*dz*dy))*dZz(ii,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+     alfa2*Kp2*dQy(ii)*(-3/(4*dz*dy))*dZz(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk+1)+   alfa2*Kp2*dQy(ii)*(4/(4*dz*dy))*dZz(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk+1)+alfa2*Kp2*dQy(ii)*(-1/(4*dz*dy))*dZz(ii,1);
end 
end

%----------------- Kp2 dZ/dz d/dz ( dZ/dy dV/dz ) ------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa2*Kp2*(Gpz2_ijk(ii,1,kk)/dz^2)*dZz(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+  alfa2*Kp2*(-(Gpz2_ijk(ii,1,kk)+Gpz2_ijk(ii,1,kk+1))/dz^2)*dZz(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+alfa2*Kp2*(Gpz2_ijk(ii,1,kk+1)/dz^2)*dZz(ii,1); 
end  
end


%++++++++++++++++++++ + KpE dens Freq^2 W ++++++++++++++++++++++;


for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk) + alfa2*KpE*dens*Freq^2; 
end  
end





%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++ Basic Boundary Conditions at the lateral edge Y=Y1 ++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++ The first equation (basic equation#1) +++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

%++++++++++++++ Fxx nx + Fxy ny = fx(z) ++++++++++++++;

%----------------- Kp1 dU/dx Y1x -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk)=MB((ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk)+alfa1*Kp1*(-1/(2*dx))*difY1(ii);
MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk)=    MB((ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk)+    alfa1*Kp1*(1/(2*dx))*difY1(ii);                                          
end 
end


%-------- i=Nx ------;

for kk=2:1:Nz-1 
MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk)+ alfa1*Kp1*(1/(2*dx))*difY1(ii); 
MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk)+alfa1*Kp1*(-4/(2*dx))*difY1(ii); 
MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk)+ alfa1*Kp1*(3/(2*dx))*difY1(ii); 
end


%----------------- Kp1 dQx dU/dQ --------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=     MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+     alfa1*Kp1*dQx(ii,1)*(-3/(2*dy))*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)+   alfa1*Kp1*dQx(ii,1)*(4/(2*dy))*difY1(ii); 
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)+alfa1*Kp1*dQx(ii,1)*(-1/(2*dy))*difY1(ii); 
end 
end


%--------------- Kp1 dZx dU/dz Y1x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa1*Kp1*(-1/(2*dz))*dZx(ii,1,kk)*difY1(ii);
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+ alfa1*Kp1*(1/(2*dz))*dZx(ii,1,kk)*difY1(ii);                                                          
end 
end



%--------------- Kp2 dQy dV/dQ Y1x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+     alfa1*Kp2*(-3/(2*dy))*dQy(ii)*difY1(ii);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)+  alfa1*Kp2*(4/(2*dy))*dQy(ii)*difY1(ii);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+alfa1*Kp2*(-1/(2*dy))*dQy(ii)*difY1(ii);
end 
end

%--------------- Kp2 dZy dV/dz Y1x ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa1*Kp2*(-1/(2*dz))*dZy(ii,1,kk)*difY1(ii);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa1*Kp2*(1/(2*dz))*dZy(ii,1,kk)*difY1(ii);                                                          
end 
end

%--------------- Kp2 dZz dW/dz Y1x ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa1*Kp2*(-1/(2*dz))*dZz(ii,1)*difY1(ii);
MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+alfa1*Kp2*(1/(2*dz))*dZz(ii,1)*difY1(ii);                                                          
end 
end



%--------------- - dQy dU/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=     MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)-     alfa1*(-3/(2*dy))*dQy(ii);
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)=  MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)-   alfa1*(4/(2*dy))*dQy(ii);
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)-alfa1*(-1/(2*dy))*dQy(ii);
end 
end

%--------------- - dZy dU/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)-alfa1*(-1/(2*dz))*dZy(ii,1,kk);
MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)- alfa1*(1/(2*dz))*dZy(ii,1,kk);                                                          
end 
end

%----------------- - dV/dx -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)-alfa1*(-1/(2*dx));
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk)=    MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk)    - alfa1*(1/(2*dx));                                          
end 
end

%-------- i=Nx ------;

for kk=2:1:Nz-1                                              
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)- alfa1*(1/(2*dx));
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)-alfa1*(-4/(2*dx));
MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)=MB((Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)- alfa1*(3/(2*dx));
end 


%--------------- - dQx dV/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)-     alfa1*(-3/(2*dy))*dQx(ii,1);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)-   alfa1*(4/(2*dy))*dQx(ii,1);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)-alfa1*(-1/(2*dy))*dQx(ii,1);
end 
end

%--------------- - dZx dV/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)-alfa1*(-1/(2*dz))*dZx(ii,1,kk);
MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB((ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)- alfa1*(1/(2*dz))*dZx(ii,1,kk);                                                          
end 
end



%++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++ The second equation (basic equation #2) +++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++;

%++++++++++ Fyx nx + Fyy ny = fy(z) +++++++++++++;

%--------------- dQy dU/dQ Y1x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)+     alfa1*(-3/(2*dy))*dQy(ii)*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)+   alfa1*(4/(2*dy))*dQy(ii)*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)+alfa1*(-1/(2*dy))*dQy(ii)*difY1(ii);
end 
end

%--------------- dZy dU/dz Y1x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa1*(-1/(2*dz))*dZy(ii,1,kk)*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+ alfa1*(1/(2*dz))*dZy(ii,1,kk)*difY1(ii);                                                          
end 
end



%----------------- dV/dx Y1x -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)+alfa1*(-1/(2*dx))*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+kk)+     alfa1*(1/(2*dx))*difY1(ii);                                          
end 
end

%------ i=Nx -------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)+ alfa1*(1/(2*dx))*difY1(Nx);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)+alfa1*(-4/(2*dx))*difY1(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)+ alfa1*(3/(2*dx))*difY1(Nx);                                          
end 

%--------------- dQx dV/dQ Y1x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)+     alfa1*(-3/(2*dy))*dQx(ii,1)*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)+   alfa1*(4/(2*dy))*dQx(ii,1)*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+alfa1*(-1/(2*dy))*dQx(ii,1)*difY1(ii);
end 
end

%--------------- dZx dV/dz Y1x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa1*(-1/(2*dz))*dZx(ii,1,kk)*difY1(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa1*(1/(2*dz))*dZx(ii,1,kk)*difY1(ii);                                                          
end 
end



%--------------- - Kp1 dQy dV/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk) -    alfa1*Kp1*(-3/(2*dy))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)-   alfa1*Kp1*(4/(2*dy))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)-alfa1*Kp1*(-1/(2*dy))*dQy(ii);
end 
end

%--------------- - Kp1 dZy dV/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)-alfa1*Kp1*(-1/(2*dz))*dZy(ii,1,kk);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)- alfa1*Kp1*(1/(2*dz))*dZy(ii,1,kk);                                                          
end 
end



%----------------- - Kp2 dU/dx -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-2)*Ny*Nz+kk)-alfa1*Kp2*(-1/(2*dx));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,ii*Ny*Nz+kk)    - alfa1*Kp2*(1/(2*dx));                                          
end 
end

%------ i=Nx -------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-3)*Ny*Nz+kk)- alfa1*Kp2*(1/(2*dx));    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-2)*Ny*Nz+kk)-alfa1*Kp2*(-4/(2*dx));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,(Nx-1)*Ny*Nz+kk)- alfa1*Kp2*(3/(2*dx));                                          
end 


%--------------- - Kp2 dQx dU/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk)-     alfa1*Kp2*(-3/(2*dy))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+Nz+kk)-   alfa1*Kp2*(4/(2*dy))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+2*Nz+kk)-alfa1*Kp2*(-1/(2*dy))*dQx(ii,1);
end 
end

%--------------- - Kp2 dZx dU/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)-alfa1*Kp2*(-1/(2*dz))*dZx(ii,1,kk);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)- alfa1*Kp2*(1/(2*dz))*dZx(ii,1,kk);                                                          
end 
end

%--------------- - Kp2 dZz dW/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)-alfa1*Kp2*(-1/(2*dz))*dZz(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1) -alfa1*Kp2*(1/(2*dz))*dZz(ii,1);                                                          
end 
end



%+++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++ The third equation (basic equation #3) ++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++;

%+++++++++++++++ Fzx nx + Fzy ny = 0 ++++++++++++++++;

%--------------- dZz dU/dz Y1x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk-1)+alfa1*(-1/(2*dz))*dZz(ii,1)*difY1(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,(ii-1)*Ny*Nz+kk+1)+ alfa1*(1/(2*dz))*dZz(ii,1)*difY1(ii);                                                          
end 
end

%----------------- dW/dx Y1x -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+kk)+alfa1*(-1/(2*dx))*difY1(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+kk)+     alfa1*(1/(2*dx))*difY1(ii);                                          
end 
end

%-------- i=Nx ---------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+kk)+ alfa1*(1/(2*dx))*difY1(ii);    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+kk)+alfa1*(-4/(2*dx))*difY1(ii);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+kk)+ alfa1*(3/(2*dx))*difY1(ii);                                          
end 



%--------------- dQx dW/dQ Y1x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk) +    alfa1*(-3/(2*dy))*dQx(ii,1)*difY1(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)+   alfa1*(4/(2*dy))*dQx(ii,1)*difY1(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)+alfa1*(-1/(2*dy))*dQx(ii,1)*difY1(ii);
end 
end

%--------------- dZx dW/dz Y1x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)+alfa1*(-1/(2*dz))*dZx(ii,1,kk)*difY1(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)+ alfa1*(1/(2*dz))*dZx(ii,1,kk)*difY1(ii);                                                          
end 
end

%--------------- - dZz dV/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)-alfa1*(-1/(2*dz))*dZz(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)- alfa1*(1/(2*dz))*dZz(ii,1);                                                          
end 
end

%--------------- - dQy dW/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk) -    alfa1*(-3/(2*dy))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+kk)-   alfa1*(4/(2*dy))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+kk)-alfa1*(-1/(2*dy))*dQy(ii);
end 
end

%--------------- - dZy dW/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk-1)-alfa1*(-1/(2*dz))*dZy(ii,1,kk);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+kk+1)- alfa1*(1/(2*dz))*dZy(ii,1,kk);                                                          
end 
end




%++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++ The lateral edge Y=Y2 (Jy=Ny) +++++++++++;
%++++++++++++++ (free lateral edge) +++++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++ The first equation +++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++ KpE dFxx/dx ++++++++++++++++++++;

%------------------- Kp1 d^2U/dx^2 ----------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=   alfa2*Kp1/dx^2; 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=-alfa2*Kp1*2/dx^2; 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk)=       alfa2*Kp1/dx^2; 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-4)*Ny*Nz+(Ny-1)*Nz+kk)=alfa2*Kp1*(-1/dx^2); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)= alfa2*Kp1*(4/dx^2); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=alfa2*Kp1*(-5/dx^2); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)= alfa2*Kp1*(2/dx^2); 
end 


%----------------- Kp1 d/dx (dQ/dx dU/dQ) --------------;

%-------------- 1: Kp1 dQ/dx d/dx ( dU/dQ ) --------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)+alfa2*Kp1*dQx(ii,Ny)*(-1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*Kp1*dQx(ii,Ny)*(4/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*Kp1*dQx(ii,Ny)*(-3/(4*dx*dy));

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*Kp1*dQx(ii,Ny)*(1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*Kp1*dQx(ii,Ny)*(-4/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*Kp1*dQx(ii,Ny)*(3/(4*dx*dy));                                  
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*Kp1*dQx(Nx,Ny)*(1/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*Kp1*dQx(Nx,Ny)*(-4/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*Kp1*dQx(Nx,Ny)*(3/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*Kp1*dQx(Nx,Ny)*(-4/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*Kp1*dQx(Nx,Ny)*(16/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*Kp1*dQx(Nx,Ny)*(-12/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)+  alfa2*Kp1*dQx(Nx,Ny)*(3/(4*dx*dy));
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*Kp1*dQx(Nx,Ny)*(-12/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)+  alfa2*Kp1*dQx(Nx,Ny)*(9/(4*dx*dy));                                  
end

%-------------- 2: Kp1 d^2Q/dxdx dU/dQ --------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*Kp1*d2Qxx(ii,Ny)*(1/(2*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*Kp1*d2Qxx(ii,Ny)*(-4/(2*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*Kp1*d2Qxx(ii,Ny)*(3/(2*dy));
end 
end


%---------------- Kp1 d/dx ( dZ/dx dU/dz ) --------------;

%-------------- 1: Kp1 dZ/dx d/dx ( dU/dz ) --------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*Kp1*dZx(ii,Ny,kk)*(1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*Kp1*dZx(ii,Ny,kk)*(-1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk-1)+    alfa2*Kp1*dZx(ii,Ny,kk)*(-1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk+1)+     alfa2*Kp1*dZx(ii,Ny,kk)*(1/(4*dx*dz));                                  
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp1*dZx(Nx,Ny,kk)*(-1/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp1*dZx(Nx,Ny,kk)*(1/(4*dx*dz));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*Kp1*dZx(Nx,Ny,kk)*(4/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*Kp1*dZx(Nx,Ny,kk)*(-4/(4*dx*dz));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp1*dZx(Nx,Ny,kk)*(-3/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp1*dZx(Nx,Ny,kk)*(3/(4*dx*dz));                                  
end 

%-------------- 2: Kp1 d2Z/dxdx dU/dz --------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp1*d2Zxx(ii,Ny,kk)*(-1/(2*dz)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp1*d2Zxx(ii,Ny,kk)*(1/(2*dz)); 
end 
end


%-------------------- Kp1 dQ/dx d/dQ ( dU/dx ) -----------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)+alfa2*Kp1*(-1/(4*dy*dx))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*Kp1*(4/(4*dy*dx))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*Kp1*(-3/(4*dy*dx))*dQx(ii,Ny); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*Kp1*(1/(4*dy*dx))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*Kp1*(-4/(4*dy*dx))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*Kp1*(3/(4*dy*dx))*dQx(ii,Ny); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*Kp1*(1/(4*dy*dx))*dQx(Nx,Ny);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*Kp1*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*Kp1*(3/(4*dy*dx))*dQx(Nx,Ny); 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*Kp1*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*Kp1*(16/(4*dy*dx))*dQx(Nx,Ny);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*Kp1*(-12/(4*dy*dx))*dQx(Nx,Ny); 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)+  alfa2*Kp1*(3/(4*dy*dx))*dQx(Nx,Ny); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*Kp1*(-12/(4*dy*dx))*dQx(Nx,Ny); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)+  alfa2*Kp1*(9/(4*dy*dx))*dQx(Nx,Ny); 
end 


%---------------- Kp1 dQ/dx d/dQ ( dQ/dx dU/dQ ) -----------------;

%-------------- 1: Kp1 dQ/dx dQ/dx d/dQ ( dU/dQ ) -----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)+alfa2*Kp1*(-1/dy^2)*dQx(ii,Ny)^2; 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*Kp1*(4/dy^2)*dQx(ii,Ny)^2; 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*Kp1*(-5/dy^2)*dQx(ii,Ny)^2; 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*Kp1*(2/dy^2)*dQx(ii,Ny)^2; 
end 
end

%------------------ 2: Kp1 d^2Q/dxdq dU/dQ --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*Kp1*d2Qxq(ii,Ny)*(1/(2*dy))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*Kp1*d2Qxq(ii,Ny)*(-4/(2*dy))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*Kp1*d2Qxq(ii,Ny)*(3/(2*dy))*dQx(ii,Ny); 
end 
end


%-------------- Kp1 dQ/dx d/dQ ( dZ/dx dU/dz ) ------------------;

%------------ 1: Kp1 dQ/dx dZ/dx d/dQ ( dU/dz ) -----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*Kp1*dZx(ii,Ny,kk)*(-1/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*Kp1*dZx(ii,Ny,kk)*(4/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp1*dZx(ii,Ny,kk)*(-3/(4*dy*dz))*dQx(ii,Ny); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*Kp1*dZx(ii,Ny,kk)*(1/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*Kp1*dZx(ii,Ny,kk)*(-4/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp1*dZx(ii,Ny,kk)*(3/(4*dy*dz))*dQx(ii,Ny); 
end 
end

%------------ 2: Kp1 dQ/dx d^2Z/dxdq  dU/dz  -----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp1*d2Zxq(ii,Ny,kk)*(-1/(2*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp1*d2Zxq(ii,Ny,kk)*(1/(2*dz))*dQx(ii,Ny); 
end 
end


%------------------- Kp1 dZ/dx d/dz ( dU/dx ) -------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*Kp1*(1/(4*dz*dx))*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk-1)+    alfa2*Kp1*(-1/(4*dz*dx))*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*Kp1*(-1/(4*dz*dx))*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk+1)+     alfa2*Kp1*(1/(4*dz*dx))*dZx(ii,Ny,kk); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp1*(-1/(4*dz*dx))*dZx(Nx,Ny,kk);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp1*(1/(4*dz*dx))*dZx(Nx,Ny,kk); 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*Kp1*(4/(4*dz*dx))*dZx(Nx,Ny,kk);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*Kp1*(-4/(4*dz*dx))*dZx(Nx,Ny,kk); 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp1*(-3/(4*dz*dx))*dZx(Nx,Ny,kk);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp1*(3/(4*dz*dx))*dZx(Nx,Ny,kk); 
end 


%------------------- Kp1 dZ/dx d/dz ( dQ/dx dU/dQ ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*Kp1*dQx(ii,Ny)*(-1/(4*dz*dy))*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*Kp1*dQx(ii,Ny)*(4/(4*dz*dy))*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp1*dQx(ii,Ny)*(-3/(4*dz*dy))*dZx(ii,Ny,kk);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*Kp1*dQx(ii,Ny)*(1/(4*dz*dy))*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*Kp1*dQx(ii,Ny)*(-4/(4*dz*dy))*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp1*dQx(ii,Ny)*(3/(4*dz*dy))*dZx(ii,Ny,kk); 
end 
end


%------------------- Kp1 dZ/dx d/dz ( dZ/dx dU/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp1*(Gpz1_ijk(ii,Ny,kk)/dz^2)*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+  alfa2*Kp1*(-(Gpz1_ijk(ii,Ny,kk)+Gpz1_ijk(ii,Ny,kk+1))/dz^2)*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*Kp1*(Gpz1_ijk(ii,Ny,kk+1)/dz^2)*dZx(ii,Ny,kk); 
end  
end


%------------------- Kp2 d/dx ( dQ/dy dV/dQ ) -------------------;

%----------------- 1: Kp2 dQ/dy d/dx ( dV/dQ ) -------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)=alfa2*Kp2*dQy(ii)*(-1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)= alfa2*Kp2*dQy(ii)*(4/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=alfa2*Kp2*dQy(ii)*(-3/(4*dx*dy));

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+kk)= alfa2*Kp2*dQy(ii)*(1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+kk)=alfa2*Kp2*dQy(ii)*(-4/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)= alfa2*Kp2*dQy(ii)*(3/(4*dx*dy));                                  
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)= alfa2*Kp2*dQy(Nx)*(1/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)=alfa2*Kp2*dQy(Nx)*(-4/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)= alfa2*Kp2*dQy(Nx)*(3/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)= alfa2*Kp2*dQy(Nx)*(-4/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)= alfa2*Kp2*dQy(Nx)*(16/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=alfa2*Kp2*dQy(Nx)*(-12/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)=  alfa2*Kp2*dQy(Nx)*(3/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)=alfa2*Kp2*dQy(Nx)*(-12/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=  alfa2*Kp2*dQy(Nx)*(9/(4*dx*dy));                                  
end


%----------------- 2: Kp2 d^2Q/dydx dV/dQ -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*Kp2*d2Qyx(ii)*(1/(2*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*Kp2*d2Qyx(ii)*(-4/(2*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*Kp2*d2Qyx(ii)*(3/(2*dy)); 
end 
end


%------------------- Kp2 d/dx ( dZ/dy dV/dz ) -------------------;

%----------------- 1: Kp2 dZ/dy d/dx ( dV/dz ) -------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*Kp2*dZy(ii,Ny,kk)*(1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*Kp2*dZy(ii,Ny,kk)*(-1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)+    alfa2*Kp2*dZy(ii,Ny,kk)*(-1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)+     alfa2*Kp2*dZy(ii,Ny,kk)*(1/(4*dx*dz)); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*dZy(Nx,Ny,kk)*(-1/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp2*dZy(Nx,Ny,kk)*(1/(4*dx*dz));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*Kp2*dZy(Nx,Ny,kk)*(4/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*Kp2*dZy(Nx,Ny,kk)*(-4/(4*dx*dz));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*dZy(Nx,Ny,kk)*(-3/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp2*dZy(Nx,Ny,kk)*(3/(4*dx*dz)); 
end 

%----------------- 2: Kp2 d^2Z/dydx dV/dz -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*d2Zyx(ii,Ny,kk)*(-1/(2*dz)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp2*d2Zyx(ii,Ny,kk)*(1/(2*dz)); 
end 
end


%--------------- Kp2 dQ/dx d/dQ ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)+alfa2*Kp2*dQy(ii)*(-1/dy^2)*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*Kp2*dQy(ii)*(4/dy^2)*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*Kp2*dQy(ii)*(-5/dy^2)*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*Kp2*dQy(ii)*(2/dy^2)*dQx(ii,Ny); 
end 
end


%--------------- Kp2 dQ/dx d/dQ ( dZ/dy dV/dz ) ------------------;

%------------- 1: Kp2 dQ/dx dZ/dy d/dQ ( dV/dz ) ------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*Kp2*dZy(ii,Ny,kk)*(-1/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*Kp2*dZy(ii,Ny,kk)*(4/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*dZy(ii,Ny,kk)*(-3/(4*dy*dz))*dQx(ii,Ny); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*Kp2*dZy(ii,Ny,kk)*(1/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*Kp2*dZy(ii,Ny,kk)*(-4/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp2*dZy(ii,Ny,kk)*(3/(4*dy*dz))*dQx(ii,Ny); 
end 
end

%------------- 2: Kp2 dQ/dx d^2Z/dydq  dV/dz ------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*d2Zyq(ii,Ny,kk)*(-1/(2*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp2*d2Zyq(ii,Ny,kk)*(1/(2*dz))*dQx(ii,Ny); 
end 
end


%--------------- Kp2 dZ/dx d/dz ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*Kp2*dQy(ii)*(-1/(4*dz*dy))*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*Kp2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,Ny,kk);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*Kp2*dQy(ii)*(1/(4*dz*dy))*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*Kp2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,Ny,kk); 
end 
end


%------------- Kp2 dZ/dx d/dz ( dZ/dy dV/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*(Gpz2_ijk(ii,Ny,kk)/dz^2)*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+  alfa2*Kp2*(-(Gpz2_ijk(ii,Ny,kk)+Gpz2_ijk(ii,Ny,kk+1))/dz^2)*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*Kp2*(Gpz2_ijk(ii,Ny,kk+1)/dz^2)*dZx(ii,Ny,kk); 
end  
end


%------------------- Kp2 d/dx ( dZ/dz dW/dz ) -----------------;

%----------------- 1: Kp2 dZ/dz d/dx ( dW/dz ) -----------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)= alfa2*Kp2*dZz(ii,Ny)*(1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=alfa2*Kp2*dZz(ii,Ny)*(-1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)=    alfa2*Kp2*dZz(ii,Ny)*(-1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)=     alfa2*Kp2*dZz(ii,Ny)*(1/(4*dx*dz));                                  
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)=alfa2*Kp2*dZz(Nx,Ny)*(-1/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)= alfa2*Kp2*dZz(Nx,Ny)*(1/(4*dx*dz));    

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)= alfa2*Kp2*dZz(Nx,Ny)*(4/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=alfa2*Kp2*dZz(Nx,Ny)*(-4/(4*dx*dz));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=alfa2*Kp2*dZz(Nx,Ny)*(-3/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)= alfa2*Kp2*dZz(Nx,Ny)*(3/(4*dx*dz));                                  
end

%----------------- 2: Kp2 d^2Z/dzdx  dW/dz -----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*d2Zzx(ii,Ny)*(-1/(2*dz)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp2*d2Zzx(ii,Ny)*(1/(2*dz)); 
end 
end


%----------------- Kp2 dQ/dx d/dQ ( dZ/dz dW/dz ) ---------------------;

%--------------- 1: Kp2 dQ/dx dZ/dz d/dQ ( dW/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*Kp2*dZz(ii,Ny)*(-1/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*Kp2*dZz(ii,Ny)*(4/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*dZz(ii,Ny)*(-3/(4*dy*dz))*dQx(ii,Ny); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*Kp2*dZz(ii,Ny)*(1/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*Kp2*dZz(ii,Ny)*(-4/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp2*dZz(ii,Ny)*(3/(4*dy*dz))*dQx(ii,Ny); 
end 
end

%--------------- 2: Kp2 dQ/dx d^2Z/dzdq  dW/dz --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*d2Zzq(ii,Ny)*(-1/(2*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp2*d2Zzq(ii,Ny)*(1/(2*dz))*dQx(ii,Ny); 
end 
end


%------------------ Kp2 dZ/dx d/dz ( dZ/dz dW/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*dZz(ii,Ny)*(1/dz^2)*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*Kp2*dZz(ii,Ny)*(-2/dz^2)*dZx(ii,Ny,kk); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*Kp2*dZz(ii,Ny)*(1/dz^2)*dZx(ii,Ny,kk); 
end  
end



%+++++++++++++++ KpE dQy 1/(2*dy) (Fxy)^Ny-2 +++++++++++++++++;

%---------------- dQy 1/(2*dy) dQy dU/dQ ------------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)+alfa2*dQy(ii)*(1/(2*dy))*(-1/(2*dy))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*dQy(ii)*(1/(2*dy))*(1/(2*dy))*dQy(ii);                              
end 
end

%----------------- dQy 1/(2*dy) dZy dU/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dQy(ii)*(1/(2*dy))*(-1/(2*dz))*dZy(ii,Ny-2,kk);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dQy(ii)*(1/(2*dy))*(1/(2*dz))*dZy(ii,Ny-2,kk);                                                          
end 
end

%------------------- dQy 1/(2*dy) dV/dx -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)+alfa2*dQy(ii)*(1/(2*dy))*(-1/(2*dx));
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+kk)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+kk)    + alfa2*dQy(ii)*(1/(2*dy))*(1/(2*dx));                                          
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQy(Nx)*(1/(2*dy))*(1/(2*dx));    
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)+alfa2*dQy(Nx)*(1/(2*dy))*(-4/(2*dx));
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQy(Nx)*(1/(2*dy))*(3/(2*dx));                                          
end 


%----------------- dQy 1/(2*dy) dQx dV/dQ ---------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)+alfa2*dQy(ii)*(1/(2*dy))*(-1/(2*dy))*dQx(ii,Ny-2);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*dQy(ii)*(1/(2*dy))*(1/(2*dy))*dQx(ii,Ny-2);                              
end 
end

%---------------- dQy 1/(2*dy) dZx dV/dz ---------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dQy(ii)*(1/(2*dy))*(-1/(2*dz))*dZx(ii,Ny-2,kk);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dQy(ii)*(1/(2*dy))*(1/(2*dz))*dZx(ii,Ny-2,kk);                                                          
end 
end



%++++++++++++++ KpE dQy (-4/(2*dy)) (Fxy)^Ny-1 +++++++++++++++++;

%---------------- dQy (-4/(2*dy)) dQy dU/dQ ------------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+alfa2*dQy(ii)*(-4/(2*dy))*(-1/(2*dy))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(ii)*(-4/(2*dy))*(1/(2*dy))*dQy(ii);                              
end 
end

%----------------- dQy (-4/(2*dy)) dZy dU/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+alfa2*dQy(ii)*(-4/(2*dy))*(-1/(2*dz))*dZy(ii,Ny-1,kk);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+ alfa2*dQy(ii)*(-4/(2*dy))*(1/(2*dz))*dZy(ii,Ny-1,kk);                                                          
end 
end

%------------------- dQy (-4/(2*dy)) dV/dx -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQy(ii)*(-4/(2*dy))*(-1/(2*dx));
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+kk)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+kk)    + alfa2*dQy(ii)*(-4/(2*dy))*(1/(2*dx));                                          
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*dQy(Nx)*(-4/(2*dy))*(1/(2*dx));    
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQy(Nx)*(-4/(2*dy))*(-4/(2*dx));
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*dQy(Nx)*(-4/(2*dy))*(3/(2*dx));                                          
end 


%----------------- dQy (-4/(2*dy)) dQx dV/dQ ---------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+alfa2*dQy(ii)*(-4/(2*dy))*(-1/(2*dy))*dQx(ii,Ny-1);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(ii)*(-4/(2*dy))*(1/(2*dy))*dQx(ii,Ny-1);                              
end 
end

%---------------- dQy (-4/(2*dy)) dZx dV/dz ---------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+alfa2*dQy(ii)*(-4/(2*dy))*(-1/(2*dz))*dZx(ii,Ny-1,kk);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+ alfa2*dQy(ii)*(-4/(2*dy))*(1/(2*dz))*dZx(ii,Ny-1,kk);                                                          
end 
end


%++++++++++++++ - KpE dQy (3/(2*dy)) (Fxx)^Ny nx +++++++++++++++;

%----------------- dQy (3/(2*dy)) Kp1 dU/dx Y2x -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*dQy(ii)*(3/(2*dy))*(-1/(2*dx))*Kp1*difY2(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk)  +   alfa2*dQy(ii)*(3/(2*dy))*(1/(2*dx))*Kp1*difY2(ii);                                          
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(Nx)*(3/(2*dy))*(1/(2*dx))*Kp1*difY2(Nx);    
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*dQy(Nx)*(3/(2*dy))*(-4/(2*dx))*Kp1*difY2(Nx);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(Nx)*(3/(2*dy))*(3/(2*dx))*Kp1*difY2(Nx);                                          
end 


%--------------- dQy (3/(2*dy)) Kp1 dQx dU/dQ Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQy(ii)*(3/(2*dy))*(1/(2*dy))*dQx(ii,Ny)*Kp1*difY2(ii);   
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQy(ii)*(3/(2*dy))*(-4/(2*dy))*dQx(ii,Ny)*Kp1*difY2(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(ii)*(3/(2*dy))*(3/(2*dy))*dQx(ii,Ny)*Kp1*difY2(ii);                              
end 
end

%--------------- dQy (3/(2*dy)) Kp1 dZx dU/dz Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dQy(ii)*(3/(2*dy))*(-1/(2*dz))*dZx(ii,Ny,kk)*Kp1*difY2(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dQy(ii)*(3/(2*dy))*(1/(2*dz))*dZx(ii,Ny,kk)*Kp1*difY2(ii);                                                          
end 
end

%--------------- dQy (3/(2*dy)) Kp2 dQy dV/dQ Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQy(ii)*(3/(2*dy))*(1/(2*dy))*dQy(ii)*Kp2*difY2(ii);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQy(ii)*(3/(2*dy))*(-4/(2*dy))*dQy(ii)*Kp2*difY2(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(ii)*(3/(2*dy))*(3/(2*dy))*dQy(ii)*Kp2*difY2(ii);                              
end 
end

%--------------- dQy (3/(2*dy)) Kp2 dZy dV/dz Y2x ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dQy(ii)*(3/(2*dy))*(-1/(2*dz))*dZy(ii,Ny,kk)*Kp2*difY2(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dQy(ii)*(3/(2*dy))*(1/(2*dz))*dZy(ii,Ny,kk)*Kp2*difY2(ii);                                                          
end 
end

%--------------- dQy (3/(2*dy)) Kp2 dZz dW/dz Y2x ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dQy(ii)*(3/(2*dy))*(-1/(2*dz))*dZz(ii,Ny)*Kp2*difY2(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dQy(ii)*(3/(2*dy))*(1/(2*dz))*dZz(ii,Ny)*Kp2*difY2(ii);                                                          
end 
end



%++++++++++++++++ - KpE dZ/dy dFxx/dz nx ++++++++++++++++++;

%-------------- dZ/dy Kp1 d/dz ( dU/dx ) Y2x  ---------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*dZy(ii,Ny,kk)*Kp1*(1/(4*dz*dx))*difY2(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk-1)+    alfa2*dZy(ii,Ny,kk)*Kp1*(-1/(4*dz*dx))*difY2(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*Kp1*(-1/(4*dz*dx))*difY2(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk+1)+     alfa2*dZy(ii,Ny,kk)*Kp1*(1/(4*dz*dx))*difY2(ii); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(Nx,Ny,kk)*Kp1*(-1/(4*dz*dx))*difY2(Nx);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZy(Nx,Ny,kk)*Kp1*(1/(4*dz*dx))*difY2(Nx);     
    
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*dZy(Nx,Ny,kk)*Kp1*(4/(4*dz*dx))*difY2(Nx);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZy(Nx,Ny,kk)*Kp1*(-4/(4*dz*dx))*difY2(Nx); 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(Nx,Ny,kk)*Kp1*(-3/(4*dz*dx))*difY2(Nx);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZy(Nx,Ny,kk)*Kp1*(3/(4*dz*dx))*difY2(Nx); 
end 


%----------- dZ/dy Kp1 d/dz ( dQ/dx dU/dQ ) Y2x ----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*Kp1*dQx(ii,Ny)*(-1/(4*dz*dy))*difY2(ii);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*dZy(ii,Ny,kk)*Kp1*dQx(ii,Ny)*(4/(4*dz*dy))*difY2(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*Kp1*dQx(ii,Ny)*(-3/(4*dz*dy))*difY2(ii);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dZy(ii,Ny,kk)*Kp1*dQx(ii,Ny)*(1/(4*dz*dy))*difY2(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*Kp1*dQx(ii,Ny)*(-4/(4*dz*dy))*difY2(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZy(ii,Ny,kk)*Kp1*dQx(ii,Ny)*(3/(4*dz*dy))*difY2(ii); 
end 
end

%------------ dZ/dy Kp1 d/dz ( dZ/dx dU/dz ) Y2x ---------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*Kp1*(Gpz1_ijk(ii,Ny,kk)/dz^2)*difY2(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+  alfa2*dZy(ii,Ny,kk)*Kp1*(-(Gpz1_ijk(ii,Ny,kk)+Gpz1_ijk(ii,Ny,kk+1))/dz^2)*difY2(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*Kp1*(Gpz1_ijk(ii,Ny,kk+1)/dz^2)*difY2(ii); 
end  
end

%-------------- dZ/dy Kp2 d/dz ( dQ/dy dV/dQ ) Y2x ---------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*Kp2*dQy(ii)*(-1/(4*dz*dy))*difY2(ii);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*dZy(ii,Ny,kk)*Kp2*dQy(ii)*(4/(4*dz*dy))*difY2(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*Kp2*dQy(ii)*(-3/(4*dz*dy))*difY2(ii);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dZy(ii,Ny,kk)*Kp2*dQy(ii)*(1/(4*dz*dy))*difY2(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*Kp2*dQy(ii)*(-4/(4*dz*dy))*difY2(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZy(ii,Ny,kk)*Kp2*dQy(ii)*(3/(4*dz*dy))*difY2(ii); 
end 
end

%------------- dZ/dy Kp2 d/dz ( dZ/dy dV/dz ) Y2x ----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*Kp2*(Gpz2_ijk(ii,Ny,kk)/dz^2)*difY2(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+  alfa2*dZy(ii,Ny,kk)*Kp2*(-(Gpz2_ijk(ii,Ny,kk)+Gpz2_ijk(ii,Ny,kk+1))/dz^2)*difY2(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*Kp2*(Gpz2_ijk(ii,Ny,kk+1)/dz^2)*difY2(ii); 
end  
end

%-------------- dZ/dy Kp2 d/dz ( dZ/dz dW/dz ) Y2x -----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*Kp2*dZz(ii,Ny)*(1/dz^2)*difY2(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dZy(ii,Ny,kk)*Kp2*dZz(ii,Ny)*(-2/dz^2)*difY2(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*Kp2*dZz(ii,Ny)*(1/dz^2)*difY2(ii); 
end  
end


%++++++++++++++++++++ KpE dZ/dz dFzx/dz +++++++++++++++++++++;

%---------------- dZ/dz d/dz ( dZ/dz dU/dz ) ----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZz(ii,Ny)*(1/dz^2)*dZz(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dZz(ii,Ny)*(-2/dz^2)*dZz(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZz(ii,Ny)*(1/dz^2)*dZz(ii,Ny); 
end  
end

%------------------- dZ/dz d/dz ( dW/dx ) -------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*dZz(ii,Ny)*(1/(4*dz*dx)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)+    alfa2*dZz(ii,Ny)*(-1/(4*dz*dx)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZz(ii,Ny)*(-1/(4*dz*dx)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)+     alfa2*dZz(ii,Ny)*(1/(4*dz*dx)); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZz(Nx,Ny)*(-1/(4*dz*dx));
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZz(Nx,Ny)*(1/(4*dz*dx));     
    
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*dZz(Nx,Ny)*(4/(4*dz*dx));
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZz(Nx,Ny)*(-4/(4*dz*dx)); 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZz(Nx,Ny)*(-3/(4*dz*dx));
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZz(Nx,Ny)*(3/(4*dz*dx)); 
end 


%------------------- dZ/dz d/dz ( dQ/dx dW/dQ ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dQx(ii,Ny)*(-1/(4*dz*dy))*dZz(ii,Ny);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*dQx(ii,Ny)*(4/(4*dz*dy))*dZz(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dQx(ii,Ny)*(-3/(4*dz*dy))*dZz(ii,Ny);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dQx(ii,Ny)*(1/(4*dz*dy))*dZz(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*dQx(ii,Ny)*(-4/(4*dz*dy))*dZz(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dQx(ii,Ny)*(3/(4*dz*dy))*dZz(ii,Ny); 
end 
end

%------------------- dZ/dz d/dz ( dZ/dx dW/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*(Gpz1_ijk(ii,Ny,kk)/dz^2)*dZz(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+  alfa2*(-(Gpz1_ijk(ii,Ny,kk)+Gpz1_ijk(ii,Ny,kk+1))/dz^2)*dZz(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*(Gpz1_ijk(ii,Ny,kk+1)/dz^2)*dZz(ii,Ny); 
end  
end



%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++++++ + KpE dens Freq^2 U ++++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk) + alfa2*KpE*dens*Freq^2;
end  
end




%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++++++ The second equation ++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%++++++++++++++++++++++ KpE dFyx/dx +++++++++++++++++++++++++;

%------------------- d/dx ( dQ/dy dU/dQ ) --------------------;

%------------------ 1: dQ/dy d/dx ( dU/dQ ) -------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)=alfa2*dQy(ii)*(-1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)= alfa2*dQy(ii)*(4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=alfa2*dQy(ii)*(-3/(4*dx*dy));

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-3)*Nz+kk)= alfa2*dQy(ii)*(1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-2)*Nz+kk)=alfa2*dQy(ii)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk)= alfa2*dQy(ii)*(3/(4*dx*dy));                                  
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)= alfa2*dQy(Nx)*(1/(4*dx*dy));     
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)=alfa2*dQy(Nx)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)= alfa2*dQy(Nx)*(3/(4*dx*dy));     

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)= alfa2*dQy(Nx)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)= alfa2*dQy(Nx)*(16/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=alfa2*dQy(Nx)*(-12/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)=  alfa2*dQy(Nx)*(3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)=alfa2*dQy(Nx)*(-12/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=  alfa2*dQy(Nx)*(9/(4*dx*dy));                                  
end

%------------------- 2: d^2Q/dydx  dU/dQ ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*d2Qyx(ii)*(1/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*d2Qyx(ii)*(-4/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*d2Qyx(ii)*(3/(2*dy));
end 
end


%--------------------- d/dx ( dZ/dy dU/dz ) ---------------------;

%------------------- 1: dZ/dy d/dx ( dU/dz ) ---------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*dZy(ii,Ny,kk)*(1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*(-1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk-1)+    alfa2*dZy(ii,Ny,kk)*(-1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk+1)+     alfa2*dZy(ii,Ny,kk)*(1/(4*dx*dz));                                  
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(Nx,Ny,kk)*(-1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZy(Nx,Ny,kk)*(1/(4*dx*dz));    
    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*dZy(Nx,Ny,kk)*(4/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZy(Nx,Ny,kk)*(-4/(4*dx*dz));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(Nx,Ny,kk)*(-3/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZy(Nx,Ny,kk)*(3/(4*dx*dz));                                  
end

%-------------------- 2: d^2Z/dydx  dU/dz ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*d2Zyx(ii,Ny,kk)*(-1/(2*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*d2Zyx(ii,Ny,kk)*(1/(2*dz)); 
end 
end


%-------------------- dQ/dx d/dQ ( dQ/dy dU/dQ ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)+alfa2*dQy(ii)*(-1/dy^2)*dQx(ii,Ny);     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQy(ii)*(4/dy^2)*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQy(ii)*(-5/dy^2)*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(ii)*(2/dy^2)*dQx(ii,Ny); 
end 
end


%-------------------- dQ/dx d/dQ ( dZ/dy dU/dz ) --------------------;

%------------------ 1: dQ/dx dZ/dy d/dQ ( dU/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*(-1/(4*dy*dz))*dQx(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*dZy(ii,Ny,kk)*(4/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*(-3/(4*dy*dz))*dQx(ii,Ny); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dZy(ii,Ny,kk)*(1/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*(-4/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZy(ii,Ny,kk)*(3/(4*dy*dz))*dQx(ii,Ny); 
end 
end

%------------------ 2: dQ/dx d^2Z/dydq  dU/dz -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*d2Zyq(ii,Ny,kk)*(-1/(2*dz))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*d2Zyq(ii,Ny,kk)*(1/(2*dz))*dQx(ii,Ny); 
end 
end


%-------------------- dZ/dx d/dz ( dQ/dy dU/dQ ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZx(ii,Ny,kk);     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,Ny,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,Ny,kk);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dQy(ii)*(1/(4*dz*dy))*dZx(ii,Ny,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,Ny,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,Ny,kk); 
end 
end


%------------------ dZ/dx d/dz ( dZ/dy dU/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*(Gpz2_ijk(ii,Ny,kk)/dz^2)*dZx(ii,Ny,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*(-(Gpz2_ijk(ii,Ny,kk)+Gpz2_ijk(ii,Ny,kk+1))/dz^2)*dZx(ii,Ny,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*(Gpz2_ijk(ii,Ny,kk+1)/dz^2)*dZx(ii,Ny,kk); 
end  
end

%------------------------ d/dx ( dV/dx ) -----------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)= alfa2*(1/dx^2); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=alfa2*(-2/dx^2); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)=     alfa2*(1/dx^2); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-4)*Ny*Nz+(Ny-1)*Nz+kk)=alfa2*(-1/dx^2);     
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)= alfa2*(4/dx^2); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=alfa2*(-5/dx^2); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)= alfa2*(2/dx^2); 
end 


%--------------------- d/dx ( dQ/dx dV/dQ ) ---------------------;

%------------------- 1: dQ/dx d/dx ( dV/dQ ) ---------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)+alfa2*dQx(ii,Ny)*(-1/(4*dx*dy));     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*dQx(ii,Ny)*(4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*dQx(ii,Ny)*(-3/(4*dx*dy));

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQx(ii,Ny)*(1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQx(ii,Ny)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQx(ii,Ny)*(3/(4*dx*dy)); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQx(Nx,Ny)*(1/(4*dx*dy));     
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQx(Nx,Ny)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQx(Nx,Ny)*(3/(4*dx*dy));
    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQx(Nx,Ny)*(-4/(4*dx*dy));     
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*dQx(Nx,Ny)*(16/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*dQx(Nx,Ny)*(-12/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)+  alfa2*dQx(Nx,Ny)*(3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQx(Nx,Ny)*(-12/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)+  alfa2*dQx(Nx,Ny)*(9/(4*dx*dy)); 
end 

%------------------- 2: d^2Q/dxdx  dV/dQ ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*d2Qxx(ii,Ny)*(1/(2*dy));     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*d2Qxx(ii,Ny)*(-4/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*d2Qxx(ii,Ny)*(3/(2*dy));
end 
end


%--------------------- d/dx ( dZ/dx dV/dz ) ---------------------;

%------------------- 1: dZ/dx d/dx ( dV/dz ) ---------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*dZx(ii,Ny,kk)*(1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZx(ii,Ny,kk)*(-1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)+    alfa2*dZx(ii,Ny,kk)*(-1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)+     alfa2*dZx(ii,Ny,kk)*(1/(4*dx*dz)); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZx(Nx,Ny,kk)*(-1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZx(Nx,Ny,kk)*(1/(4*dx*dz));    
    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*dZx(Nx,Ny,kk)*(4/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZx(Nx,Ny,kk)*(-4/(4*dx*dz));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZx(Nx,Ny,kk)*(-3/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZx(Nx,Ny,kk)*(3/(4*dx*dz)); 
end

%------------------- 2: d^2Z/dxdx  dV/dz ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*d2Zxx(ii,Ny,kk)*(-1/(2*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*d2Zxx(ii,Ny,kk)*(1/(2*dz)); 
end 
end


%-------------------- dQ/dx d/dQ ( dV/dx ) ---------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)+alfa2*(-1/(4*dy*dx))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*(4/(4*dy*dx))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*(-3/(4*dy*dx))*dQx(ii,Ny); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*(1/(4*dy*dx))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*(-4/(4*dy*dx))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*(3/(4*dy*dx))*dQx(ii,Ny); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*(1/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*(3/(4*dy*dx))*dQx(Nx,Ny); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*(16/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*(-12/(4*dy*dx))*dQx(Nx,Ny); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)+  alfa2*(3/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*(-12/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)+  alfa2*(9/(4*dy*dx))*dQx(Nx,Ny); 
end 


%------------------- dQ/dx d/dQ ( dQ/dx dV/dQ ) --------------------;

%----------------- 1: dQ/dx dQ/dx d/dQ ( dV/dQ ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)+alfa2*(-1/dy^2)*dQx(ii,Ny)^2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*(4/dy^2)*dQx(ii,Ny)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*(-5/dy^2)*dQx(ii,Ny)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*(2/dy^2)*dQx(ii,Ny)^2;                                                                            
end 
end

%----------------- 2: dQ/dx d^2Q/dxdq  dV/dQ --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*d2Qxq(ii,Ny)*(1/(2*dy))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*d2Qxq(ii,Ny)*(-4/(2*dy))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*d2Qxq(ii,Ny)*(3/(2*dy))*dQx(ii,Ny);                                                                            
end 
end


%-------------------- dQ/dx d/dQ ( dZ/dx dV/dz ) --------------------;

%------------------ 1: dQ/dx dZ/dx d/dQ ( dV/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dZx(ii,Ny,kk)*(-1/(4*dy*dz))*dQx(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*dZx(ii,Ny,kk)*(4/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZx(ii,Ny,kk)*(-3/(4*dy*dz))*dQx(ii,Ny); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dZx(ii,Ny,kk)*(1/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*dZx(ii,Ny,kk)*(-4/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZx(ii,Ny,kk)*(3/(4*dy*dz))*dQx(ii,Ny);                                                                                         
end 
end

%------------------ 2: dQ/dx d^2Z/dxdq  dV/dz --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*d2Zxq(ii,Ny,kk)*(-1/(2*dz))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*d2Zxq(ii,Ny,kk)*(1/(2*dz))*dQx(ii,Ny);                                                                                         
end 
end


%-------------------- dZ/dx d/dz ( dV/dx ) --------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*(1/(4*dz*dx))*dZx(ii,Ny,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)+    alfa2*(-1/(4*dz*dx))*dZx(ii,Ny,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*(-1/(4*dz*dx))*dZx(ii,Ny,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)+     alfa2*(1/(4*dz*dx))*dZx(ii,Ny,kk); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*(-1/(4*dz*dx))*dZx(Nx,Ny,kk);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*(1/(4*dz*dx))*dZx(Nx,Ny,kk); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*(4/(4*dz*dx))*dZx(Nx,Ny,kk);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*(-4/(4*dz*dx))*dZx(Nx,Ny,kk); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*(-3/(4*dz*dx))*dZx(Nx,Ny,kk);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*(3/(4*dz*dx))*dZx(Nx,Ny,kk); 
end 


%-------------------- dZ/dx d/dz ( dQ/dx dV/dQ ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dQx(ii,Ny)*(-1/(4*dz*dy))*dZx(ii,Ny,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*dQx(ii,Ny)*(4/(4*dz*dy))*dZx(ii,Ny,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dQx(ii,Ny)*(-3/(4*dz*dy))*dZx(ii,Ny,kk);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dQx(ii,Ny)*(1/(4*dz*dy))*dZx(ii,Ny,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*dQx(ii,Ny)*(-4/(4*dz*dy))*dZx(ii,Ny,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dQx(ii,Ny)*(3/(4*dz*dy))*dZx(ii,Ny,kk); 
end 
end

%-------------------- dZ/dx d/dz ( dZ/dx dV/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*(Gpz1_ijk(ii,Ny,kk)/dz^2)*dZx(ii,Ny,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*(-(Gpz1_ijk(ii,Ny,kk)+Gpz1_ijk(ii,Ny,kk+1))/dz^2)*dZx(ii,Ny,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*(Gpz1_ijk(ii,Ny,kk+1)/dz^2)*dZx(ii,Ny,kk);                                                                            
end  
end



%++++++++++++++  KpE dQy 1/(2*dy) (Fyy)^Ny-2 +++++++++++++++++;

%-------------- dQy 1/(2*dy) Kp1 dQy dV/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)+alfa2*dQy(ii)*(1/(2*dy))*(-1/(2*dy))*dQy(ii)*Kp1;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*dQy(ii)*(1/(2*dy))*(1/(2*dy))*dQy(ii)*Kp1;                              
end 
end

%-------------- dQy 1/(2*dy) Kp1 dZy dV/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dQy(ii)*(1/(2*dy))*(-1/(2*dz))*dZy(ii,Ny-2,kk)*Kp1;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dQy(ii)*(1/(2*dy))*(1/(2*dz))*dZy(ii,Ny-2,kk)*Kp1;                                                          
end 
end

%---------------- dQy 1/(2*dy) Kp2 dU/dx -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)+alfa2*dQy(ii)*(1/(2*dy))*(-1/(2*dx))*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-3)*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-3)*Nz+kk)    + alfa2*dQy(ii)*(1/(2*dy))*(1/(2*dx))*Kp2;                                          
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQy(Nx)*(1/(2*dy))*(1/(2*dx))*Kp2;
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)+alfa2*dQy(Nx)*(1/(2*dy))*(-4/(2*dx))*Kp2;
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQy(Nx)*(1/(2*dy))*(3/(2*dx))*Kp2;                                          
end 


%-------------- dQy 1/(2*dy) Kp2 dQx dU/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)+alfa2*dQy(ii)*(1/(2*dy))*(-1/(2*dy))*dQx(ii,Ny-2)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*dQy(ii)*(1/(2*dy))*(1/(2*dy))*dQx(ii,Ny-2)*Kp2;                              
end 
end

%-------------- dQy 1/(2*dy) Kp2 dZx dU/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dQy(ii)*(1/(2*dy))*(-1/(2*dz))*dZx(ii,Ny-2,kk)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dQy(ii)*(1/(2*dy))*(1/(2*dz))*dZx(ii,Ny-2,kk)*Kp2;                                                          
end 
end

%--------------- dQy 1/(2*dy) Kp2 dZz dW/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dQy(ii)*(1/(2*dy))*(-1/(2*dz))*dZz(ii,Ny-2)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dQy(ii)*(1/(2*dy))*(1/(2*dz))*dZz(ii,Ny-2)*Kp2;                                                          
end 
end


%++++++++++++++ KpE dQy (-4/(2*dy)) (Fyy)^Ny-1 +++++++++++++++++;

%-------------- dQy (-4/(2*dy)) Kp1 dQy dV/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+alfa2*dQy(ii)*(-4/(2*dy))*(-1/(2*dy))*dQy(ii)*Kp1;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(ii)*(-4/(2*dy))*(1/(2*dy))*dQy(ii)*Kp1;                              
end 
end

%-------------- dQy (-4/(2*dy)) Kp1 dZy dV/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+alfa2*dQy(ii)*(-4/(2*dy))*(-1/(2*dz))*dZy(ii,Ny-1,kk)*Kp1;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+ alfa2*dQy(ii)*(-4/(2*dy))*(1/(2*dz))*dZy(ii,Ny-1,kk)*Kp1;                                                          
end 
end

%---------------- dQy (-4/(2*dy)) Kp2 dU/dx -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQy(ii)*(-4/(2*dy))*(-1/(2*dx))*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-2)*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-2)*Nz+kk)    + alfa2*dQy(ii)*(-4/(2*dy))*(1/(2*dx))*Kp2;                                          
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*dQy(Nx)*(-4/(2*dy))*(1/(2*dx))*Kp2;
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQy(Nx)*(-4/(2*dy))*(-4/(2*dx))*Kp2;
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*dQy(Nx)*(-4/(2*dy))*(3/(2*dx))*Kp2;                                          
end 


%-------------- dQy 1/dy Kp2 dQx dU/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+alfa2*dQy(ii)*(-4/(2*dy))*(-1/(2*dy))*dQx(ii,Ny-1)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(ii)*(-4/(2*dy))*(1/(2*dy))*dQx(ii,Ny-1)*Kp2;                              
end 
end

%-------------- dQy 1/dy Kp2 dZx dU/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+alfa2*dQy(ii)*(-4/(2*dy))*(-1/(2*dz))*dZx(ii,Ny-1,kk)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+ alfa2*dQy(ii)*(-4/(2*dy))*(1/(2*dz))*dZx(ii,Ny-1,kk)*Kp2;                                                          
end 
end

%--------------- dQy 1/dy Kp2 dZz dW/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+alfa2*dQy(ii)*(-4/(2*dy))*(-1/(2*dz))*dZz(ii,Ny-1)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+ alfa2*dQy(ii)*(-4/(2*dy))*(1/(2*dz))*dZz(ii,Ny-1)*Kp2;                                                          
end 
end



%++++++++++++++ - KpE dQy (3/(2*dy)) (Fyx)^Ny nx +++++++++++++++;

%---------------- dQy (3/(2*dy)) dQy dU/dQ Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQy(ii)*(3/(2*dy))*(1/(2*dy))*dQy(ii)*difY2(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQy(ii)*(3/(2*dy))*(-4/(2*dy))*dQy(ii)*difY2(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(ii)*(3/(2*dy))*(3/(2*dy))*dQy(ii)*difY2(ii);                              
end 
end

%---------------- dQy (3/(2*dy)) dZy dU/dz Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dQy(ii)*(3/(2*dy))*(-1/(2*dz))*dZy(ii,Ny,kk)*difY2(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dQy(ii)*(3/(2*dy))*(1/(2*dz))*dZy(ii,Ny,kk)*difY2(ii);                                                          
end 
end

%------------------ dQy (3/(2*dy)) dV/dx Y2x -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*dQy(ii)*(3/(2*dy))*(-1/(2*dx))*difY2(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk) +    alfa2*dQy(ii)*(3/(2*dy))*(1/(2*dx))*difY2(ii);                                          
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(Nx)*(3/(2*dy))*(1/(2*dx))*difY2(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*dQy(Nx)*(3/(2*dy))*(-4/(2*dx))*difY2(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(Nx)*(3/(2*dy))*(3/(2*dx))*difY2(Nx);                                          
end 


%--------------- dQy (3/(2*dy)) dQx dV/dQ Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQy(ii)*(3/(2*dy))*(1/(2*dy))*dQx(ii,Ny)*difY2(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQy(ii)*(3/(2*dy))*(-4/(2*dy))*dQx(ii,Ny)*difY2(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(ii)*(3/(2*dy))*(3/(2*dy))*dQx(ii,Ny)*difY2(ii);                              
end 
end

%--------------- dQy (3/(2*dy)) dZx dV/dz Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dQy(ii)*(3/(2*dy))*(-1/(2*dz))*dZx(ii,Ny,kk)*difY2(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dQy(ii)*(3/(2*dy))*(1/(2*dz))*dZx(ii,Ny,kk)*difY2(ii);                                                          
end 
end


%+++++++++++++++++++++++ - KpE dZ/dy dFyx/dz nx ++++++++++++++++++++++++;

%-------------------- dZ/dy d/dz ( dQ/dy dU/dQ ) Y2x --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*dQy(ii)*(-1/(4*dz*dy))*difY2(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*dZy(ii,Ny,kk)*dQy(ii)*(4/(4*dz*dy))*difY2(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*dQy(ii)*(-3/(4*dz*dy))*difY2(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dZy(ii,Ny,kk)*dQy(ii)*(1/(4*dz*dy))*difY2(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*dQy(ii)*(-4/(4*dz*dy))*difY2(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZy(ii,Ny,kk)*dQy(ii)*(3/(4*dz*dy))*difY2(ii); 
end 
end

%--------------------- dZ/dy d/dz ( dZ/dy dU/dz ) Y2x ------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*(Gpz2_ijk(ii,Ny,kk)/dz^2)*difY2(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*dZy(ii,Ny,kk)*(-(Gpz2_ijk(ii,Ny,kk)+Gpz2_ijk(ii,Ny,kk+1))/dz^2)*difY2(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*(Gpz2_ijk(ii,Ny,kk+1)/dz^2)*difY2(ii); 
end  
end


%-------------------- dZ/dy d/dz ( dV/dx ) Y2x --------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*dZy(ii,Ny,kk)*(1/(4*dz*dx))*difY2(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)+    alfa2*dZy(ii,Ny,kk)*(-1/(4*dz*dx))*difY2(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*(-1/(4*dz*dx))*difY2(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)+     alfa2*dZy(ii,Ny,kk)*(1/(4*dz*dx))*difY2(ii); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(Nx,Ny,kk)*(-1/(4*dz*dx))*difY2(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZy(Nx,Ny,kk)*(1/(4*dz*dx))*difY2(Nx); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*dZy(Nx,Ny,kk)*(4/(4*dz*dx))*difY2(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZy(Nx,Ny,kk)*(-4/(4*dz*dx))*difY2(Nx); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(Nx,Ny,kk)*(-3/(4*dz*dx))*difY2(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZy(Nx,Ny,kk)*(3/(4*dz*dx))*difY2(Nx); 
end 


%---------------- dZ/dy d/dz ( dQ/dx dV/dQ ) Y2x --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*dQx(ii,Ny)*(-1/(4*dz*dy))*difY2(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*dZy(ii,Ny,kk)*dQx(ii,Ny)*(4/(4*dz*dy))*difY2(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*dQx(ii,Ny)*(-3/(4*dz*dy))*difY2(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dZy(ii,Ny,kk)*dQx(ii,Ny)*(1/(4*dz*dy))*difY2(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*dQx(ii,Ny)*(-4/(4*dz*dy))*difY2(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZy(ii,Ny,kk)*dQx(ii,Ny)*(3/(4*dz*dy))*difY2(ii); 
end 
end

%---------------- dZ/dy d/dz ( dZ/dx dV/dz ) Y2x --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*(Gpz1_ijk(ii,Ny,kk)/dz^2)*difY2(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*dZy(ii,Ny,kk)*(-(Gpz1_ijk(ii,Ny,kk)+Gpz1_ijk(ii,Ny,kk+1))/dz^2)*difY2(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*(Gpz1_ijk(ii,Ny,kk+1)/dz^2)*difY2(ii);                                                                            
end  
end


%+++++++++++++++++++++++ KpE dZ/dz dFyz/dz ++++++++++++++++++++++++;

%------------------- dZ/dz d/dz ( dZ/dz dV/dz ) ------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZz(ii,Ny)*(1/dz^2)*dZz(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dZz(ii,Ny)*(-2/dz^2)*dZz(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZz(ii,Ny)*(1/dz^2)*dZz(ii,Ny); 
end  
end

%-------------------- dZ/dz d/dz ( dQ/dy dW/dQ ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZz(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*dQy(ii)*(4/(4*dz*dy))*dZz(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dQy(ii)*(-3/(4*dz*dy))*dZz(ii,Ny);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dQy(ii)*(1/(4*dz*dy))*dZz(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*dQy(ii)*(-4/(4*dz*dy))*dZz(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dQy(ii)*(3/(4*dz*dy))*dZz(ii,Ny); 
end 
end

%------------------ dZ/dz d/dz ( dZ/dy dW/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*alfa2*(Gpz2_ijk(ii,Ny,kk)/dz^2)*dZz(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*(-(Gpz2_ijk(ii,Ny,kk)+Gpz2_ijk(ii,Ny,kk+1))/dz^2)*dZz(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*(Gpz2_ijk(ii,Ny,kk+1)/dz^2)*dZz(ii,Ny); 
end  
end



%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++++++ + KpE dens Freq^2 V ++++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk) + alfa2*KpE*dens*Freq^2;
end  
end




%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++++++ The third equation ++++++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++++++++ KpE dFzx/dx ++++++++++++++++++++++++++;

%--------------------- d/dx ( dZ/dz dU/dz ) ---------------------;

%------------------- 1: dZ/dz d/dx ( dU/dz ) ---------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)= alfa2*dZz(ii,Ny)*(1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=alfa2*dZz(ii,Ny)*(-1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk-1)=    alfa2*dZz(ii,Ny)*(-1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk+1)=     alfa2*dZz(ii,Ny)*(1/(4*dx*dz));                                  
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)=alfa2*dZz(Nx,Ny)*(-1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)= alfa2*dZz(Nx,Ny)*(1/(4*dx*dz));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)= alfa2*dZz(Nx,Ny)*(4/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=alfa2*dZz(Nx,Ny)*(-4/(4*dx*dz));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)= alfa2*dZz(Nx,Ny)*(-3/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=  alfa2*dZz(Nx,Ny)*(3/(4*dx*dz));                                  
end

%------------------- 2: d^2Z/dzdx  dU/dz ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*d2Zzx(ii,Ny)*(-1/(2*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*d2Zzx(ii,Ny)*(1/(2*dz)); 
end 
end


%-------------------- dQ/dx d/dQ ( dZ/dz dU/dz ) --------------------;

%------------------ 1: dQ/dx dZ/dz d/dQ ( dU/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dZz(ii,Ny)*(-1/(4*dy*dz))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*dZz(ii,Ny)*(4/(4*dy*dz))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZz(ii,Ny)*(-3/(4*dy*dz))*dQx(ii,Ny); 

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dZz(ii,Ny)*(1/(4*dy*dz))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*dZz(ii,Ny)*(-4/(4*dy*dz))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZz(ii,Ny)*(3/(4*dy*dz))*dQx(ii,Ny); 
end 
end

%------------------ 2: dQ/dx d^2Z/dzdq  dU/dz --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*d2Zzq(ii,Ny)*(-1/(2*dz))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*d2Zzq(ii,Ny)*(1/(2*dz))*dQx(ii,Ny); 
end 
end


%------------------- dZ/dx d/dz ( dZ/dz dU/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZz(ii,Ny)*(1/dz^2)*dZx(ii,Ny,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dZz(ii,Ny)*(-2/dz^2)*dZx(ii,Ny,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZz(ii,Ny)*(1/dz^2)*dZx(ii,Ny,kk); 
end  
end

%------------------------ d/dx ( dW/dx ) -----------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)= alfa2*(1/dx^2); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=alfa2*(-2/dx^2); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)=     alfa2*(1/dx^2); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-4)*Ny*Nz+(Ny-1)*Nz+kk)=alfa2*(-1/dx^2); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)= alfa2*(4/dx^2); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=alfa2*(-5/dx^2); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)= alfa2*(2/dx^2); 
end 


%--------------------- d/dx ( dQ/dx dW/dQ ) ---------------------;

%------------------- 1: dQ/dx d/dx ( dW/dQ ) ---------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)+alfa2*dQx(ii,Ny)*(-1/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*dQx(ii,Ny)*(4/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*dQx(ii,Ny)*(-3/(4*dx*dy));

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQx(ii,Ny)*(1/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQx(ii,Ny)*(-4/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQx(ii,Ny)*(3/(4*dx*dy)); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQx(Nx,Ny)*(1/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQx(Nx,Ny)*(-4/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQx(Nx,Ny)*(3/(4*dx*dy));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQx(Nx,Ny)*(-4/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*dQx(Nx,Ny)*(16/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*dQx(Nx,Ny)*(-12/(4*dx*dy));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)+  alfa2*dQx(Nx,Ny)*(3/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQx(Nx,Ny)*(-12/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)+  alfa2*dQx(Nx,Ny)*(9/(4*dx*dy)); 
end

%------------------- 2: d^2Q/dxdx  dW/dQ ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*d2Qxx(ii,Ny)*(1/(2*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*d2Qxx(ii,Ny)*(-4/(2*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*d2Qxx(ii,Ny)*(3/(2*dy));
end 
end


%--------------------- d/dx ( dZ/dx dW/dz ) ---------------------;

%------------------- 1: dZ/dx d/dx ( dW/dz ) ---------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*dZx(ii,Ny,kk)*(1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZx(ii,Ny,kk)*(-1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)+    alfa2*dZx(ii,Ny,kk)*(-1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)+     alfa2*dZx(ii,Ny,kk)*(1/(4*dx*dz)); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZx(Nx,Ny,kk)*(-1/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZx(Nx,Ny,kk)*(1/(4*dx*dz));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*dZx(Nx,Ny,kk)*(4/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZx(Nx,Ny,kk)*(-4/(4*dx*dz));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZx(Nx,Ny,kk)*(-3/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZx(Nx,Ny,kk)*(3/(4*dx*dz)); 
end

%------------------- 2: d^2Z/dxdx  dW/dz ---------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*d2Zxx(ii,Ny,kk)*(-1/(2*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*d2Zxx(ii,Ny,kk)*(1/(2*dz)); 
end 
end


%-------------------- dQ/dx d/dQ ( dW/dx ) ---------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+kk)+alfa2*(-1/(4*dy*dx))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*(4/(4*dy*dx))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*(-3/(4*dy*dx))*dQx(ii,Ny); 

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*(1/(4*dy*dx))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*(-4/(4*dy*dx))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*(3/(4*dy*dx))*dQx(ii,Ny); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*(1/(4*dy*dx))*dQx(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*(3/(4*dy*dx))*dQx(Nx,Ny);     

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*(16/(4*dy*dx))*dQx(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*(-12/(4*dy*dx))*dQx(Nx,Ny); 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+kk)+  alfa2*(3/(4*dy*dx))*dQx(Nx,Ny); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*(-12/(4*dy*dx))*dQx(Nx,Ny); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)+  alfa2*(9/(4*dy*dx))*dQx(Nx,Ny); 
end 


%------------------- dQ/dx d/dQ ( dQ/dx dW/dQ ) --------------------;

%----------------- 1: dQ/dx dQ/dx d/dQ ( dW/dQ ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)+alfa2*(-1/dy^2)*dQx(ii,Ny)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*(4/dy^2)*dQx(ii,Ny)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*(-5/dy^2)*dQx(ii,Ny)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*(2/dy^2)*dQx(ii,Ny)^2;                                                                            
end 
end

%----------------- 2: dQ/dx d^2Q/dxdq  dW/dQ --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*d2Qxq(ii,Ny)*(1/(2*dy))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*d2Qxq(ii,Ny)*(-4/(2*dy))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*d2Qxq(ii,Ny)*(3/(2*dy))*dQx(ii,Ny);                                                                            
end 
end


%-------------------- dQ/dx d/dQ ( dZ/dx dW/dz ) --------------------;

%------------------ 1: dQ/dx dZ/dx d/dQ ( dW/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dZx(ii,Ny,kk)*(-1/(4*dy*dz))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*dZx(ii,Ny,kk)*(4/(4*dy*dz))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZx(ii,Ny,kk)*(-3/(4*dy*dz))*dQx(ii,Ny); 

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dZx(ii,Ny,kk)*(1/(4*dy*dz))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*dZx(ii,Ny,kk)*(-4/(4*dy*dz))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZx(ii,Ny,kk)*(3/(4*dy*dz))*dQx(ii,Ny);                                                                                         
end 
end

%------------------ 2: dQ/dx d^2Z/dxdq  dW/dz --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*d2Zxq(ii,Ny,kk)*(-1/(2*dz))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*d2Zxq(ii,Ny,kk)*(1/(2*dz))*dQx(ii,Ny);                                                                                         
end 
end


%-------------------- dZ/dx d/dz ( dW/dx ) --------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*(1/(4*dz*dx))*dZx(ii,Ny,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)+    alfa2*(-1/(4*dz*dx))*dZx(ii,Ny,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*(-1/(4*dz*dx))*dZx(ii,Ny,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)+     alfa2*(1/(4*dz*dx))*dZx(ii,Ny,kk); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*(-1/(4*dz*dx))*dZx(Nx,Ny,kk);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*alfa2*(1/(4*dz*dx))*dZx(Nx,Ny,kk); 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*(4/(4*dz*dx))*dZx(Nx,Ny,kk);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*(-4/(4*dz*dx))*dZx(Nx,Ny,kk); 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*(-3/(4*dz*dx))*dZx(Nx,Ny,kk); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*(3/(4*dz*dx))*dZx(Nx,Ny,kk); 
end 


%-------------------- dZ/dx d/dz ( dQ/dx dW/dQ ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dQx(ii,Ny)*(-1/(4*dz*dy))*dZx(ii,Ny,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*dQx(ii,Ny)*(4/(4*dz*dy))*dZx(ii,Ny,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dQx(ii,Ny)*(-3/(4*dz*dy))*dZx(ii,Ny,kk);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dQx(ii,Ny)*(1/(4*dz*dy))*dZx(ii,Ny,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*dQx(ii,Ny)*(-4/(4*dz*dy))*dZx(ii,Ny,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dQx(ii,Ny)*(3/(4*dz*dy))*dZx(ii,Ny,kk); 
end 
end


%-------------------- dZ/dx d/dz ( dZ/dx dW/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*(Gpz1_ijk(ii,Ny,kk)/dz^2)*dZx(ii,Ny,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*(-(Gpz1_ijk(ii,Ny,kk)+Gpz1_ijk(ii,Ny,kk+1))/dz^2)*dZx(ii,Ny,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*(Gpz1_ijk(ii,Ny,kk+1)/dz^2)*dZx(ii,Ny,kk);                                                                            
end  
end



%++++++++++++++ KpE dQy 1/(2*dy) (Fzy)^Ny-2 +++++++++++++++++;

%---------------- dQy 1/(2*dy) dZz dV/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dQy(ii)*(1/(2*dy))*(-1/(2*dz))*dZz(ii,Ny-2);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dQy(ii)*(1/(2*dy))*(1/(2*dz))*dZz(ii,Ny-2);                                                          
end 
end

%---------------- dQy 1/(2*dy) dQy dW/dQ ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+kk)+alfa2*dQy(ii)*(1/(2*dy))*(-1/(2*dy))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+ alfa2*dQy(ii)*(1/(2*dy))*(1/(2*dy))*dQy(ii);                              
end 
end

%---------------- dQy 1/(2*dy) dZy dW/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dQy(ii)*(1/(2*dy))*(-1/(2*dz))*dZy(ii,Ny-2,kk);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dQy(ii)*(1/(2*dy))*(1/(2*dz))*dZy(ii,Ny-2,kk);                                                          
end 
end


%++++++++++++++ KpE dQy (-4/(2*dy)) (Fzy)^Ny-1 ++++++++++++++;

%---------------- dQy (-4/(2*dy)) dZz dV/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+alfa2*dQy(ii)*(-4/(2*dy))*(-1/(2*dz))*dZz(ii,Ny-1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+ alfa2*dQy(ii)*(-4/(2*dy))*(1/(2*dz))*dZz(ii,Ny-1);                                                          
end 
end

%---------------- dQy (-4/(2*dy)) dQy dW/dQ ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+alfa2*dQy(ii)*(-4/(2*dy))*(-1/(2*dy))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(ii)*(-4/(2*dy))*(1/(2*dy))*dQy(ii);                              
end 
end

%---------------- dQy (-4/(2*dy)) dZy dW/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+alfa2*dQy(ii)*(-4/(2*dy))*(-1/(2*dz))*dZy(ii,Ny-1,kk);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+ alfa2*dQy(ii)*(-4/(2*dy))*(1/(2*dz))*dZy(ii,Ny-1,kk);                                                          
end 
end


%+++++++++++++ - KpE dQy 3/(2*dy) (Fzx)^Ny nx +++++++++++++;

%---------------- dQy 3/(2*dy) dZz dU/dz Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dQy(ii)*(3/(2*dy))*(-1/(2*dz))*dZz(ii,Ny)*difY2(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dQy(ii)*(3/(2*dy))*(1/(2*dz))*dZz(ii,Ny)*difY2(ii);                                                          
end 
end

%----------------- dQy 3/(2*dy) dW/dx Y2x -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*dQy(ii)*(3/(2*dy))*(-1/(2*dx))*difY2(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk) +    alfa2*dQy(ii)*(3/(2*dy))*(1/(2*dx))*difY2(ii);                                          
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(Nx)*(3/(2*dy))*(1/(2*dx))*difY2(Nx);    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*dQy(Nx)*(3/(2*dy))*(-4/(2*dx))*difY2(Nx);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(Nx)*(3/(2*dy))*(3/(2*dx))*difY2(Nx);                                          
end 


%--------------- dQy 3/(2*dy) dQx dW/dQ Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa2*dQy(ii)*(3/(2*dy))*(1/(2*dy))*dQx(ii,Ny)*difY2(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa2*dQy(ii)*(3/(2*dy))*(-4/(2*dy))*dQx(ii,Ny)*difY2(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dQy(ii)*(3/(2*dy))*(3/(2*dy))*dQx(ii,Ny)*difY2(ii);                              
end 
end

%--------------- dQy 3/(2*dy) dZx dW/dz Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dQy(ii)*(3/(2*dy))*(-1/(2*dz))*dZx(ii,Ny,kk)*difY2(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dQy(ii)*(3/(2*dy))*(1/(2*dz))*dZx(ii,Ny,kk)*difY2(ii);                                                          
end 
end



%+++++++++++++++++++ - KpE dZ/dy dFzx/dz nx ++++++++++++++++++++;

%------------------- dZ/dy d/dz ( dZ/dz dU/dz ) Y2x -----------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*dZz(ii,Ny)*(1/dz^2)*difY2(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*dZy(ii,Ny,kk)*dZz(ii,Ny)*(-2/dz^2)*difY2(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*dZz(ii,Ny)*(1/dz^2)*difY2(ii); 
end  
end

%-------------------- dZ/dy d/dz ( dW/dx ) Y2x --------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*dZy(ii,Ny,kk)*(1/(4*dz*dx))*difY2(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk-1)+    alfa2*dZy(ii,Ny,kk)*(-1/(4*dz*dx))*difY2(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*(-1/(4*dz*dx))*difY2(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk+1)+     alfa2*dZy(ii,Ny,kk)*(1/(4*dz*dx))*difY2(ii); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(Nx,Ny,kk)*(-1/(4*dz*dx))*difY2(Nx);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZy(Nx,Ny,kk)*(1/(4*dz*dx))*difY2(Nx);     
    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*dZy(Nx,Ny,kk)*(4/(4*dz*dx))*difY2(Nx);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZy(Nx,Ny,kk)*(-4/(4*dz*dx))*difY2(Nx); 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(Nx,Ny,kk)*(-3/(4*dz*dx))*difY2(Nx); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZy(Nx,Ny,kk)*(3/(4*dz*dx))*difY2(Nx); 
end 


%-------------------- dZ/dy d/dz ( dQ/dx dW/dQ ) Y2x --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*dQx(ii,Ny)*(-1/(4*dz*dy))*difY2(ii);     
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*dZy(ii,Ny,kk)*dQx(ii,Ny)*(4/(4*dz*dy))*difY2(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*dQx(ii,Ny)*(-3/(4*dz*dy))*difY2(ii);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*dZy(ii,Ny,kk)*dQx(ii,Ny)*(1/(4*dz*dy))*difY2(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*dQx(ii,Ny)*(-4/(4*dz*dy))*difY2(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*dZy(ii,Ny,kk)*dQx(ii,Ny)*(3/(4*dz*dy))*difY2(ii); 
end 
end

%-------------------- dZ/dy d/dz ( dZ/dx dW/dz ) Y2x --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*dZy(ii,Ny,kk)*(Gpz1_ijk(ii,Ny,kk)/dz^2)*difY2(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*dZy(ii,Ny,kk)*(-(Gpz1_ijk(ii,Ny,kk)+Gpz1_ijk(ii,Ny,kk+1))/dz^2)*difY2(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*dZy(ii,Ny,kk)*(Gpz1_ijk(ii,Ny,kk+1)/dz^2)*difY2(ii);                                                                            
end  
end


%+++++++++++++++++++++++ KpE dZ/dz dFzz/dz ++++++++++++++++++++++++;

%------------------ alfa2*Kp1 dZ/dz d/dz ( dZ/dz dW/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp1*dZz(ii,Ny)*(1/dz^2)*dZz(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa2*Kp1*dZz(ii,Ny)*(-2/dz^2)*dZz(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*Kp1*dZz(ii,Ny)*(1/dz^2)*dZz(ii,Ny); 
end  
end

%------------------- Kp2 dZ/dz d/dz ( dU/dx ) --------------------;

for ii=2:1:Nx-1 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*Kp2*(1/(4*dz*dx))*dZz(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk-1)+    alfa2*Kp2*(-1/(4*dz*dx))*dZz(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*Kp2*(-1/(4*dz*dx))*dZz(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk+1)+     alfa2*Kp2*(1/(4*dz*dx))*dZz(ii,Ny); 
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*(-1/(4*dz*dx))*dZz(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp2*(1/(4*dz*dx))*dZz(Nx,Ny);     
    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk-1)+ alfa2*Kp2*(4/(4*dz*dx))*dZz(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*Kp2*(-4/(4*dz*dx))*dZz(Nx,Ny); 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*(-3/(4*dz*dx))*dZz(Nx,Ny); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp2*(3/(4*dz*dx))*dZz(Nx,Ny); 
end 


%---------------- Kp2 dZ/dz d/dz ( dQ/dx dU/dQ ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*Kp2*dQx(ii,Ny)*(-1/(4*dz*dy))*dZz(ii,Ny);     
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*Kp2*dQx(ii,Ny)*(4/(4*dz*dy))*dZz(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*dQx(ii,Ny)*(-3/(4*dz*dy))*dZz(ii,Ny);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*Kp2*dQx(ii,Ny)*(1/(4*dz*dy))*dZz(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*Kp2*dQx(ii,Ny)*(-4/(4*dz*dy))*dZz(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp2*dQx(ii,Ny)*(3/(4*dz*dy))*dZz(ii,Ny); 
end 
end

%---------------- Kp2 dZ/dz d/dz ( dZ/dx dU/dz ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*(Gpz1_ijk(ii,Ny,kk)/dz^2)*dZz(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*Kp2*(-(Gpz1_ijk(ii,Ny,kk)+Gpz1_ijk(ii,Ny,kk+1))/dz^2)*dZz(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*Kp2*(Gpz1_ijk(ii,Ny,kk+1)/dz^2)*dZz(ii,Ny);                                                                            
end  
end

%------------------ Kp2 dZ/dz d/dz ( dQ/dy dV/dQ ) --------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk-1)+alfa2*Kp2*dQy(ii)*(-1/(4*dz*dy))*dZz(ii,Ny);     
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk-1)+ alfa2*Kp2*dQy(ii)*(4/(4*dz*dy))*dZz(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*dQy(ii)*(-3/(4*dz*dy))*dZz(ii,Ny);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk+1)+ alfa2*Kp2*dQy(ii)*(1/(4*dz*dy))*dZz(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk+1)+alfa2*Kp2*dQy(ii)*(-4/(4*dz*dy))*dZz(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa2*Kp2*dQy(ii)*(3/(4*dz*dy))*dZz(ii,Ny); 
end 
end

%------------------ Kp2 dZ/dz d/dz ( dZ/dy dV/dz ) -------------------;

for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa2*Kp2*(Gpz2_ijk(ii,Ny,kk)/dz^2)*dZz(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+alfa2*Kp2*(-(Gpz2_ijk(ii,Ny,kk)+Gpz2_ijk(ii,Ny,kk+1))/dz^2)*dZz(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+alfa2*Kp2*(Gpz2_ijk(ii,Ny,kk+1)/dz^2)*dZz(ii,Ny); 
end  
end


%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++++++ + KpE dens Freq^2 W ++++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


for ii=2:1:Nx 
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk) + alfa2*KpE*dens*Freq^2; 
end  
end



%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++ Basic Boundary Conditions at the lateral edge Y=Y2 ++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++ The first equation (basic equation #1) ++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++ KpE Fxx nx ++++++++++++++++++;

%--------------- - Kp1 dU/dx Y2x -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)-alfa1*Kp1*(-1/(2*dx))*difY2(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk)  -   alfa1*Kp1*(1/(2*dx))*difY2(ii);                                          
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)- alfa1*Kp1*(1/(2*dx))*difY2(Nx);    
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)-alfa1*Kp1*(-4/(2*dx))*difY2(Nx);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)- alfa1*Kp1*(3/(2*dx))*difY2(Nx);                                          
end 


%--------------- - Kp1 dQx dU/dQ Y2x ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)- alfa1*Kp1*(1/(2*dy))*dQx(ii,Ny)*difY2(ii);   
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)-alfa1*Kp1*(-4/(2*dy))*dQx(ii,Ny)*difY2(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)- alfa1*Kp1*(3/(2*dy))*dQx(ii,Ny)*difY2(ii);                              
end 
end

%--------------- - Kp1 dZx dU/dz Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)-alfa1*Kp1*(-1/(2*dz))*dZx(ii,Ny,kk)*difY2(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)- alfa1*Kp1*(1/(2*dz))*dZx(ii,Ny,kk)*difY2(ii);                                                          
end 
end

%--------------- - Kp2 dQy dV/dQ Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)- alfa1*Kp2*(1/(2*dy))*dQy(ii)*difY2(ii);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)-alfa1*Kp2*(-4/(2*dy))*dQy(ii)*difY2(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)- alfa1*Kp2*(3/(2*dy))*dQy(ii)*difY2(ii);                              
end 
end

%--------------- - Kp2 dZy dV/dz Y2x ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)-alfa1*Kp2*(-1/(2*dz))*dZy(ii,Ny,kk)*difY2(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)- alfa1*Kp2*(1/(2*dz))*dZy(ii,Ny,kk)*difY2(ii);                                                          
end 
end

%--------------- - Kp2 dZz dW/dz Y2x ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)-alfa1*Kp2*(-1/(2*dz))*dZz(ii,Ny)*difY2(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)- alfa1*Kp2*(1/(2*dz))*dZz(ii,Ny)*difY2(ii);                                                          
end 
end



%++++++++++++++++ KpE Fxy ny +++++++++++++++++;

%---------------- dQy dU/dQ ------------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa1*(1/(2*dy))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa1*(-4/(2*dy))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa1*(3/(2*dy))*dQy(ii);                              
end 
end

%---------------- dZy dU/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa1*(-1/(2*dz))*dZy(ii,Ny,kk);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa1*(1/(2*dz))*dZy(ii,Ny,kk);                                                          
end 
end

%------------------ dV/dx -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa1*(-1/(2*dx));
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)    + alfa1*(1/(2*dx));                                          
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa1*(1/(2*dx));    
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa1*(-4/(2*dx));
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa1*(3/(2*dx));                                          
end 


%--------------- dQx dV/dQ ---------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa1*(1/(2*dy))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa1*(-4/(2*dy))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa1*(3/(2*dy))*dQx(ii,Ny);                              
end 
end

%-------------- dZx dV/dz ---------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa1*(-1/(2*dz))*dZx(ii,Ny,kk);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa1*(1/(2*dz))*dZx(ii,Ny,kk);                                                          
end 
end




%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++ The second equation (basic equation #2) +++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++ KpE Fyx nx +++++++++++++++++++;

%---------------- - dQy dU/dQ Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)- alfa1*(1/(2*dy))*dQy(ii)*difY2(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)-alfa1*(-4/(2*dy))*dQy(ii)*difY2(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)- alfa1*(3/(2*dy))*dQy(ii)*difY2(ii);                              
end 
end

%---------------- - dZy dU/dz Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)-alfa1*(-1/(2*dz))*dZy(ii,Ny,kk)*difY2(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)- alfa1*(1/(2*dz))*dZy(ii,Ny,kk)*difY2(ii);                                                          
end 
end

%------------------ - dV/dx Y2x -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)-alfa1*(-1/(2*dx))*difY2(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk) -    alfa1*(1/(2*dx))*difY2(ii);                                          
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)- alfa1*(1/(2*dx))*difY2(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)-alfa1*(-4/(2*dx))*difY2(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)- alfa1*(3/(2*dx))*difY2(Nx);                                          
end 


%--------------- - dQx dV/dQ Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)- alfa1*(1/(2*dy))*dQx(ii,Ny)*difY2(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)-alfa1*(-4/(2*dy))*dQx(ii,Ny)*difY2(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)- alfa1*(3/(2*dy))*dQx(ii,Ny)*difY2(ii);                              
end 
end

%--------------- - dZx dV/dz Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)-alfa1*(-1/(2*dz))*dZx(ii,Ny,kk)*difY2(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)- alfa1*(1/(2*dz))*dZx(ii,Ny,kk)*difY2(ii);                                                          
end 
end



%+++++++++++++++++ KpE Fyy ny +++++++++++++++++;

%-------------- Kp1 dQy dV/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa1*(1/(2*dy))*dQy(ii)*Kp1;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa1*(-4/(2*dy))*dQy(ii)*Kp1;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa1*(3/(2*dy))*dQy(ii)*Kp1;                              
end 
end

%-------------- Kp1 dZy dV/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa1*(-1/(2*dz))*dZy(ii,Ny,kk)*Kp1;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa1*(1/(2*dz))*dZy(ii,Ny,kk)*Kp1;                                                          
end 
end

%---------------- Kp2 dU/dx -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa1*(-1/(2*dx))*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,ii*Ny*Nz+(Ny-1)*Nz+kk)    + alfa1*(1/(2*dx))*Kp2;                                          
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa1*(1/(2*dx))*Kp2;
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)+alfa1*(-4/(2*dx))*Kp2;
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa1*(3/(2*dx))*Kp2;                                          
end 


%-------------- Kp2 dQx dU/dQ -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa1*(1/(2*dy))*dQx(ii,Ny)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa1*(-4/(2*dy))*dQx(ii,Ny)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa1*(3/(2*dy))*dQx(ii,Ny)*Kp2;                              
end 
end

%-------------- Kp2 dZx dU/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa1*(-1/(2*dz))*dZx(ii,Ny,kk)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa1*(1/(2*dz))*dZx(ii,Ny,kk)*Kp2;                                                          
end 
end

%--------------- Kp2 dZz dW/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa1*(-1/(2*dz))*dZz(ii,Ny)*Kp2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa1*(1/(2*dz))*dZz(ii,Ny)*Kp2;                                                          
end 
end



%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++ The third equation (basic equation #3) +++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++ KpE Fzx nx ++++++++++++++++++;

%---------------- - dZz dU/dz Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)-alfa1*(-1/(2*dz))*dZz(ii,Ny)*difY2(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)- alfa1*(1/(2*dz))*dZz(ii,Ny)*difY2(ii);                                                          
end 
end

%----------------- - dW/dx Y2x -----------------;

for ii=2:1:Nx-1
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+kk)-alfa1*(-1/(2*dx))*difY2(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+kk) -    alfa1*(1/(2*dx))*difY2(ii);                                          
end 
end

%-------- i=Nx --------;

for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+kk)- alfa1*(1/(2*dx))*difY2(Nx);    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+kk)-alfa1*(-4/(2*dx))*difY2(Nx);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+kk)- alfa1*(3/(2*dx))*difY2(Nx);                                          
end 


%--------------- - dQx dW/dQ Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)- alfa1*(1/(2*dy))*dQx(ii,Ny)*difY2(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)-alfa1*(-4/(2*dy))*dQx(ii,Ny)*difY2(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)- alfa1*(3/(2*dy))*dQx(ii,Ny)*difY2(ii);                              
end 
end

%--------------- - dZx dW/dz Y2x -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)-alfa1*(-1/(2*dz))*dZx(ii,Ny,kk)*difY2(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)- alfa1*(1/(2*dz))*dZx(ii,Ny,kk)*difY2(ii);                                                          
end 
end



%++++++++++++++++ KpE Fzy ny +++++++++++++++++;

%----------------- dZz dV/dz ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa1*(-1/(2*dz))*dZz(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa1*(1/(2*dz))*dZz(ii,Ny);                                                          
end 
end

%---------------- dQy dW/dQ ----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+kk)+ alfa1*(1/(2*dy))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+kk)+alfa1*(-4/(2*dy))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk)+ alfa1*(3/(2*dy))*dQy(ii);                              
end 
end

%---------------- dZy dW/dz -----------------;

for ii=2:1:Nx
for kk=2:1:Nz-1                                              
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk-1)+alfa1*(-1/(2*dz))*dZy(ii,Ny,kk);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+kk+1)+ alfa1*(1/(2*dz))*dZy(ii,Ny,kk);                                                          
end 
end





%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++ Ice Shelf Surface (k=1) +++++++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++++ The first equation ++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

%++++++++++++++++++++++ KpE dFxx/dx +++++++++++++++++++++++;

%------------------- Kp1 d/dx ( dU/dx ) -------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)= alfa2*Kp1*(1/dx^2); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=alfa2*Kp1*(-2/dx^2); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)=     alfa2*Kp1*(1/dx^2);                                        
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-4)*Ny*Nz+(jj-1)*Nz+1)=alfa2*Kp1*(-1/dx^2);     
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)= alfa2*Kp1*(4/dx^2); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=alfa2*Kp1*(-5/dx^2); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)= alfa2*Kp1*(2/dx^2);                                        
end 

%----------------- Kp1 d/dx ( dQ/dx dU/dQ ) --------------;

%--------------- 1: Kp1 dQ/dx d/dx ( dU/dQ ) --------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp1*dQx(ii,jj)*(1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+jj*Nz+1)+   alfa2*Kp1*dQx(ii,jj)*(-1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-2)*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-2)*Nz+1)+   alfa2*Kp1*dQx(ii,jj)*(-1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+jj*Nz+1)=        MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+jj*Nz+1)+        alfa2*Kp1*dQx(ii,jj)*(1/(4*dx*dy));                                  
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB((ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+1)+     alfa2*Kp1*dQx(ii,1)*(3/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+Nz+1)+ alfa2*Kp1*dQx(ii,1)*(-4/(4*dx*dy));
MB((ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+2*Nz+1)+alfa2*Kp1*dQx(ii,1)*(1/(4*dx*dy));

MB((ii-1)*Ny*Nz+1,ii*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,ii*Ny*Nz+1)+     alfa2*Kp1*dQx(ii,1)*(-3/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+1,ii*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,ii*Ny*Nz+Nz+1)+   alfa2*Kp1*dQx(ii,1)*(4/(4*dx*dy));
MB((ii-1)*Ny*Nz+1,ii*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,ii*Ny*Nz+2*Nz+1)+alfa2*Kp1*dQx(ii,1)*(-1/(4*dx*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*Kp1*dQx(ii,Ny)*(-1/(4*dx*dy));     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*Kp1*dQx(ii,Ny)*(4/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp1*dQx(ii,Ny)*(-3/(4*dx*dy));

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp1*dQx(ii,Ny)*(1/(4*dx*dy));
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*dQx(ii,Ny)*(-4/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp1*dQx(ii,Ny)*(3/(4*dx*dy));                                  
end

%---- i=Nx ----;

MB((Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+1)=     MB((Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+1)+     alfa2*Kp1*dQx(Nx,1)*(-3/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+Nz+1)=  MB((Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+Nz+1)+   alfa2*Kp1*dQx(Nx,1)*(4/(4*dx*dy));
MB((Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+2*Nz+1)=MB((Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+2*Nz+1)+alfa2*Kp1*dQx(Nx,1)*(-1/(4*dx*dy));

MB((Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+1)=     MB((Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+1)+    alfa2*Kp1*dQx(Nx,1)*(12/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+Nz+1)=  MB((Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+Nz+1)+alfa2*Kp1*dQx(Nx,1)*(-16/(4*dx*dy));
MB((Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+2*Nz+1)=MB((Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+2*Nz+1)+alfa2*Kp1*dQx(Nx,1)*(4/(4*dx*dy));

MB((Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+1)=     MB((Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+1)+     alfa2*Kp1*dQx(Nx,1)*(-9/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+Nz+1)=  MB((Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+Nz+1)+  alfa2*Kp1*dQx(Nx,1)*(12/(4*dx*dy));
MB((Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+2*Nz+1)=MB((Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+2*Nz+1)+alfa2*Kp1*dQx(Nx,1)*(-3/(4*dx*dy));


for jj=2:1:Ny-1
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp1*dQx(Nx,jj)*(-1/(4*dx*dy));    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp1*dQx(Nx,jj)*(4/(4*dx*dy));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp1*dQx(Nx,jj)*(-3/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+jj*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+jj*Nz+1)+ alfa2*Kp1*dQx(Nx,jj)*(1/(4*dx*dy));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+jj*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+jj*Nz+1)+alfa2*Kp1*dQx(Nx,jj)*(-4/(4*dx*dy));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+jj*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+jj*Nz+1)+ alfa2*Kp1*dQx(Nx,jj)*(3/(4*dx*dy));                                  
end 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp1*dQx(Nx,Ny)*(1/(4*dx*dy));
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*dQx(Nx,Ny)*(-4/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp1*dQx(Nx,Ny)*(3/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp1*dQx(Nx,Ny)*(-4/(4*dx*dy));
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*Kp1*dQx(Nx,Ny)*(16/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp1*dQx(Nx,Ny)*(-12/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)+  alfa2*Kp1*dQx(Nx,Ny)*(3/(4*dx*dy));
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*dQx(Nx,Ny)*(-12/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)+  alfa2*Kp1*dQx(Nx,Ny)*(9/(4*dx*dy));


%---------------- 2: Kp1 d^2Q/dxdx  dU/dQ ----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp1*d2Qxx(ii,jj)*(-1/(2*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*Kp1*d2Qxx(ii,jj)*(1/(2*dy)); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*Kp1*d2Qxx(ii,1)*(-3/(2*dy)); 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+   alfa2*Kp1*d2Qxx(ii,1)*(4/(2*dy));
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp1*d2Qxx(ii,1)*(-1/(2*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp1*d2Qxx(ii,Ny)*(1/(2*dy));     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*d2Qxx(ii,Ny)*(-4/(2*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp1*d2Qxx(ii,Ny)*(3/(2*dy));
end


%--------------------- Kp1 d/dx ( dZ/dx dU/dz ) --------------;

%------------------- 1: Kp1 dZ/dx d/dx ( dU/dz ) --------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp1*dZx(ii,jj,1)*(3/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp1*dZx(ii,jj,1)*(-4/(4*dx*dz));
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp1*dZx(ii,jj,1)*(1/(4*dx*dz));

MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*dZx(ii,jj,1)*(-3/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp1*dZx(ii,jj,1)*(4/(4*dx*dz));
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp1*dZx(ii,jj,1)*(-1/(4*dx*dz));
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*dZx(Nx,jj,1)*(-3/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp1*dZx(Nx,jj,1)*(4/(4*dx*dz));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp1*dZx(Nx,jj,1)*(-1/(4*dx*dz));    
    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp1*dZx(Nx,jj,1)*(12/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp1*dZx(Nx,jj,1)*(-16/(4*dx*dz));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+3)+  alfa2*Kp1*dZx(Nx,jj,1)*(4/(4*dx*dz));

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*dZx(Nx,jj,1)*(-9/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp1*dZx(Nx,jj,1)*(12/(4*dx*dz));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp1*dZx(Nx,jj,1)*(-3/(4*dx*dz));
end

%------------------- 2: Kp1 d^2Z/dxdx  dU/dz  --------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*d2Zxx(ii,jj,1)*(-3/(2*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp1*d2Zxx(ii,jj,1)*(4/(2*dz));
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp1*d2Zxx(ii,jj,1)*(-1/(2*dz));
end 
end


%-------------------- Kp1 dQ/dx d/dQ ( dU/dx ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp1*(1/(4*dy*dx))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-2)*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-2)*Nz+1)+   alfa2*Kp1*(-1/(4*dy*dx))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+jj*Nz+1)+   alfa2*Kp1*(-1/(4*dy*dx))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+jj*Nz+1)=        MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+jj*Nz+1)+        alfa2*Kp1*(1/(4*dy*dx))*dQx(ii,jj);                                                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB((ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+1)+     alfa2*Kp1*(3/(4*dy*dx))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+Nz+1)+ alfa2*Kp1*(-4/(4*dy*dx))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+2*Nz+1)+alfa2*Kp1*(1/(4*dy*dx))*dQx(ii,1);

MB((ii-1)*Ny*Nz+1,ii*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,ii*Ny*Nz+1)+     alfa2*Kp1*(-3/(4*dy*dx))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+1,ii*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,ii*Ny*Nz+Nz+1)+   alfa2*Kp1*(4/(4*dy*dx))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,ii*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,ii*Ny*Nz+2*Nz+1)+alfa2*Kp1*(-1/(4*dy*dx))*dQx(ii,1);
end

%----- j=Ny ----;

for ii=2:1:Nx-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*Kp1*(-1/(4*dy*dx))*dQx(ii,Ny);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*Kp1*(4/(4*dy*dx))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp1*(-3/(4*dy*dx))*dQx(ii,Ny);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp1*(1/(4*dy*dx))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*(-4/(4*dy*dx))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp1*(3/(4*dy*dx))*dQx(ii,Ny);                                                                                          
end

%---- i=Nx ----;

MB((Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+1)=     MB((Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+1)+     alfa2*Kp1*(-3/(4*dy*dx))*dQx(Nx,1); 
MB((Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+Nz+1)=  MB((Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+Nz+1)+   alfa2*Kp1*(4/(4*dy*dx))*dQx(Nx,1);
MB((Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+2*Nz+1)=MB((Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+2*Nz+1)+alfa2*Kp1*(-1/(4*dy*dx))*dQx(Nx,1);

MB((Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+1)=     MB((Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+1)+    alfa2*Kp1*(12/(4*dy*dx))*dQx(Nx,1); 
MB((Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+Nz+1)=  MB((Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+Nz+1)+alfa2*Kp1*(-16/(4*dy*dx))*dQx(Nx,1);
MB((Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+2*Nz+1)=MB((Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+2*Nz+1)+alfa2*Kp1*(4/(4*dy*dx))*dQx(Nx,1);

MB((Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+1)=     MB((Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+1)+     alfa2*Kp1*(-9/(4*dy*dx))*dQx(Nx,1); 
MB((Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+Nz+1)=  MB((Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+Nz+1)+  alfa2*Kp1*(12/(4*dy*dx))*dQx(Nx,1);
MB((Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+2*Nz+1)=MB((Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+2*Nz+1)+alfa2*Kp1*(-3/(4*dy*dx))*dQx(Nx,1);

for jj=2:1:Ny-1
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp1*(-1/(4*dy*dx))*dQx(Nx,jj);     
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp1*(4/(4*dy*dx))*dQx(Nx,jj); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp1*(-3/(4*dy*dx))*dQx(Nx,jj);

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+jj*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+jj*Nz+1)+ alfa2*Kp1*(1/(4*dy*dx))*dQx(Nx,jj); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+jj*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+jj*Nz+1)+alfa2*Kp1*(-4/(4*dy*dx))*dQx(Nx,jj); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+jj*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+jj*Nz+1)+ alfa2*Kp1*(3/(4*dy*dx))*dQx(Nx,jj);                                                                                          
end 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp1*(1/(4*dy*dx))*dQx(Nx,Ny);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp1*(3/(4*dy*dx))*dQx(Nx,Ny);

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp1*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*Kp1*(16/(4*dy*dx))*dQx(Nx,Ny);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp1*(-12/(4*dy*dx))*dQx(Nx,Ny);

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)+  alfa2*Kp1*(3/(4*dy*dx))*dQx(Nx,Ny);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*(-12/(4*dy*dx))*dQx(Nx,Ny); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)+  alfa2*Kp1*(9/(4*dy*dx))*dQx(Nx,Ny);


%---------------- Kp1 dQ/dx d/dQ ( dQ/dx dU/dQ ) ------------------;

%--------------- 1: Kp1 dQ/dx dQ/dx d/dQ ( dU/dQ ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp1*(1/dy^2)*dQx(ii,jj)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*(-2/dy^2)*dQx(ii,jj)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*Kp1*(1/dy^2)*dQx(ii,jj)^2;                                                                             
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+      alfa2*Kp1*(2/dy^2)*dQx(ii,1)^2; 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+  alfa2*Kp1*(-5/dy^2)*dQx(ii,1)^2; 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+ alfa2*Kp1*(4/dy^2)*dQx(ii,1)^2;
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3*Nz+1)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3*Nz+1)+alfa2*Kp1*(-1/dy^2)*dQx(ii,1)^2;
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-4)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-4)*Nz+1)+alfa2*Kp1*(-1/dy^2)*dQx(ii,Ny)^2;     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp1*(4/dy^2)*dQx(ii,Ny)^2; 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*(-5/dy^2)*dQx(ii,Ny)^2; 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp1*(2/dy^2)*dQx(ii,Ny)^2;                                                                                          
end

%---------------- 2: Kp1 dQ/dx d^2Q/dxdq  dU/dQ ------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp1*d2Qxq(ii,jj)*(-1/(2*dy))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*Kp1*d2Qxq(ii,jj)*(1/(2*dy))*dQx(ii,jj);                                                                             
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*Kp1*d2Qxq(ii,1)*(-3/(2*dy))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+   alfa2*Kp1*d2Qxq(ii,1)*(4/(2*dy))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp1*d2Qxq(ii,1)*(-1/(2*dy))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp1*d2Qxq(ii,Ny)*(1/(2*dy))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*d2Qxq(ii,Ny)*(-4/(2*dy))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp1*d2Qxq(ii,Ny)*(3/(2*dy))*dQx(ii,Ny);                                                                                          
end


%---------------- Kp1 dQ/dx d/dQ ( dZ/dx dU/dz ) ------------------;

%-------------- 1: Kp1 dQ/dx dZ/dx d/dQ ( dU/dz ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp1*dZx(ii,jj,1)*(3/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*Kp1*dZx(ii,jj,1)*(-4/(4*dy*dz))*dQx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*Kp1*dZx(ii,jj,1)*(1/(4*dy*dz))*dQx(ii,jj);

MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*Kp1*dZx(ii,jj,1)*(-3/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*Kp1*dZx(ii,jj,1)*(4/(4*dy*dz))*dQx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*Kp1*dZx(ii,jj,1)*(-1/(4*dy*dz))*dQx(ii,jj);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*Kp1*dZx(ii,1,1)*(9/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+alfa2*Kp1*dZx(ii,1,1)*(-12/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp1*dZx(ii,1,1)*(3/(4*dy*dz))*dQx(ii,1);

MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)+    alfa2*Kp1*dZx(ii,1,1)*(-12/(4*dy*dz))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)+  alfa2*Kp1*dZx(ii,1,1)*(16/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)+alfa2*Kp1*dZx(ii,1,1)*(-4/(4*dy*dz))*dQx(ii,1);

MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)+     alfa2*Kp1*dZx(ii,1,1)*(3/(4*dy*dz))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)+ alfa2*Kp1*dZx(ii,1,1)*(-4/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)+alfa2*Kp1*dZx(ii,1,1)*(1/(4*dy*dz))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*Kp1*dZx(ii,Ny,1)*(-3/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*dZx(ii,Ny,1)*(12/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp1*dZx(ii,Ny,1)*(-9/(4*dy*dz))*dQx(ii,Ny);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*Kp1*dZx(ii,Ny,1)*(4/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*Kp1*dZx(ii,Ny,1)*(-16/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*Kp1*dZx(ii,Ny,1)*(12/(4*dy*dz))*dQx(ii,Ny);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*Kp1*dZx(ii,Ny,1)*(-1/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*Kp1*dZx(ii,Ny,1)*(4/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*Kp1*dZx(ii,Ny,1)*(-3/(4*dy*dz))*dQx(ii,Ny);
end

%-------------- 2: Kp1 dQ/dx d^2Z/dxdq  dU/dz -----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*d2Zxq(ii,jj,1)*(-3/(2*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp1*d2Zxq(ii,jj,1)*(4/(2*dz))*dQx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp1*d2Zxq(ii,jj,1)*(-1/(2*dz))*dQx(ii,jj);
end 
end


%------------------- Kp1 dZ/dx d/dz ( dU/dx ) -------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp1*(3/(4*dz*dx))*dZx(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp1*(-4/(4*dz*dx))*dZx(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp1*(1/(4*dz*dx))*dZx(ii,jj,1);

MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*(-3/(4*dz*dx))*dZx(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp1*(4/(4*dz*dx))*dZx(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp1*(-1/(4*dz*dx))*dZx(ii,jj,1);
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*(-3/(4*dz*dx))*dZx(Nx,jj,1);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp1*(4/(4*dz*dx))*dZx(Nx,jj,1);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp1*(-1/(4*dz*dx))*dZx(Nx,jj,1);
    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp1*(12/(4*dz*dx))*dZx(Nx,jj,1);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp1*(-16/(4*dz*dx))*dZx(Nx,jj,1);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+3)+  alfa2*Kp1*(4/(4*dz*dx))*dZx(Nx,jj,1);

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*(-9/(4*dz*dx))*dZx(Nx,jj,1); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp1*(12/(4*dz*dx))*dZx(Nx,jj,1);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp1*(-3/(4*dz*dx))*dZx(Nx,jj,1);
end 


%--------------- Kp1 dZ/dx d/dz ( dQ/dx dU/dQ ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp1*dQx(ii,jj)*(3/(4*dz*dy))*dZx(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*Kp1*dQx(ii,jj)*(-4/(4*dz*dy))*dZx(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*Kp1*dQx(ii,jj)*(1/(4*dz*dy))*dZx(ii,jj,1); 

MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*Kp1*dQx(ii,jj)*(-3/(4*dz*dy))*dZx(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*Kp1*dQx(ii,jj)*(4/(4*dz*dy))*dZx(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*Kp1*dQx(ii,jj)*(-1/(4*dz*dy))*dZx(ii,jj,1);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*Kp1*dQx(ii,1)*(9/(4*dz*dy))*dZx(ii,1,1); 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+alfa2*Kp1*dQx(ii,1)*(-12/(4*dz*dy))*dZx(ii,1,1);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp1*dQx(ii,1)*(3/(4*dz*dy))*dZx(ii,1,1);

MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)+    alfa2*Kp1*dQx(ii,1)*(-12/(4*dz*dy))*dZx(ii,1,1); 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)+  alfa2*Kp1*dQx(ii,1)*(16/(4*dz*dy))*dZx(ii,1,1);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)+alfa2*Kp1*dQx(ii,1)*(-4/(4*dz*dy))*dZx(ii,1,1);

MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)+     alfa2*Kp1*dQx(ii,1)*(3/(4*dz*dy))*dZx(ii,1,1); 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)+ alfa2*Kp1*dQx(ii,1)*(-4/(4*dz*dy))*dZx(ii,1,1);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)+alfa2*Kp1*dQx(ii,1)*(1/(4*dz*dy))*dZx(ii,1,1);
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*Kp1*dQx(ii,Ny)*(-3/(4*dz*dy))*dZx(ii,Ny,1);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*dQx(ii,Ny)*(12/(4*dz*dy))*dZx(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp1*dQx(ii,Ny)*(-9/(4*dz*dy))*dZx(ii,Ny,1);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*Kp1*dQx(ii,Ny)*(4/(4*dz*dy))*dZx(ii,Ny,1);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*Kp1*dQx(ii,Ny)*(-16/(4*dz*dy))*dZx(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*Kp1*dQx(ii,Ny)*(12/(4*dz*dy))*dZx(ii,Ny,1);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*Kp1*dQx(ii,Ny)*(-1/(4*dz*dy))*dZx(ii,Ny,1);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*Kp1*dQx(ii,Ny)*(4/(4*dz*dy))*dZx(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*Kp1*dQx(ii,Ny)*(-3/(4*dz*dy))*dZx(ii,Ny,1);
end


%----------------- Kp1 dZ/dx d/dz ( dZ/dx dU/dz ) -------------------;

%--------------- 1: Kp1 dZ/dx dZ/dx d/dz ( dU/dz ) ------------------;


for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp1*(2/dz^2)*dZx(ii,jj,1)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp1*(-5/dz^2)*dZx(ii,jj,1)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp1*(4/dz^2)*dZx(ii,jj,1)^2;
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+4)+alfa2*Kp1*(-1/dz^2)*dZx(ii,jj,1)^2;
end 
end

%--------------- 2: Kp1 dZ/dx d^2Z/dxdz  dU/dz ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*d2Zxz(ii,jj,1)*(-3/(2*dz))*dZx(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp1*d2Zxz(ii,jj,1)*(4/(2*dz))*dZx(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp1*d2Zxz(ii,jj,1)*(-1/(2*dz))*dZx(ii,jj,1);
end 
end


%------------------- Kp2 d/dx ( dQ/dy dV/dQ ) -------------------;

%----------------- 1: Kp2 dQ/dy d/dx ( dV/dQ ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp2*dQy(ii)*(1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+1)+   alfa2*Kp2*dQy(ii)*(-1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+1)+   alfa2*Kp2*dQy(ii)*(-1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+1)=        MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+1)+        alfa2*Kp2*dQy(ii)*(1/(4*dx*dy));
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+1)+     alfa2*Kp2*dQy(ii)*(3/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+1)+ alfa2*Kp2*dQy(ii)*(-4/(4*dx*dy));
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+1)+alfa2*Kp2*dQy(ii)*(1/(4*dx*dy));

MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+1)+     alfa2*Kp2*dQy(ii)*(-3/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+Nz+1)+   alfa2*Kp2*dQy(ii)*(4/(4*dx*dy));
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+1)+alfa2*Kp2*dQy(ii)*(-1/(4*dx*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*Kp2*dQy(ii)*(-1/(4*dx*dy));     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*Kp2*dQy(ii)*(4/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp2*dQy(ii)*(-3/(4*dx*dy));

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp2*dQy(ii)*(1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*dQy(ii)*(-4/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp2*dQy(ii)*(3/(4*dx*dy));                                  
end

%---- i=Nx ----;

MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+1)=     MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+1)+     alfa2*Kp2*dQy(Nx)*(-3/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+1)=  MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+1)+   alfa2*Kp2*dQy(Nx)*(4/(4*dx*dy));
MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+1)=MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+1)+alfa2*Kp2*dQy(Nx)*(-1/(4*dx*dy));

MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+1)=     MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+1)+    alfa2*Kp2*dQy(Nx)*(12/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+1)=  MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+1)+alfa2*Kp2*dQy(Nx)*(-16/(4*dx*dy));
MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+1)=MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+1)+alfa2*Kp2*dQy(Nx)*(4/(4*dx*dy));

MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+1)=     MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+1)+     alfa2*Kp2*dQy(Nx)*(-9/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+1)=  MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+1)+  alfa2*Kp2*dQy(Nx)*(12/(4*dx*dy));
MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+1)=MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+1)+alfa2*Kp2*dQy(Nx)*(-3/(4*dx*dy));

for jj=2:1:Ny-1
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp2*dQy(Nx)*(-1/(4*dx*dy));    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp2*dQy(Nx)*(4/(4*dx*dy));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp2*dQy(Nx)*(-3/(4*dx*dy)); 

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+1)+ alfa2*Kp2*dQy(Nx)*(1/(4*dx*dy));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+1)+alfa2*Kp2*dQy(Nx)*(-4/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+1)+ alfa2*Kp2*dQy(Nx)*(3/(4*dx*dy));                                  
end 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp2*dQy(Nx)*(1/(4*dx*dy));
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*dQy(Nx)*(-4/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp2*dQy(Nx)*(3/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp2*dQy(Nx)*(-4/(4*dx*dy));
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*Kp2*dQy(Nx)*(16/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp2*dQy(Nx)*(-12/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)+  alfa2*Kp2*dQy(Nx)*(3/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*dQy(Nx)*(-12/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)+  alfa2*Kp2*dQy(Nx)*(9/(4*dx*dy)); 

%----------------- 2: Kp2 d^2Q/dydx  dV/dQ -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp2*d2Qyx(ii)*(-1/(2*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*Kp2*d2Qyx(ii)*(1/(2*dy)); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*Kp2*d2Qyx(ii)*(-3/(2*dy)); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+   alfa2*Kp2*d2Qyx(ii)*(4/(2*dy));
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp2*d2Qyx(ii)*(-1/(2*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp2*d2Qyx(ii)*(1/(2*dy));     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*d2Qyx(ii)*(-4/(2*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp2*d2Qyx(ii)*(3/(2*dy));
end


%------------------ Kp2 d/dx ( dZ/dy dV/dz ) -------------------;

%---------------- 1: Kp2 dZ/dy d/dx ( dV/dz ) ------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp2*dZy(ii,jj,1)*(3/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*dZy(ii,jj,1)*(-4/(4*dx*dz));
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp2*dZy(ii,jj,1)*(1/(4*dx*dz));

MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*dZy(ii,jj,1)*(-3/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*dZy(ii,jj,1)*(4/(4*dx*dz));
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*dZy(ii,jj,1)*(-1/(4*dx*dz));
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*dZy(Nx,jj,1)*(-3/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*dZy(Nx,jj,1)*(4/(4*dx*dz));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*dZy(Nx,jj,1)*(-1/(4*dx*dz));    
    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp2*dZy(Nx,jj,1)*(12/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*dZy(Nx,jj,1)*(-16/(4*dx*dz));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)+  alfa2*Kp2*dZy(Nx,jj,1)*(4/(4*dx*dz));

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*dZy(Nx,jj,1)*(-9/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*dZy(Nx,jj,1)*(12/(4*dx*dz));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*dZy(Nx,jj,1)*(-3/(4*dx*dz));
end

%------------------- 2: Kp2 d^2Z/dydx  dV/dz --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*d2Zyx(ii,jj,1)*(-3/(2*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*d2Zyx(ii,jj,1)*(4/(2*dz));
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*d2Zyx(ii,jj,1)*(-1/(2*dz));
end 
end


%------------------- Kp2 dQ/dx d/dQ ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp2*dQy(ii)*(1/dy^2)*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*dQy(ii)*(-2/dy^2)*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*Kp2*dQy(ii)*(1/dy^2)*dQx(ii,jj);                                                                             
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+      alfa2*Kp2*dQy(ii)*(2/dy^2)*dQx(ii,1); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+  alfa2*Kp2*dQy(ii)*(-5/dy^2)*dQx(ii,1); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+ alfa2*Kp2*dQy(ii)*(4/dy^2)*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+1)+alfa2*Kp2*dQy(ii)*(-1/dy^2)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+1)+alfa2*Kp2*dQy(ii)*(-1/dy^2)*dQx(ii,Ny);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp2*dQy(ii)*(4/dy^2)*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*dQy(ii)*(-5/dy^2)*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp2*dQy(ii)*(2/dy^2)*dQx(ii,Ny);                                                                                          
end


%------------------- Kp2 dQ/dx d/dQ ( dZ/dy dV/dz ) -------------------;

%----------------- 1: Kp2 dQ/dx dZ/dy d/dQ ( dV/dz ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp2*dZy(ii,jj,1)*(3/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*Kp2*dZy(ii,jj,1)*(-4/(4*dy*dz))*dQx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*Kp2*dZy(ii,jj,1)*(1/(4*dy*dz))*dQx(ii,jj);

MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*Kp2*dZy(ii,jj,1)*(-3/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*Kp2*dZy(ii,jj,1)*(4/(4*dy*dz))*dQx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*Kp2*dZy(ii,jj,1)*(-1/(4*dy*dz))*dQx(ii,jj);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*Kp2*dZy(ii,1,1)*(9/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+alfa2*Kp2*dZy(ii,1,1)*(-12/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp2*dZy(ii,1,1)*(3/(4*dy*dz))*dQx(ii,1);

MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+    alfa2*Kp2*dZy(ii,1,1)*(-12/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+  alfa2*Kp2*dZy(ii,1,1)*(16/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*Kp2*dZy(ii,1,1)*(-4/(4*dy*dz))*dQx(ii,1);

MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*Kp2*dZy(ii,1,1)*(3/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+ alfa2*Kp2*dZy(ii,1,1)*(-4/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*Kp2*dZy(ii,1,1)*(1/(4*dy*dz))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*Kp2*dZy(ii,Ny,1)*(-3/(4*dy*dz))*dQx(ii,Ny);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*dZy(ii,Ny,1)*(12/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp2*dZy(ii,Ny,1)*(-9/(4*dy*dz))*dQx(ii,Ny);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*Kp2*dZy(ii,Ny,1)*(4/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*Kp2*dZy(ii,Ny,1)*(-16/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*Kp2*dZy(ii,Ny,1)*(12/(4*dy*dz))*dQx(ii,Ny);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*Kp2*dZy(ii,Ny,1)*(-1/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*Kp2*dZy(ii,Ny,1)*(4/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*Kp2*dZy(ii,Ny,1)*(-3/(4*dy*dz))*dQx(ii,Ny);
end

%----------------- 2: Kp2 dQ/dx d^2Z/dydq  dV/dz -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*d2Zyq(ii,jj,1)*(-3/(2*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*d2Zyq(ii,jj,1)*(4/(2*dz))*dQx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*d2Zyq(ii,jj,1)*(-1/(2*dz))*dQx(ii,jj);
end 
end


%------------------- Kp2 dZ/dx d/dz ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*Kp2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*Kp2*dQy(ii)*(1/(4*dz*dy))*dZx(ii,jj,1); 

MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*Kp2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*Kp2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*Kp2*dQy(ii)*(-1/(4*dz*dy))*dZx(ii,jj,1);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*Kp2*dQy(ii)*(9/(4*dz*dy))*dZx(ii,1,1); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+alfa2*Kp2*dQy(ii)*(-12/(4*dz*dy))*dZx(ii,1,1);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,1,1);

MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+    alfa2*Kp2*dQy(ii)*(-12/(4*dz*dy))*dZx(ii,1,1); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+  alfa2*Kp2*dQy(ii)*(16/(4*dz*dy))*dZx(ii,1,1);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*Kp2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,1,1);

MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*Kp2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,1,1); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+ alfa2*Kp2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,1,1);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*Kp2*dQy(ii)*(1/(4*dz*dy))*dZx(ii,1,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*Kp2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,Ny,1);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*dQy(ii)*(12/(4*dz*dy))*dZx(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp2*dQy(ii)*(-9/(4*dz*dy))*dZx(ii,Ny,1);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*Kp2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,Ny,1);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*Kp2*dQy(ii)*(-16/(4*dz*dy))*dZx(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*Kp2*dQy(ii)*(12/(4*dz*dy))*dZx(ii,Ny,1);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*Kp2*dQy(ii)*(-1/(4*dz*dy))*dZx(ii,Ny,1);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*Kp2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*Kp2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,Ny,1);
end


%--------------- Kp2 dZ/dx d/dz ( dZ/dy dV/dz ) -----------------;

%-------------- 1: Kp2 dZ/dx dZ/dy d/dz ( dV/dz ) ----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp2*dZy(ii,jj,1)*(2/dz^2)*dZx(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*dZy(ii,jj,1)*(-5/dz^2)*dZx(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp2*dZy(ii,jj,1)*(4/dz^2)*dZx(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)+alfa2*Kp2*dZy(ii,jj,1)*(-1/dz^2)*dZx(ii,jj,1);
end 
end

%-------------- 2: Kp2 dZ/dx d^2Z/dydz  dV/dz -----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*d2Zyz(ii,jj,1)*(-3/(2*dz))*dZx(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*d2Zyz(ii,jj,1)*(4/(2*dz))*dZx(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*d2Zyz(ii,jj,1)*(-1/(2*dz))*dZx(ii,jj,1);
end 
end


%------------------ Kp2 d/dx ( dZ/dz dW/dz ) -------------------;

%----------------- 1: Kp2 dZ/dz d/dx ( dW/dz ) ------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp2*dZz(ii,jj)*(3/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*dZz(ii,jj)*(-4/(4*dx*dz));
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp2*dZz(ii,jj)*(1/(4*dx*dz));

MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*dZz(ii,jj)*(-3/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*dZz(ii,jj)*(4/(4*dx*dz));
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*dZz(ii,jj)*(-1/(4*dx*dz));
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*dZz(Nx,jj)*(-3/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*dZz(Nx,jj)*(4/(4*dx*dz));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*dZz(Nx,jj)*(-1/(4*dx*dz));    
    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp2*dZz(Nx,jj)*(12/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*dZz(Nx,jj)*(-16/(4*dx*dz));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)+  alfa2*Kp2*dZz(Nx,jj)*(4/(4*dx*dz));

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*dZz(Nx,jj)*(-9/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*dZz(Nx,jj)*(12/(4*dx*dz));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*dZz(Nx,jj)*(-3/(4*dx*dz));
end

%----------------- 2: Kp2 d^2Z/dzdx  dW/dz ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*d2Zzx(ii,jj)*(-3/(2*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*d2Zzx(ii,jj)*(4/(2*dz));
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*d2Zzx(ii,jj)*(-1/(2*dz));
end 
end


%---------------- Kp2 dQ/dx d/dQ ( dZ/dz dW/dz ) ------------------;

%-------------- 1: Kp2 dQ/dx dZ/dz d/dQ ( dW/dz ) ------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp2*dZz(ii,jj)*(3/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*Kp2*dZz(ii,jj)*(-4/(4*dy*dz))*dQx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*Kp2*dZz(ii,jj)*(1/(4*dy*dz))*dQx(ii,jj);

MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*Kp2*dZz(ii,jj)*(-3/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*Kp2*dZz(ii,jj)*(4/(4*dy*dz))*dQx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*Kp2*dZz(ii,jj)*(-1/(4*dy*dz))*dQx(ii,jj);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*Kp2*dZz(ii,1)*(9/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+alfa2*Kp2*dZz(ii,1)*(-12/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp2*dZz(ii,1)*(3/(4*dy*dz))*dQx(ii,1);

MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+    alfa2*Kp2*dZz(ii,1)*(-12/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+  alfa2*Kp2*dZz(ii,1)*(16/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*Kp2*dZz(ii,1)*(-4/(4*dy*dz))*dQx(ii,1);

MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*Kp2*dZz(ii,1)*(3/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+ alfa2*Kp2*dZz(ii,1)*(-4/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*Kp2*dZz(ii,1)*(1/(4*dy*dz))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*Kp2*dZz(ii,Ny)*(-3/(4*dy*dz))*dQx(ii,Ny);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*dZz(ii,Ny)*(12/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp2*dZz(ii,Ny)*(-9/(4*dy*dz))*dQx(ii,Ny); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*Kp2*dZz(ii,Ny)*(4/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*Kp2*dZz(ii,Ny)*(-16/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*Kp2*dZz(ii,Ny)*(12/(4*dy*dz))*dQx(ii,Ny);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*Kp2*dZz(ii,Ny)*(-1/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*Kp2*dZz(ii,Ny)*(4/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*Kp2*dZz(ii,Ny)*(-3/(4*dy*dz))*dQx(ii,Ny);
end

%-------------- 2: Kp2 dQ/dx d^2Z/dzdq  dW/dz ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*d2Zzq(ii,jj)*(-3/(2*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*d2Zzq(ii,jj)*(4/(2*dz))*dQx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*d2Zzq(ii,jj)*(-1/(2*dz))*dQx(ii,jj);
end 
end


%---------------- Kp2 dZ/dx d/dz ( dZ/dz dW/dz ) ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp2*dZz(ii,jj)*(2/dz^2)*dZx(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*dZz(ii,jj)*(-5/dz^2)*dZx(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp2*dZz(ii,jj)*(4/dz^2)*dZx(ii,jj,1);  
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)+alfa2*Kp2*dZz(ii,jj)*(-1/dz^2)*dZx(ii,jj,1);  
end 
end



%++++++++++++++++++++++++ KpE dFxy/dy ++++++++++++++++++++++++++++;

%---------------- dQ/dy d/dQ ( dQ/dy dU/dQ ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*(1/dy^2)*dQy(ii)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-2/dy^2)*dQy(ii)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*(1/dy^2)*dQy(ii)^2;                                                                             
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+      alfa2*(2/dy^2)*dQy(ii)^2; 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+  alfa2*(-5/dy^2)*dQy(ii)^2; 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+ alfa2*(4/dy^2)*dQy(ii)^2;
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3*Nz+1)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3*Nz+1)+alfa2*(-1/dy^2)*dQy(ii)^2;
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-4)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-4)*Nz+1)+alfa2*(-1/dy^2)*dQy(ii)^2;     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(4/dy^2)*dQy(ii)^2; 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-5/dy^2)*dQy(ii)^2; 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(2/dy^2)*dQy(ii)^2;                                                                                          
end


%----------------- dQ/dy d/dQ ( dZ/dy dU/dz ) -----------------;

%--------------- 1: dQ/dy dZ/dy d/dQ ( dU/dz ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dZy(ii,jj,1)*(3/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*dZy(ii,jj,1)*(-4/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*dZy(ii,jj,1)*(1/(4*dy*dz))*dQy(ii);

MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*dZy(ii,jj,1)*(-3/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*dZy(ii,jj,1)*(4/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*dZy(ii,jj,1)*(-1/(4*dy*dz))*dQy(ii);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*dZy(ii,1,1)*(9/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+alfa2*dZy(ii,1,1)*(-12/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*dZy(ii,1,1)*(3/(4*dy*dz))*dQy(ii); 

MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)+    alfa2*dZy(ii,1,1)*(-12/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)+  alfa2*dZy(ii,1,1)*(16/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)+alfa2*dZy(ii,1,1)*(-4/(4*dy*dz))*dQy(ii);

MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)+     alfa2*dZy(ii,1,1)*(3/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)+ alfa2*dZy(ii,1,1)*(-4/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)+alfa2*dZy(ii,1,1)*(1/(4*dy*dz))*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dZy(ii,Ny,1)*(-3/(4*dy*dz))*dQy(ii);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dZy(ii,Ny,1)*(12/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dZy(ii,Ny,1)*(-9/(4*dy*dz))*dQy(ii); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*dZy(ii,Ny,1)*(4/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*dZy(ii,Ny,1)*(-16/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*dZy(ii,Ny,1)*(12/(4*dy*dz))*dQy(ii);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*dZy(ii,Ny,1)*(-1/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*dZy(ii,Ny,1)*(4/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*dZy(ii,Ny,1)*(-3/(4*dy*dz))*dQy(ii);
end

%--------------- 2: dQ/dy d^2Z/dydq  dU/dz -----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zyq(ii,jj,1)*(-3/(2*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zyq(ii,jj,1)*(4/(2*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zyq(ii,jj,1)*(-1/(2*dz))*dQy(ii);
end 
end


%--------------- dZ/dy d/dz ( dQ/dy dU/dQ ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dQy(ii)*(3/(4*dz*dy))*dZy(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*dQy(ii)*(1/(4*dz*dy))*dZy(ii,jj,1); 

MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*dQy(ii)*(4/(4*dz*dy))*dZy(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZy(ii,jj,1);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*dQy(ii)*(9/(4*dz*dy))*dZy(ii,1,1); 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+alfa2*dQy(ii)*(-12/(4*dz*dy))*dZy(ii,1,1);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*dQy(ii)*(3/(4*dz*dy))*dZy(ii,1,1);

MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)+    alfa2*dQy(ii)*(-12/(4*dz*dy))*dZy(ii,1,1); 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)+  alfa2*dQy(ii)*(16/(4*dz*dy))*dZy(ii,1,1);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)+alfa2*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,1,1);

MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)+     alfa2*dQy(ii)*(3/(4*dz*dy))*dZy(ii,1,1); 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)+ alfa2*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,1,1);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)+alfa2*dQy(ii)*(1/(4*dz*dy))*dZy(ii,1,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,Ny,1);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQy(ii)*(12/(4*dz*dy))*dZy(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dQy(ii)*(-9/(4*dz*dy))*dZy(ii,Ny,1);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*dQy(ii)*(4/(4*dz*dy))*dZy(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*dQy(ii)*(-16/(4*dz*dy))*dZy(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*dQy(ii)*(12/(4*dz*dy))*dZy(ii,Ny,1);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZy(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*dQy(ii)*(4/(4*dz*dy))*dZy(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,Ny,1);
end


%-------------- dZ/dy d/dz ( dZ/dy dU/dz ) -----------------;

%------------ 1: dZ/dy dZ/dy d/dz ( dU/dz ) -----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(2/dz^2)*dZy(ii,jj,1)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-5/dz^2)*dZy(ii,jj,1)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*(4/dz^2)*dZy(ii,jj,1)^2;
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+4)+alfa2*(-1/dz^2)*dZy(ii,jj,1)^2;
end 
end

%------------ 2: dZ/dy d^2Z/dydz  dU/dz -----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zyz(ii,jj,1)*(-3/(2*dz))*dZy(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zyz(ii,jj,1)*(4/(2*dz))*dZy(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zyz(ii,jj,1)*(-1/(2*dz))*dZy(ii,jj,1);
end 
end


%----------------- dQ/dy d/dQ ( dV/dx ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+1)+alfa2*(1/(4*dy*dx))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+1)+   alfa2*(-1/(4*dy*dx))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+1)+   alfa2*(-1/(4*dy*dx))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+1)=        MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+1)+        alfa2*(1/(4*dy*dx))*dQy(ii);                                                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+1)+     alfa2*(3/(4*dy*dx))*dQy(ii);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+1)+ alfa2*(-4/(4*dy*dx))*dQy(ii); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+1)+alfa2*(1/(4*dy*dx))*dQy(ii); 

MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+1)+     alfa2*(-3/(4*dy*dx))*dQy(ii); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+Nz+1)+   alfa2*(4/(4*dy*dx))*dQy(ii);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+1)+alfa2*(-1/(4*dy*dx))*dQy(ii);
end

%----- j=Ny ----;

for ii=2:1:Nx-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*(-1/(4*dy*dx))*dQy(ii);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*(4/(4*dy*dx))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*(-3/(4*dy*dx))*dQy(ii); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(1/(4*dy*dx))*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-4/(4*dy*dx))*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(3/(4*dy*dx))*dQy(ii);                                                                                          
end

%---- i=Nx ----;

MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+1)=     MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+1)+     alfa2*(-3/(4*dy*dx))*dQy(Nx);
MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+1)=  MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+1)+   alfa2*(4/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+1)=MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+1)+alfa2*(-1/(4*dy*dx))*dQy(Nx); 

MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+1)=     MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+1)+    alfa2*(12/(4*dy*dx))*dQy(Nx);
MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+1)=  MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+1)+alfa2*(-16/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+1)=MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+1)+alfa2*(4/(4*dy*dx))*dQy(Nx); 

MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+1)=     MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+1)+     alfa2*(-9/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+1)=  MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+1)+  alfa2*(12/(4*dy*dx))*dQy(Nx);
MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+1)=MB((Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+1)+alfa2*(-3/(4*dy*dx))*dQy(Nx);

for jj=2:1:Ny-1
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+1)+alfa2*(-1/(4*dy*dx))*dQy(Nx);     
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*(4/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*(-3/(4*dy*dx))*dQy(Nx);

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+1)+ alfa2*(1/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+1)+alfa2*(-4/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+1)+ alfa2*(3/(4*dy*dx))*dQy(Nx);                                                                                          
end 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(1/(4*dy*dx))*dQy(Nx);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-4/(4*dy*dx))*dQy(Nx);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(3/(4*dy*dx))*dQy(Nx);

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(-4/(4*dy*dx))*dQy(Nx);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*(16/(4*dy*dx))*dQy(Nx);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*(-12/(4*dy*dx))*dQy(Nx);

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)+  alfa2*(3/(4*dy*dx))*dQy(Nx);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-12/(4*dy*dx))*dQy(Nx);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)+  alfa2*(9/(4*dy*dx))*dQy(Nx);


%------------------ dQ/dy d/dQ ( dQ/dx dV/dQ ) -----------------;

%---------------- 1: dQ/dy dQ/dx d/dQ ( dV/dQ ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dQx(ii,jj)*(1/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*dQx(ii,jj)*(-2/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*dQx(ii,jj)*(1/dy^2)*dQy(ii);                                                                             
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+      alfa2*dQx(ii,1)*(2/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+  alfa2*dQx(ii,1)*(-5/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+ alfa2*dQx(ii,1)*(4/dy^2)*dQy(ii);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+1)+alfa2*dQx(ii,1)*(-1/dy^2)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+1)+alfa2*dQx(ii,Ny)*(-1/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*dQx(ii,Ny)*(4/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQx(ii,Ny)*(-5/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*dQx(ii,Ny)*(2/dy^2)*dQy(ii);                                                                                          
end

%---------------- 2: dQ/dy d^2Q/dxdq  dV/dQ ------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*d2Qxq(ii,jj)*(-1/(2*dy))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*d2Qxq(ii,jj)*(1/(2*dy))*dQy(ii);                                                                             
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*d2Qxq(ii,1)*(-3/(2*dy))*dQy(ii); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+   alfa2*d2Qxq(ii,1)*(4/(2*dy))*dQy(ii); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*d2Qxq(ii,1)*(-1/(2*dy))*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*d2Qxq(ii,Ny)*(1/(2*dy))*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*d2Qxq(ii,Ny)*(-4/(2*dy))*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*d2Qxq(ii,Ny)*(3/(2*dy))*dQy(ii);                                                                                          
end


%----------------- dQ/dy d/dQ ( dZ/dx dV/dz ) -----------------;

%--------------- 1: dQ/dy dZ/dx d/dQ ( dV/dz ) ----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dZx(ii,jj,1)*(3/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*dZx(ii,jj,1)*(-4/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*dZx(ii,jj,1)*(1/(4*dy*dz))*dQy(ii);

MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*dZx(ii,jj,1)*(-3/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*dZx(ii,jj,1)*(4/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*dZx(ii,jj,1)*(-1/(4*dy*dz))*dQy(ii);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*dZx(ii,1,1)*(9/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+alfa2*dZx(ii,1,1)*(-12/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*dZx(ii,1,1)*(3/(4*dy*dz))*dQy(ii); 

MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+    alfa2*dZx(ii,1,1)*(-12/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+  alfa2*dZx(ii,1,1)*(16/(4*dy*dz))*dQy(ii);  
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*dZx(ii,1,1)*(-4/(4*dy*dz))*dQy(ii);  

MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*dZx(ii,1,1)*(3/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+ alfa2*dZx(ii,1,1)*(-4/(4*dy*dz))*dQy(ii);  
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*dZx(ii,1,1)*(1/(4*dy*dz))*dQy(ii);  
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dZx(ii,Ny,1)*(-3/(4*dy*dz))*dQy(ii);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dZx(ii,Ny,1)*(12/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dZx(ii,Ny,1)*(-9/(4*dy*dz))*dQy(ii); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*dZx(ii,Ny,1)*(4/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*dZx(ii,Ny,1)*(-16/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*dZx(ii,Ny,1)*(12/(4*dy*dz))*dQy(ii);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*dZx(ii,Ny,1)*(-1/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*dZx(ii,Ny,1)*(4/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*dZx(ii,Ny,1)*(-3/(4*dy*dz))*dQy(ii);
end

%-------------- 2: dQ/dy d^2Z/dxdq  dV/dz ----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zxq(ii,jj,1)*(-3/(2*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zxq(ii,jj,1)*(4/(2*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zxq(ii,jj,1)*(-1/(2*dz))*dQy(ii);
end 
end


%----------------- dZ/dy d/dz ( dV/dx ) -----------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(3/(4*dz*dx))*dZy(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-4/(4*dz*dx))*dZy(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*(1/(4*dz*dx))*dZy(ii,jj,1);

MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-3/(4*dz*dx))*dZy(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(4/(4*dz*dx))*dZy(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-1/(4*dz*dx))*dZy(ii,jj,1);
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-3/(4*dz*dx))*dZy(Nx,jj,1);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(4/(4*dz*dx))*dZy(Nx,jj,1); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-1/(4*dz*dx))*dZy(Nx,jj,1); 
    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(12/(4*dz*dx))*dZy(Nx,jj,1);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-16/(4*dz*dx))*dZy(Nx,jj,1); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)+  alfa2*(4/(4*dz*dx))*dZy(Nx,jj,1); 

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-9/(4*dz*dx))*dZy(Nx,jj,1);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(12/(4*dz*dx))*dZy(Nx,jj,1);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-3/(4*dz*dx))*dZy(Nx,jj,1);
end 


%----------------- dZ/dy d/dz ( dQ/dx dV/dQ ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dQx(ii,jj)*(3/(4*dz*dy))*dZy(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*dQx(ii,jj)*(-4/(4*dz*dy))*dZy(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*dQx(ii,jj)*(1/(4*dz*dy))*dZy(ii,jj,1); 

MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*dQx(ii,jj)*(-3/(4*dz*dy))*dZy(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*dQx(ii,jj)*(4/(4*dz*dy))*dZy(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*dQx(ii,jj)*(-1/(4*dz*dy))*dZy(ii,jj,1);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*dQx(ii,1)*(9/(4*dz*dy))*dZy(ii,1,1); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+alfa2*dQx(ii,1)*(-12/(4*dz*dy))*dZy(ii,1,1); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*dQx(ii,1)*(3/(4*dz*dy))*dZy(ii,1,1); 

MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+    alfa2*dQx(ii,1)*(-12/(4*dz*dy))*dZy(ii,1,1); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+  alfa2*dQx(ii,1)*(16/(4*dz*dy))*dZy(ii,1,1);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*dQx(ii,1)*(-4/(4*dz*dy))*dZy(ii,1,1);

MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*dQx(ii,1)*(3/(4*dz*dy))*dZy(ii,1,1); 
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+ alfa2*dQx(ii,1)*(-4/(4*dz*dy))*dZy(ii,1,1);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*dQx(ii,1)*(1/(4*dz*dy))*dZy(ii,1,1);
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dQx(ii,Ny)*(-3/(4*dz*dy))*dZy(ii,Ny,1);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQx(ii,Ny)*(12/(4*dz*dy))*dZy(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dQx(ii,Ny)*(-9/(4*dz*dy))*dZy(ii,Ny,1);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*dQx(ii,Ny)*(4/(4*dz*dy))*dZy(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*dQx(ii,Ny)*(-16/(4*dz*dy))*dZy(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*dQx(ii,Ny)*(12/(4*dz*dy))*dZy(ii,Ny,1); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*dQx(ii,Ny)*(-1/(4*dz*dy))*dZy(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*dQx(ii,Ny)*(4/(4*dz*dy))*dZy(ii,Ny,1); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*dQx(ii,Ny)*(-3/(4*dz*dy))*dZy(ii,Ny,1); 
end


%---------------- dZ/dy d/dz ( dZ/dx dV/dz ) -----------------;

%-------------- 1: dZ/dy dZ/dx d/dz ( dV/dz ) ----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*dZx(ii,jj,1)*(2/dz^2)*dZy(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZx(ii,jj,1)*(-5/dz^2)*dZy(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*dZx(ii,jj,1)*(4/dz^2)*dZy(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)+alfa2*dZx(ii,jj,1)*(-1/dz^2)*dZy(ii,jj,1);
end 
end

%-------------- 2: dZ/dy d^2Z/dxdz dV/dz ----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zxz(ii,jj,1)*(-3/(2*dz))*dZy(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zxz(ii,jj,1)*(4/(2*dz))*dZy(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zxz(ii,jj,1)*(-1/(2*dz))*dZy(ii,jj,1);
end 
end



%+++++++++ KpE (-1/H) (-3/(2 dz)) ( Fxx dHs/dx + Fxy dHs/dy )^1 ++++++++++;

%--------------- Kp1 (-1/H) (-3/(2*dz)) dHs/dx dU/dx --------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*(-1/(2*dx))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)+     alfa2*Kp1*(1/(2*dx))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp1*(1/(2*dx))*(-3/(2*dz))*dZz(Nx,jj)*dHsx(Nx,jj);    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*(-4/(2*dx))*(-3/(2*dz))*dZz(Nx,jj)*dHsx(Nx,jj);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp1*(3/(2*dx))*(-3/(2*dz))*dZz(Nx,jj)*dHsx(Nx,jj);                                                          
end 


%-------------- Kp1 (-1/H) (-3/(2*dz)) dHs/dx dQ/dx dU/dQ ---------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp1*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dQx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*Kp1*(1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dQx(ii,jj);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*Kp1*(-3/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsx(ii,1)*dQx(ii,1);            
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+   alfa2*Kp1*(4/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsx(ii,1)*dQx(ii,1); 
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp1*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsx(ii,1)*dQx(ii,1); 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp1*(1/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsx(ii,Ny)*dQx(ii,Ny);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*(-4/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsx(ii,Ny)*dQx(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp1*(3/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsx(ii,Ny)*dQx(ii,Ny);                                                          
end


%-------------- Kp1 (-1/H) (-3/(2*dz)) dHs/dx dZ/dx dU/dz ----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*(-3/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZx(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp1*(4/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZx(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp1*(-1/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZx(ii,jj,1);
end 
end


%--------------- Kp2 (-1/H) (-3/(2*dz)) dHs/dx dQ/dy dV/dQ ---------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dQy(ii);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*Kp2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dQy(ii);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*Kp2*(-3/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsx(ii,1)*dQy(ii);            
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+   alfa2*Kp2*(4/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsx(ii,1)*dQy(ii);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsx(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsx(ii,Ny)*dQy(ii);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*(-4/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsx(ii,Ny)*dQy(ii);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp2*(3/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsx(ii,Ny)*dQy(ii);                                                          
end


%----------- Kp2 (-1/H) (-3/(2*dz)) dHs/dx dZ/dy dV/dz --------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*(-3/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZy(ii,jj,1);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*(4/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZy(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*(-1/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZy(ii,jj,1);
end 
end

%---------- Kp2 (-1/H) (-3/(2*dz)) dHs/dx dZ/dz dW/dz -------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*(-3/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZz(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*(4/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZz(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*(-1/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZz(ii,jj);
end 
end


%---------------- (-1/H) (-3/(2*dz)) dHs/dy dQ/dy dU/dQ ----------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dQy(ii);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dQy(ii);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*(-3/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsy(ii,1)*dQy(ii);            
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+   alfa2*(4/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsy(ii,1)*dQy(ii);
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsy(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsy(ii,Ny)*dQy(ii);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-4/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsy(ii,Ny)*dQy(ii);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(3/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsy(ii,Ny)*dQy(ii);                                                          
end


%------------- (-1/H) (-3/(2*dz)) dHs/dy dZ/dy dU/dz ----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-3/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZy(ii,jj,1);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(4/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZy(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-1/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZy(ii,jj,1);
end 
end


%------------------ (-1/H) (-3/(2*dz)) dHs/dy dV/dx -----------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-1/(2*dx))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)+     alfa2*(1/(2*dx))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(1/(2*dx))*(-3/(2*dz))*dZz(Nx,jj)*dHsy(Nx,jj);    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-4/(2*dx))*(-3/(2*dz))*dZz(Nx,jj)*dHsy(Nx,jj);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(3/(2*dx))*(-3/(2*dz))*dZz(Nx,jj)*dHsy(Nx,jj);                                                          
end 


%-------------- (-1/H) (-3/(2*dz)) dHs/dy dQ/dx dV/dQ ---------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dQx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dQx(ii,jj);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*(-3/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsy(ii,1)*dQx(ii,1);            
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+   alfa2*(4/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsy(ii,1)*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsy(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsy(ii,Ny)*dQx(ii,Ny);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-4/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsy(ii,Ny)*dQx(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(3/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsy(ii,Ny)*dQx(ii,Ny);                                                          
end


%---------------- (-1/H) (-3/(2*dz)) dHs/dy dZ/dx dV/dz ----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-3/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZx(ii,jj,1);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(4/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZx(ii,jj,1); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-1/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZx(ii,jj,1);
end 
end



%++++++++++++++++++ KpE (-1/H) 4/(2 dz) ( Fxz )^2 +++++++++++++++++++;

%------------------ (-1/H) (4/(2*dz)) dZ/dz dU/dz -------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-1/(2*dz))*(4/(2*dz))*dZz(ii,jj)*dZz(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*(1/(2*dz))*(4/(2*dz))*dZz(ii,jj)*dZz(ii,jj);                                                          
end 
end

%-------------------- (-1/H) (4/(2*dz)) dW/dx ------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-1/(2*dx))*(4/(2*dz))*dZz(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)+     alfa2*(1/(2*dx))*(4/(2*dz))*dZz(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(1/(2*dx))*(4/(2*dz))*dZz(Nx,jj);    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-4/(2*dx))*(4/(2*dz))*dZz(Nx,jj);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(3/(2*dx))*(4/(2*dz))*dZz(Nx,jj);                                                          
end 


%---------------- (-1/H) (4/(2*dz)) dQ/dx dW/dQ ------------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*(-1/(2*dy))*(4/(2*dz))*dZz(ii,jj)*dQx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+     alfa2*(1/(2*dy))*(4/(2*dz))*dZz(ii,jj)*dQx(ii,jj);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+     alfa2*(-3/(2*dy))*(4/(2*dz))*dZz(ii,1)*dQx(ii,1);            
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+   alfa2*(4/(2*dy))*(4/(2*dz))*dZz(ii,1)*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*(-1/(2*dy))*(4/(2*dz))*dZz(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+ alfa2*(1/(2*dy))*(4/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*(-4/(2*dy))*(4/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*(3/(2*dy))*(4/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);                                                          
end


%----------------- (-1/H) (4/(2*dz)) dZ/dx dW/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-1/(2*dz))*(4/(2*dz))*dZz(ii,jj)*dZx(ii,jj,2);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*(1/(2*dz))*(4/(2*dz))*dZz(ii,jj)*dZx(ii,jj,2);                                                          
end 
end



%++++++++++++++++++ KpE (-1/H) (-1/(2 dz)) ( Fxz )^3 +++++++++++++++++++;

%--------------------- (-1/H) (-1/(2*dz)) dZ/dz dU/dz --------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZz(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+4)+ alfa2*(1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZz(ii,jj);                                                          
end 
end

%-------------------- (-1/H) (-1/(2*dz)) dW/dx ------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-1/(2*dx))*(-1/(2*dz))*dZz(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)+     alfa2*(1/(2*dx))*(-1/(2*dz))*dZz(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*(1/(2*dx))*(-1/(2*dz))*dZz(Nx,jj);    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-4/(2*dx))*(-1/(2*dz))*dZz(Nx,jj);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*(3/(2*dx))*(-1/(2*dz))*dZz(Nx,jj);                                                          
end 


%---------------- (-1/H) (-1/(2*dz)) dQ/dx dW/dQ ------------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+alfa2*(-1/(2*dy))*(-1/(2*dz))*dZz(ii,jj)*dQx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+     alfa2*(1/(2*dy))*(-1/(2*dz))*dZz(ii,jj)*dQx(ii,jj);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*(-3/(2*dy))*(-1/(2*dz))*dZz(ii,1)*dQx(ii,1);            
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+   alfa2*(4/(2*dy))*(-1/(2*dz))*dZz(ii,1)*dQx(ii,1);
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*(-1/(2*dy))*(-1/(2*dz))*dZz(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+ alfa2*(1/(2*dy))*(-1/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+alfa2*(-4/(2*dy))*(-1/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+ alfa2*(3/(2*dy))*(-1/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);                                                          
end


%----------------- (-1/H) (-1/(2*dz)) dZ/dx dW/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZx(ii,jj,3);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)+ alfa2*(1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZx(ii,jj,3);                                                          
end 
end



%+++++++++++++++++++ + KpE dens Freq^2 U ++++++++++++++++++++++;

for ii=2:1:Nx 
for jj=1:1:Ny 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1) + alfa2*KpE*dens*Freq^2;
end 
end




%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++++++++ The second equation ++++++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

%+++++++++++++++++++++++++ KpE dFyx/dx +++++++++++++++++++++++++++;

%--------------------- d/dx ( dQ/dy dU/dQ ) ---------------------;

%------------------- 1: dQ/dy d/dx ( dU/dQ ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-2)*Nz+1)+alfa2*dQy(ii)*(1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+jj*Nz+1)+   alfa2*dQy(ii)*(-1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-2)*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-2)*Nz+1)+   alfa2*dQy(ii)*(-1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+jj*Nz+1)=        MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+jj*Nz+1)+        alfa2*dQy(ii)*(1/(4*dx*dy));                                  
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+1)+     alfa2*dQy(ii)*(3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+Nz+1)+ alfa2*dQy(ii)*(-4/(4*dx*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+2*Nz+1)+alfa2*dQy(ii)*(1/(4*dx*dy));

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,ii*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,ii*Ny*Nz+1)+     alfa2*dQy(ii)*(-3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,ii*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,ii*Ny*Nz+Nz+1)+   alfa2*dQy(ii)*(4/(4*dx*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,ii*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,ii*Ny*Nz+2*Nz+1)+alfa2*dQy(ii)*(-1/(4*dx*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dQy(ii)*(-1/(4*dx*dy));     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*dQy(ii)*(4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dQy(ii)*(-3/(4*dx*dy)); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*dQy(ii)*(1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQy(ii)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*dQy(ii)*(3/(4*dx*dy));                                  
end

%---- i=Nx ----;

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+1)+     alfa2*dQy(Nx)*(-3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+Nz+1)+   alfa2*dQy(Nx)*(4/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+2*Nz+1)+alfa2*dQy(Nx)*(-1/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+1)+    alfa2*dQy(Nx)*(12/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+Nz+1)+alfa2*dQy(Nx)*(-16/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+2*Nz+1)+alfa2*dQy(Nx)*(4/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+1)+     alfa2*dQy(Nx)*(-9/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+Nz+1)+  alfa2*dQy(Nx)*(12/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+2*Nz+1)+alfa2*dQy(Nx)*(-3/(4*dx*dy));


for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-2)*Nz+1)+alfa2*dQy(Nx)*(-1/(4*dx*dy));    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dQy(Nx)*(4/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*dQy(Nx)*(-3/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+jj*Nz+1)+ alfa2*dQy(Nx)*(1/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+jj*Nz+1)+alfa2*dQy(Nx)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+jj*Nz+1)+ alfa2*dQy(Nx)*(3/(4*dx*dy));                                  
end 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*dQy(Nx)*(1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQy(Nx)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*dQy(Nx)*(3/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*dQy(Nx)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*dQy(Nx)*(16/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dQy(Nx)*(-12/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)+  alfa2*dQy(Nx)*(3/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQy(Nx)*(-12/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)+  alfa2*dQy(Nx)*(9/(4*dx*dy)); 

%------------------- 2: d^2Q/dydx  dU/dQ --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*d2Qyx(ii)*(-1/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*d2Qyx(ii)*(1/(2*dy)); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*d2Qyx(ii)*(-3/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+   alfa2*d2Qyx(ii)*(4/(2*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*d2Qyx(ii)*(-1/(2*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*d2Qyx(ii)*(1/(2*dy));     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*d2Qyx(ii)*(-4/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*d2Qyx(ii)*(3/(2*dy)); 
end


%--------------------- d/dx ( dZ/dy dU/dz ) ---------------------;

%------------------- 1: dZ/dy d/dx ( dU/dz ) ---------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*dZy(ii,jj,1)*(3/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZy(ii,jj,1)*(-4/(4*dx*dz));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*dZy(ii,jj,1)*(1/(4*dx*dz));

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)+alfa2*dZy(ii,jj,1)*(-3/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+2)+ alfa2*dZy(ii,jj,1)*(4/(4*dx*dz));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+3)+alfa2*dZy(ii,jj,1)*(-1/(4*dx*dz));
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+alfa2*dZy(Nx,jj,1)*(-3/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*dZy(Nx,jj,1)*(4/(4*dx*dz));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+3)+alfa2*dZy(Nx,jj,1)*(-1/(4*dx*dz));    
    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*dZy(Nx,jj,1)*(12/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZy(Nx,jj,1)*(-16/(4*dx*dz));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+3)+  alfa2*dZy(Nx,jj,1)*(4/(4*dx*dz));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*dZy(Nx,jj,1)*(-9/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZy(Nx,jj,1)*(12/(4*dx*dz));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*dZy(Nx,jj,1)*(-3/(4*dx*dz));
end

%------------------- 2: d^2Z/dydx  dU/dz ---------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zyx(ii,jj,1)*(-3/(2*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zyx(ii,jj,1)*(4/(2*dz));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zyx(ii,jj,1)*(-1/(2*dz));
end 
end


%-------------------- dQ/dx d/dQ ( dQ/dy dU/dQ ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dQy(ii)*(1/dy^2)*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*dQy(ii)*(-2/dy^2)*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*dQy(ii)*(1/dy^2)*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+      alfa2*dQy(ii)*(2/dy^2)*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+  alfa2*dQy(ii)*(-5/dy^2)*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+ alfa2*dQy(ii)*(4/dy^2)*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3*Nz+1)+alfa2*dQy(ii)*(-1/dy^2)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-4)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-4)*Nz+1)+alfa2*dQy(ii)*(-1/dy^2)*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*dQy(ii)*(4/dy^2)*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQy(ii)*(-5/dy^2)*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*dQy(ii)*(2/dy^2)*dQx(ii,Ny);                                                                                          
end


%-------------------- dQ/dx d/dQ ( dZ/dy dU/dz ) --------------------;

%------------------ 1: dQ/dx dZ/dy d/dQ ( dU/dz ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dZy(ii,jj,1)*(3/(4*dy*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*dZy(ii,jj,1)*(-4/(4*dy*dz))*dQx(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*dZy(ii,jj,1)*(1/(4*dy*dz))*dQx(ii,jj);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*dZy(ii,jj,1)*(-3/(4*dy*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*dZy(ii,jj,1)*(4/(4*dy*dz))*dQx(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*dZy(ii,jj,1)*(-1/(4*dy*dz))*dQx(ii,jj);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*dZy(ii,1,1)*(9/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+alfa2*dZy(ii,1,1)*(-12/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*dZy(ii,1,1)*(3/(4*dy*dz))*dQx(ii,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)+    alfa2*dZy(ii,1,1)*(-12/(4*dy*dz))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)+  alfa2*dZy(ii,1,1)*(16/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)+alfa2*dZy(ii,1,1)*(-4/(4*dy*dz))*dQx(ii,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)+     alfa2*dZy(ii,1,1)*(3/(4*dy*dz))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)+ alfa2*dZy(ii,1,1)*(-4/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)+alfa2*dZy(ii,1,1)*(1/(4*dy*dz))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dZy(ii,Ny,1)*(-3/(4*dy*dz))*dQx(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dZy(ii,Ny,1)*(12/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dZy(ii,Ny,1)*(-9/(4*dy*dz))*dQx(ii,Ny); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*dZy(ii,Ny,1)*(4/(4*dy*dz))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*dZy(ii,Ny,1)*(-16/(4*dy*dz))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*dZy(ii,Ny,1)*(12/(4*dy*dz))*dQx(ii,Ny);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*dZy(ii,Ny,1)*(-1/(4*dy*dz))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*dZy(ii,Ny,1)*(4/(4*dy*dz))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*dZy(ii,Ny,1)*(-3/(4*dy*dz))*dQx(ii,Ny);
end

%------------------ 2: dQ/dx d^2Z/dydq  dU/dz -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zyq(ii,jj,1)*(-3/(2*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zyq(ii,jj,1)*(4/(2*dz))*dQx(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zyq(ii,jj,1)*(-1/(2*dz))*dQx(ii,jj);
end 
end


%-------------------- dZ/dx d/dz ( dQ/dy dU/dQ ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*dQy(ii)*(1/(4*dz*dy))*dZx(ii,jj,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZx(ii,jj,1);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*dQy(ii)*(9/(4*dz*dy))*dZx(ii,1,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+alfa2*dQy(ii)*(-12/(4*dz*dy))*dZx(ii,1,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,1,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)+    alfa2*dQy(ii)*(-12/(4*dz*dy))*dZx(ii,1,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)+  alfa2*dQy(ii)*(16/(4*dz*dy))*dZx(ii,1,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)+alfa2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,1,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)+     alfa2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,1,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)+ alfa2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,1,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)+alfa2*dQy(ii)*(1/(4*dz*dy))*dZx(ii,1,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,Ny,1);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQy(ii)*(12/(4*dz*dy))*dZx(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dQy(ii)*(-9/(4*dz*dy))*dZx(ii,Ny,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*dQy(ii)*(-16/(4*dz*dy))*dZx(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*dQy(ii)*(12/(4*dz*dy))*dZx(ii,Ny,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZx(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,Ny,1);
end


%------------------ dZ/dx d/dz ( dZ/dy dU/dz ) --------------------;

%----------------- 1: dZ/dx dZ/dy d/dz ( dU/dz ) -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*dZy(ii,jj,1)*(2/dz^2)*dZx(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZy(ii,jj,1)*(-5/dz^2)*dZx(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*dZy(ii,jj,1)*(4/dz^2)*dZx(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+4)+alfa2*dZy(ii,jj,1)*(-1/dz^2)*dZx(ii,jj,1);
end 
end

%----------------- 2: dZ/dx d^2Z/dydz  dU/dz -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zyz(ii,jj,1)*(-3/(2*dz))*dZx(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zyz(ii,jj,1)*(4/(2*dz))*dZx(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zyz(ii,jj,1)*(-1/(2*dz))*dZx(ii,jj,1);
end 
end


%------------------------ d/dx ( dV/dx ) -----------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(1/dx^2); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-2/dx^2); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)+     alfa2*(1/dx^2); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-4)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-4)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-1/dx^2);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(4/dx^2); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-5/dx^2); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(2/dx^2);                                        
end 


%--------------------- d/dx ( dQ/dx dV/dQ ) ---------------------;

%------------------- 1: dQ/dx d/dx ( dV/dQ ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+1)+alfa2*dQx(ii,jj)*(1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+1)+   alfa2*dQx(ii,jj)*(-1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+1)+   alfa2*dQx(ii,jj)*(-1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+1)=        MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+1)+        alfa2*dQx(ii,jj)*(1/(4*dx*dy)); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+1)+     alfa2*dQx(ii,1)*(3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+1)+ alfa2*dQx(ii,1)*(-4/(4*dx*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+1)+alfa2*dQx(ii,1)*(1/(4*dx*dy));

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+1)+     alfa2*dQx(ii,1)*(-3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+Nz+1)+   alfa2*dQx(ii,1)*(4/(4*dx*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+1)+alfa2*dQx(ii,1)*(-1/(4*dx*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dQx(ii,Ny)*(-1/(4*dx*dy));    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*dQx(ii,Ny)*(4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dQx(ii,Ny)*(-3/(4*dx*dy));

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*dQx(ii,Ny)*(1/(4*dx*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQx(ii,Ny)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*dQx(ii,Ny)*(3/(4*dx*dy));                                  
end

%---- i=Nx ----;

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+1)+     alfa2*dQx(Nx,1)*(-3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+1)+   alfa2*dQx(Nx,1)*(4/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+1)+alfa2*dQx(Nx,1)*(-1/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+1)+    alfa2*dQx(Nx,1)*(12/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+1)+alfa2*dQx(Nx,1)*(-16/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+1)+alfa2*dQx(Nx,1)*(4/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+1)+     alfa2*dQx(Nx,1)*(-9/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+1)+  alfa2*dQx(Nx,1)*(12/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+1)+alfa2*dQx(Nx,1)*(-3/(4*dx*dy));

for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+1)+alfa2*dQx(Nx,jj)*(-1/(4*dx*dy));    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dQx(Nx,jj)*(4/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*dQx(Nx,jj)*(-3/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+1)+ alfa2*dQx(Nx,jj)*(1/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+1)+alfa2*dQx(Nx,jj)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+1)+ alfa2*dQx(Nx,jj)*(3/(4*dx*dy));                                  
end 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*dQx(Nx,Ny)*(1/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQx(Nx,Ny)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*dQx(Nx,Ny)*(3/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*dQx(Nx,Ny)*(-4/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*dQx(Nx,Ny)*(16/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dQx(Nx,Ny)*(-12/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)+  alfa2*dQx(Nx,Ny)*(3/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQx(Nx,Ny)*(-12/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)+  alfa2*dQx(Nx,Ny)*(9/(4*dx*dy));

%------------------- 2: d^2Q/dxdx  dV/dQ  ---------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*d2Qxx(ii,jj)*(-1/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*d2Qxx(ii,jj)*(1/(2*dy)); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*d2Qxx(ii,1)*(-3/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+   alfa2*d2Qxx(ii,1)*(4/(2*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*d2Qxx(ii,1)*(-1/(2*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*d2Qxx(ii,Ny)*(1/(2*dy));    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*d2Qxx(ii,Ny)*(-4/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*d2Qxx(ii,Ny)*(3/(2*dy));
end


%--------------------- d/dx ( dZ/dx dV/dz ) ---------------------;

%------------------- 1: dZ/dx d/dx ( dV/dz ) ---------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*dZx(ii,jj,1)*(3/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZx(ii,jj,1)*(-4/(4*dx*dz));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*dZx(ii,jj,1)*(1/(4*dx*dz));

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)+alfa2*dZx(ii,jj,1)*(-3/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)+ alfa2*dZx(ii,jj,1)*(4/(4*dx*dz));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)+alfa2*dZx(ii,jj,1)*(-1/(4*dx*dz));
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+alfa2*dZx(Nx,jj,1)*(-3/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*dZx(Nx,jj,1)*(4/(4*dx*dz));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)+alfa2*dZx(Nx,jj,1)*(-1/(4*dx*dz));    
    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*dZx(Nx,jj,1)*(12/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZx(Nx,jj,1)*(-16/(4*dx*dz));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)+  alfa2*dZx(Nx,jj,1)*(4/(4*dx*dz));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*dZx(Nx,jj,1)*(-9/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZx(Nx,jj,1)*(12/(4*dx*dz));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*dZx(Nx,jj,1)*(-3/(4*dx*dz));
end

%------------------- 2: d^2Z/dxdx  dV/dz  ---------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zxx(ii,jj,1)*(-3/(2*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zxx(ii,jj,1)*(4/(2*dz));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zxx(ii,jj,1)*(-1/(2*dz));
end 
end


%-------------------- dQ/dx d/dQ ( dV/dx ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+1)+alfa2*(1/(4*dy*dx))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+1)+   alfa2*(-1/(4*dy*dx))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+1)+   alfa2*(-1/(4*dy*dx))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+1)=        MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+1)+        alfa2*(1/(4*dy*dx))*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+1)+     alfa2*(3/(4*dy*dx))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+1)+ alfa2*(-4/(4*dy*dx))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+1)+alfa2*(1/(4*dy*dx))*dQx(ii,1); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+1)+     alfa2*(-3/(4*dy*dx))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+Nz+1)+   alfa2*(4/(4*dy*dx))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+1)+alfa2*(-1/(4*dy*dx))*dQx(ii,1);
end

%----- j=Ny ----;

for ii=2:1:Nx-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*(-1/(4*dy*dx))*dQx(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*(4/(4*dy*dx))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*(-3/(4*dy*dx))*dQx(ii,Ny); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(1/(4*dy*dx))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-4/(4*dy*dx))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(3/(4*dy*dx))*dQx(ii,Ny);                                                                                          
end

%---- i=Nx ----;

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+1)+     alfa2*(-3/(4*dy*dx))*dQx(Nx,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+1)+   alfa2*(4/(4*dy*dx))*dQx(Nx,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+1)+alfa2*(-1/(4*dy*dx))*dQx(Nx,1);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+1)+    alfa2*(12/(4*dy*dx))*dQx(Nx,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+1)+alfa2*(-16/(4*dy*dx))*dQx(Nx,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+1)+alfa2*(4/(4*dy*dx))*dQx(Nx,1);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+1)+     alfa2*(-9/(4*dy*dx))*dQx(Nx,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+1)+  alfa2*(12/(4*dy*dx))*dQx(Nx,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+1)+alfa2*(-3/(4*dy*dx))*dQx(Nx,1);


for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+1)+alfa2*(-1/(4*dy*dx))*dQx(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*(4/(4*dy*dx))*dQx(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*(-3/(4*dy*dx))*dQx(Nx,jj);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+1)+ alfa2*(1/(4*dy*dx))*dQx(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+1)+alfa2*(-4/(4*dy*dx))*dQx(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+1)+ alfa2*(3/(4*dy*dx))*dQx(Nx,jj);                                                                                          
end 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(1/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(3/(4*dy*dx))*dQx(Nx,Ny);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*(16/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*(-12/(4*dy*dx))*dQx(Nx,Ny);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)+  alfa2*(3/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-12/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)+  alfa2*(9/(4*dy*dx))*dQx(Nx,Ny);


%------------------- dQ/dx d/dQ ( dQ/dx dV/dQ ) --------------------;

%----------------- 1: dQ/dx dQ/dx d/dQ ( dV/dQ ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*(1/dy^2)*dQx(ii,jj)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-2/dy^2)*dQx(ii,jj)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*(1/dy^2)*dQx(ii,jj)^2; 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+      alfa2*(2/dy^2)*dQx(ii,1)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+  alfa2*(-5/dy^2)*dQx(ii,1)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+ alfa2*(4/dy^2)*dQx(ii,1)^2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+1)+alfa2*(-1/dy^2)*dQx(ii,1)^2;
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+1)+alfa2*(-1/dy^2)*dQx(ii,Ny)^2;    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(4/dy^2)*dQx(ii,Ny)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-5/dy^2)*dQx(ii,Ny)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(2/dy^2)*dQx(ii,Ny)^2;                                                                                          
end

%----------------- 2: dQ/dx d2Q/dxdq  dV/dQ  --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*d2Qxq(ii,jj)*(-1/(2*dy))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*d2Qxq(ii,jj)*(1/(2*dy))*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*d2Qxq(ii,1)*(-3/(2*dy))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+   alfa2*d2Qxq(ii,1)*(4/(2*dy))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*d2Qxq(ii,1)*(-1/(2*dy))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*d2Qxq(ii,Ny)*(1/(2*dy))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*d2Qxq(ii,Ny)*(-4/(2*dy))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*d2Qxq(ii,Ny)*(3/(2*dy))*dQx(ii,Ny);                                                                                          
end


%-------------------- dQ/dx d/dQ ( dZ/dx dV/dz ) --------------------;

%------------------ 1: dQ/dx dZ/dx d/dQ ( dV/dz ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dZx(ii,jj,1)*(3/(4*dy*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*dZx(ii,jj,1)*(-4/(4*dy*dz))*dQx(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*dZx(ii,jj,1)*(1/(4*dy*dz))*dQx(ii,jj);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*dZx(ii,jj,1)*(-3/(4*dy*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*dZx(ii,jj,1)*(4/(4*dy*dz))*dQx(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*dZx(ii,jj,1)*(-1/(4*dy*dz))*dQx(ii,jj);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*dZx(ii,1,1)*(9/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+alfa2*dZx(ii,1,1)*(-12/(4*dy*dz))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*dZx(ii,1,1)*(3/(4*dy*dz))*dQx(ii,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+    alfa2*dZx(ii,1,1)*(-12/(4*dy*dz))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+  alfa2*dZx(ii,1,1)*(16/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*dZx(ii,1,1)*(-4/(4*dy*dz))*dQx(ii,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*dZx(ii,1,1)*(3/(4*dy*dz))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+ alfa2*dZx(ii,1,1)*(-4/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*dZx(ii,1,1)*(1/(4*dy*dz))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dZx(ii,Ny,1)*(-3/(4*dy*dz))*dQx(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dZx(ii,Ny,1)*(12/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dZx(ii,Ny,1)*(-9/(4*dy*dz))*dQx(ii,Ny); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*dZx(ii,Ny,1)*(4/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*dZx(ii,Ny,1)*(-16/(4*dy*dz))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*dZx(ii,Ny,1)*(12/(4*dy*dz))*dQx(ii,Ny);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*dZx(ii,Ny,1)*(-1/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*dZx(ii,Ny,1)*(4/(4*dy*dz))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*dZx(ii,Ny,1)*(-3/(4*dy*dz))*dQx(ii,Ny);
end

%------------------ 2: dQ/dx d^2Z/dxdq  dV/dz  --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zxq(ii,jj,1)*(-3/(2*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zxq(ii,jj,1)*(4/(2*dz))*dQx(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zxq(ii,jj,1)*(-1/(2*dz))*dQx(ii,jj);
end 
end


%-------------------- dZ/dx d/dz ( dV/dx ) --------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(3/(4*dz*dx))*dZx(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-4/(4*dz*dx))*dZx(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*(1/(4*dz*dx))*dZx(ii,jj,1); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-3/(4*dz*dx))*dZx(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(4/(4*dz*dx))*dZx(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-1/(4*dz*dx))*dZx(ii,jj,1);
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-3/(4*dz*dx))*dZx(Nx,jj,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(4/(4*dz*dx))*dZx(Nx,jj,1); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-1/(4*dz*dx))*dZx(Nx,jj,1);    
    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(12/(4*dz*dx))*dZx(Nx,jj,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-16/(4*dz*dx))*dZx(Nx,jj,1); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)+  alfa2*(4/(4*dz*dx))*dZx(Nx,jj,1);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-9/(4*dz*dx))*dZx(Nx,jj,1); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(12/(4*dz*dx))*dZx(Nx,jj,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-3/(4*dz*dx))*dZx(Nx,jj,1);
end 


%-------------------- dZ/dx d/dz ( dQ/dx dV/dQ ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dQx(ii,jj)*(3/(4*dz*dy))*dZx(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*dQx(ii,jj)*(-4/(4*dz*dy))*dZx(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*dQx(ii,jj)*(1/(4*dz*dy))*dZx(ii,jj,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*dQx(ii,jj)*(-3/(4*dz*dy))*dZx(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*dQx(ii,jj)*(4/(4*dz*dy))*dZx(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*dQx(ii,jj)*(-1/(4*dz*dy))*dZx(ii,jj,1);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*dQx(ii,1)*(9/(4*dz*dy))*dZx(ii,1,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+alfa2*dQx(ii,1)*(-12/(4*dz*dy))*dZx(ii,1,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*dQx(ii,1)*(3/(4*dz*dy))*dZx(ii,1,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+    alfa2*dQx(ii,1)*(-12/(4*dz*dy))*dZx(ii,1,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+  alfa2*dQx(ii,1)*(16/(4*dz*dy))*dZx(ii,1,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*dQx(ii,1)*(-4/(4*dz*dy))*dZx(ii,1,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*dQx(ii,1)*(3/(4*dz*dy))*dZx(ii,1,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+ alfa2*dQx(ii,1)*(-4/(4*dz*dy))*dZx(ii,1,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*dQx(ii,1)*(1/(4*dz*dy))*dZx(ii,1,1);
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dQx(ii,Ny)*(-3/(4*dz*dy))*dZx(ii,Ny,1);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQx(ii,Ny)*(12/(4*dz*dy))*dZx(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dQx(ii,Ny)*(-9/(4*dz*dy))*dZx(ii,Ny,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*dQx(ii,Ny)*(4/(4*dz*dy))*dZx(ii,Ny,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*dQx(ii,Ny)*(-16/(4*dz*dy))*dZx(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*dQx(ii,Ny)*(12/(4*dz*dy))*dZx(ii,Ny,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*dQx(ii,Ny)*(-1/(4*dz*dy))*dZx(ii,Ny,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*dQx(ii,Ny)*(4/(4*dz*dy))*dZx(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*dQx(ii,Ny)*(-3/(4*dz*dy))*dZx(ii,Ny,1);
end


%-------------------- dZ/dx d/dz ( dZ/dx dV/dz ) --------------------;

%------------------ 1: dZ/dx dZ/dx d/dz ( dV/dz ) --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(2/dz^2)*dZx(ii,jj,1)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-5/dz^2)*dZx(ii,jj,1)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*(4/dz^2)*dZx(ii,jj,1)^2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)+alfa2*(-1/dz^2)*dZx(ii,jj,1)^2;
end 
end

%------------------- 2: dZ/dx d^2Z/dxdz  dV/dz  --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zxz(ii,jj,1)*(-3/(2*dz))*dZx(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zxz(ii,jj,1)*(4/(2*dz))*dZx(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zxz(ii,jj,1)*(-1/(2*dz))*dZx(ii,jj,1); 
end 
end



%+++++++++++++++++++++++++ KpE dFyy/dy +++++++++++++++++++++++++++;

%-------------- Kp1 dQ/dy d/dQ ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp1*(1/dy^2)*dQy(ii)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*(-2/dy^2)*dQy(ii)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*Kp1*(1/dy^2)*dQy(ii)^2; 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+      alfa2*Kp1*(2/dy^2)*dQy(ii)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+  alfa2*Kp1*(-5/dy^2)*dQy(ii)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+ alfa2*Kp1*(4/dy^2)*dQy(ii)^2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+1)+alfa2*Kp1*(-1/dy^2)*dQy(ii)^2;
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+1)+alfa2*Kp1*(-1/dy^2)*dQy(ii)^2;    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp1*(4/dy^2)*dQy(ii)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*(-5/dy^2)*dQy(ii)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp1*(2/dy^2)*dQy(ii)^2;                                                                                          
end


%-------------- Kp1 dQ/dy d/dQ ( dZ/dy dV/dz ) -------------------;

%------------ 1: Kp1 dQ/dy dZ/dy d/dQ ( dV/dz ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp1*dZy(ii,jj,1)*(3/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*Kp1*dZy(ii,jj,1)*(-4/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*Kp1*dZy(ii,jj,1)*(1/(4*dy*dz))*dQy(ii); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*Kp1*dZy(ii,jj,1)*(-3/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*Kp1*dZy(ii,jj,1)*(4/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*Kp1*dZy(ii,jj,1)*(-1/(4*dy*dz))*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*Kp1*dZy(ii,1,1)*(9/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+alfa2*Kp1*dZy(ii,1,1)*(-12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp1*dZy(ii,1,1)*(3/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+    alfa2*Kp1*dZy(ii,1,1)*(-12/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+  alfa2*Kp1*dZy(ii,1,1)*(16/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*Kp1*dZy(ii,1,1)*(-4/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*Kp1*dZy(ii,1,1)*(3/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+ alfa2*Kp1*dZy(ii,1,1)*(-4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*Kp1*dZy(ii,1,1)*(1/(4*dy*dz))*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*Kp1*dZy(ii,Ny,1)*(-3/(4*dy*dz))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*dZy(ii,Ny,1)*(12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp1*dZy(ii,Ny,1)*(-9/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*Kp1*dZy(ii,Ny,1)*(4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*Kp1*dZy(ii,Ny,1)*(-16/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*Kp1*dZy(ii,Ny,1)*(12/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*Kp1*dZy(ii,Ny,1)*(-1/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*Kp1*dZy(ii,Ny,1)*(4/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*Kp1*dZy(ii,Ny,1)*(-3/(4*dy*dz))*dQy(ii);
end

%------------ 2: Kp1 dQ/dy d^2Z/dydq  dV/dz  -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*d2Zyq(ii,jj,1)*(-3/(2*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp1*d2Zyq(ii,jj,1)*(4/(2*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp1*d2Zyq(ii,jj,1)*(-1/(2*dz))*dQy(ii); 
end 
end


%-------------- Kp1 dZ/dy d/dz ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp1*dQy(ii)*(3/(4*dz*dy))*dZy(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*Kp1*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*Kp1*dQy(ii)*(1/(4*dz*dy))*dZy(ii,jj,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*Kp1*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*Kp1*dQy(ii)*(4/(4*dz*dy))*dZy(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*Kp1*dQy(ii)*(-1/(4*dz*dy))*dZy(ii,jj,1);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*Kp1*dQy(ii)*(9/(4*dz*dy))*dZy(ii,1,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+alfa2*Kp1*dQy(ii)*(-12/(4*dz*dy))*dZy(ii,1,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp1*dQy(ii)*(3/(4*dz*dy))*dZy(ii,1,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+    alfa2*Kp1*dQy(ii)*(-12/(4*dz*dy))*dZy(ii,1,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+  alfa2*Kp1*dQy(ii)*(16/(4*dz*dy))*dZy(ii,1,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*Kp1*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,1,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*Kp1*dQy(ii)*(3/(4*dz*dy))*dZy(ii,1,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+ alfa2*Kp1*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,1,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*Kp1*dQy(ii)*(1/(4*dz*dy))*dZy(ii,1,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*Kp1*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,Ny,1);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*dQy(ii)*(12/(4*dz*dy))*dZy(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp1*dQy(ii)*(-9/(4*dz*dy))*dZy(ii,Ny,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*Kp1*dQy(ii)*(4/(4*dz*dy))*dZy(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*Kp1*dQy(ii)*(-16/(4*dz*dy))*dZy(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*Kp1*dQy(ii)*(12/(4*dz*dy))*dZy(ii,Ny,1); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*Kp1*dQy(ii)*(-1/(4*dz*dy))*dZy(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*Kp1*dQy(ii)*(4/(4*dz*dy))*dZy(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*Kp1*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,Ny,1);
end


%-------------- Kp1 dZ/dy d/dz ( dZ/dy dV/dz ) -------------------;

%------------ 1: Kp1 dZ/dy dZ/dy d/dz ( dV/dz ) -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp1*(2/dz^2)*dZy(ii,jj,1)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp1*(-5/dz^2)*dZy(ii,jj,1)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp1*(4/dz^2)*dZy(ii,jj,1)^2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)+alfa2*Kp1*(-1/dz^2)*dZy(ii,jj,1)^2;
end 
end

%------------- 2: Kp1 dZ/dy d^2Z/dydz  dV/dz -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*d2Zyz(ii,jj,1)*(-3/(2*dz))*dZy(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp1*d2Zyz(ii,jj,1)*(4/(2*dz))*dZy(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp1*d2Zyz(ii,jj,1)*(-1/(2*dz))*dZy(ii,jj,1);
end 
end


%------------------ Kp2 dQ/dy d/dQ ( dU/dx ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp2*(1/(4*dy*dx))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-2)*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-2)*Nz+1)+   alfa2*Kp2*(-1/(4*dy*dx))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+jj*Nz+1)+   alfa2*Kp2*(-1/(4*dy*dx))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+jj*Nz+1)=        MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+jj*Nz+1)+        alfa2*Kp2*(1/(4*dy*dx))*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+1)+     alfa2*Kp2*(3/(4*dy*dx))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+Nz+1)+ alfa2*Kp2*(-4/(4*dy*dx))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-2)*Ny*Nz+2*Nz+1)+alfa2*Kp2*(1/(4*dy*dx))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,ii*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,ii*Ny*Nz+1)+     alfa2*Kp2*(-3/(4*dy*dx))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,ii*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,ii*Ny*Nz+Nz+1)+   alfa2*Kp2*(4/(4*dy*dx))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,ii*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,ii*Ny*Nz+2*Nz+1)+alfa2*Kp2*(-1/(4*dy*dx))*dQy(ii);
end

%----- j=Ny ----;

for ii=2:1:Nx-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*Kp2*(-1/(4*dy*dx))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*Kp2*(4/(4*dy*dx))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp2*(-3/(4*dy*dx))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp2*(1/(4*dy*dx))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*(-4/(4*dy*dx))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,ii*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp2*(3/(4*dy*dx))*dQy(ii);                                                                                          
end

%---- i=Nx ----;

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+1)+     alfa2*Kp2*(-3/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+Nz+1)+   alfa2*Kp2*(4/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-3)*Ny*Nz+2*Nz+1)+alfa2*Kp2*(-1/(4*dy*dx))*dQy(Nx);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+1)+    alfa2*Kp2*(12/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+Nz+1)+alfa2*Kp2*(-16/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-2)*Ny*Nz+2*Nz+1)+alfa2*Kp2*(4/(4*dy*dx))*dQy(Nx);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+1)+     alfa2*Kp2*(-9/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+Nz+1)+  alfa2*Kp2*(12/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,(Nx-1)*Ny*Nz+2*Nz+1)+alfa2*Kp2*(-3/(4*dy*dx))*dQy(Nx);

for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp2*(-1/(4*dy*dx))*dQy(Nx);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp2*(4/(4*dy*dx))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp2*(-3/(4*dy*dx))*dQy(Nx);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+jj*Nz+1)+ alfa2*Kp2*(1/(4*dy*dx))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+jj*Nz+1)+alfa2*Kp2*(-4/(4*dy*dx))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+jj*Nz+1)+ alfa2*Kp2*(3/(4*dy*dx))*dQy(Nx);                                                                                          
end 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp2*(1/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*(-4/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp2*(3/(4*dy*dx))*dQy(Nx);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp2*(-4/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*Kp2*(16/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp2*(-12/(4*dy*dx))*dQy(Nx); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)+  alfa2*Kp2*(3/(4*dy*dx))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*(-12/(4*dy*dx))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)+  alfa2*Kp2*(9/(4*dy*dx))*dQy(Nx);


%----------------- Kp2 dQ/dy d/dQ ( dQ/dx dU/dQ ) -----------------;

%--------------- 1: Kp2 dQ/dy dQ/dx d/dQ ( dU/dQ ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp2*dQx(ii,jj)*(1/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*dQx(ii,jj)*(-2/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*Kp2*dQx(ii,jj)*(1/dy^2)*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+      alfa2*Kp2*dQx(ii,1)*(2/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+  alfa2*Kp2*dQx(ii,1)*(-5/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+ alfa2*Kp2*dQx(ii,1)*(4/dy^2)*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3*Nz+1)+alfa2*Kp2*dQx(ii,1)*(-1/dy^2)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-4)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-4)*Nz+1)+alfa2*Kp2*dQx(ii,Ny)*(-1/dy^2)*dQy(ii);     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp2*dQx(ii,Ny)*(4/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*dQx(ii,Ny)*(-5/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp2*dQx(ii,Ny)*(2/dy^2)*dQy(ii);                                                                                          
end

%--------------- 2: Kp2 dQ/dy d^2Q/dxdq  dU/dQ  -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp2*d2Qxq(ii,jj)*(-1/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*Kp2*d2Qxq(ii,jj)*(1/(2*dy))*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*Kp2*d2Qxq(ii,1)*(-3/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+   alfa2*Kp2*d2Qxq(ii,1)*(4/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp2*d2Qxq(ii,1)*(-1/(2*dy))*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp2*d2Qxq(ii,Ny)*(1/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*d2Qxq(ii,Ny)*(-4/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp2*d2Qxq(ii,Ny)*(3/(2*dy))*dQy(ii);                                                                                          
end


%----------------- Kp2 dQ/dy d/dQ ( dZ/dx dU/dz ) -----------------;

%--------------- 1: Kp2 dQ/dy dZ/dx d/dQ ( dU/dz ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp2*dZx(ii,jj,1)*(3/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*Kp2*dZx(ii,jj,1)*(-4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*Kp2*dZx(ii,jj,1)*(1/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*Kp2*dZx(ii,jj,1)*(-3/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*Kp2*dZx(ii,jj,1)*(4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*Kp2*dZx(ii,jj,1)*(-1/(4*dy*dz))*dQy(ii);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*Kp2*dZx(ii,1,1)*(9/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+alfa2*Kp2*dZx(ii,1,1)*(-12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp2*dZx(ii,1,1)*(3/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)+    alfa2*Kp2*dZx(ii,1,1)*(-12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)+  alfa2*Kp2*dZx(ii,1,1)*(16/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)+alfa2*Kp2*dZx(ii,1,1)*(-4/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)+     alfa2*Kp2*dZx(ii,1,1)*(3/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)+ alfa2*Kp2*dZx(ii,1,1)*(-4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)+alfa2*Kp2*dZx(ii,1,1)*(1/(4*dy*dz))*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*Kp2*dZx(ii,Ny,1)*(-3/(4*dy*dz))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*dZx(ii,Ny,1)*(12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp2*dZx(ii,Ny,1)*(-9/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*Kp2*dZx(ii,Ny,1)*(4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*Kp2*dZx(ii,Ny,1)*(-16/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*Kp2*dZx(ii,Ny,1)*(12/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*Kp2*dZx(ii,Ny,1)*(-1/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*Kp2*dZx(ii,Ny,1)*(4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*Kp2*dZx(ii,Ny,1)*(-3/(4*dy*dz))*dQy(ii);
end

%--------------- 2: Kp2 dQ/dy d^2Z/dxdq  dU/dz ----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*d2Zxq(ii,jj,1)*(-3/(2*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*d2Zxq(ii,jj,1)*(4/(2*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*d2Zxq(ii,jj,1)*(-1/(2*dz))*dQy(ii);
end 
end


%------------------- Kp2 dZ/dy d/dz ( dU/dx ) -------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp2*(3/(4*dz*dx))*dZy(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*(-4/(4*dz*dx))*dZy(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp2*(1/(4*dz*dx))*dZy(ii,jj,1); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*(-3/(4*dz*dx))*dZy(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*(4/(4*dz*dx))*dZy(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*(-1/(4*dz*dx))*dZy(ii,jj,1);
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*(-3/(4*dz*dx))*dZy(Nx,jj,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*(4/(4*dz*dx))*dZy(Nx,jj,1); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*(-1/(4*dz*dx))*dZy(Nx,jj,1);    
    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp2*(12/(4*dz*dx))*dZy(Nx,jj,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*(-16/(4*dz*dx))*dZy(Nx,jj,1); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+3)+  alfa2*Kp2*(4/(4*dz*dx))*dZy(Nx,jj,1); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*(-9/(4*dz*dx))*dZy(Nx,jj,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*(12/(4*dz*dx))*dZy(Nx,jj,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*(-3/(4*dz*dx))*dZy(Nx,jj,1);
end 


%----------------- Kp2 dZ/dy d/dz ( dQ/dx dU/dQ ) ------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp2*dQx(ii,jj)*(3/(4*dz*dy))*dZy(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*Kp2*dQx(ii,jj)*(-4/(4*dz*dy))*dZy(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*Kp2*dQx(ii,jj)*(1/(4*dz*dy))*dZy(ii,jj,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*Kp2*dQx(ii,jj)*(-3/(4*dz*dy))*dZy(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*Kp2*dQx(ii,jj)*(4/(4*dz*dy))*dZy(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*Kp2*dQx(ii,jj)*(-1/(4*dz*dy))*dZy(ii,jj,1);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*Kp2*dQx(ii,1)*(9/(4*dz*dy))*dZy(ii,1,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+alfa2*Kp2*dQx(ii,1)*(-12/(4*dz*dy))*dZy(ii,1,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp2*dQx(ii,1)*(3/(4*dz*dy))*dZy(ii,1,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)+    alfa2*Kp2*dQx(ii,1)*(-12/(4*dz*dy))*dZy(ii,1,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)+  alfa2*Kp2*dQx(ii,1)*(16/(4*dz*dy))*dZy(ii,1,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)+alfa2*Kp2*dQx(ii,1)*(-4/(4*dz*dy))*dZy(ii,1,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)+     alfa2*Kp2*dQx(ii,1)*(3/(4*dz*dy))*dZy(ii,1,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)+ alfa2*Kp2*dQx(ii,1)*(-4/(4*dz*dy))*dZy(ii,1,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)+alfa2*Kp2*dQx(ii,1)*(1/(4*dz*dy))*dZy(ii,1,1);
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*Kp2*dQx(ii,Ny)*(-3/(4*dz*dy))*dZy(ii,Ny,1);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*dQx(ii,Ny)*(12/(4*dz*dy))*dZy(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp2*dQx(ii,Ny)*(-9/(4*dz*dy))*dZy(ii,Ny,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*Kp2*dQx(ii,Ny)*(4/(4*dz*dy))*dZy(ii,Ny,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*Kp2*dQx(ii,Ny)*(-16/(4*dz*dy))*dZy(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*Kp2*dQx(ii,Ny)*(12/(4*dz*dy))*dZy(ii,Ny,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*Kp2*dQx(ii,Ny)*(-1/(4*dz*dy))*dZy(ii,Ny,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*Kp2*dQx(ii,Ny)*(4/(4*dz*dy))*dZy(ii,Ny,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*Kp2*dQx(ii,Ny)*(-3/(4*dz*dy))*dZy(ii,Ny,1);
end


%---------------- Kp2 dZ/dy d/dz ( dZ/dx dU/dz ) ------------------;

%-------------- 1: Kp2 dZ/dy dZ/dx d/dz ( dU/dz ) ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp2*dZx(ii,jj,1)*(2/dz^2)*dZy(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*dZx(ii,jj,1)*(-5/dz^2)*dZy(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp2*dZx(ii,jj,1)*(4/dz^2)*dZy(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+4)+alfa2*Kp2*dZx(ii,jj,1)*(-1/dz^2)*dZy(ii,jj,1);
end 
end

%-------------- 2: Kp2 dZ/dy d^2Z/dxdz  dU/dz  ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*d2Zxz(ii,jj,1)*(-3/(2*dz))*dZy(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*d2Zxz(ii,jj,1)*(4/(2*dz))*dZy(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*d2Zxz(ii,jj,1)*(-1/(2*dz))*dZy(ii,jj,1); 
end 
end


%---------------- Kp2 dQ/dy d/dQ ( dZ/dz dW/dz ) ------------------;

%-------------- 1: Kp2 dQ/dy dZ/dz d/dQ ( dW/dz ) ------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*Kp2*dZz(ii,jj)*(3/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*Kp2*dZz(ii,jj)*(-4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*Kp2*dZz(ii,jj)*(1/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*Kp2*dZz(ii,jj)*(-3/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*Kp2*dZz(ii,jj)*(4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*Kp2*dZz(ii,jj)*(-1/(4*dy*dz))*dQy(ii);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*Kp2*dZz(ii,1)*(9/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+alfa2*Kp2*dZz(ii,1)*(-12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp2*dZz(ii,1)*(3/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+    alfa2*Kp2*dZz(ii,1)*(-12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+  alfa2*Kp2*dZz(ii,1)*(16/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*Kp2*dZz(ii,1)*(-4/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*Kp2*dZz(ii,1)*(3/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+ alfa2*Kp2*dZz(ii,1)*(-4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*Kp2*dZz(ii,1)*(1/(4*dy*dz))*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*Kp2*dZz(ii,Ny)*(-3/(4*dy*dz))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*dZz(ii,Ny)*(12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*Kp2*dZz(ii,Ny)*(-9/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*Kp2*dZz(ii,Ny)*(4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*Kp2*dZz(ii,Ny)*(-16/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*Kp2*dZz(ii,Ny)*(12/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*Kp2*dZz(ii,Ny)*(-1/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*Kp2*dZz(ii,Ny)*(4/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*Kp2*dZz(ii,Ny)*(-3/(4*dy*dz))*dQy(ii);
end

%-------------- 2: Kp2 dQ/dy d^2Z/dzdq  dW/dz ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*d2Zzq(ii,jj)*(-3/(2*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*d2Zzq(ii,jj)*(4/(2*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*d2Zzq(ii,jj)*(-1/(2*dz))*dQy(ii);
end 
end


%---------------- Kp2 dZ/dy d/dz ( dZ/dz dW/dz ) ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp2*dZz(ii,jj)*(2/dz^2)*dZy(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*dZz(ii,jj)*(-5/dz^2)*dZy(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp2*dZz(ii,jj)*(4/dz^2)*dZy(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)+alfa2*Kp2*dZz(ii,jj)*(-1/dz^2)*dZy(ii,jj,1);
end 
end



%+++++++++ KpE (-1/H) (-3/(2 dz)) ( Fyx dHs/dx + Fyy dHs/dy )^1 ++++++++++;

%--------------- (-1/H) (-3/(2*dz)) dHs/dx dQ/dy dU/dQ ----------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*(-3/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsx(ii,1)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+   alfa2*(4/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsx(ii,1)*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsx(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsx(ii,Ny)*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-4/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsx(ii,Ny)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(3/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsx(ii,Ny)*dQy(ii); 
end


%---------- (-1/H) (-3/(2*dz)) dHs/dx dZ/dy dU/dz -------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-3/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZy(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(4/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZy(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-1/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZy(ii,jj,1);
end 
end


%-------------- (-1/H) (-3/(2*dz)) dHs/dx dV/dx ----------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-1/(2*dx))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)+     alfa2*(1/(2*dx))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(1/(2*dx))*(-3/(2*dz))*dZz(Nx,jj)*dHsx(Nx,jj);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-4/(2*dx))*(-3/(2*dz))*dZz(Nx,jj)*dHsx(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(3/(2*dx))*(-3/(2*dz))*dZz(Nx,jj)*dHsx(Nx,jj); 
end 


%------------ (-1/H) (-3/(2*dz)) dHs/dx dQ/dx dV/dQ ---------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*(-3/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsx(ii,1)*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+   alfa2*(4/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsx(ii,1)*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsx(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsx(ii,Ny)*dQx(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-4/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsx(ii,Ny)*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(3/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsx(ii,Ny)*dQx(ii,Ny); 
end


%------------- (-1/H) (-3/(2*dz)) dHs/dx dZ/dx dV/dz ---------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-3/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZx(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(4/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZx(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-1/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZx(ii,jj,1);
end 
end


%------------ Kp1 (-1/H) (-3/(2*dz)) dHs/dy dQ/dy dV/dQ ---------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp1*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*Kp1*(1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*Kp1*(-3/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsy(ii,1)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+   alfa2*Kp1*(4/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsy(ii,1)*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp1*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsy(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp1*(1/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsy(ii,Ny)*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp1*(-4/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsy(ii,Ny)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp1*(3/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsy(ii,Ny)*dQy(ii); 
end


%------------- Kp1 (-1/H) (-3/(2*dz)) dHs/dy dZ/dy dV/dz --------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*(-3/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZy(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp1*(4/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZy(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp1*(-1/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZy(ii,jj,1);
end 
end


%--------------- Kp2 (-1/H) (-3/(2*dz)) dHs/dy dU/dx -----------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*(-1/(2*dx))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)+     alfa2*Kp2*(1/(2*dx))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp2*(1/(2*dx))*(-3/(2*dz))*dZz(Nx,jj)*dHsy(Nx,jj);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*(-4/(2*dx))*(-3/(2*dz))*dZz(Nx,jj)*dHsy(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*Kp2*(3/(2*dx))*(-3/(2*dz))*dZz(Nx,jj)*dHsy(Nx,jj); 
end 


%------------- Kp2 (-1/H) (-3/(2*dz)) dHs/dy dQ/dx dU/dQ --------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*Kp2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*Kp2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*Kp2*(-3/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsy(ii,1)*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+   alfa2*Kp2*(4/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsy(ii,1)*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*Kp2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsy(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*Kp2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsy(ii,Ny)*dQx(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*Kp2*(-4/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsy(ii,Ny)*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*Kp2*(3/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsy(ii,Ny)*dQx(ii,Ny); 
end


%------------ Kp2 (-1/H) (-3/(2*dz)) dHs/dy dZ/dx dU/dz --------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*(-3/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZx(ii,jj,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*(4/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZx(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*(-1/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZx(ii,jj,1);
end 
end


%------------- Kp2 (-1/H) (-3/(2*dz)) dHs/dy dZ/dz dW/dz --------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*(-3/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZz(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*(4/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZz(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*(-1/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZz(ii,jj);
end 
end



%++++++++++++++++++ KpE (-1/H) 4/(2 dz) ( Fyz )^2 +++++++++++++++++++++;

%-------------------- (-1/H) 4/(2*dz) dZ/dz dV/dz --------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-1/(2*dz))*(4/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*(1/(2*dz))*(4/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
end 
end


%------------------- (-1/H) 4/(2*dz) dQ/dy dW/dQ ------------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*(-1/(2*dy))*(4/(2*dz))*dZz(ii,jj)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+     alfa2*(1/(2*dy))*(4/(2*dz))*dZz(ii,jj)*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+     alfa2*(-3/(2*dy))*(4/(2*dz))*dZz(ii,1)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+   alfa2*(4/(2*dy))*(4/(2*dz))*dZz(ii,1)*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*(-1/(2*dy))*(4/(2*dz))*dZz(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+ alfa2*(1/(2*dy))*(4/(2*dz))*dZz(ii,Ny)*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*(-4/(2*dy))*(4/(2*dz))*dZz(ii,Ny)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*(3/(2*dy))*(4/(2*dz))*dZz(ii,Ny)*dQy(ii); 
end


%----------------- (-1/H) 4/(2*dz) dZ/dy dW/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-1/(2*dz))*(4/(2*dz))*dZz(ii,jj)*dZy(ii,jj,2); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*(1/(2*dz))*(4/(2*dz))*dZz(ii,jj)*dZy(ii,jj,2); 
end 
end



%++++++++++++++++++ KpE (-1/H) (-1/(2 dz)) ( Fyz )^3 ++++++++++++++++++++;

%-------------------- (-1/H) (-1/(2*dz)) dZ/dz dV/dz --------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)+ alfa2*(1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
end 
end


%------------------- (-1/H) (-1/(2*dz)) dQ/dy dW/dQ ------------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+alfa2*(-1/(2*dy))*(-1/(2*dz))*dZz(ii,jj)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+     alfa2*(1/(2*dy))*(-1/(2*dz))*dZz(ii,jj)*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*(-3/(2*dy))*(-1/(2*dz))*dZz(ii,1)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+   alfa2*(4/(2*dy))*(-1/(2*dz))*dZz(ii,1)*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*(-1/(2*dy))*(-1/(2*dz))*dZz(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+ alfa2*(1/(2*dy))*(-1/(2*dz))*dZz(ii,Ny)*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+alfa2*(-4/(2*dy))*(-1/(2*dz))*dZz(ii,Ny)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+ alfa2*(3/(2*dy))*(-1/(2*dz))*dZz(ii,Ny)*dQy(ii); 
end


%----------------- (-1/H) (-1/(2*dz)) dZ/dy dW/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZy(ii,jj,3); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)+ alfa2*(1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZy(ii,jj,3); 
end 
end



%+++++++++++++++++++ + KpE dens Freq^2 V ++++++++++++++++++++++;

for ii=2:1:Nx 
for jj=1:1:Ny 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1) + alfa2*KpE*dens*Freq^2;
end 
end




%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++++++++ The third equation ++++++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

%+++++++++++++++++++++++++ KpE dFzx/dx +++++++++++++++++++++++++++;

%--------------------- d/dx ( dZ/dz dU/dz ) ---------------------;

%------------------- 1: dZ/dz d/dx ( dU/dz ) ---------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*dZz(ii,jj)*(3/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZz(ii,jj)*(-4/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*dZz(ii,jj)*(1/(4*dx*dz)); 

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)+alfa2*dZz(ii,jj)*(-3/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+2)+ alfa2*dZz(ii,jj)*(4/(4*dx*dz));
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+3)+alfa2*dZz(ii,jj)*(-1/(4*dx*dz));
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+alfa2*dZz(Nx,jj)*(-3/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*dZz(Nx,jj)*(4/(4*dx*dz));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+3)+alfa2*dZz(Nx,jj)*(-1/(4*dx*dz));    
    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*dZz(Nx,jj)*(12/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZz(Nx,jj)*(-16/(4*dx*dz));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+3)+  alfa2*dZz(Nx,jj)*(4/(4*dx*dz));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*dZz(Nx,jj)*(-9/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZz(Nx,jj)*(12/(4*dx*dz));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*dZz(Nx,jj)*(-3/(4*dx*dz));
end

%------------------- 2: d^2Z/dzdx  dU/dz ---------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zzx(ii,jj)*(-3/(2*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zzx(ii,jj)*(4/(2*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zzx(ii,jj)*(-1/(2*dz)); 
end 
end


%---------------- dQ/dx d/dQ ( dZ/dz dU/dz ) ------------------;

%-------------- 1: dQ/dx dZ/dz d/dQ ( dU/dz ) ------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dZz(ii,jj)*(3/(4*dy*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*dZz(ii,jj)*(-4/(4*dy*dz))*dQx(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*dZz(ii,jj)*(1/(4*dy*dz))*dQx(ii,jj);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*dZz(ii,jj)*(-3/(4*dy*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*dZz(ii,jj)*(4/(4*dy*dz))*dQx(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*dZz(ii,jj)*(-1/(4*dy*dz))*dQx(ii,jj);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa2*dZz(ii,1)*(9/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+alfa2*dZz(ii,1)*(-12/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa2*dZz(ii,1)*(3/(4*dy*dz))*dQx(ii,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)+    alfa2*dZz(ii,1)*(-12/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)+  alfa2*dZz(ii,1)*(16/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)+alfa2*dZz(ii,1)*(-4/(4*dy*dz))*dQx(ii,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)+     alfa2*dZz(ii,1)*(3/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)+ alfa2*dZz(ii,1)*(-4/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)+alfa2*dZz(ii,1)*(1/(4*dy*dz))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dZz(ii,Ny)*(-3/(4*dy*dz))*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dZz(ii,Ny)*(12/(4*dy*dz))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dZz(ii,Ny)*(-9/(4*dy*dz))*dQx(ii,Ny);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*dZz(ii,Ny)*(4/(4*dy*dz))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*dZz(ii,Ny)*(-16/(4*dy*dz))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*dZz(ii,Ny)*(12/(4*dy*dz))*dQx(ii,Ny);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*dZz(ii,Ny)*(-1/(4*dy*dz))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*dZz(ii,Ny)*(4/(4*dy*dz))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*dZz(ii,Ny)*(-3/(4*dy*dz))*dQx(ii,Ny);
end

%----------------- 2: dQ/dx d^2Z/dzdq  dU/dz ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zzq(ii,jj)*(-3/(2*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zzq(ii,jj)*(4/(2*dz))*dQx(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zzq(ii,jj)*(-1/(2*dz))*dQx(ii,jj);
end 
end


%----------------- dZ/dx d/dz ( dZ/dz dU/dz ) --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*dZz(ii,jj)*(2/dz^2)*dZx(ii,jj,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZz(ii,jj)*(-5/dz^2)*dZx(ii,jj,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*dZz(ii,jj)*(4/dz^2)*dZx(ii,jj,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+4)+alfa2*dZz(ii,jj)*(-1/dz^2)*dZx(ii,jj,1);
end 
end


%---------------------- d/dx ( dW/dx ) -----------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(1/dx^2); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-2/dx^2); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)+    alfa2*(1/dx^2); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-4)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-4)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-1/dx^2);     
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(4/dx^2); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-5/dx^2); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(2/dx^2);                                        
end 


%--------------------- d/dx ( dQ/dx dW/dQ ) ---------------------;

%-------------------- 1: dQ/dx d/dx ( dW/dQ ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+1)+alfa2*dQx(ii,jj)*(1/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+1)+   alfa2*dQx(ii,jj)*(-1/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+1)+   alfa2*dQx(ii,jj)*(-1/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+1)=        MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+1)+        alfa2*dQx(ii,jj)*(1/(4*dx*dy)); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+1)+     alfa2*dQx(ii,1)*(3/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+1)+ alfa2*dQx(ii,1)*(-4/(4*dx*dy));
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+1)+alfa2*dQx(ii,1)*(1/(4*dx*dy));

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+1)+     alfa2*dQx(ii,1)*(-3/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+Nz+1)+   alfa2*dQx(ii,1)*(4/(4*dx*dy));
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+2*Nz+1)+alfa2*dQx(ii,1)*(-1/(4*dx*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dQx(ii,Ny)*(-1/(4*dx*dy));    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*dQx(ii,Ny)*(4/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dQx(ii,Ny)*(-3/(4*dx*dy));

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*dQx(ii,Ny)*(1/(4*dx*dy));
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQx(ii,Ny)*(-4/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*dQx(ii,Ny)*(3/(4*dx*dy));                                  
end

%---- i=Nx ----;

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+1)+     alfa2*dQx(Nx,1)*(-3/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+1)+   alfa2*dQx(Nx,1)*(4/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+1)+alfa2*dQx(Nx,1)*(-1/(4*dx*dy));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+1)+    alfa2*dQx(Nx,1)*(12/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+1)+alfa2*dQx(Nx,1)*(-16/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+1)+alfa2*dQx(Nx,1)*(4/(4*dx*dy));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1)+     alfa2*dQx(Nx,1)*(-9/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+1)+  alfa2*dQx(Nx,1)*(12/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+1)+alfa2*dQx(Nx,1)*(-3/(4*dx*dy));

for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+1)+alfa2*dQx(Nx,jj)*(-1/(4*dx*dy));    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dQx(Nx,jj)*(4/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*dQx(Nx,jj)*(-3/(4*dx*dy));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+1)+ alfa2*dQx(Nx,jj)*(1/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+1)+alfa2*dQx(Nx,jj)*(-4/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+1)+ alfa2*dQx(Nx,jj)*(3/(4*dx*dy));                                  
end 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*dQx(Nx,Ny)*(1/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQx(Nx,Ny)*(-4/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*dQx(Nx,Ny)*(3/(4*dx*dy));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*dQx(Nx,Ny)*(-4/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*dQx(Nx,Ny)*(16/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dQx(Nx,Ny)*(-12/(4*dx*dy));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)+  alfa2*dQx(Nx,Ny)*(3/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQx(Nx,Ny)*(-12/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)+  alfa2*dQx(Nx,Ny)*(9/(4*dx*dy));

%-------------------- 2: d^2Q/dxdx  dW/dQ --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*d2Qxx(ii,jj)*(-1/(2*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*d2Qxx(ii,jj)*(1/(2*dy)); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*d2Qxx(ii,1)*(-3/(2*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+   alfa2*d2Qxx(ii,1)*(4/(2*dy));
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*d2Qxx(ii,1)*(-1/(2*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*d2Qxx(ii,Ny)*(1/(2*dy));    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*d2Qxx(ii,Ny)*(-4/(2*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*d2Qxx(ii,Ny)*(3/(2*dy));
end


%--------------------- d/dx ( dZ/dx dW/dz ) ---------------------;

%------------------- 1: dZ/dx d/dx ( dW/dz ) ---------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*dZx(ii,jj,1)*(3/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZx(ii,jj,1)*(-4/(4*dx*dz));
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*dZx(ii,jj,1)*(1/(4*dx*dz));

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)+alfa2*dZx(ii,jj,1)*(-3/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)+ alfa2*dZx(ii,jj,1)*(4/(4*dx*dz));
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)+alfa2*dZx(ii,jj,1)*(-1/(4*dx*dz));
end 
end

%---- i=Nx ----;

for jj=1:1:Ny 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+alfa2*dZx(Nx,jj,1)*(-3/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*dZx(Nx,jj,1)*(4/(4*dx*dz));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)+alfa2*dZx(Nx,jj,1)*(-1/(4*dx*dz));    

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*dZx(Nx,jj,1)*(12/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZx(Nx,jj,1)*(-16/(4*dx*dz));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)+  alfa2*dZx(Nx,jj,1)*(4/(4*dx*dz));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*dZx(Nx,jj,1)*(-9/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZx(Nx,jj,1)*(12/(4*dx*dz));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*dZx(Nx,jj,1)*(-3/(4*dx*dz));
end

%------------------- 2: d^2Z/dxdx  dW/dz ---------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zxx(ii,jj,1)*(-3/(2*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zxx(ii,jj,1)*(4/(2*dz));
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zxx(ii,jj,1)*(-1/(2*dz));
end 
end


%-------------------- dQ/dx d/dQ ( dW/dx ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+1)+alfa2*(1/(4*dy*dx))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+1)+   alfa2*(-1/(4*dy*dx))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+1)+   alfa2*(-1/(4*dy*dx))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+1)=        MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+1)+        alfa2*(1/(4*dy*dx))*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+1)+     alfa2*(3/(4*dy*dx))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+1)+ alfa2*(-4/(4*dy*dx))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+1)+alfa2*(1/(4*dy*dx))*dQx(ii,1); 

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+1)+     alfa2*(-3/(4*dy*dx))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+Nz+1)+   alfa2*(4/(4*dy*dx))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+2*Nz+1)+alfa2*(-1/(4*dy*dx))*dQx(ii,1);
end

%----- j=Ny ----;

for ii=2:1:Nx-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*(-1/(4*dy*dx))*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*(4/(4*dy*dx))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*(-3/(4*dy*dx))*dQx(ii,Ny);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(1/(4*dy*dx))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-4/(4*dy*dx))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(3/(4*dy*dx))*dQx(ii,Ny);                                                                                          
end

%---- i=Nx ----;

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+1)+     alfa2*(-3/(4*dy*dx))*dQx(Nx,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+1)+   alfa2*(4/(4*dy*dx))*dQx(Nx,1); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+1)+alfa2*(-1/(4*dy*dx))*dQx(Nx,1);

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+1)+    alfa2*(12/(4*dy*dx))*dQx(Nx,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+1)+alfa2*(-16/(4*dy*dx))*dQx(Nx,1); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+1)+alfa2*(4/(4*dy*dx))*dQx(Nx,1);

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1)+     alfa2*(-9/(4*dy*dx))*dQx(Nx,1); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+1)+  alfa2*(12/(4*dy*dx))*dQx(Nx,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+1)+alfa2*(-3/(4*dy*dx))*dQx(Nx,1);


for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+1)+alfa2*(-1/(4*dy*dx))*dQx(Nx,jj);    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*(4/(4*dy*dx))*dQx(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*(-3/(4*dy*dx))*dQx(Nx,jj);

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+1)+ alfa2*(1/(4*dy*dx))*dQx(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+1)+alfa2*(-4/(4*dy*dx))*dQx(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+1)+ alfa2*(3/(4*dy*dx))*dQx(Nx,jj);                                                                                          
end 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(1/(4*dy*dx))*dQx(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(3/(4*dy*dx))*dQx(Nx,Ny);

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+1)+ alfa2*(16/(4*dy*dx))*dQx(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*(-12/(4*dy*dx))*dQx(Nx,Ny);

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+1)+  alfa2*(3/(4*dy*dx))*dQx(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-12/(4*dy*dx))*dQx(Nx,Ny); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+1)+  alfa2*(9/(4*dy*dx))*dQx(Nx,Ny);


%------------------- dQ/dx d/dQ ( dQ/dx dW/dQ ) --------------------;

%----------------- 1: dQ/dx dQ/dx d/dQ ( dW/dQ ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*(1/dy^2)*dQx(ii,jj)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-2/dy^2)*dQx(ii,jj)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*(1/dy^2)*dQx(ii,jj)^2; 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+      alfa2*(2/dy^2)*dQx(ii,1)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+  alfa2*(-5/dy^2)*dQx(ii,1)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+ alfa2*(4/dy^2)*dQx(ii,1)^2;
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+1)+alfa2*(-1/dy^2)*dQx(ii,1)^2;
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+1)+alfa2*(-1/dy^2)*dQx(ii,Ny)^2;    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(4/dy^2)*dQx(ii,Ny)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-5/dy^2)*dQx(ii,Ny)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(2/dy^2)*dQx(ii,Ny)^2;                                                                                          
end

%------------------- 2: dQ/dx d^2Q/dxdq  dW/dQ ----------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*d2Qxq(ii,jj)*(-1/(2*dy))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*d2Qxq(ii,jj)*(1/(2*dy))*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*d2Qxq(ii,1)*(-3/(2*dy))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+   alfa2*d2Qxq(ii,1)*(4/(2*dy))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*d2Qxq(ii,1)*(-1/(2*dy))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*d2Qxq(ii,Ny)*(1/(2*dy))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*d2Qxq(ii,Ny)*(-4/(2*dy))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*d2Qxq(ii,Ny)*(3/(2*dy))*dQx(ii,Ny);                                                                                          
end


%-------------------- dQ/dx d/dQ ( dZ/dx dW/dz ) --------------------;

%------------------ 1: dQ/dx dZ/dx d/dQ ( dW/dz ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dZx(ii,jj,1)*(3/(4*dy*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*dZx(ii,jj,1)*(-4/(4*dy*dz))*dQx(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*dZx(ii,jj,1)*(1/(4*dy*dz))*dQx(ii,jj);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*dZx(ii,jj,1)*(-3/(4*dy*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*dZx(ii,jj,1)*(4/(4*dy*dz))*dQx(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*dZx(ii,jj,1)*(-1/(4*dy*dz))*dQx(ii,jj);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*dZx(ii,1,1)*(9/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+alfa2*dZx(ii,1,1)*(-12/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*dZx(ii,1,1)*(3/(4*dy*dz))*dQx(ii,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+    alfa2*dZx(ii,1,1)*(-12/(4*dy*dz))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+  alfa2*dZx(ii,1,1)*(16/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*dZx(ii,1,1)*(-4/(4*dy*dz))*dQx(ii,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*dZx(ii,1,1)*(3/(4*dy*dz))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+ alfa2*dZx(ii,1,1)*(-4/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*dZx(ii,1,1)*(1/(4*dy*dz))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dZx(ii,Ny,1)*(-3/(4*dy*dz))*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dZx(ii,Ny,1)*(12/(4*dy*dz))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dZx(ii,Ny,1)*(-9/(4*dy*dz))*dQx(ii,Ny);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*dZx(ii,Ny,1)*(4/(4*dy*dz))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*dZx(ii,Ny,1)*(-16/(4*dy*dz))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*dZx(ii,Ny,1)*(12/(4*dy*dz))*dQx(ii,Ny);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*dZx(ii,Ny,1)*(-1/(4*dy*dz))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*dZx(ii,Ny,1)*(4/(4*dy*dz))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*dZx(ii,Ny,1)*(-3/(4*dy*dz))*dQx(ii,Ny);
end

%----------------- 2: dQ/dx d^2Z/dxdq  dW/dz -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zxq(ii,jj,1)*(-3/(2*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zxq(ii,jj,1)*(4/(2*dz))*dQx(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zxq(ii,jj,1)*(-1/(2*dz))*dQx(ii,jj);
end 
end


%-------------------- dZ/dx d/dz ( dW/dx ) --------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(3/(4*dz*dx))*dZx(ii,jj,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-4/(4*dz*dx))*dZx(ii,jj,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*(1/(4*dz*dx))*dZx(ii,jj,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-3/(4*dz*dx))*dZx(ii,jj,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(4/(4*dz*dx))*dZx(ii,jj,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-1/(4*dz*dx))*dZx(ii,jj,1);
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-3/(4*dz*dx))*dZx(Nx,jj,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(4/(4*dz*dx))*dZx(Nx,jj,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-1/(4*dz*dx))*dZx(Nx,jj,1);    
    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(12/(4*dz*dx))*dZx(Nx,jj,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-16/(4*dz*dx))*dZx(Nx,jj,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+3)+  alfa2*(4/(4*dz*dx))*dZx(Nx,jj,1);

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-9/(4*dz*dx))*dZx(Nx,jj,1); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(12/(4*dz*dx))*dZx(Nx,jj,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-3/(4*dz*dx))*dZx(Nx,jj,1);
end 


%-------------------- dZ/dx d/dz ( dQ/dx dW/dQ ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dQx(ii,jj)*(3/(4*dz*dy))*dZx(ii,jj,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*dQx(ii,jj)*(-4/(4*dz*dy))*dZx(ii,jj,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*dQx(ii,jj)*(1/(4*dz*dy))*dZx(ii,jj,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*dQx(ii,jj)*(-3/(4*dz*dy))*dZx(ii,jj,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*dQx(ii,jj)*(4/(4*dz*dy))*dZx(ii,jj,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*dQx(ii,jj)*(-1/(4*dz*dy))*dZx(ii,jj,1);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*dQx(ii,1)*(9/(4*dz*dy))*dZx(ii,1,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+alfa2*dQx(ii,1)*(-12/(4*dz*dy))*dZx(ii,1,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*dQx(ii,1)*(3/(4*dz*dy))*dZx(ii,1,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+    alfa2*dQx(ii,1)*(-12/(4*dz*dy))*dZx(ii,1,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+  alfa2*dQx(ii,1)*(16/(4*dz*dy))*dZx(ii,1,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*dQx(ii,1)*(-4/(4*dz*dy))*dZx(ii,1,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*dQx(ii,1)*(3/(4*dz*dy))*dZx(ii,1,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+ alfa2*dQx(ii,1)*(-4/(4*dz*dy))*dZx(ii,1,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*dQx(ii,1)*(1/(4*dz*dy))*dZx(ii,1,1);
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dQx(ii,Ny)*(-3/(4*dz*dy))*dZx(ii,Ny,1);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQx(ii,Ny)*(12/(4*dz*dy))*dZx(ii,Ny,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dQx(ii,Ny)*(-9/(4*dz*dy))*dZx(ii,Ny,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*dQx(ii,Ny)*(4/(4*dz*dy))*dZx(ii,Ny,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*dQx(ii,Ny)*(-16/(4*dz*dy))*dZx(ii,Ny,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*dQx(ii,Ny)*(12/(4*dz*dy))*dZx(ii,Ny,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*dQx(ii,Ny)*(-1/(4*dz*dy))*dZx(ii,Ny,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*dQx(ii,Ny)*(4/(4*dz*dy))*dZx(ii,Ny,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*dQx(ii,Ny)*(-3/(4*dz*dy))*dZx(ii,Ny,1);
end


%-------------------- dZ/dx d/dz ( dZ/dx dW/dz ) --------------------;

%------------------ 1: dZ/dx dZ/dx d/dz ( dW/dz ) --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(2/dz^2)*dZx(ii,jj,1)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-5/dz^2)*dZx(ii,jj,1)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*(4/dz^2)*dZx(ii,jj,1)^2;
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)+alfa2*(-1/dz^2)*dZx(ii,jj,1)^2;
end 
end

%------------------ 2: dZ/dx d^2Z/dxdz  dW/dz --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zxz(ii,jj,1)*(-3/(2*dz))*dZx(ii,jj,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zxz(ii,jj,1)*(4/(2*dz))*dZx(ii,jj,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zxz(ii,jj,1)*(-1/(2*dz))*dZx(ii,jj,1);
end 
end



%+++++++++++++++++++++++++ KpE dFzy/dy +++++++++++++++++++++++++++;

%------------------ dQ/dy d/dQ ( dZ/dz dV/dz ) -------------------;

%---------------- 1: dQ/dy dZ/dz d/dQ ( dV/dz ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dZz(ii,jj)*(3/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*dZz(ii,jj)*(-4/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*dZz(ii,jj)*(1/(4*dy*dz))*dQy(ii);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*dZz(ii,jj)*(-3/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*dZz(ii,jj)*(4/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*dZz(ii,jj)*(-1/(4*dy*dz))*dQy(ii);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*dZz(ii,1)*(9/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+alfa2*dZz(ii,1)*(-12/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*dZz(ii,1)*(3/(4*dy*dz))*dQy(ii);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+    alfa2*dZz(ii,1)*(-12/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+  alfa2*dZz(ii,1)*(16/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*dZz(ii,1)*(-4/(4*dy*dz))*dQy(ii);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*dZz(ii,1)*(3/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+ alfa2*dZz(ii,1)*(-4/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*dZz(ii,1)*(1/(4*dy*dz))*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dZz(ii,Ny)*(-3/(4*dy*dz))*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dZz(ii,Ny)*(12/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dZz(ii,Ny)*(-9/(4*dy*dz))*dQy(ii); 

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*dZz(ii,Ny)*(4/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*dZz(ii,Ny)*(-16/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*dZz(ii,Ny)*(12/(4*dy*dz))*dQy(ii);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*dZz(ii,Ny)*(-1/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*dZz(ii,Ny)*(4/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*dZz(ii,Ny)*(-3/(4*dy*dz))*dQy(ii);
end

%---------------- 2: dQ/dy d^2Z/dzdq  dV/dz -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zzq(ii,jj)*(-3/(2*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zzq(ii,jj)*(4/(2*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zzq(ii,jj)*(-1/(2*dz))*dQy(ii);
end 
end


%----------------- dZ/dy d/dz ( dZ/dz dV/dz ) ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*dZz(ii,jj)*(2/dz^2)*dZy(ii,jj,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*dZz(ii,jj)*(-5/dz^2)*dZy(ii,jj,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*dZz(ii,jj)*(4/dz^2)*dZy(ii,jj,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)+alfa2*dZz(ii,jj)*(-1/dz^2)*dZy(ii,jj,1);
end 
end


%---------------- dQ/dy d/dQ ( dQ/dy dW/dQ ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*(1/dy^2)*dQy(ii)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-2/dy^2)*dQy(ii)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*(1/dy^2)*dQy(ii)^2; 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+      alfa2*(2/dy^2)*dQy(ii)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+  alfa2*(-5/dy^2)*dQy(ii)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+ alfa2*(4/dy^2)*dQy(ii)^2;
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+1)+alfa2*(-1/dy^2)*dQy(ii)^2;
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+1)+alfa2*(-1/dy^2)*dQy(ii)^2;     
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(4/dy^2)*dQy(ii)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-5/dy^2)*dQy(ii)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(2/dy^2)*dQy(ii)^2;                                                                                          
end


%-------------- dQ/dy d/dQ ( dZ/dy dW/dz ) -------------------;

%------------ 1: dQ/dy dZ/dy d/dQ ( dW/dz ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dZy(ii,jj,1)*(3/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*dZy(ii,jj,1)*(-4/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*dZy(ii,jj,1)*(1/(4*dy*dz))*dQy(ii);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*dZy(ii,jj,1)*(-3/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*dZy(ii,jj,1)*(4/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*dZy(ii,jj,1)*(-1/(4*dy*dz))*dQy(ii);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*dZy(ii,1,1)*(9/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+alfa2*dZy(ii,1,1)*(-12/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*dZy(ii,1,1)*(3/(4*dy*dz))*dQy(ii);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+    alfa2*dZy(ii,1,1)*(-12/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+  alfa2*dZy(ii,1,1)*(16/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*dZy(ii,1,1)*(-4/(4*dy*dz))*dQy(ii);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*dZy(ii,1,1)*(3/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+ alfa2*dZy(ii,1,1)*(-4/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*dZy(ii,1,1)*(1/(4*dy*dz))*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dZy(ii,Ny,1)*(-3/(4*dy*dz))*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dZy(ii,Ny,1)*(12/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dZy(ii,Ny,1)*(-9/(4*dy*dz))*dQy(ii);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*dZy(ii,Ny,1)*(4/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*dZy(ii,Ny,1)*(-16/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*dZy(ii,Ny,1)*(12/(4*dy*dz))*dQy(ii);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*dZy(ii,Ny,1)*(-1/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*dZy(ii,Ny,1)*(4/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*dZy(ii,Ny,1)*(-3/(4*dy*dz))*dQy(ii);
end

%------------ 2: dQ/dy d^2Z/dydq  dW/dz -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zyq(ii,jj,1)*(-3/(2*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zyq(ii,jj,1)*(4/(2*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zyq(ii,jj,1)*(-1/(2*dz))*dQy(ii);
end 
end


%--------------- dZ/dy d/dz ( dQ/dy dW/dQ ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+ alfa2*dQy(ii)*(3/(4*dz*dy))*dZy(ii,jj,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,jj,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+ alfa2*dQy(ii)*(1/(4*dz*dy))*dZy(ii,jj,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+alfa2*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,jj,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+ alfa2*dQy(ii)*(4/(4*dz*dy))*dZy(ii,jj,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZy(ii,jj,1);
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*dQy(ii)*(9/(4*dz*dy))*dZy(ii,1,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+alfa2*dQy(ii)*(-12/(4*dz*dy))*dZy(ii,1,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*dQy(ii)*(3/(4*dz*dy))*dZy(ii,1,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+    alfa2*dQy(ii)*(-12/(4*dz*dy))*dZy(ii,1,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+  alfa2*dQy(ii)*(16/(4*dz*dy))*dZy(ii,1,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,1,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*dQy(ii)*(3/(4*dz*dy))*dZy(ii,1,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+ alfa2*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,1,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*dQy(ii)*(1/(4*dz*dy))*dZy(ii,1,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+alfa2*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,Ny,1);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*dQy(ii)*(12/(4*dz*dy))*dZy(ii,Ny,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+alfa2*dQy(ii)*(-9/(4*dz*dy))*dZy(ii,Ny,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+  alfa2*dQy(ii)*(4/(4*dz*dy))*dZy(ii,Ny,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*dQy(ii)*(-16/(4*dz*dy))*dZy(ii,Ny,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*dQy(ii)*(12/(4*dz*dy))*dZy(ii,Ny,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZy(ii,Ny,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+ alfa2*dQy(ii)*(4/(4*dz*dy))*dZy(ii,Ny,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+alfa2*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,Ny,1);
end


%--------------- dZ/dy d/dz ( dZ/dy dW/dz ) -------------------;

%------------- 1: dZ/dy dZ/dy d/dz ( dW/dz ) -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(2/dz^2)*dZy(ii,jj,1)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*(-5/dz^2)*dZy(ii,jj,1)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*(4/dz^2)*dZy(ii,jj,1)^2;
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)+alfa2*(-1/dz^2)*dZy(ii,jj,1)^2;
end 
end

%--------------- 2: dZ/dy d^2Z/dydz  dW/dz -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*d2Zyz(ii,jj,1)*(-3/(2*dz))*dZy(ii,jj,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*d2Zyz(ii,jj,1)*(4/(2*dz))*dZy(ii,jj,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*d2Zyz(ii,jj,1)*(-1/(2*dz))*dZy(ii,jj,1);
end 
end


%+++++++++ KpE (-1/H) (-3/(2 dz)) ( Fzx dHs/dx + Fzy dHs/dy )^1 ++++++++++;

%---------------- (-1/H) (-3/(2*dz)) dHs/dx dZ/dz dU/dz ----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-3/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(4/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZz(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-1/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZz(ii,jj);
end 
end


%----------------- (-1/H) (-3/(2*dz)) dHs/dx dW/dx --------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-1/(2*dx))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)+     alfa2*(1/(2*dx))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(1/(2*dx))*(-3/(2*dz))*dZz(Nx,jj)*dHsx(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-4/(2*dx))*(-3/(2*dz))*dZz(Nx,jj)*dHsx(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa2*(3/(2*dx))*(-3/(2*dz))*dZz(Nx,jj)*dHsx(Nx,jj); 
end 


%---------------- (-1/H) (-3/(2*dz)) dHs/dx dQ/dx dW/dQ -----------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*(-3/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsx(ii,1)*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+   alfa2*(4/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsx(ii,1)*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsx(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsx(ii,Ny)*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-4/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsx(ii,Ny)*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(3/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsx(ii,Ny)*dQx(ii,Ny); 
end


%-------------- (-1/H) (-3/(2*dz)) dHs/dx dZ/dx dW/dz -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-3/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZx(ii,jj,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(4/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZx(ii,jj,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-1/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj)*dZx(ii,jj,1);
end 
end


%-------------- (-1/H) (-3/(2*dz)) dHs/dy dZ/dz dV/dz -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-3/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(4/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZz(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-1/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZz(ii,jj);
end 
end


%--------------- (-1/H) (-3/(2*dz)) dHs/dy dQ/dy dW/dQ ------------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa2*(-3/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsy(ii,1)*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+   alfa2*(4/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsy(ii,1)*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa2*(-1/(2*dy))*(-3/(2*dz))*dZz(ii,1)*dHsy(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa2*(1/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsy(ii,Ny)*dQy(ii);     
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa2*(-4/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsy(ii,Ny)*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa2*(3/(2*dy))*(-3/(2*dz))*dZz(ii,Ny)*dHsy(ii,Ny)*dQy(ii); 
end


%--------------- (-1/H) (-3/(2*dz)) dHs/dy dZ/dy dW/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*(-3/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZy(ii,jj,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*(4/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZy(ii,jj,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa2*(-1/(2*dz))*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj)*dZy(ii,jj,1);
end 
end



%++++++++++++++++++ KpE (-1/H) 4/(2 dz) ( Fzz )^2 +++++++++++++++++++++;

%----------------  Kp1 (-1/H) 4/(2*dz) dZ/dz dW/dz -------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp1*(-1/(2*dz))*(4/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp1*(1/(2*dz))*(4/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
end 
end


%--------------------  Kp2 (-1/H) 4/(2*dz) dU/dx ------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*(-1/(2*dx))*(4/(2*dz))*dZz(ii,jj);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+2)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+2)+     alfa2*Kp2*(1/(2*dx))*(4/(2*dz))*dZz(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*(1/(2*dx))*(4/(2*dz))*dZz(Nx,jj);    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*(-4/(2*dx))*(4/(2*dz))*dZz(Nx,jj);            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa2*Kp2*(3/(2*dx))*(4/(2*dz))*dZz(Nx,jj);                                                          
end 


%-----------------  Kp2 (-1/H) 4/(2*dz) dQ/dx dU/dQ ------------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*Kp2*(-1/(2*dy))*(4/(2*dz))*dZz(ii,jj)*dQx(ii,jj);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+2)+     alfa2*Kp2*(1/(2*dy))*(4/(2*dz))*dZz(ii,jj)*dQx(ii,jj);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2)+     alfa2*Kp2*(-3/(2*dy))*(4/(2*dz))*dZz(ii,1)*dQx(ii,1);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+2)+   alfa2*Kp2*(4/(2*dy))*(4/(2*dz))*dZz(ii,1)*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+2)+alfa2*Kp2*(-1/(2*dy))*(4/(2*dz))*dZz(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+ alfa2*Kp2*(1/(2*dy))*(4/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*Kp2*(-4/(2*dy))*(4/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*Kp2*(3/(2*dy))*(4/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);                                                          
end


%------------------  Kp2 (-1/H) 4/(2*dz) dZ/dx dU/dz -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*(-1/(2*dz))*(4/(2*dz))*dZz(ii,jj)*dZx(ii,jj,2);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp2*(1/(2*dz))*(4/(2*dz))*dZz(ii,jj)*dZx(ii,jj,2);                                                          
end 
end


%-----------------  Kp2 (-1/H) 4/(2*dz) dQ/dy dV/dQ ------------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+2)+alfa2*Kp2*(-1/(2*dy))*(4/(2*dz))*dZz(ii,jj)*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+2)+     alfa2*Kp2*(1/(2*dy))*(4/(2*dz))*dZz(ii,jj)*dQy(ii);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2)+     alfa2*Kp2*(-3/(2*dy))*(4/(2*dz))*dZz(ii,1)*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+2)+   alfa2*Kp2*(4/(2*dy))*(4/(2*dz))*dZz(ii,1)*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+2)+alfa2*Kp2*(-1/(2*dy))*(4/(2*dz))*dZz(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+2)+ alfa2*Kp2*(1/(2*dy))*(4/(2*dz))*dZz(ii,Ny)*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+2)+alfa2*Kp2*(-4/(2*dy))*(4/(2*dz))*dZz(ii,Ny)*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+2)+ alfa2*Kp2*(3/(2*dy))*(4/(2*dz))*dZz(ii,Ny)*dQy(ii);                                                          
end


%-------------------  Kp2 (-1/H) 4/(2*dz) dZ/dy dV/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa2*Kp2*(-1/(2*dz))*(4/(2*dz))*dZz(ii,jj)*dZy(ii,jj,2);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp2*(1/(2*dz))*(4/(2*dz))*dZz(ii,jj)*dZy(ii,jj,2);                                                          
end 
end



%++++++++++++++++++ KpE (-1/H) (-1/(2 dz)) ( Fzz )^3 +++++++++++++++++++++;

%----------------  Kp1 (-1/H) (-1/(2*dz)) dZ/dz dW/dz -------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp1*(-1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)+ alfa2*Kp1*(1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
end 
end


%--------------------  Kp2 (-1/H) (-1/(2*dz)) dU/dx ------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*(-1/(2*dx))*(-1/(2*dz))*dZz(ii,jj);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+3)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+3)+     alfa2*Kp2*(1/(2*dx))*(-1/(2*dz))*dZz(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp2*(1/(2*dx))*(-1/(2*dz))*dZz(Nx,jj);    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+3)+alfa2*Kp2*(-4/(2*dx))*(-1/(2*dz))*dZz(Nx,jj);            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+3)+ alfa2*Kp2*(3/(2*dx))*(-1/(2*dz))*dZz(Nx,jj);                                                          
end 


%-----------------  Kp2 (-1/H) (-1/(2*dz)) dQ/dx dU/dQ ------------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+3)+alfa2*Kp2*(-1/(2*dy))*(-1/(2*dz))*dZz(ii,jj)*dQx(ii,jj);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+3)+     alfa2*Kp2*(1/(2*dy))*(-1/(2*dz))*dZz(ii,jj)*dQx(ii,jj);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+3)+     alfa2*Kp2*(-3/(2*dy))*(-1/(2*dz))*dZz(ii,1)*dQx(ii,1);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+3)+   alfa2*Kp2*(4/(2*dy))*(-1/(2*dz))*dZz(ii,1)*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+3)+alfa2*Kp2*(-1/(2*dy))*(-1/(2*dz))*dZz(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+ alfa2*Kp2*(1/(2*dy))*(-1/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+alfa2*Kp2*(-4/(2*dy))*(-1/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+ alfa2*Kp2*(3/(2*dy))*(-1/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);                                                          
end


%-----------------  Kp2 (-1/H) (-1/(2*dz)) dZ/dx dU/dz -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*(-1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZx(ii,jj,3);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+4)+ alfa2*Kp2*(1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZx(ii,jj,3);                                                          
end 
end


%-----------------  Kp2 (-1/H) (-1/(2*dz)) dQ/dy dV/dQ ------------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+3)+alfa2*Kp2*(-1/(2*dy))*(-1/(2*dz))*dZz(ii,jj)*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+3)+     alfa2*Kp2*(1/(2*dy))*(-1/(2*dz))*dZz(ii,jj)*dQy(ii);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+3)+     alfa2*Kp2*(-3/(2*dy))*(-1/(2*dz))*dZz(ii,1)*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+3)+   alfa2*Kp2*(4/(2*dy))*(-1/(2*dz))*dZz(ii,1)*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+3)+alfa2*Kp2*(-1/(2*dy))*(-1/(2*dz))*dZz(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+3)+ alfa2*Kp2*(1/(2*dy))*(-1/(2*dz))*dZz(ii,Ny)*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+3)+alfa2*Kp2*(-4/(2*dy))*(-1/(2*dz))*dZz(ii,Ny)*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+3)+ alfa2*Kp2*(3/(2*dy))*(-1/(2*dz))*dZz(ii,Ny)*dQy(ii);                                                          
end


%------------------  Kp2 (-1/H) (-1/(2*dz)) dZ/dy dV/dz -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+alfa2*Kp2*(-1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZy(ii,jj,3);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+4)+ alfa2*Kp2*(1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZy(ii,jj,3);                                                          
end 
end


%+++++++++++++++++++ + KpE dens Freq^2 W ++++++++++++++++++++++;

for ii=2:1:Nx 
for jj=1:1:Ny 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1) + alfa2*KpE*dens*Freq^2;
end 
end



%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++ Basic Boundary Conditions at Ice Shelf Surface +++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++ The first equation (basic equation #1) +++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%++++++++++++++++++ - KpE Fxx dHs/dx ++++++++++++++++++++++;

%----------------- - Kp1 dU/dx dHs/dx -------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)-alfa1*Kp1*(-1/(2*dx))*dHsx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)-     alfa1*Kp1*(1/(2*dx))*dHsx(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)- alfa1*Kp1*(1/(2*dx))*dHsx(Nx,jj);    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)-alfa1*Kp1*(-4/(2*dx))*dHsx(Nx,jj);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)- alfa1*Kp1*(3/(2*dx))*dHsx(Nx,jj);                                                          
end 


%-------------- - Kp1 dQ/dx dU/dQ dHs/dx ---------------;

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)-     alfa1*Kp1*(-3/(2*dy))*dQx(ii,1)*dHsx(ii,1);     
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)-   alfa1*Kp1*(4/(2*dy))*dQx(ii,1)*dHsx(ii,1);            
MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)-alfa1*Kp1*(-1/(2*dy))*dQx(ii,1)*dHsx(ii,1);                                                          
end


%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)-alfa1*Kp1*(-1/(2*dy))*dQx(ii,jj)*dHsx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)-     alfa1*Kp1*(1/(2*dy))*dQx(ii,jj)*dHsx(ii,jj);                                                          
end 
end


%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)- alfa1*Kp1*(1/(2*dy))*dQx(ii,Ny)*dHsx(ii,Ny);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)-alfa1*Kp1*(-4/(2*dy))*dQx(ii,Ny)*dHsx(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)- alfa1*Kp1*(3/(2*dy))*dQx(ii,Ny)*dHsx(ii,Ny);                                                          
end


%-------------- - Kp1 dZ/dx dU/dz dHs/dx ----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)-alfa1*Kp1*(-3/(2*dz))*dZx(ii,jj,1)*dHsx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)- alfa1*Kp1*(4/(2*dz))*dZx(ii,jj,1)*dHsx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)-alfa1*Kp1*(-1/(2*dz))*dZx(ii,jj,1)*dHsx(ii,jj);
end 
end


%--------------- - Kp2 dQ/dy dV/dQ dHs/dx ---------------;

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)-     alfa1*Kp2*(-3/(2*dy))*dQy(ii)*dHsx(ii,1);    
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)-   alfa1*Kp2*(4/(2*dy))*dQy(ii)*dHsx(ii,1);            
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)-alfa1*Kp2*(-1/(2*dy))*dQy(ii)*dHsx(ii,1);                                                          
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)-alfa1*Kp2*(-1/(2*dy))*dQy(ii)*dHsx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)-     alfa1*Kp2*(1/(2*dy))*dQy(ii)*dHsx(ii,jj);                                                          
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)- alfa1*Kp2*(1/(2*dy))*dQy(ii)*dHsx(ii,Ny);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)-alfa1*Kp2*(-4/(2*dy))*dQy(ii)*dHsx(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)- alfa1*Kp2*(3/(2*dy))*dQy(ii)*dHsx(ii,Ny);                                                          
end


%----------- - Kp2 dZ/dy dV/dz dHs/dx --------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)-alfa1*Kp2*(-3/(2*dz))*dZy(ii,jj,1)*dHsx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)- alfa1*Kp2*(4/(2*dz))*dZy(ii,jj,1)*dHsx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)-alfa1*Kp2*(-1/(2*dz))*dZy(ii,jj,1)*dHsx(ii,jj);
end 
end

%---------- - Kp2 dZ/dz dW/dz dHs/dx -------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)-alfa1*Kp2*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)- alfa1*Kp2*(4/(2*dz))*dZz(ii,jj)*dHsx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)-alfa1*Kp2*(-1/(2*dz))*dZz(ii,jj)*dHsx(ii,jj);
end 
end


%++++++++++++++++++ - KpE Fxy dHs/dy ++++++++++++++++++++++;


%---------------- - dQ/dy dU/dQ dHs/dy ----------------;

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+1)-     alfa1*(-3/(2*dy))*dQy(ii)*dHsy(ii,1);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+Nz+1)-   alfa1*(4/(2*dy))*dQy(ii)*dHsy(ii,1);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)-alfa1*(-1/(2*dy))*dQy(ii)*dHsy(ii,1);                                                          
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)-alfa1*(-1/(2*dy))*dQy(ii)*dHsy(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)-     alfa1*(1/(2*dy))*dQy(ii)*dHsy(ii,jj);                                                          
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)- alfa1*(1/(2*dy))*dQy(ii)*dHsy(ii,Ny);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)-alfa1*(-4/(2*dy))*dQy(ii)*dHsy(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)- alfa1*(3/(2*dy))*dQy(ii)*dHsy(ii,Ny);                                                          
end


%------------- - dZ/dy dU/dz dHs/dy ----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)-alfa1*(-3/(2*dz))*dZy(ii,jj,1)*dHsy(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)- alfa1*(4/(2*dz))*dZy(ii,jj,1)*dHsy(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)-alfa1*(-1/(2*dz))*dZy(ii,jj,1)*dHsy(ii,jj);
end 
end


%------------------ - dV/dx dHs/dy -----------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)-alfa1*(-1/(2*dx))*dHsy(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)-     alfa1*(1/(2*dx))*dHsy(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)- alfa1*(1/(2*dx))*dHsy(Nx,jj);    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)-alfa1*(-4/(2*dx))*dHsy(Nx,jj);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)- alfa1*(3/(2*dx))*dHsy(Nx,jj);                                                          
end 


%-------------- - dQ/dx dV/dQ dHs/dy ---------------;

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)-     alfa1*(-3/(2*dy))*dQx(ii,1)*dHsy(ii,1);    
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)-   alfa1*(4/(2*dy))*dQx(ii,1)*dHsy(ii,1);            
MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)-alfa1*(-1/(2*dy))*dQx(ii,1)*dHsy(ii,1);                                                          
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)-alfa1*(-1/(2*dy))*dQx(ii,jj)*dHsy(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)-     alfa1*(1/(2*dy))*dQx(ii,jj)*dHsy(ii,jj);                                                          
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)- alfa1*(1/(2*dy))*dQx(ii,Ny)*dHsy(ii,Ny);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)-alfa1*(-4/(2*dy))*dQx(ii,Ny)*dHsy(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)- alfa1*(3/(2*dy))*dQx(ii,Ny)*dHsy(ii,Ny);                                                          
end


%---------------- - dZ/dx dV/dz dHs/dy ----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)-alfa1*(-3/(2*dz))*dZx(ii,jj,1)*dHsy(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)- alfa1*(4/(2*dz))*dZx(ii,jj,1)*dHsy(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)-alfa1*(-1/(2*dz))*dZx(ii,jj,1)*dHsy(ii,jj);
end 
end



%+++++++++++++++++++++++ KpE Fxz ++++++++++++++++++++++;

%------------------ dZ/dz dU/dz -------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa1*(-3/(2*dz))*dZz(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa1*(4/(2*dz))*dZz(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa1*(-1/(2*dz))*dZz(ii,jj);                                                          
end 
end

%-------------------- dW/dx ------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)+alfa1*(-1/(2*dx));            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)+     alfa1*(1/(2*dx));                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+ alfa1*(1/(2*dx));    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+alfa1*(-4/(2*dx));            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa1*(3/(2*dx));                                                          
end 


%---------------- dQ/dx dW/dQ ------------------;

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa1*(-3/(2*dy))*dQx(ii,1);    
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+   alfa1*(4/(2*dy))*dQx(ii,1);            
MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB((ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa1*(-1/(2*dy))*dQx(ii,1);                                                          
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa1*(-1/(2*dy))*dQx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa1*(1/(2*dy))*dQx(ii,jj);                                                          
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa1*(1/(2*dy))*dQx(ii,Ny);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa1*(-4/(2*dy))*dQx(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa1*(3/(2*dy))*dQx(ii,Ny);                                                          
end


%----------------- dZ/dx dW/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa1*(-3/(2*dz))*dZx(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa1*(4/(2*dz))*dZx(ii,jj,1);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa1*(-1/(2*dz))*dZx(ii,jj,1);                                                          
end 
end




%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++ The second equation (basic equation #2) +++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++++++ - KpE Fyx dHs/dx ++++++++++++++++++++++++;

%-------------------- - dQ/dy dU/dQ dHs/dx -----------------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)-     alfa1*(-3/(2*dy))*dQy(ii)*dHsx(ii,1);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)-   alfa1*(4/(2*dy))*dQy(ii)*dHsx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)-alfa1*(-1/(2*dy))*dQy(ii)*dHsx(ii,1); 
end

%---- 1<j<Ny ----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)-alfa1*(-1/(2*dy))*dQy(ii)*dHsx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)-     alfa1*(1/(2*dy))*dQy(ii)*dHsx(ii,jj); 
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)- alfa1*(1/(2*dy))*dQy(ii)*dHsx(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)-alfa1*(-4/(2*dy))*dQy(ii)*dHsx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)- alfa1*(3/(2*dy))*dQy(ii)*dHsx(ii,Ny); 
end


%---------- - dZ/dy dU/dz dHs/dx -------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)-alfa1*(-3/(2*dz))*dZy(ii,jj,1)*dHsx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)- alfa1*(4/(2*dz))*dZy(ii,jj,1)*dHsx(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)-alfa1*(-1/(2*dz))*dZy(ii,jj,1)*dHsx(ii,jj);
end 
end


%-------------- - dV/dx dHs/dx ----------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)-alfa1*(-1/(2*dx))*dHsx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)-     alfa1*(1/(2*dx))*dHsx(ii,jj); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)- alfa1*(1/(2*dx))*dHsx(Nx,jj);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)-alfa1*(-4/(2*dx))*dHsx(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)- alfa1*(3/(2*dx))*dHsx(Nx,jj); 
end 


%------------ - dQ/dx dV/dQ dHs/dx ---------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)-     alfa1*(-3/(2*dy))*dQx(ii,1)*dHsx(ii,1);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)-   alfa1*(4/(2*dy))*dQx(ii,1)*dHsx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)-alfa1*(-1/(2*dy))*dQx(ii,1)*dHsx(ii,1); 
end

%---- 1<j<Ny ----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)-alfa1*(-1/(2*dy))*dQx(ii,jj)*dHsx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)-     alfa1*(1/(2*dy))*dQx(ii,jj)*dHsx(ii,jj); 
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)- alfa1*(1/(2*dy))*dQx(ii,Ny)*dHsx(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)-alfa1*(-4/(2*dy))*dQx(ii,Ny)*dHsx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)- alfa1*(3/(2*dy))*dQx(ii,Ny)*dHsx(ii,Ny); 
end


%------------- - dZ/dx dV/dz dHs/dx ---------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)-alfa1*(-3/(2*dz))*dZx(ii,jj,1)*dHsx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)- alfa1*(4/(2*dz))*dZx(ii,jj,1)*dHsx(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)-alfa1*(-1/(2*dz))*dZx(ii,jj,1)*dHsx(ii,jj);
end 
end


%+++++++++++++++++++++++ - KpE Fyy dHs/dy ++++++++++++++++++++++++;


%-------------------- - Kp1 dQ/dy dV/dQ dHs/dy -------------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)-     alfa1*Kp1*(-3/(2*dy))*dQy(ii)*dHsy(ii,1);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)-   alfa1*Kp1*(4/(2*dy))*dQy(ii)*dHsy(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)-alfa1*Kp1*(-1/(2*dy))*dQy(ii)*dHsy(ii,1); 
end

%---- 1<j<Ny ----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)-alfa1*Kp1*(-1/(2*dy))*dQy(ii)*dHsy(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)-     alfa1*Kp1*(1/(2*dy))*dQy(ii)*dHsy(ii,jj); 
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)- alfa1*Kp1*(1/(2*dy))*dQy(ii)*dHsy(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)-alfa1*Kp1*(-4/(2*dy))*dQy(ii)*dHsy(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)- alfa1*Kp1*(3/(2*dy))*dQy(ii)*dHsy(ii,Ny); 
end


%------------- - Kp1 dZ/dy dV/dz dHs/dy --------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)-alfa1*Kp1*(-3/(2*dz))*dZy(ii,jj,1)*dHsy(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)- alfa1*Kp1*(4/(2*dz))*dZy(ii,jj,1)*dHsy(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)-alfa1*Kp1*(-1/(2*dz))*dZy(ii,jj,1)*dHsy(ii,jj);
end 
end


%--------------- - Kp2 dU/dx dHs/dy -----------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)-alfa1*Kp2*(-1/(2*dx))*dHsy(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)-     alfa1*Kp2*(1/(2*dx))*dHsy(ii,jj); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)- alfa1*Kp2*(1/(2*dx))*dHsy(Nx,jj);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)-alfa1*Kp2*(-4/(2*dx))*dHsy(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)- alfa1*Kp2*(3/(2*dx))*dHsy(Nx,jj); 
end 


%------------- - Kp2 dQ/dx dU/dQ dHs/dy--------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)-     alfa1*Kp2*(-3/(2*dy))*dQx(ii,1)*dHsy(ii,1);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)-   alfa1*Kp2*(4/(2*dy))*dQx(ii,1)*dHsy(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)-alfa1*Kp2*(-1/(2*dy))*dQx(ii,1)*dHsy(ii,1); 
end

%---- 1<j<Ny ----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)-alfa1*Kp2*(-1/(2*dy))*dQx(ii,jj)*dHsy(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)-     alfa1*Kp2*(1/(2*dy))*dQx(ii,jj)*dHsy(ii,jj); 
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)- alfa1*Kp2*(1/(2*dy))*dQx(ii,Ny)*dHsy(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)-alfa1*Kp2*(-4/(2*dy))*dQx(ii,Ny)*dHsy(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)- alfa1*Kp2*(3/(2*dy))*dQx(ii,Ny)*dHsy(ii,Ny); 
end


%------------ - Kp2 dZ/dx dU/dz dHs/dy --------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)-alfa1*Kp2*(-3/(2*dz))*dZx(ii,jj,1)*dHsy(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)- alfa1*Kp2*(4/(2*dz))*dZx(ii,jj,1)*dHsy(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)-alfa1*Kp2*(-1/(2*dz))*dZx(ii,jj,1)*dHsy(ii,jj);
end 
end


%------------- - Kp2 dZ/dz dW/dz dHs/dy --------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)-alfa1*Kp2*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)- alfa1*Kp2*(4/(2*dz))*dZz(ii,jj)*dHsy(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)-alfa1*Kp2*(-1/(2*dz))*dZz(ii,jj)*dHsy(ii,jj);
end 
end



%+++++++++++++++++++++++ KpE Fyz +++++++++++++++++++++++;

%--------------------- dZ/dz dV/dz --------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa1*(-3/(2*dz))*dZz(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa1*(4/(2*dz))*dZz(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa1*(-1/(2*dz))*dZz(ii,jj); 
end 
end


%------------------- dQ/dy dW/dQ ------------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa1*(-3/(2*dy))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+   alfa1*(4/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa1*(-1/(2*dy))*dQy(ii); 
end

%---- 1<j<Ny ----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa1*(-1/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa1*(1/(2*dy))*dQy(ii); 
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa1*(1/(2*dy))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa1*(-4/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa1*(3/(2*dy))*dQy(ii); 
end


%----------------- dZ/dy dW/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa1*(-3/(2*dz))*dZy(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa1*(4/(2*dz))*dZy(ii,jj,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa1*(-1/(2*dz))*dZy(ii,jj,1); 
end 
end




%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++ The third equation (basic equation #3) ++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%++++++++++++++++++++++++ - KpE Fzx dHs/dx +++++++++++++++++++;

%--------------------- - dZ/dz dU/dz dHs/dx -------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)-alfa1*(-3/(2*dz))*dZz(ii,jj)*dHsx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)- alfa1*(4/(2*dz))*dZz(ii,jj)*dHsx(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)-alfa1*(-1/(2*dz))*dZz(ii,jj)*dHsx(ii,jj);
end 
end


%--------------------- - dW/dx dHs/dx ---------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+1)-alfa1*(-1/(2*dx))*dHsx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+1)-     alfa1*(1/(2*dx))*dHsx(ii,jj); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+1)- alfa1*(1/(2*dx))*dHsx(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+1)-alfa1*(-4/(2*dx))*dHsx(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1)- alfa1*(3/(2*dx))*dHsx(Nx,jj); 
end 


%------------------- - dQ/dx dW/dQ dHs/dx -----------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)-     alfa1*(-3/(2*dy))*dQx(ii,1)*dHsx(ii,1);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)-   alfa1*(4/(2*dy))*dQx(ii,1)*dHsx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)-alfa1*(-1/(2*dy))*dQx(ii,1)*dHsx(ii,1); 
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)-alfa1*(-1/(2*dy))*dQx(ii,jj)*dHsx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)-     alfa1*(1/(2*dy))*dQx(ii,jj)*dHsx(ii,jj); 
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)- alfa1*(1/(2*dy))*dQx(ii,Ny)*dHsx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)-alfa1*(-4/(2*dy))*dQx(ii,Ny)*dHsx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)- alfa1*(3/(2*dy))*dQx(ii,Ny)*dHsx(ii,Ny); 
end


%------------------ - dZ/dx dW/dz dHs/dx -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)-alfa1*(-3/(2*dz))*dZx(ii,jj,1)*dHsx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)- alfa1*(4/(2*dz))*dZx(ii,jj,1)*dHsx(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)-alfa1*(-1/(2*dz))*dZx(ii,jj,1)*dHsx(ii,jj);
end 
end



%+++++++++++++++++++ - KpE Fzy dHs/dy +++++++++++++++++++;


%----------------- - dZ/dz dV/dz dHs/dy -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)-alfa1*(-3/(2*dz))*dZz(ii,jj)*dHsy(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)- alfa1*(4/(2*dz))*dZz(ii,jj)*dHsy(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)-alfa1*(-1/(2*dz))*dZz(ii,jj)*dHsy(ii,jj);
end 
end


%--------------- - dQ/dy dW/dQ dHs/dy ------------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1)-     alfa1*(-3/(2*dy))*dQy(ii)*dHsy(ii,1);     
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)-   alfa1*(4/(2*dy))*dQy(ii)*dHsy(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)-alfa1*(-1/(2*dy))*dQy(ii)*dHsy(ii,1); 
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)-alfa1*(-1/(2*dy))*dQy(ii)*dHsy(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)-     alfa1*(1/(2*dy))*dQy(ii)*dHsy(ii,jj); 
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)- alfa1*(1/(2*dy))*dQy(ii)*dHsy(ii,Ny);     
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)-alfa1*(-4/(2*dy))*dQy(ii)*dHsy(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)- alfa1*(3/(2*dy))*dQy(ii)*dHsy(ii,Ny); 
end


%--------------- - dZ/dy dW/dz dHs/dy ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)-alfa1*(-3/(2*dz))*dZy(ii,jj,1)*dHsy(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)- alfa1*(4/(2*dz))*dZy(ii,jj,1)*dHsy(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)-alfa1*(-1/(2*dz))*dZy(ii,jj,1)*dHsy(ii,jj);
end 
end



%+++++++++++++++++++++ KpE Fzz +++++++++++++++++++++++;

%------------------ Kp1 dZ/dz dW/dz -------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa1*Kp1*(-3/(2*dz))*dZz(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa1*Kp1*(4/(2*dz))*dZz(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa1*Kp1*(-1/(2*dz))*dZz(ii,jj); 
end 
end


%-------------------- Kp2 dU/dx ------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-2)*Ny*Nz+(jj-1)*Nz+1)+alfa1*Kp2*(-1/(2*dx));            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,ii*Ny*Nz+(jj-1)*Nz+1)+     alfa1*Kp2*(1/(2*dx));                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-3)*Ny*Nz+(jj-1)*Nz+1)+ alfa1*Kp2*(1/(2*dx));    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-2)*Ny*Nz+(jj-1)*Nz+1)+alfa1*Kp2*(-4/(2*dx));            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+1,(Nx-1)*Ny*Nz+(jj-1)*Nz+1)+ alfa1*Kp2*(3/(2*dx));                                                          
end 


%-----------------  Kp2 dQ/dx dU/dQ ------------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+1)+     alfa1*Kp2*(-3/(2*dy))*dQx(ii,1);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+Nz+1)+   alfa1*Kp2*(4/(2*dy))*dQx(ii,1);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,(ii-1)*Ny*Nz+2*Nz+1)+alfa1*Kp2*(-1/(2*dy))*dQx(ii,1);                                                          
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa1*Kp2*(-1/(2*dy))*dQx(ii,jj);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+jj*Nz+1)+     alfa1*Kp2*(1/(2*dy))*dQx(ii,jj);                                                          
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa1*Kp2*(1/(2*dy))*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa1*Kp2*(-4/(2*dy))*dQx(ii,Ny);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa1*Kp2*(3/(2*dy))*dQx(ii,Ny);                                                          
end


%------------------ Kp2 dZ/dx dU/dz -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa1*Kp2*(-3/(2*dz))*dZx(ii,jj,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa1*Kp2*(4/(2*dz))*dZx(ii,jj,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa1*Kp2*(-1/(2*dz))*dZx(ii,jj,1);                                                          
end 
end


%-----------------  Kp2 dQ/dy dV/dQ ------------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+1)+     alfa1*Kp2*(-3/(2*dy))*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+1)+   alfa1*Kp2*(4/(2*dy))*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+1)+alfa1*Kp2*(-1/(2*dy))*dQy(ii);                                                          
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+1)+alfa1*Kp2*(-1/(2*dy))*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+1)+     alfa1*Kp2*(1/(2*dy))*dQy(ii);                                                          
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+1)+ alfa1*Kp2*(1/(2*dy))*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+1)+alfa1*Kp2*(-4/(2*dy))*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+1)+ alfa1*Kp2*(3/(2*dy))*dQy(ii);                                                          
end


%-------------------  Kp2 dZ/dy dV/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1)+alfa1*Kp2*(-3/(2*dz))*dZy(ii,jj,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+2)+ alfa1*Kp2*(4/(2*dz))*dZy(ii,jj,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+1,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+3)+alfa1*Kp2*(-1/(2*dz))*dZy(ii,jj,1);                                                          
end 
end






%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++++ Ice-shelf base (k=Nz) +++++++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++++ The first equation ++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++;



%++++++++++++++++++++++ KpE dFxx/dx ++++++++++++++++++++++++;

%------------------- Kp1 d/dx ( dU/dx ) --------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)= alfa2*Kp1*(1/dx^2); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=alfa2*Kp1*(-2/dx^2); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)=     alfa2*Kp1*(1/dx^2);                                        
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-4)*Ny*Nz+(jj-1)*Nz+Nz)=alfa2*Kp1*(-1/dx^2);     
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)= alfa2*Kp1*(4/dx^2); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=alfa2*Kp1*(-5/dx^2); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)= alfa2*Kp1*(2/dx^2);                                        
end 

%----------------- Kp1 d/dx ( dQ/dx dU/dQ ) --------------;

%--------------- 1: Kp1 dQ/dx d/dx ( dU/dQ ) --------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp1*dQx(ii,jj)*(1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp1*dQx(ii,jj)*(-1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-2)*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-2)*Nz+Nz)+   alfa2*Kp1*dQx(ii,jj)*(-1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+jj*Nz+Nz)=        MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+jj*Nz+Nz)+        alfa2*Kp1*dQx(ii,jj)*(1/(4*dx*dy));                                  
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB((ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz)+      alfa2*Kp1*dQx(ii,1)*(3/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz+Nz)+  alfa2*Kp1*dQx(ii,1)*(-4/(4*dx*dy));
MB((ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+2*Nz+Nz)+ alfa2*Kp1*dQx(ii,1)*(1/(4*dx*dy));

MB((ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz)+     alfa2*Kp1*dQx(ii,1)*(-3/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz+Nz)+   alfa2*Kp1*dQx(ii,1)*(4/(4*dx*dy));
MB((ii-1)*Ny*Nz+Nz,ii*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,ii*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*dQx(ii,1)*(-1/(4*dx*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)+alfa2*Kp1*dQx(ii,Ny)*(-1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*Kp1*dQx(ii,Ny)*(4/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*Kp1*dQx(ii,Ny)*(-3/(4*dx*dy));

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-3)*Nz+Nz)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp1*dQx(ii,Ny)*(1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-2)*Nz+Nz)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*dQx(ii,Ny)*(-4/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-1)*Nz+Nz)=    MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp1*dQx(ii,Ny)*(3/(4*dx*dy));                                  
end

%---- i=Nx ----;

MB((Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz)=     MB((Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz)+     alfa2*Kp1*dQx(Nx,1)*(-3/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz+Nz)=  MB((Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz+Nz)+   alfa2*Kp1*dQx(Nx,1)*(4/(4*dx*dy));
MB((Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+2*Nz+Nz)=MB((Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*dQx(Nx,1)*(-1/(4*dx*dy));

MB((Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz)=     MB((Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz)+     alfa2*Kp1*dQx(Nx,1)*(12/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz+Nz)=  MB((Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz+Nz)+ alfa2*Kp1*dQx(Nx,1)*(-16/(4*dx*dy));
MB((Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+2*Nz+Nz)=MB((Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+2*Nz+Nz)+ alfa2*Kp1*dQx(Nx,1)*(4/(4*dx*dy));

MB((Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz)=     MB((Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz)+     alfa2*Kp1*dQx(Nx,1)*(-9/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz+Nz)=  MB((Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp1*dQx(Nx,1)*(12/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+2*Nz+Nz)=MB((Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*dQx(Nx,1)*(-3/(4*dx*dy)); 

for jj=2:1:Ny-1
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp1*dQx(Nx,jj)*(-1/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+jj*Nz+Nz)=    MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp1*dQx(Nx,jj)*(1/(4*dx*dy));
    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp1*dQx(Nx,jj)*(4/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+jj*Nz+Nz)=    MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp1*dQx(Nx,jj)*(-4/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp1*dQx(Nx,jj)*(-3/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+jj*Nz+Nz)=    MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp1*dQx(Nx,jj)*(3/(4*dx*dy));                                  
end 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp1*dQx(Nx,Ny)*(1/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*dQx(Nx,Ny)*(-4/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp1*dQx(Nx,Ny)*(3/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp1*dQx(Nx,Ny)*(-4/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*Kp1*dQx(Nx,Ny)*(16/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*Kp1*dQx(Nx,Ny)*(-12/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*Kp1*dQx(Nx,Ny)*(3/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*dQx(Nx,Ny)*(-12/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*Kp1*dQx(Nx,Ny)*(9/(4*dx*dy));


%----------------- 2: Kp1 d^2Q/dxdx  dU/dQ ----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp1*d2Qxx(ii,jj)*(-1/(2*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp1*d2Qxx(ii,jj)*(1/(2*dy)); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*Kp1*d2Qxx(ii,1)*(-3/(2*dy)); 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*Kp1*d2Qxx(ii,1)*(4/(2*dy));
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*d2Qxx(ii,1)*(-1/(2*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp1*d2Qxx(ii,Ny)*(1/(2*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*d2Qxx(ii,Ny)*(-4/(2*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp1*d2Qxx(ii,Ny)*(3/(2*dy));
end


%--------------------- Kp1 d/dx ( dZ/dx dU/dz ) ----------------;

%------------------- 1: Kp1 dZ/dx d/dx ( dU/dz ) ---------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*Kp1*dZx(ii,jj,Nz)*(-1/(4*dx*dz));     
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*Kp1*dZx(ii,jj,Nz)*(4/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+  alfa2*Kp1*dZx(ii,jj,Nz)*(-3/(4*dx*dz));

MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp1*dZx(ii,jj,Nz)*(1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*dZx(ii,jj,Nz)*(-4/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp1*dZx(ii,jj,Nz)*(3/(4*dx*dz));                                  
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp1*dZx(Nx,jj,Nz)*(1/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*dZx(Nx,jj,Nz)*(-4/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp1*dZx(Nx,jj,Nz)*(3/(4*dx*dz));
    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*Kp1*dZx(Nx,jj,Nz)*(-4/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*dZx(Nx,jj,Nz)*(16/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*Kp1*dZx(Nx,jj,Nz)*(-12/(4*dx*dz));

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+  alfa2*Kp1*dZx(Nx,jj,Nz)*(3/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*dZx(Nx,jj,Nz)*(-12/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+    alfa2*Kp1*dZx(Nx,jj,Nz)*(9/(4*dx*dz));                                  
end

%------------------ 2: Kp1 d^2Z/dxdx  dU/dz --------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp1*d2Zxx(ii,jj,Nz)*(1/(2*dz));     
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*d2Zxx(ii,jj,Nz)*(-4/(2*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp1*d2Zxx(ii,jj,Nz)*(3/(2*dz));
end 
end


%-------------------- Kp1 dQ/dx d/dQ ( dU/dx ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp1*(1/(4*dy*dx))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-2)*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-2)*Nz+Nz)+   alfa2*Kp1*(-1/(4*dy*dx))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp1*(-1/(4*dy*dx))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+jj*Nz+Nz)=        MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+jj*Nz+Nz)+        alfa2*Kp1*(1/(4*dy*dx))*dQx(ii,jj);                                                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB((ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz)+     alfa2*Kp1*(3/(4*dy*dx))*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz+Nz)+ alfa2*Kp1*(-4/(4*dy*dx))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*(1/(4*dy*dx))*dQx(ii,1); 

MB((ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz)+     alfa2*Kp1*(-3/(4*dy*dx))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz+Nz)+   alfa2*Kp1*(4/(4*dy*dx))*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,ii*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,ii*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*(-1/(4*dy*dx))*dQx(ii,1);
end

%----- j=Ny ----;

for ii=2:1:Nx-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)+alfa2*Kp1*(-1/(4*dy*dx))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*Kp1*(4/(4*dy*dx))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*Kp1*(-3/(4*dy*dx))*dQx(ii,Ny); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp1*(1/(4*dy*dx))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*(-4/(4*dy*dx))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp1*(3/(4*dy*dx))*dQx(ii,Ny);                                                                                          
end

%---- i=Nx ----;

MB((Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz)=     MB((Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz)+     alfa2*Kp1*(-3/(4*dy*dx))*dQx(Nx,1);
MB((Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz+Nz)=  MB((Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz+Nz)+   alfa2*Kp1*(4/(4*dy*dx))*dQx(Nx,1); 
MB((Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+2*Nz+Nz)=MB((Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*(-1/(4*dy*dx))*dQx(Nx,1); 

MB((Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz)=     MB((Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz)+    alfa2*Kp1*(12/(4*dy*dx))*dQx(Nx,1);
MB((Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz+Nz)=  MB((Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz+Nz)+alfa2*Kp1*(-16/(4*dy*dx))*dQx(Nx,1); 
MB((Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+2*Nz+Nz)=MB((Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*(4/(4*dy*dx))*dQx(Nx,1); 

MB((Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz)=     MB((Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz)+     alfa2*Kp1*(-9/(4*dy*dx))*dQx(Nx,1); 
MB((Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz+Nz)=  MB((Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp1*(12/(4*dy*dx))*dQx(Nx,1);
MB((Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+2*Nz+Nz)=MB((Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*(-3/(4*dy*dx))*dQx(Nx,1);


for jj=2:1:Ny-1
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp1*(-1/(4*dy*dx))*dQx(Nx,jj);     
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*Kp1*(4/(4*dy*dx))*dQx(Nx,jj); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp1*(-3/(4*dy*dx))*dQx(Nx,jj);

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+jj*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+jj*Nz+Nz)+ alfa2*Kp1*(1/(4*dy*dx))*dQx(Nx,jj); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+jj*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+jj*Nz+Nz)+alfa2*Kp1*(-4/(4*dy*dx))*dQx(Nx,jj); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+jj*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+jj*Nz+Nz)+ alfa2*Kp1*(3/(4*dy*dx))*dQx(Nx,jj);                                                                                          
end


MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp1*(1/(4*dy*dx))*dQx(Nx,Ny);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp1*(3/(4*dy*dx))*dQx(Nx,Ny); 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp1*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*Kp1*(16/(4*dy*dx))*dQx(Nx,Ny);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*Kp1*(-12/(4*dy*dx))*dQx(Nx,Ny); 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*Kp1*(3/(4*dy*dx))*dQx(Nx,Ny);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*(-12/(4*dy*dx))*dQx(Nx,Ny);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*Kp1*(9/(4*dy*dx))*dQx(Nx,Ny);


%---------------- Kp1 dQ/dx d/dQ ( dQ/dx dU/dQ ) ------------------;

%-------------- 1: Kp1 dQ/dx dQ/dx d/dQ ( dU/dQ ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*Kp1*(1/dy^2)*dQx(ii,jj)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*Kp1*(-2/dy^2)*dQx(ii,jj)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp1*(1/dy^2)*dQx(ii,jj)^2;                                                                             
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+      alfa2*Kp1*(2/dy^2)*dQx(ii,1)^2; 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp1*(-5/dy^2)*dQx(ii,1)^2; 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+ alfa2*Kp1*(4/dy^2)*dQx(ii,1)^2;
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+3*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+3*Nz+Nz)+alfa2*Kp1*(-1/dy^2)*dQx(ii,1)^2;
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)+alfa2*Kp1*(-1/dy^2)*dQx(ii,Ny)^2;     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp1*(4/dy^2)*dQx(ii,Ny)^2; 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*(-5/dy^2)*dQx(ii,Ny)^2; 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp1*(2/dy^2)*dQx(ii,Ny)^2;                                                                                          
end

%-------------- 2: Kp1 dQ/dx d^2Q/dxdq dU/dQ -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp1*(-1/(2*dy))*d2Qxq(ii,jj)*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp1*(1/(2*dy))*d2Qxq(ii,jj)*dQx(ii,jj);                                                                             
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*Kp1*(-3/(2*dy))*d2Qxq(ii,1)*dQx(ii,1); 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*Kp1*(4/(2*dy))*d2Qxq(ii,1)*dQx(ii,1); 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*(-1/(2*dy))*d2Qxq(ii,1)*dQx(ii,1);

end

%----- j=Ny -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp1*(1/(2*dy))*d2Qxq(ii,Ny)*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*(-4/(2*dy))*d2Qxq(ii,Ny)*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp1*(3/(2*dy))*d2Qxq(ii,Ny)*dQx(ii,Ny);                                                                                          
end


%---------------- Kp1 dQ/dx d/dQ ( dZ/dx dU/dz ) ------------------;

%-------------- 1: Kp1 dQ/dx dZ/dx d/dQ ( dU/dz ) ------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*Kp1*dZx(ii,jj,Nz)*(-1/(4*dy*dz))*dQx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*Kp1*dZx(ii,jj,Nz)*(4/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*Kp1*dZx(ii,jj,Nz)*(-3/(4*dy*dz))*dQx(ii,jj);

MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*Kp1*dZx(ii,jj,Nz)*(1/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*Kp1*dZx(ii,jj,Nz)*(-4/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp1*dZx(ii,jj,Nz)*(3/(4*dy*dz))*dQx(ii,jj);                                                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)+     alfa2*Kp1*dZx(ii,1,Nz)*(-3/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*Kp1*dZx(ii,1,Nz)*(4/(4*dy*dz))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*Kp1*dZx(ii,1,Nz)*(-1/(4*dy*dz))*dQx(ii,1); 
    
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)+    alfa2*Kp1*dZx(ii,1,Nz)*(12/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*Kp1*dZx(ii,1,Nz)*(-16/(4*dy*dz))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*Kp1*dZx(ii,1,Nz)*(4/(4*dy*dz))*dQx(ii,1); 

MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*Kp1*dZx(ii,1,Nz)*(-9/(4*dy*dz))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp1*dZx(ii,1,Nz)*(12/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*dZx(ii,1,Nz)*(-3/(4*dy*dz))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*Kp1*dZx(ii,Ny,Nz)*(1/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*Kp1*dZx(ii,Ny,Nz)*(-4/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*Kp1*dZx(ii,Ny,Nz)*(3/(4*dy*dz))*dQx(ii,Ny); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*Kp1*dZx(ii,Ny,Nz)*(-4/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*Kp1*dZx(ii,Ny,Nz)*(16/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*Kp1*dZx(ii,Ny,Nz)*(-12/(4*dy*dz))*dQx(ii,Ny); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*Kp1*dZx(ii,Ny,Nz)*(3/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*dZx(ii,Ny,Nz)*(-12/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*Kp1*dZx(ii,Ny,Nz)*(9/(4*dy*dz))*dQx(ii,Ny);                                                                                          
end

%-------------- 2: Kp1 dQ/dx d^2Z/dxdq dU/dz ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp1*d2Zxq(ii,jj,Nz)*(1/(2*dz))*dQx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*d2Zxq(ii,jj,Nz)*(-4/(2*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp1*d2Zxq(ii,jj,Nz)*(3/(2*dz))*dQx(ii,jj);
end 
end


%------------------- Kp1 dZ/dx d/dz ( dU/dx ) -------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*Kp1*(-1/(4*dz*dx))*dZx(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*Kp1*(4/(4*dz*dx))*dZx(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+  alfa2*Kp1*(-3/(4*dz*dx))*dZx(ii,jj,Nz); 

MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp1*(1/(4*dz*dx))*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*(-4/(4*dz*dx))*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp1*(3/(4*dz*dx))*dZx(ii,jj,Nz);                                                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp1*(1/(4*dz*dx))*dZx(Nx,jj,Nz);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*(-4/(4*dz*dx))*dZx(Nx,jj,Nz);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp1*(3/(4*dz*dx))*dZx(Nx,jj,Nz);     

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*Kp1*(-4/(4*dz*dx))*dZx(Nx,jj,Nz);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*(16/(4*dz*dx))*dZx(Nx,jj,Nz);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*Kp1*(-12/(4*dz*dx))*dZx(Nx,jj,Nz); 

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+  alfa2*Kp1*(3/(4*dz*dx))*dZx(Nx,jj,Nz); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*(-12/(4*dz*dx))*dZx(Nx,jj,Nz); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+    alfa2*Kp1*(9/(4*dz*dx))*dZx(Nx,jj,Nz);                                                                                          
end 


%--------------- Kp1 dZ/dx d/dz ( dQ/dx dU/dQ ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*Kp1*dQx(ii,jj)*(-1/(4*dz*dy))*dZx(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*Kp1*dQx(ii,jj)*(4/(4*dz*dy))*dZx(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*Kp1*dQx(ii,jj)*(-3/(4*dz*dy))*dZx(ii,jj,Nz); 

MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*Kp1*dQx(ii,jj)*(1/(4*dz*dy))*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*Kp1*dQx(ii,jj)*(-4/(4*dz*dy))*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp1*dQx(ii,jj)*(3/(4*dz*dy))*dZx(ii,jj,Nz);                                                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)+     alfa2*Kp1*dQx(ii,1)*(-3/(4*dz*dy))*dZx(ii,1,Nz); 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*Kp1*dQx(ii,1)*(4/(4*dz*dy))*dZx(ii,1,Nz);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*Kp1*dQx(ii,1)*(-1/(4*dz*dy))*dZx(ii,1,Nz);    

MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)+    alfa2*Kp1*dQx(ii,1)*(12/(4*dz*dy))*dZx(ii,1,Nz); 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*Kp1*dQx(ii,1)*(-16/(4*dz*dy))*dZx(ii,1,Nz);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*Kp1*dQx(ii,1)*(4/(4*dz*dy))*dZx(ii,1,Nz);

MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*Kp1*dQx(ii,1)*(-9/(4*dz*dy))*dZx(ii,1,Nz); 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp1*dQx(ii,1)*(12/(4*dz*dy))*dZx(ii,1,Nz);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*dQx(ii,1)*(-3/(4*dz*dy))*dZx(ii,1,Nz);

end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*Kp1*dQx(ii,Ny)*(1/(4*dz*dy))*dZx(ii,Ny,Nz);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*Kp1*dQx(ii,Ny)*(-4/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*Kp1*dQx(ii,Ny)*(3/(4*dz*dy))*dZx(ii,Ny,Nz);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*Kp1*dQx(ii,Ny)*(-4/(4*dz*dy))*dZx(ii,Ny,Nz);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*Kp1*dQx(ii,Ny)*(16/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*Kp1*dQx(ii,Ny)*(-12/(4*dz*dy))*dZx(ii,Ny,Nz);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*Kp1*dQx(ii,Ny)*(3/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*dQx(ii,Ny)*(-12/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*Kp1*dQx(ii,Ny)*(9/(4*dz*dy))*dZx(ii,Ny,Nz);                                                                                          
end


%----------------- Kp1 dZ/dx d/dz ( dZ/dx dU/dz ) -------------------;

%-------------- 1: Kp1 dZ/dx dZ/dx d/dz ( dU/dz ) -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*Kp1*(-1/dz^2)*dZx(ii,jj,Nz)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp1*(4/dz^2)*dZx(ii,jj,Nz)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*(-5/dz^2)*dZx(ii,jj,Nz)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp1*(2/dz^2)*dZx(ii,jj,Nz)^2;                                                                              
end 
end

%---------------- 2: Kp1 dZ/dx d^2Z/dxdz  dU/dz -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp1*d2Zxz(ii,jj,Nz)*(1/(2*dz))*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*d2Zxz(ii,jj,Nz)*(-4/(2*dz))*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp1*d2Zxz(ii,jj,Nz)*(3/(2*dz))*dZx(ii,jj,Nz);                                                                              
end 
end


%------------------- Kp2 d/dx ( dQ/dy dV/dQ ) -------------------;

%----------------- 1: Kp2 dQ/dy d/dx ( dV/dQ ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp2*dQy(ii)*(1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp2*dQy(ii)*(-1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+Nz)+   alfa2*Kp2*dQy(ii)*(-1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+Nz)=        MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+Nz)+        alfa2*Kp2*dQy(ii)*(1/(4*dx*dy));
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz)+     alfa2*Kp2*dQy(ii)*(3/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+Nz)+ alfa2*Kp2*dQy(ii)*(-4/(4*dx*dy));
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*dQy(ii)*(1/(4*dx*dy));

MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz)+     alfa2*Kp2*dQy(ii)*(-3/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz+Nz)+   alfa2*Kp2*dQy(ii)*(4/(4*dx*dy));
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*dQy(ii)*(-1/(4*dx*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx-1
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)+alfa2*Kp2*dQy(ii)*(-1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*Kp2*dQy(ii)*(4/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*Kp2*dQy(ii)*(-3/(4*dx*dy));

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp2*dQy(ii)*(1/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*dQy(ii)*(-4/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp2*dQy(ii)*(3/(4*dx*dy));                                  
end

%---- i=Nx ----;

MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz)=     MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz)+     alfa2*Kp2*dQy(Nx)*(-3/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+Nz)=  MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+Nz)+   alfa2*Kp2*dQy(Nx)*(4/(4*dx*dy));
MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+Nz)=MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*dQy(Nx)*(-1/(4*dx*dy));

MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz)=     MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz)+    alfa2*Kp2*dQy(Nx)*(12/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+Nz)=  MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+Nz)+alfa2*Kp2*dQy(Nx)*(-16/(4*dx*dy));
MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+Nz)=MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*dQy(Nx)*(4/(4*dx*dy));

MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz)=     MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz)+     alfa2*Kp2*dQy(Nx)*(-9/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+Nz)=  MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp2*dQy(Nx)*(12/(4*dx*dy));
MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+Nz)=MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*dQy(Nx)*(-3/(4*dx*dy));

for jj=2:1:Ny-1
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp2*dQy(Nx)*(-1/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+Nz)=    MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp2*dQy(Nx)*(1/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp2*dQy(Nx)*(4/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+Nz)=    MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp2*dQy(Nx)*(-4/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp2*dQy(Nx)*(-3/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+Nz)=    MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp2*dQy(Nx)*(3/(4*dx*dy));                                  
end 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp2*dQy(Nx)*(1/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*dQy(Nx)*(-4/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp2*dQy(Nx)*(3/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp2*dQy(Nx)*(-4/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*Kp2*dQy(Nx)*(16/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*Kp2*dQy(Nx)*(-12/(4*dx*dy));

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*Kp2*dQy(Nx)*(3/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*dQy(Nx)*(-12/(4*dx*dy)); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*Kp2*dQy(Nx)*(9/(4*dx*dy));


%------------------ 2: Kp2 d^2Q/dydx dV/dQ --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp2*d2Qyx(ii)*(-1/(2*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp2*d2Qyx(ii)*(1/(2*dy)); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*Kp2*d2Qyx(ii)*(-3/(2*dy)); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*Kp2*d2Qyx(ii)*(4/(2*dy));
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*d2Qyx(ii)*(-1/(2*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp2*d2Qyx(ii)*(1/(2*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*d2Qyx(ii)*(-4/(2*dy)); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp2*d2Qyx(ii)*(3/(2*dy));
end


%------------------ Kp2 d/dx ( dZ/dy dV/dz ) -------------------;

%--------------- 1: Kp2 dZ/dy d/dx ( dV/dz ) -------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*Kp2*dZy(ii,jj,Nz)*(-1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*Kp2*dZy(ii,jj,Nz)*(4/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+  alfa2*Kp2*dZy(ii,jj,Nz)*(-3/(4*dx*dz));

MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*dZy(ii,jj,Nz)*(1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*dZy(ii,jj,Nz)*(-4/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*dZy(ii,jj,Nz)*(3/(4*dx*dz));                                   
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*dZy(Nx,jj,Nz)*(1/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*dZy(Nx,jj,Nz)*(-4/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*dZy(Nx,jj,Nz)*(3/(4*dx*dz));

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*Kp2*dZy(Nx,jj,Nz)*(-4/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*dZy(Nx,jj,Nz)*(16/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*Kp2*dZy(Nx,jj,Nz)*(-12/(4*dx*dz));

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+  alfa2*Kp2*dZy(Nx,jj,Nz)*(3/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*dZy(Nx,jj,Nz)*(-12/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+    alfa2*Kp2*dZy(Nx,jj,Nz)*(9/(4*dx*dz));                                  
end

%--------------- 2: Kp2 d^2Z/dydx  dV/dz -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*d2Zyx(ii,jj,Nz)*(1/(2*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*d2Zyx(ii,jj,Nz)*(-4/(2*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*d2Zyx(ii,jj,Nz)*(3/(2*dz));
end 
end


%------------------- Kp2 dQ/dx d/dQ ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*Kp2*dQy(ii)*(1/dy^2)*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*Kp2*dQy(ii)*(-2/dy^2)*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp2*dQy(ii)*(1/dy^2)*dQx(ii,jj);                                                                             
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+      alfa2*Kp2*dQy(ii)*(2/dy^2)*dQx(ii,1); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp2*dQy(ii)*(-5/dy^2)*dQx(ii,1); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+ alfa2*Kp2*dQy(ii)*(4/dy^2)*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+Nz)+alfa2*Kp2*dQy(ii)*(-1/dy^2)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)+alfa2*Kp2*dQy(ii)*(-1/dy^2)*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp2*dQy(ii)*(4/dy^2)*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*dQy(ii)*(-5/dy^2)*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp2*dQy(ii)*(2/dy^2)*dQx(ii,Ny);                                                                                          
end


%------------------- Kp2 dQ/dx d/dQ ( dZ/dy dV/dz ) -------------------;

%----------------- 1: Kp2 dQ/dx dZ/dy d/dQ ( dV/dz ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*Kp2*dZy(ii,jj,Nz)*(-1/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*Kp2*dZy(ii,jj,Nz)*(4/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*Kp2*dZy(ii,jj,Nz)*(-3/(4*dy*dz))*dQx(ii,jj);

MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*Kp2*dZy(ii,jj,Nz)*(1/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*Kp2*dZy(ii,jj,Nz)*(-4/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp2*dZy(ii,jj,Nz)*(3/(4*dy*dz))*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*Kp2*dZy(ii,1,Nz)*(-3/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*Kp2*dZy(ii,1,Nz)*(4/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*Kp2*dZy(ii,1,Nz)*(-1/(4*dy*dz))*dQx(ii,1);

MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+    alfa2*Kp2*dZy(ii,1,Nz)*(12/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*Kp2*dZy(ii,1,Nz)*(-16/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*Kp2*dZy(ii,1,Nz)*(4/(4*dy*dz))*dQx(ii,1);

MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*Kp2*dZy(ii,1,Nz)*(-9/(4*dy*dz))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp2*dZy(ii,1,Nz)*(12/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*dZy(ii,1,Nz)*(-3/(4*dy*dz))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*Kp2*dZy(ii,Ny,Nz)*(1/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*Kp2*dZy(ii,Ny,Nz)*(-4/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*Kp2*dZy(ii,Ny,Nz)*(3/(4*dy*dz))*dQx(ii,Ny); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*Kp2*dZy(ii,Ny,Nz)*(-4/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*Kp2*dZy(ii,Ny,Nz)*(16/(4*dy*dz))*dQx(ii,Ny); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*Kp2*dZy(ii,Ny,Nz)*(-12/(4*dy*dz))*dQx(ii,Ny); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*Kp2*dZy(ii,Ny,Nz)*(3/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*dZy(ii,Ny,Nz)*(-12/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*Kp2*dZy(ii,Ny,Nz)*(9/(4*dy*dz))*dQx(ii,Ny);                                                                                          
end

%----------------- 2: Kp2 dQ/dx d^2Z/dydq  dV/dz -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*d2Zyq(ii,jj,Nz)*(1/(2*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*d2Zyq(ii,jj,Nz)*(-4/(2*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*d2Zyq(ii,jj,Nz)*(3/(2*dz))*dQx(ii,jj);
end 
end


%------------------- Kp2 dZ/dx d/dz ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*Kp2*dQy(ii)*(-1/(4*dz*dy))*dZx(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*Kp2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*Kp2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,jj,Nz); 

MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*Kp2*dQy(ii)*(1/(4*dz*dy))*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*Kp2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,jj,Nz); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*Kp2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,1,Nz); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*Kp2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,1,Nz);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*Kp2*dQy(ii)*(-1/(4*dz*dy))*dZx(ii,1,Nz);

MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+    alfa2*Kp2*dQy(ii)*(12/(4*dz*dy))*dZx(ii,1,Nz); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*Kp2*dQy(ii)*(-16/(4*dz*dy))*dZx(ii,1,Nz);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*Kp2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,1,Nz);

MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*Kp2*dQy(ii)*(-9/(4*dz*dy))*dZx(ii,1,Nz); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp2*dQy(ii)*(12/(4*dz*dy))*dZx(ii,1,Nz);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,1,Nz);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*Kp2*dQy(ii)*(1/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*Kp2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*Kp2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,Ny,Nz);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*Kp2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*Kp2*dQy(ii)*(16/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*Kp2*dQy(ii)*(-12/(4*dz*dy))*dZx(ii,Ny,Nz);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*Kp2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*dQy(ii)*(-12/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*Kp2*dQy(ii)*(9/(4*dz*dy))*dZx(ii,Ny,Nz);                                                                                          
end


%--------------- Kp2 dZ/dx d/dz ( dZ/dy dV/dz ) -----------------;

%------------- 1: Kp2 dZ/dx dZ/dy d/dz ( dV/dz ) -----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*Kp2*dZy(ii,jj,Nz)*(-1/dz^2)*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*dZy(ii,jj,Nz)*(4/dz^2)*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*dZy(ii,jj,Nz)*(-5/dz^2)*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*dZy(ii,jj,Nz)*(2/dz^2)*dZx(ii,jj,Nz);                                                                              
end 
end

%------------- 2: Kp2 dZ/dx d^2Z/dydz  dV/dz -----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*d2Zyz(ii,jj,Nz)*(1/(2*dz))*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*d2Zyz(ii,jj,Nz)*(-4/(2*dz))*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*d2Zyz(ii,jj,Nz)*(3/(2*dz))*dZx(ii,jj,Nz);                                                                              
end 
end


%------------------ Kp2 d/dx ( dZ/dz dW/dz ) -------------------;

%---------------- 1: Kp2 dZ/dz d/dx ( dW/dz ) -------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*Kp2*dZz(ii,jj)*(-1/(4*dx*dz));     
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*Kp2*dZz(ii,jj)*(4/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+  alfa2*Kp2*dZz(ii,jj)*(-3/(4*dx*dz));

MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*dZz(ii,jj)*(1/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*dZz(ii,jj)*(-4/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*dZz(ii,jj)*(3/(4*dx*dz));                                  
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*dZz(Nx,jj)*(1/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*dZz(Nx,jj)*(-4/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*dZz(Nx,jj)*(3/(4*dx*dz));

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*Kp2*dZz(Nx,jj)*(-4/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*dZz(Nx,jj)*(16/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*Kp2*dZz(Nx,jj)*(-12/(4*dx*dz));

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+  alfa2*Kp2*dZz(Nx,jj)*(3/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*dZz(Nx,jj)*(-12/(4*dx*dz)); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+    alfa2*Kp2*dZz(Nx,jj)*(9/(4*dx*dz));                                  
end

%---------------- 2: Kp2 d^2Z/dzdx  dW/dz -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*d2Zzx(ii,jj)*(1/(2*dz));     
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*d2Zzx(ii,jj)*(-4/(2*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*d2Zzx(ii,jj)*(3/(2*dz));
end 
end


%---------------- Kp2 dQ/dx d/dQ ( dZ/dz dW/dz ) ------------------;

%------------- 1: Kp2 dQ/dx dZ/dz d/dQ ( dW/dz ) ------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*Kp2*dZz(ii,jj)*(-1/(4*dy*dz))*dQx(ii,jj);     
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*Kp2*dZz(ii,jj)*(4/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*Kp2*dZz(ii,jj)*(-3/(4*dy*dz))*dQx(ii,jj);

MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*Kp2*dZz(ii,jj)*(1/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*Kp2*dZz(ii,jj)*(-4/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp2*dZz(ii,jj)*(3/(4*dy*dz))*dQx(ii,jj);                                                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*Kp2*dZz(ii,1)*(-3/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*Kp2*dZz(ii,1)*(4/(4*dy*dz))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*Kp2*dZz(ii,1)*(-1/(4*dy*dz))*dQx(ii,1); 


MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+    alfa2*Kp2*dZz(ii,1)*(12/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*Kp2*dZz(ii,1)*(-16/(4*dy*dz))*dQx(ii,1); 
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*Kp2*dZz(ii,1)*(4/(4*dy*dz))*dQx(ii,1); 

MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*Kp2*dZz(ii,1)*(-9/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp2*dZz(ii,1)*(12/(4*dy*dz))*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*dZz(ii,1)*(-3/(4*dy*dz))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*Kp2*dZz(ii,Ny)*(1/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*Kp2*dZz(ii,Ny)*(-4/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*Kp2*dZz(ii,Ny)*(3/(4*dy*dz))*dQx(ii,Ny); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*Kp2*dZz(ii,Ny)*(-4/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*Kp2*dZz(ii,Ny)*(16/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*Kp2*dZz(ii,Ny)*(-12/(4*dy*dz))*dQx(ii,Ny); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*Kp2*dZz(ii,Ny)*(3/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*dZz(ii,Ny)*(-12/(4*dy*dz))*dQx(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*Kp2*dZz(ii,Ny)*(9/(4*dy*dz))*dQx(ii,Ny);                                                                                          
end

%------------- 2: Kp2 dQ/dx d^2Z/dzdq  dW/dz ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*dZz(ii,jj)*(1/(2*dz))*dQx(ii,jj);     
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*dZz(ii,jj)*(-4/(2*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*dZz(ii,jj)*(3/(2*dz))*dQx(ii,jj);
end 
end


%---------------- Kp2 dZ/dx d/dz ( dZ/dz dW/dz ) ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*Kp2*dZz(ii,jj)*(-1/dz^2)*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*dZz(ii,jj)*(4/dz^2)*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*dZz(ii,jj)*(-5/dz^2)*dZx(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*dZz(ii,jj)*(2/dz^2)*dZx(ii,jj,Nz);                                                                              
end 
end



%++++++++++++++++++++++++ KpE dFxy/dy ++++++++++++++++++++++++++++;

%---------------- dQ/dy d/dQ ( dQ/dy dU/dQ ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*(1/dy^2)*dQy(ii)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(-2/dy^2)*dQy(ii)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*(1/dy^2)*dQy(ii)^2;                                                                             
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+      alfa2*(2/dy^2)*dQy(ii)^2; 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*(-5/dy^2)*dQy(ii)^2; 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+ alfa2*(4/dy^2)*dQy(ii)^2; 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+3*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+3*Nz+Nz)+alfa2*(-1/dy^2)*dQy(ii)^2; 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)+alfa2*(-1/dy^2)*dQy(ii)^2; 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(4/dy^2)*dQy(ii)^2; 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-5/dy^2)*dQy(ii)^2; 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(2/dy^2)*dQy(ii)^2;                                                                                          
end


%----------------- dQ/dy d/dQ ( dZ/dy dU/dz ) -----------------;

%--------------- 1: dQ/dy dZ/dy d/dQ ( dU/dz ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*dZy(ii,jj,Nz)*(-1/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*dZy(ii,jj,Nz)*(4/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*dZy(ii,jj,Nz)*(-3/(4*dy*dz))*dQy(ii);

MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*dZy(ii,jj,Nz)*(1/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*dZy(ii,jj,Nz)*(-4/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*dZy(ii,jj,Nz)*(3/(4*dy*dz))*dQy(ii);                                                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)+     alfa2*dZy(ii,1,Nz)*(-3/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*dZy(ii,1,Nz)*(4/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*dZy(ii,1,Nz)*(-1/(4*dy*dz))*dQy(ii); 

MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)+    alfa2*dZy(ii,1,Nz)*(12/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*dZy(ii,1,Nz)*(-16/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*dZy(ii,1,Nz)*(4/(4*dy*dz))*dQy(ii); 

MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*dZy(ii,1,Nz)*(-9/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dZy(ii,1,Nz)*(12/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*dZy(ii,1,Nz)*(-3/(4*dy*dz))*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*dZy(ii,Ny,Nz)*(1/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*dZy(ii,Ny,Nz)*(-4/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*dZy(ii,Ny,Nz)*(3/(4*dy*dz))*dQy(ii); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*dZy(ii,Ny,Nz)*(-4/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*dZy(ii,Ny,Nz)*(16/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*dZy(ii,Ny,Nz)*(-12/(4*dy*dz))*dQy(ii); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dZy(ii,Ny,Nz)*(3/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dZy(ii,Ny,Nz)*(-12/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dZy(ii,Ny,Nz)*(9/(4*dy*dz))*dQy(ii);                                                                                          
end

%--------------- 2: dQ/dy d^2Z/dydq  dU/dz -----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zyq(ii,jj,Nz)*(1/(2*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zyq(ii,jj,Nz)*(-4/(2*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zyq(ii,jj,Nz)*(3/(2*dz))*dQy(ii);
end 
end


%--------------- dZ/dy d/dz ( dQ/dy dU/dQ ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZy(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*dQy(ii)*(4/(4*dz*dy))*dZy(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,jj,Nz); 

MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*dQy(ii)*(1/(4*dz*dy))*dZy(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*dQy(ii)*(3/(4*dz*dy))*dZy(ii,jj,Nz);                                                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)+     alfa2*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,1,Nz); 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*dQy(ii)*(4/(4*dz*dy))*dZy(ii,1,Nz);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZy(ii,1,Nz);

MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)+    alfa2*dQy(ii)*(12/(4*dz*dy))*dZy(ii,1,Nz); 
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*dQy(ii)*(-16/(4*dz*dy))*dZy(ii,1,Nz);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*dQy(ii)*(4/(4*dz*dy))*dZy(ii,1,Nz);

MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*dQy(ii)*(-9/(4*dz*dy))*dZy(ii,1,Nz);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dQy(ii)*(12/(4*dz*dy))*dZy(ii,1,Nz);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,1,Nz);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*dQy(ii)*(1/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*dQy(ii)*(3/(4*dz*dy))*dZy(ii,Ny,Nz);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*dQy(ii)*(16/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*dQy(ii)*(-12/(4*dz*dy))*dZy(ii,Ny,Nz);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dQy(ii)*(3/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQy(ii)*(-12/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dQy(ii)*(9/(4*dz*dy))*dZy(ii,Ny,Nz);                                                                                          
end


%-------------- dZ/dy d/dz ( dZ/dy dU/dz ) -----------------;

%------------ 1: dZ/dy dZ/dy d/dz ( dU/dz ) ----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*(-1/dz^2)*dZy(ii,jj,Nz)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(4/dz^2)*dZy(ii,jj,Nz)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-5/dz^2)*dZy(ii,jj,Nz)^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(2/dz^2)*dZy(ii,jj,Nz)^2;                                                                              
end 
end

%------------ 2: dZ/dy d^2Z/dydz  dU/dz ----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zyz(ii,jj,Nz)*(1/(2*dz))*dZy(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zyz(ii,jj,Nz)*(-4/(2*dz))*dZy(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zyz(ii,jj,Nz)*(3/(2*dz))*dZy(ii,jj,Nz);                                                                              
end 
end


%----------------- dQ/dy d/dQ ( dV/dx ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*(1/(4*dy*dx))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+Nz)+   alfa2*(-1/(4*dy*dx))*dQy(ii);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+Nz)+   alfa2*(-1/(4*dy*dx))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+Nz)=        MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+Nz)+        alfa2*(1/(4*dy*dx))*dQy(ii);                                                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz)+     alfa2*(3/(4*dy*dx))*dQy(ii);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+Nz)+ alfa2*(-4/(4*dy*dx))*dQy(ii); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+Nz)+alfa2*(1/(4*dy*dx))*dQy(ii); 

MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz)+     alfa2*(-3/(4*dy*dx))*dQy(ii); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz+Nz)+   alfa2*(4/(4*dy*dx))*dQy(ii);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+Nz)+alfa2*(-1/(4*dy*dx))*dQy(ii);
end

%----- j=Ny ----;

for ii=2:1:Nx-1 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)+alfa2*(-1/(4*dy*dx))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*(4/(4*dy*dx))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*(-3/(4*dy*dx))*dQy(ii); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(1/(4*dy*dx))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-4/(4*dy*dx))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(3/(4*dy*dx))*dQy(ii);                                                                                          
end

%---- i=Nx ----;

MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz)=     MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz)+     alfa2*(-3/(4*dy*dx))*dQy(Nx);
MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+Nz)=  MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+Nz)+   alfa2*(4/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+Nz)=MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+Nz)+alfa2*(-1/(4*dy*dx))*dQy(Nx); 

MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz)=     MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz)+    alfa2*(12/(4*dy*dx))*dQy(Nx);
MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+Nz)=  MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+Nz)+alfa2*(-16/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+Nz)=MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+Nz)+alfa2*(4/(4*dy*dx))*dQy(Nx); 

MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz)=     MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz)+     alfa2*(-9/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+Nz)=  MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+Nz)+  alfa2*(12/(4*dy*dx))*dQy(Nx);  
MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+Nz)=MB((Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+Nz)+alfa2*(-3/(4*dy*dx))*dQy(Nx);  

for jj=2:1:Ny-1
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*(-1/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*(4/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*(-3/(4*dy*dx))*dQy(Nx);

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+Nz)+ alfa2*(1/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+Nz)+alfa2*(-4/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+Nz)+ alfa2*(3/(4*dy*dx))*dQy(Nx);                                                                                          
end 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(1/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-4/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(3/(4*dy*dx))*dQy(Nx); 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(-4/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*(16/(4*dy*dx))*dQy(Nx); 
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*(-12/(4*dy*dx))*dQy(Nx); 

MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*(3/(4*dy*dx))*dQy(Nx);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-12/(4*dy*dx))*dQy(Nx);
MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*(9/(4*dy*dx))*dQy(Nx);


%------------------ dQ/dy d/dQ ( dQ/dx dV/dQ ) -----------------;

%---------------- 1: dQ/dy dQ/dx d/dQ ( dV/dQ ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*dQx(ii,jj)*(1/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*dQx(ii,jj)*(-2/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*dQx(ii,jj)*(1/dy^2)*dQy(ii);                                                                             
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+      alfa2*dQx(ii,1)*(2/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dQx(ii,1)*(-5/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+ alfa2*dQx(ii,1)*(4/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+Nz)+alfa2*dQx(ii,1)*(-1/dy^2)*dQy(ii);    
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)+alfa2*dQx(ii,Ny)*(-1/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*dQx(ii,Ny)*(4/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQx(ii,Ny)*(-5/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*dQx(ii,Ny)*(2/dy^2)*dQy(ii);                                                                                          
end

%---------------- 2: dQ/dy d^2Q/dxdq  dV/dQ -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*d2Qxq(ii,jj)*(-1/(2*dy))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*d2Qxq(ii,jj)*(1/(2*dy))*dQy(ii);                                                                             
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*d2Qxq(ii,1)*(-3/(2*dy))*dQy(ii); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*d2Qxq(ii,1)*(4/(2*dy))*dQy(ii); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*d2Qxq(ii,1)*(-1/(2*dy))*dQy(ii); 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*d2Qxq(ii,Ny)*(1/(2*dy))*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*d2Qxq(ii,Ny)*(-4/(2*dy))*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*d2Qxq(ii,Ny)*(3/(2*dy))*dQy(ii);                                                                                          
end


%----------------- dQ/dy d/dQ ( dZ/dx dV/dz ) -----------------;

%--------------- 1: dQ/dy dZ/dx d/dQ ( dV/dz ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*dZx(ii,jj,Nz)*(-1/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*dZx(ii,jj,Nz)*(4/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*dZx(ii,jj,Nz)*(-3/(4*dy*dz))*dQy(ii);

MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*dZx(ii,jj,Nz)*(1/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*dZx(ii,jj,Nz)*(-4/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*dZx(ii,jj,Nz)*(3/(4*dy*dz))*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*dZx(ii,1,Nz)*(-3/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*dZx(ii,1,Nz)*(4/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*dZx(ii,1,Nz)*(-1/(4*dy*dz))*dQy(ii); 

MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+    alfa2*dZx(ii,1,Nz)*(12/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*dZx(ii,1,Nz)*(-16/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*dZx(ii,1,Nz)*(4/(4*dy*dz))*dQy(ii); 

MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*dZx(ii,1,Nz)*(-9/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dZx(ii,1,Nz)*(12/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*dZx(ii,1,Nz)*(-3/(4*dy*dz))*dQy(ii);

end

%----- j=Ny -----;

for ii=2:1:Nx 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*dZx(ii,Ny,Nz)*(1/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*dZx(ii,Ny,Nz)*(-4/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*dZx(ii,Ny,Nz)*(3/(4*dy*dz))*dQy(ii); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*dZx(ii,Ny,Nz)*(-4/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*dZx(ii,Ny,Nz)*(16/(4*dy*dz))*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*dZx(ii,Ny,Nz)*(-12/(4*dy*dz))*dQy(ii); 

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dZx(ii,Ny,Nz)*(3/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dZx(ii,Ny,Nz)*(-12/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dZx(ii,Ny,Nz)*(9/(4*dy*dz))*dQy(ii);                                                                                          
end

%-------------- 2: dQ/dy d^2Z/dxdq  dV/dz -----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zxq(ii,jj,Nz)*(1/(2*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zxq(ii,jj,Nz)*(-4/(2*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zxq(ii,jj,Nz)*(3/(2*dz))*dQy(ii);
end 
end


%----------------- dZ/dy d/dz ( dV/dx ) -----------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*(-1/(4*dz*dx))*dZy(ii,jj,Nz);    
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*(4/(4*dz*dx))*dZy(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+  alfa2*(-3/(4*dz*dx))*dZy(ii,jj,Nz); 

MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(1/(4*dz*dx))*dZy(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-4/(4*dz*dx))*dZy(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(3/(4*dz*dx))*dZy(ii,jj,Nz);                                                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(1/(4*dz*dx))*dZy(Nx,jj,Nz);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-4/(4*dz*dx))*dZy(Nx,jj,Nz);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(3/(4*dz*dx))*dZy(Nx,jj,Nz); 

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*(-4/(4*dz*dx))*dZy(Nx,jj,Nz);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(16/(4*dz*dx))*dZy(Nx,jj,Nz);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*(-12/(4*dz*dx))*dZy(Nx,jj,Nz); 

MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+  alfa2*(3/(4*dz*dx))*dZy(Nx,jj,Nz); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-12/(4*dz*dx))*dZy(Nx,jj,Nz); 
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+    alfa2*(9/(4*dz*dx))*dZy(Nx,jj,Nz);                                                                                          
end 


%----------------- dZ/dy d/dz ( dQ/dx dV/dQ ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*dQx(ii,jj)*(-1/(4*dz*dy))*dZy(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*dQx(ii,jj)*(4/(4*dz*dy))*dZy(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*dQx(ii,jj)*(-3/(4*dz*dy))*dZy(ii,jj,Nz); 

MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*dQx(ii,jj)*(1/(4*dz*dy))*dZy(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*dQx(ii,jj)*(-4/(4*dz*dy))*dZy(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*dQx(ii,jj)*(3/(4*dz*dy))*dZy(ii,jj,Nz); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*dQx(ii,1)*(-3/(4*dz*dy))*dZy(ii,1,Nz); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*dQx(ii,1)*(4/(4*dz*dy))*dZy(ii,1,Nz);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*dQx(ii,1)*(-1/(4*dz*dy))*dZy(ii,1,Nz);

MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+    alfa2*dQx(ii,1)*(12/(4*dz*dy))*dZy(ii,1,Nz); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*dQx(ii,1)*(-16/(4*dz*dy))*dZy(ii,1,Nz);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*dQx(ii,1)*(4/(4*dz*dy))*dZy(ii,1,Nz);

MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*dQx(ii,1)*(-9/(4*dz*dy))*dZy(ii,1,Nz); 
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dQx(ii,1)*(12/(4*dz*dy))*dZy(ii,1,Nz);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*dQx(ii,1)*(-3/(4*dz*dy))*dZy(ii,1,Nz);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*dQx(ii,Ny)*(1/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*dQx(ii,Ny)*(-4/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*dQx(ii,Ny)*(3/(4*dz*dy))*dZy(ii,Ny,Nz);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*dQx(ii,Ny)*(-4/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*dQx(ii,Ny)*(16/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*dQx(ii,Ny)*(-12/(4*dz*dy))*dZy(ii,Ny,Nz);

MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dQx(ii,Ny)*(3/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQx(ii,Ny)*(-12/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dQx(ii,Ny)*(9/(4*dz*dy))*dZy(ii,Ny,Nz);                                                                                          
end


%---------------- dZ/dy d/dz ( dZ/dx dV/dz ) -----------------;

%-------------- 1: dZ/dy dZ/dx d/dz ( dV/dz ) -----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*dZx(ii,jj,Nz)*(-1/dz^2)*dZy(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*dZx(ii,jj,Nz)*(4/dz^2)*dZy(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZx(ii,jj,Nz)*(-5/dz^2)*dZy(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*dZx(ii,jj,Nz)*(2/dz^2)*dZy(ii,jj,Nz);                                                                              
end 
end

%---------------- 2: dZ/dy d^2Z/dxdz  dV/dz ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zxz(ii,jj,Nz)*(1/(2*dz))*dZy(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zxz(ii,jj,Nz)*(-4/(2*dz))*dZy(ii,jj,Nz); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zxz(ii,jj,Nz)*(3/(2*dz))*dZy(ii,jj,Nz);                                                                              
end 
end



%+++++++++ KpE (-1/H) 3/(2 dz) ( Fyx dHb/dx + Fyy dHb/dy )^Nz +++++++++++;

%----------------- Kp1 (-1/H) 3/(2*dz) dHb/dx dU/dx ------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*Kp1*(3/(2*dz))*(-1/(2*dx))*dZz(ii,jj)*dHbx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)+     alfa2*Kp1*(3/(2*dz))*(1/(2*dx))*dZz(ii,jj)*dHbx(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*Kp1*(3/(2*dz))*(1/(2*dx))*dZz(Nx,jj)*dHbx(Nx,jj);       
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*Kp1*(3/(2*dz))*(-4/(2*dx))*dZz(Nx,jj)*dHbx(Nx,jj);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*Kp1*(3/(2*dz))*(3/(2*dx))*dZz(Nx,jj)*dHbx(Nx,jj);                                                          
end 


%---------------- Kp1 (-1/H) 3/(2*dz) dHb/dx dQ/dx dU/dQ ----------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp1*(3/(2*dz))*(-1/(2*dy))*dZz(ii,jj)*dHbx(ii,jj)*dQx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp1*(3/(2*dz))*(1/(2*dy))*dZz(ii,jj)*dHbx(ii,jj)*dQx(ii,jj);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*Kp1*(3/(2*dz))*(-3/(2*dy))*dZz(ii,1)*dHbx(ii,1)*dQx(ii,1);            
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*Kp1*(3/(2*dz))*(4/(2*dy))*dZz(ii,1)*dHbx(ii,1)*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*(3/(2*dz))*(-1/(2*dy))*dZz(ii,1)*dHbx(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp1*(3/(2*dz))*(1/(2*dy))*dZz(ii,Ny)*dHbx(ii,Ny)*dQx(ii,Ny);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*(3/(2*dz))*(-4/(2*dy))*dZz(ii,Ny)*dHbx(ii,Ny)*dQx(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp1*(3/(2*dz))*(3/(2*dy))*dZz(ii,Ny)*dHbx(ii,Ny)*dQx(ii,Ny);                                                          
end


%--------------- Kp1 (-1/H) 3/(2*dz) dHb/dx dZ/dx dU/dz ----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp1*(3/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZx(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*(3/(2*dz))*(-4/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZx(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp1*(3/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZx(ii,jj,Nz);                                                          
end 
end


%--------------- Kp2 (-1/H) 3/(2*dz) dHb/dx dQ/dy dV/dQ -----------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp2*(3/(2*dz))*(-1/(2*dy))*dZz(ii,jj)*dHbx(ii,jj)*dQy(ii);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp2*(3/(2*dz))*(1/(2*dy))*dZz(ii,jj)*dHbx(ii,jj)*dQy(ii);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*Kp2*(3/(2*dz))*(-3/(2*dy))*dZz(ii,1)*dHbx(ii,1)*dQy(ii);            
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*Kp2*(3/(2*dz))*(4/(2*dy))*dZz(ii,1)*dHbx(ii,1)*dQy(ii);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*(3/(2*dz))*(-1/(2*dy))*dZz(ii,1)*dHbx(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp2*(3/(2*dz))*(1/(2*dy))*dZz(ii,Ny)*dHbx(ii,Ny)*dQy(ii);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*(3/(2*dz))*(-4/(2*dy))*dZz(ii,Ny)*dHbx(ii,Ny)*dQy(ii);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp2*(3/(2*dz))*(3/(2*dy))*dZz(ii,Ny)*dHbx(ii,Ny)*dQy(ii);                                                          
end


%------------ Kp2 (-1/H) 3/(2*dz) dHb/dx dZ/dy dV/dz ---------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*(3/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZy(ii,jj,Nz);    
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*(3/(2*dz))*(-4/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZy(ii,jj,Nz);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*(3/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZy(ii,jj,Nz);
end 
end

%----------  Kp2 (-1/H) 3/(2*dz) dHb/dx dZ/dz dW/dz -------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*(3/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZz(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*(3/(2*dz))*(-4/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZz(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*(3/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZz(ii,jj); 
end 
end


%------------- (- 1/H) 3/(2*dz) dHb/dy dQ/dy dU/dQ ----------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*(3/(2*dz))*(-1/(2*dy))*dZz(ii,jj)*dHby(ii,jj)*dQy(ii);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*(3/(2*dz))*(1/(2*dy))*dZz(ii,jj)*dHby(ii,jj)*dQy(ii);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*(3/(2*dz))*(-3/(2*dy))*dZz(ii,1)*dHby(ii,1)*dQy(ii);            
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*(3/(2*dz))*(4/(2*dy))*dZz(ii,1)*dHby(ii,1)*dQy(ii);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*(3/(2*dz))*(-1/(2*dy))*dZz(ii,1)*dHby(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(3/(2*dz))*(1/(2*dy))*dZz(ii,Ny)*dHby(ii,Ny)*dQy(ii);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(3/(2*dz))*(-4/(2*dy))*dZz(ii,Ny)*dHby(ii,Ny)*dQy(ii);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(3/(2*dz))*(3/(2*dy))*dZz(ii,Ny)*dHby(ii,Ny)*dQy(ii);                                                          
end


%-------------- (- 1/H) 3/(2*dz) dHb/dy dZ/dy dU/dz ----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(3/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZy(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(3/(2*dz))*(-4/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZy(ii,jj,Nz);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(3/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZy(ii,jj,Nz);                                                          
end 
end


%---------------- (- 1/H) 3/(2*dz) dHb/dy dV/dx -----------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(3/(2*dz))*(-1/(2*dx))*dZz(ii,jj)*dHby(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)+     alfa2*(3/(2*dz))*(1/(2*dx))*dZz(ii,jj)*dHby(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*(3/(2*dz))*(1/(2*dx))*dZz(Nx,jj)*dHby(Nx,jj);    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(3/(2*dz))*(-4/(2*dx))*dZz(Nx,jj)*dHby(Nx,jj);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*(3/(2*dz))*(3/(2*dx))*dZz(Nx,jj)*dHby(Nx,jj);                                                          
end 


%------------ (- 1/H) 3/(2*dz) dHb/dy dQ/dx dV/dQ ---------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*(3/(2*dz))*(-1/(2*dy))*dZz(ii,jj)*dHby(ii,jj)*dQx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*(3/(2*dz))*(1/(2*dy))*dZz(ii,jj)*dHby(ii,jj)*dQx(ii,jj);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*(3/(2*dz))*(-3/(2*dy))*dZz(ii,1)*dHby(ii,1)*dQx(ii,1);            
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*(3/(2*dz))*(4/(2*dy))*dZz(ii,1)*dHby(ii,1)*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*(3/(2*dz))*(-1/(2*dy))*dZz(ii,1)*dHby(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(3/(2*dz))*(1/(2*dy))*dZz(ii,Ny)*dHby(ii,Ny)*dQx(ii,Ny);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(3/(2*dz))*(-4/(2*dy))*dZz(ii,Ny)*dHby(ii,Ny)*dQx(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(3/(2*dz))*(3/(2*dy))*dZz(ii,Ny)*dHby(ii,Ny)*dQx(ii,Ny);                                                          
end


%--------------- (- 1/H) 3/(2*dz) dHb/dy dZ/dx dV/dz ---------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(3/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZx(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(3/(2*dz))*(-4/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZx(ii,jj,Nz);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(3/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZx(ii,jj,Nz);                                                          
end 
end


%++++++++++++++++  KpE (-1/H) (-4/(2 dz)) ( Fxz )^Nz-1 +++++++++++++++++++;

%------------------  (-1/H) (-4/(2*dz)) dZ/dz dU/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*(-4/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZz(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(-4/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dZz(ii,jj);                                                          
end 
end

%------------------- (-1/H) (-4/(2*dz)) dW/dx ---------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-4/(2*dz))*(-1/(2*dx))*dZz(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)+     alfa2*(-4/(2*dz))*(1/(2*dx))*dZz(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*(-4/(2*dz))*(1/(2*dx))*dZz(Nx,jj);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-4/(2*dz))*(-4/(2*dx))*dZz(Nx,jj);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*(-4/(2*dz))*(3/(2*dx))*dZz(Nx,jj);                                                          
end 


%----------------- (-1/H) (-4/(2*dz)) dQ/dx dW/dQ ------------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+alfa2*(-4/(2*dz))*(-1/(2*dy))*dZz(ii,jj)*dQx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+     alfa2*(-4/(2*dz))*(1/(2*dy))*dZz(ii,jj)*dQx(ii,jj);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+     alfa2*(-4/(2*dz))*(-3/(2*dy))*dZz(ii,1)*dQx(ii,1);            
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+   alfa2*(-4/(2*dz))*(4/(2*dy))*dZz(ii,1)*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*(-4/(2*dz))*(-1/(2*dy))*dZz(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*(-4/(2*dz))*(1/(2*dy))*dZz(ii,Ny)*dQx(ii,Ny);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+alfa2*(-4/(2*dz))*(-4/(2*dy))*dZz(ii,Ny)*dQx(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+ alfa2*(-4/(2*dz))*(3/(2*dy))*dZz(ii,Ny)*dQx(ii,Ny);                                                          
end


%----------------- (-1/H) (-4/(2*dz)) dZ/dx dW/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*(-4/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZx(ii,jj,Nz-1);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(-4/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dZx(ii,jj,Nz-1);                                                          
end 
end


%++++++++++++++++  KpE (-1/H) (1/(2 dz)) ( Fxz )^Nz-2 +++++++++++++++++++;

%------------------  (-1/H) (1/(2*dz)) dZ/dz dU/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*(1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZz(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*(1/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dZz(ii,jj);                                                          
end 
end

%------------------- (-1/H) (1/(2*dz)) dW/dx ---------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*(1/(2*dz))*(-1/(2*dx))*dZz(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)+     alfa2*(1/(2*dz))*(1/(2*dx))*dZz(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(1/(2*dz))*(1/(2*dx))*dZz(Nx,jj);
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*(1/(2*dz))*(-4/(2*dx))*dZz(Nx,jj);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(1/(2*dz))*(3/(2*dx))*dZz(Nx,jj);                                                          
end 


%----------------- (-1/H) (1/(2*dz)) dQ/dx dW/dQ ------------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*(1/(2*dz))*(-1/(2*dy))*dZz(ii,jj)*dQx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+     alfa2*(1/(2*dz))*(1/(2*dy))*dZz(ii,jj)*dQx(ii,jj);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*(1/(2*dz))*(-3/(2*dy))*dZz(ii,1)*dQx(ii,1);            
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*(1/(2*dz))*(4/(2*dy))*dZz(ii,1)*dQx(ii,1);
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*(1/(2*dz))*(-1/(2*dy))*dZz(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*(1/(2*dz))*(1/(2*dy))*dZz(ii,Ny)*dQx(ii,Ny);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*(1/(2*dz))*(-4/(2*dy))*dZz(ii,Ny)*dQx(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*(1/(2*dz))*(3/(2*dy))*dZz(ii,Ny)*dQx(ii,Ny);                                                          
end


%----------------- (-1/H) (1/(2*dz)) dZ/dx dW/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*(1/(2*dz))*(-1/(2*dz))*dZz(ii,jj)*dZx(ii,jj,Nz-2);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*(1/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dZx(ii,jj,Nz-2);                                                          
end 
end



%-----------------  KpE (-1/H) 3/(2 dz) dHb/dx P' -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj) + alfa2*KpE*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj);                                                          
end 
end



%+++++++++++++++++++ + KpE dens Freq^2 U ++++++++++++++++++++++;

for ii=2:1:Nx 
for jj=1:1:Ny 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz) + alfa2*KpE*dens*Freq^2;
end 
end




%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++++++ The second equation +++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++++++++ KpE dFyx/dx +++++++++++++++++++++++++++;

%---------------------- d/dx ( dQ/dy dU/dQ ) ---------------------;

%-------------------- 1: dQ/dy d/dx ( dU/dQ ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*dQy(ii)*(1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+jj*Nz+Nz)+   alfa2*dQy(ii)*(-1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-2)*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-2)*Nz+Nz)+   alfa2*dQy(ii)*(-1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+jj*Nz+Nz)=        MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+jj*Nz+Nz)+        alfa2*dQy(ii)*(1/(4*dx*dy));                                  
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz)+     alfa2*dQy(ii)*(3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz+Nz)+ alfa2*dQy(ii)*(-4/(4*dx*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+2*Nz+Nz)+alfa2*dQy(ii)*(1/(4*dx*dy));

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz)+     alfa2*dQy(ii)*(-3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz+Nz)+   alfa2*dQy(ii)*(4/(4*dx*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,ii*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,ii*Ny*Nz+2*Nz+Nz)+alfa2*dQy(ii)*(-1/(4*dx*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)+alfa2*dQy(ii)*(-1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*dQy(ii)*(4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*dQy(ii)*(-3/(4*dx*dy));

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*dQy(ii)*(1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQy(ii)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*dQy(ii)*(3/(4*dx*dy));                                  
end

%---- i=Nx ----;

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz)+     alfa2*dQy(Nx)*(-3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz+Nz)+   alfa2*dQy(Nx)*(4/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+2*Nz+Nz)+alfa2*dQy(Nx)*(-1/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz)+    alfa2*dQy(Nx)*(12/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz+Nz)+alfa2*dQy(Nx)*(-16/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+2*Nz+Nz)+alfa2*dQy(Nx)*(4/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz)+     alfa2*dQy(Nx)*(-9/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz+Nz)+  alfa2*dQy(Nx)*(12/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+2*Nz+Nz)+alfa2*dQy(Nx)*(-3/(4*dx*dy));

for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*dQy(Nx)*(-1/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*dQy(Nx)*(4/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*dQy(Nx)*(-3/(4*dx*dy)); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+jj*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+jj*Nz+Nz)+ alfa2*dQy(Nx)*(1/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+jj*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+jj*Nz+Nz)+alfa2*dQy(Nx)*(-4/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+jj*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+jj*Nz+Nz)+ alfa2*dQy(Nx)*(3/(4*dx*dy));                                  
end 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*dQy(Nx)*(1/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQy(Nx)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*dQy(Nx)*(3/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*dQy(Nx)*(-4/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*dQy(Nx)*(16/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*dQy(Nx)*(-12/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dQy(Nx)*(3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQy(Nx)*(-12/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dQy(Nx)*(9/(4*dx*dy));


%---------------------- 2: d^2Q/dydx dU/dQ ----------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*d2Qyx(ii)*(-1/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*d2Qyx(ii)*(1/(2*dy)); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*d2Qyx(ii)*(-3/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*d2Qyx(ii)*(4/(2*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*d2Qyx(ii)*(-1/(2*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*d2Qyx(ii)*(1/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*d2Qyx(ii)*(-4/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*d2Qyx(ii)*(3/(2*dy));
end


%--------------------- d/dx ( dZ/dy dU/dz ) ---------------------;

%------------------- 1: dZ/dy d/dx ( dU/dz ) ---------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*dZy(ii,jj,Nz)*(-1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*dZy(ii,jj,Nz)*(4/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+  alfa2*dZy(ii,jj,Nz)*(-3/(4*dx*dz));

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*dZy(ii,jj,Nz)*(1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZy(ii,jj,Nz)*(-4/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*dZy(ii,jj,Nz)*(3/(4*dx*dz));                                  
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*dZy(Nx,jj,Nz)*(1/(4*dx*dz));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZy(Nx,jj,Nz)*(-4/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*dZy(Nx,jj,Nz)*(3/(4*dx*dz));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*dZy(Nx,jj,Nz)*(-4/(4*dx*dz));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZy(Nx,jj,Nz)*(16/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*dZy(Nx,jj,Nz)*(-12/(4*dx*dz));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+  alfa2*dZy(Nx,jj,Nz)*(3/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZy(Nx,jj,Nz)*(-12/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+    alfa2*dZy(Nx,jj,Nz)*(9/(4*dx*dz));                                  
end

%------------------- 2: d^2Z/dydx  dU/dz ---------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zyx(ii,jj,Nz)*(1/(2*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zyx(ii,jj,Nz)*(-4/(2*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zyx(ii,jj,Nz)*(3/(2*dz));
end 
end


%-------------------- dQ/dx d/dQ ( dQ/dy dU/dQ ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*dQy(ii)*(1/dy^2)*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*dQy(ii)*(-2/dy^2)*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*dQy(ii)*(1/dy^2)*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+      alfa2*dQy(ii)*(2/dy^2)*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dQy(ii)*(-5/dy^2)*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+ alfa2*dQy(ii)*(4/dy^2)*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+3*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+3*Nz+Nz)+alfa2*dQy(ii)*(-1/dy^2)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)+alfa2*dQy(ii)*(-1/dy^2)*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*dQy(ii)*(4/dy^2)*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQy(ii)*(-5/dy^2)*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*dQy(ii)*(2/dy^2)*dQx(ii,Ny);                                                                                          
end


%-------------------- dQ/dx d/dQ ( dZ/dy dU/dz ) --------------------;

%------------------ 1: dQ/dx dZ/dy d/dQ ( dU/dz ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*dZy(ii,jj,Nz)*(-1/(4*dy*dz))*dQx(ii,jj);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*dZy(ii,jj,Nz)*(4/(4*dy*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*dZy(ii,jj,Nz)*(-3/(4*dy*dz))*dQx(ii,jj);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*dZy(ii,jj,Nz)*(1/(4*dy*dz))*dQx(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*dZy(ii,jj,Nz)*(-4/(4*dy*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*dZy(ii,jj,Nz)*(3/(4*dy*dz))*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)+     alfa2*dZy(ii,1,Nz)*(-3/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*dZy(ii,1,Nz)*(4/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*dZy(ii,1,Nz)*(-1/(4*dy*dz))*dQx(ii,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)+    alfa2*dZy(ii,1,Nz)*(12/(4*dy*dz))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*dZy(ii,1,Nz)*(-16/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*dZy(ii,1,Nz)*(4/(4*dy*dz))*dQx(ii,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*dZy(ii,1,Nz)*(-9/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dZy(ii,1,Nz)*(12/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*dZy(ii,1,Nz)*(-3/(4*dy*dz))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*dZy(ii,Ny,Nz)*(1/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*dZy(ii,Ny,Nz)*(-4/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*dZy(ii,Ny,Nz)*(3/(4*dy*dz))*dQx(ii,Ny);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*dZy(ii,Ny,Nz)*(-4/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*dZy(ii,Ny,Nz)*(16/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*dZy(ii,Ny,Nz)*(-12/(4*dy*dz))*dQx(ii,Ny); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dZy(ii,Ny,Nz)*(3/(4*dy*dz))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dZy(ii,Ny,Nz)*(-12/(4*dy*dz))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dZy(ii,Ny,Nz)*(9/(4*dy*dz))*dQx(ii,Ny);                                                                                          
end

%------------------ 2: dQ/dx d^2Z/dydq  dU/dz --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zyq(ii,jj,Nz)*(1/(2*dz))*dQx(ii,jj);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zyq(ii,jj,Nz)*(-4/(2*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zyq(ii,jj,Nz)*(3/(2*dz))*dQx(ii,jj);
end 
end


%-------------------- dZ/dx d/dz ( dQ/dy dU/dQ ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZx(ii,jj,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,jj,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,jj,Nz); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*dQy(ii)*(1/(4*dz*dy))*dZx(ii,jj,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,jj,Nz); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)+     alfa2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,1,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,1,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZx(ii,1,Nz);    

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)+    alfa2*dQy(ii)*(12/(4*dz*dy))*dZx(ii,1,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*dQy(ii)*(-16/(4*dz*dy))*dZx(ii,1,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*dQy(ii)*(4/(4*dz*dy))*dZx(ii,1,Nz);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*dQy(ii)*(-9/(4*dz*dy))*dZx(ii,1,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dQy(ii)*(12/(4*dz*dy))*dZx(ii,1,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*dQy(ii)*(-3/(4*dz*dy))*dZx(ii,1,Nz);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*dQy(ii)*(1/(4*dz*dy))*dZx(ii,Ny,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,Ny,Nz);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*dQy(ii)*(-4/(4*dz*dy))*dZx(ii,Ny,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*dQy(ii)*(16/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*dQy(ii)*(-12/(4*dz*dy))*dZx(ii,Ny,Nz);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dQy(ii)*(3/(4*dz*dy))*dZx(ii,Ny,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQy(ii)*(-12/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dQy(ii)*(9/(4*dz*dy))*dZx(ii,Ny,Nz);                                                                                          
end


%------------------- dZ/dx d/dz ( dZ/dy dU/dz ) --------------------;

%----------------- 1: dZ/dx dZ/dy d/dz ( dU/dz ) --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*dZy(ii,jj,Nz)*(-1/dz^2)*dZx(ii,jj,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*dZy(ii,jj,Nz)*(4/dz^2)*dZx(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZy(ii,jj,Nz)*(-5/dz^2)*dZx(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*dZy(ii,jj,Nz)*(2/dz^2)*dZx(ii,jj,Nz); 
end 
end

%----------------- 2: dZ/dx d^2Z/dydz  dU/dz --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zyz(ii,jj,Nz)*(1/(2*dz))*dZx(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zyz(ii,jj,Nz)*(-4/(2*dz))*dZx(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zyz(ii,jj,Nz)*(3/(2*dz))*dZx(ii,jj,Nz); 
end 
end


%------------------------ d/dx ( dV/dx ) -----------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(1/dx^2); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(-2/dx^2); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)+    alfa2*(1/dx^2); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-4)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-4)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(-1/dx^2);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(4/dx^2); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(-5/dx^2); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(2/dx^2);                                        
end 


%--------------------- d/dx ( dQ/dx dV/dQ ) ---------------------;

%------------------- 1: dQ/dx d/dx ( dV/dQ ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*dQx(ii,jj)*(1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+Nz)+   alfa2*dQx(ii,jj)*(-1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+Nz)+   alfa2*dQx(ii,jj)*(-1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+Nz)=        MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+Nz)+        alfa2*dQx(ii,jj)*(1/(4*dx*dy)); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz)+     alfa2*dQx(ii,1)*(3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+Nz)+ alfa2*dQx(ii,1)*(-4/(4*dx*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+Nz)+alfa2*dQx(ii,1)*(1/(4*dx*dy));

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz)+     alfa2*dQx(ii,1)*(-3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz+Nz)+   alfa2*dQx(ii,1)*(4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+Nz)+alfa2*dQx(ii,1)*(-1/(4*dx*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)+alfa2*dQx(ii,Ny)*(-1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*dQx(ii,Ny)*(4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*dQx(ii,Ny)*(-3/(4*dx*dy));

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*dQx(ii,Ny)*(1/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQx(ii,Ny)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*dQx(ii,Ny)*(3/(4*dx*dy));                                  
end

%---- i=Nx ----;

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz)+     alfa2*dQx(Nx,1)*(-3/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+Nz)+   alfa2*dQx(Nx,1)*(4/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+Nz)+alfa2*dQx(Nx,1)*(-1/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz)+    alfa2*dQx(Nx,1)*(12/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+Nz)+alfa2*dQx(Nx,1)*(-16/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+Nz)+alfa2*dQx(Nx,1)*(4/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz)+     alfa2*dQx(Nx,1)*(-9/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+Nz)+  alfa2*dQx(Nx,1)*(12/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+Nz)+alfa2*dQx(Nx,1)*(-3/(4*dx*dy));

for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*dQx(Nx,jj)*(-1/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*dQx(Nx,jj)*(4/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*dQx(Nx,jj)*(-3/(4*dx*dy)); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+Nz)+ alfa2*dQx(Nx,jj)*(1/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+Nz)+alfa2*dQx(Nx,jj)*(-4/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+Nz)+ alfa2*dQx(Nx,jj)*(3/(4*dx*dy));                                  
end 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*dQx(Nx,Ny)*(1/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQx(Nx,Ny)*(-4/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*dQx(Nx,Ny)*(3/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*dQx(Nx,Ny)*(-4/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*dQx(Nx,Ny)*(16/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*dQx(Nx,Ny)*(-12/(4*dx*dy));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dQx(Nx,Ny)*(3/(4*dx*dy));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQx(Nx,Ny)*(-12/(4*dx*dy)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dQx(Nx,Ny)*(9/(4*dx*dy));


%------------------- 2: d^2Q/dxdx  dV/dQ  --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*d2Qxx(ii,jj)*(-1/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*d2Qxx(ii,jj)*(1/(2*dy)); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*d2Qxx(ii,1)*(-3/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*d2Qxx(ii,1)*(4/(2*dy));
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*d2Qxx(ii,1)*(-1/(2*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*d2Qxx(ii,Ny)*(1/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*d2Qxx(ii,Ny)*(-4/(2*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*d2Qxx(ii,Ny)*(3/(2*dy));
end


%--------------------- d/dx ( dZ/dx dV/dz ) ---------------------;

%------------------- 1: dZ/dx d/dx ( dV/dz ) ---------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*dZx(ii,jj,Nz)*(-1/(4*dx*dz));     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*dZx(ii,jj,Nz)*(4/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+  alfa2*dZx(ii,jj,Nz)*(-3/(4*dx*dz));

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*dZx(ii,jj,Nz)*(1/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZx(ii,jj,Nz)*(-4/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*dZx(ii,jj,Nz)*(3/(4*dx*dz)); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*dZx(Nx,jj,Nz)*(1/(4*dx*dz));    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZx(Nx,jj,Nz)*(-4/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*dZx(Nx,jj,Nz)*(3/(4*dx*dz));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*dZx(Nx,jj,Nz)*(-4/(4*dx*dz));    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZx(Nx,jj,Nz)*(16/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*dZx(Nx,jj,Nz)*(-12/(4*dx*dz));

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+  alfa2*dZx(Nx,jj,Nz)*(3/(4*dx*dz));
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZx(Nx,jj,Nz)*(-12/(4*dx*dz)); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+    alfa2*dZx(Nx,jj,Nz)*(9/(4*dx*dz));                                  
end

%------------------- 2: d^2Z/dxdx  dV/dz ---------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zxx(ii,jj,Nz)*(1/(2*dz));     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zxx(ii,jj,Nz)*(-4/(2*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zxx(ii,jj,Nz)*(3/(2*dz));
end 
end


%-------------------- dQ/dx d/dQ ( dV/dx ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*(1/(4*dy*dx))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+Nz)+   alfa2*(-1/(4*dy*dx))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+Nz)+   alfa2*(-1/(4*dy*dx))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+Nz)=        MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+Nz)+        alfa2*(1/(4*dy*dx))*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz)+     alfa2*(3/(4*dy*dx))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+Nz)+ alfa2*(-4/(4*dy*dx))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+Nz)+alfa2*(1/(4*dy*dx))*dQx(ii,1); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz)+     alfa2*(-3/(4*dy*dx))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+Nz+Nz)+   alfa2*(4/(4*dy*dx))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+2*Nz+Nz)+alfa2*(-1/(4*dy*dx))*dQx(ii,1);
end

%----- j=Ny ----;

for ii=2:1:Nx-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)+alfa2*(-1/(4*dy*dx))*dQx(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*(4/(4*dy*dx))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*(-3/(4*dy*dx))*dQx(ii,Ny); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(1/(4*dy*dx))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-4/(4*dy*dx))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(3/(4*dy*dx))*dQx(ii,Ny);                                                                                          
end

%---- i=Nx ----;

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz)+     alfa2*(-3/(4*dy*dx))*dQx(Nx,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+Nz)+   alfa2*(4/(4*dy*dx))*dQx(Nx,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+Nz)+alfa2*(-1/(4*dy*dx))*dQx(Nx,1);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz)+    alfa2*(12/(4*dy*dx))*dQx(Nx,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+Nz)+alfa2*(-16/(4*dy*dx))*dQx(Nx,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+Nz)+alfa2*(4/(4*dy*dx))*dQx(Nx,1);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz)+     alfa2*(-9/(4*dy*dx))*dQx(Nx,1); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+Nz)+  alfa2*(12/(4*dy*dx))*dQx(Nx,1);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+Nz)+alfa2*(-3/(4*dy*dx))*dQx(Nx,1);

for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*(-1/(4*dy*dx))*dQx(Nx,jj);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*(4/(4*dy*dx))*dQx(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*(-3/(4*dy*dx))*dQx(Nx,jj);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+Nz)+ alfa2*(1/(4*dy*dx))*dQx(Nx,jj);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+Nz)+alfa2*(-4/(4*dy*dx))*dQx(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+Nz)+ alfa2*(3/(4*dy*dx))*dQx(Nx,jj);                                                                                          
end 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(1/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(3/(4*dy*dx))*dQx(Nx,Ny); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*(16/(4*dy*dx))*dQx(Nx,Ny);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*(-12/(4*dy*dx))*dQx(Nx,Ny); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*(3/(4*dy*dx))*dQx(Nx,Ny); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-12/(4*dy*dx))*dQx(Nx,Ny); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*(9/(4*dy*dx))*dQx(Nx,Ny);


%------------------- dQ/dx d/dQ ( dQ/dx dV/dQ ) --------------------;

%----------------- 1: dQ/dx dQ/dx d/dQ ( dV/dQ ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*(1/dy^2)*dQx(ii,jj)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(-2/dy^2)*dQx(ii,jj)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*(1/dy^2)*dQx(ii,jj)^2; 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+      alfa2*(2/dy^2)*dQx(ii,1)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*(-5/dy^2)*dQx(ii,1)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+ alfa2*(4/dy^2)*dQx(ii,1)^2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+Nz)+alfa2*(-1/dy^2)*dQx(ii,1)^2;
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)+alfa2*(-1/dy^2)*dQx(ii,Ny)^2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(4/dy^2)*dQx(ii,Ny)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-5/dy^2)*dQx(ii,Ny)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(2/dy^2)*dQx(ii,Ny)^2;                                                                                          
end

%----------------- 2: dQ/dx d^2Q/dxdq  dV/dQ  --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*d2Qxq(ii,jj)*(-1/(2*dy))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*d2Qxq(ii,jj)*(1/(2*dy))*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*d2Qxq(ii,1)*(-3/(2*dy))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*d2Qxq(ii,1)*(4/(2*dy))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*d2Qxq(ii,1)*(-1/(2*dy))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*d2Qxq(ii,Ny)*(1/(2*dy))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*d2Qxq(ii,Ny)*(-4/(2*dy))*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*d2Qxq(ii,Ny)*(3/(2*dy))*dQx(ii,Ny);                                                                                          
end


%-------------------- dQ/dx d/dQ ( dZ/dx dV/dz ) --------------------;

%------------------ 1: dQ/dx dZ/dx d/dQ ( dV/dz ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*dZx(ii,jj,Nz)*(-1/(4*dy*dz))*dQx(ii,jj);     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*dZx(ii,jj,Nz)*(4/(4*dy*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*dZx(ii,jj,Nz)*(-3/(4*dy*dz))*dQx(ii,jj);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*dZx(ii,jj,Nz)*(1/(4*dy*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*dZx(ii,jj,Nz)*(-4/(4*dy*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*dZx(ii,jj,Nz)*(3/(4*dy*dz))*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*dZx(ii,1,Nz)*(-3/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*dZx(ii,1,Nz)*(4/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*dZx(ii,1,Nz)*(-1/(4*dy*dz))*dQx(ii,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+    alfa2*dZx(ii,1,Nz)*(12/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*dZx(ii,1,Nz)*(-16/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*dZx(ii,1,Nz)*(4/(4*dy*dz))*dQx(ii,1);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*dZx(ii,1,Nz)*(-9/(4*dy*dz))*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dZx(ii,1,Nz)*(12/(4*dy*dz))*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*dZx(ii,1,Nz)*(-3/(4*dy*dz))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*dZx(ii,Ny,Nz)*(1/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*dZx(ii,Ny,Nz)*(-4/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*dZx(ii,Ny,Nz)*(3/(4*dy*dz))*dQx(ii,Ny); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*dZx(ii,Ny,Nz)*(-4/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*dZx(ii,Ny,Nz)*(16/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*dZx(ii,Ny,Nz)*(-12/(4*dy*dz))*dQx(ii,Ny); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dZx(ii,Ny,Nz)*(3/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dZx(ii,Ny,Nz)*(-12/(4*dy*dz))*dQx(ii,Ny);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dZx(ii,Ny,Nz)*(9/(4*dy*dz))*dQx(ii,Ny);                                                                                          
end

%------------------ 2: dQ/dx d^2Z/dxdq  dV/dz --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zxq(ii,jj,Nz)*(1/(2*dz))*dQx(ii,jj);     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zxq(ii,jj,Nz)*(-4/(2*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zxq(ii,jj,Nz)*(3/(2*dz))*dQx(ii,jj);
end 
end


%-------------------- dZ/dx d/dz ( dV/dx ) --------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*(-1/(4*dz*dx))*dZx(ii,jj,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*(4/(4*dz*dx))*dZx(ii,jj,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+  alfa2*(-3/(4*dz*dx))*dZx(ii,jj,Nz); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(1/(4*dz*dx))*dZx(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-4/(4*dz*dx))*dZx(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(3/(4*dz*dx))*dZx(ii,jj,Nz); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(1/(4*dz*dx))*dZx(Nx,jj,Nz);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-4/(4*dz*dx))*dZx(Nx,jj,Nz);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(3/(4*dz*dx))*dZx(Nx,jj,Nz); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*(-4/(4*dz*dx))*dZx(Nx,jj,Nz);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(16/(4*dz*dx))*dZx(Nx,jj,Nz);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*(-12/(4*dz*dx))*dZx(Nx,jj,Nz); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+  alfa2*(3/(4*dz*dx))*dZx(Nx,jj,Nz); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-12/(4*dz*dx))*dZx(Nx,jj,Nz); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+    alfa2*(9/(4*dz*dx))*dZx(Nx,jj,Nz);                                                                                          
end 


%-------------------- dZ/dx d/dz ( dQ/dx dV/dQ ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*dQx(ii,jj)*(-1/(4*dz*dy))*dZx(ii,jj,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*dQx(ii,jj)*(4/(4*dz*dy))*dZx(ii,jj,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*dQx(ii,jj)*(-3/(4*dz*dy))*dZx(ii,jj,Nz); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*dQx(ii,jj)*(1/(4*dz*dy))*dZx(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*dQx(ii,jj)*(-4/(4*dz*dy))*dZx(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*dQx(ii,jj)*(3/(4*dz*dy))*dZx(ii,jj,Nz); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*dQx(ii,1)*(-3/(4*dz*dy))*dZx(ii,1,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*dQx(ii,1)*(4/(4*dz*dy))*dZx(ii,1,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*dQx(ii,1)*(-1/(4*dz*dy))*dZx(ii,1,Nz);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+    alfa2*dQx(ii,1)*(12/(4*dz*dy))*dZx(ii,1,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*dQx(ii,1)*(-16/(4*dz*dy))*dZx(ii,1,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*dQx(ii,1)*(4/(4*dz*dy))*dZx(ii,1,Nz);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*dQx(ii,1)*(-9/(4*dz*dy))*dZx(ii,1,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dQx(ii,1)*(12/(4*dz*dy))*dZx(ii,1,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*dQx(ii,1)*(-3/(4*dz*dy))*dZx(ii,1,Nz);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*dQx(ii,Ny)*(1/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*dQx(ii,Ny)*(-4/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*dQx(ii,Ny)*(3/(4*dz*dy))*dZx(ii,Ny,Nz);    
    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*dQx(ii,Ny)*(-4/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*dQx(ii,Ny)*(16/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*dQx(ii,Ny)*(-12/(4*dz*dy))*dZx(ii,Ny,Nz);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dQx(ii,Ny)*(3/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQx(ii,Ny)*(-12/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dQx(ii,Ny)*(9/(4*dz*dy))*dZx(ii,Ny,Nz);                                                                                          
end


%-------------------- dZ/dx d/dz ( dZ/dx dV/dz ) --------------------;

%----------------- 1: dZ/dx dZ/dx d/dz ( dV/dz ) --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*(-1/dz^2)*dZx(ii,jj,Nz)^2;     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(4/dz^2)*dZx(ii,jj,Nz)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-5/dz^2)*dZx(ii,jj,Nz)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(2/dz^2)*dZx(ii,jj,Nz)^2; 
end 
end

%----------------- 2: dZ/dx d^2Z/dxdz  dV/dz --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zxz(ii,jj,Nz)*(1/(2*dz))*dZx(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zxz(ii,jj,Nz)*(-4/(2*dz))*dZx(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zxz(ii,jj,Nz)*(3/(2*dz))*dZx(ii,jj,Nz); 
end 
end



%+++++++++++++++++++++++++ KpE dFyy/dy +++++++++++++++++++++++++++;

%-------------- Kp1 dQ/dy d/dQ ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*Kp1*(1/dy^2)*dQy(ii)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*Kp1*(-2/dy^2)*dQy(ii)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp1*(1/dy^2)*dQy(ii)^2; 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+      alfa2*Kp1*(2/dy^2)*dQy(ii)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp1*(-5/dy^2)*dQy(ii)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+ alfa2*Kp1*(4/dy^2)*dQy(ii)^2;
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+Nz)+alfa2*Kp1*(-1/dy^2)*dQy(ii)^2;
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)+alfa2*Kp1*(-1/dy^2)*dQy(ii)^2;     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp1*(4/dy^2)*dQy(ii)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*(-5/dy^2)*dQy(ii)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp1*(2/dy^2)*dQy(ii)^2;                                                                                          
end


%-------------- Kp1 dQ/dy d/dQ ( dZ/dy dV/dz ) -------------------;

%------------- 1: Kp1 dQ/dy dZ/dy d/dQ ( dV/dz ) ------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*Kp1*dZy(ii,jj,Nz)*(-1/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*Kp1*dZy(ii,jj,Nz)*(4/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*Kp1*dZy(ii,jj,Nz)*(-3/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*Kp1*dZy(ii,jj,Nz)*(1/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*Kp1*dZy(ii,jj,Nz)*(-4/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp1*dZy(ii,jj,Nz)*(3/(4*dy*dz))*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*Kp1*dZy(ii,1,Nz)*(-3/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*Kp1*dZy(ii,1,Nz)*(4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*Kp1*dZy(ii,1,Nz)*(-1/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+    alfa2*Kp1*dZy(ii,1,Nz)*(12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*Kp1*dZy(ii,1,Nz)*(-16/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*Kp1*dZy(ii,1,Nz)*(4/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*Kp1*dZy(ii,1,Nz)*(-9/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp1*dZy(ii,1,Nz)*(12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*dZy(ii,1,Nz)*(-3/(4*dy*dz))*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*Kp1*dZy(ii,Ny,Nz)*(1/(4*dy*dz))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*Kp1*dZy(ii,Ny,Nz)*(-4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*Kp1*dZy(ii,Ny,Nz)*(3/(4*dy*dz))*dQy(ii); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*Kp1*dZy(ii,Ny,Nz)*(-4/(4*dy*dz))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*Kp1*dZy(ii,Ny,Nz)*(16/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*Kp1*dZy(ii,Ny,Nz)*(-12/(4*dy*dz))*dQy(ii); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*Kp1*dZy(ii,Ny,Nz)*(3/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*dZy(ii,Ny,Nz)*(-12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*Kp1*dZy(ii,Ny,Nz)*(9/(4*dy*dz))*dQy(ii);                                                                                          
end

%------------- 2: Kp1 dQ/dy d^2Z/dydq  dV/dz  ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp1*d2Zyq(ii,jj,Nz)*(1/(2*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*d2Zyq(ii,jj,Nz)*(-4/(2*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp1*d2Zyq(ii,jj,Nz)*(3/(2*dz))*dQy(ii);
end 
end


%-------------- Kp1 dZ/dy d/dz ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*Kp1*dQy(ii)*(-1/(4*dz*dy))*dZy(ii,jj,Nz);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*Kp1*dQy(ii)*(4/(4*dz*dy))*dZy(ii,jj,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*Kp1*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,jj,Nz); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*Kp1*dQy(ii)*(1/(4*dz*dy))*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*Kp1*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp1*dQy(ii)*(3/(4*dz*dy))*dZy(ii,jj,Nz); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*Kp1*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,1,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*Kp1*dQy(ii)*(4/(4*dz*dy))*dZy(ii,1,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*Kp1*dQy(ii)*(-1/(4*dz*dy))*dZy(ii,1,Nz);    
    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+    alfa2*Kp1*dQy(ii)*(12/(4*dz*dy))*dZy(ii,1,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*Kp1*dQy(ii)*(-16/(4*dz*dy))*dZy(ii,1,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*Kp1*dQy(ii)*(4/(4*dz*dy))*dZy(ii,1,Nz);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*Kp1*dQy(ii)*(-9/(4*dz*dy))*dZy(ii,1,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp1*dQy(ii)*(12/(4*dz*dy))*dZy(ii,1,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,1,Nz);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*Kp1*dQy(ii)*(1/(4*dz*dy))*dZy(ii,Ny,Nz);     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*Kp1*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*Kp1*dQy(ii)*(3/(4*dz*dy))*dZy(ii,Ny,Nz);    
    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*Kp1*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,Ny,Nz);     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*Kp1*dQy(ii)*(16/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*Kp1*dQy(ii)*(-12/(4*dz*dy))*dZy(ii,Ny,Nz);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*Kp1*dQy(ii)*(3/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*dQy(ii)*(-12/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*Kp1*dQy(ii)*(9/(4*dz*dy))*dZy(ii,Ny,Nz);                                                                                          
end


%------------- Kp1 dZ/dy d/dz ( dZ/dy dV/dz ) -------------------;

%------------ 1: Kp1 dZ/dy dZ/dy d/dz ( dV/dz ) ----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*Kp1*(-1/dz^2)*dZy(ii,jj,Nz)^2;     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp1*(4/dz^2)*dZy(ii,jj,Nz)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*(-5/dz^2)*dZy(ii,jj,Nz)^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp1*(2/dz^2)*dZy(ii,jj,Nz)^2; 
end 
end

%------------ 2: Kp1 dZ/dy d^2Z/dydz  dV/dz ----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp1*d2Zyz(ii,jj,Nz)*(1/(2*dz))*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*d2Zyz(ii,jj,Nz)*(-4/(2*dz))*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp1*d2Zyz(ii,jj,Nz)*(3/(2*dz))*dZy(ii,jj,Nz); 
end 
end


%------------------ Kp2 dQ/dy d/dQ ( dU/dx ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp2*(1/(4*dy*dx))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-2)*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-2)*Nz+Nz)+   alfa2*Kp2*(-1/(4*dy*dx))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp2*(-1/(4*dy*dx))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+jj*Nz+Nz)=        MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+jj*Nz+Nz)+        alfa2*Kp2*(1/(4*dy*dx))*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz)+     alfa2*Kp2*(3/(4*dy*dx))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+Nz+Nz)+ alfa2*Kp2*(-4/(4*dy*dx))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-2)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*(1/(4*dy*dx))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz)+     alfa2*Kp2*(-3/(4*dy*dx))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,ii*Ny*Nz+Nz+Nz)+   alfa2*Kp2*(4/(4*dy*dx))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,ii*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,ii*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*(-1/(4*dy*dx))*dQy(ii);
end

%----- j=Ny ----;

for ii=2:1:Nx-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)+alfa2*Kp2*(-1/(4*dy*dx))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*Kp2*(4/(4*dy*dx))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*Kp2*(-3/(4*dy*dx))*dQy(ii); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp2*(1/(4*dy*dx))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*(-4/(4*dy*dx))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,ii*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp2*(3/(4*dy*dx))*dQy(ii);                                                                                          
end

%---- i=Nx ----;

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz)+     alfa2*Kp2*(-3/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+Nz+Nz)+   alfa2*Kp2*(4/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-3)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*(-1/(4*dy*dx))*dQy(Nx);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz)+    alfa2*Kp2*(12/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+Nz+Nz)+alfa2*Kp2*(-16/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-2)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*(4/(4*dy*dx))*dQy(Nx);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz)+     alfa2*Kp2*(-9/(4*dy*dx))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp2*(12/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,(Nx-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*(-3/(4*dy*dx))*dQy(Nx);


for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp2*(-1/(4*dy*dx))*dQy(Nx);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*Kp2*(4/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp2*(-3/(4*dy*dx))*dQy(Nx);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+jj*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+jj*Nz+Nz)+ alfa2*Kp2*(1/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+jj*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+jj*Nz+Nz)+alfa2*Kp2*(-4/(4*dy*dx))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+jj*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+jj*Nz+Nz)+ alfa2*Kp2*(3/(4*dy*dx))*dQy(Nx);                                                                                          
end 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp2*(1/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*(-4/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp2*(3/(4*dy*dx))*dQy(Nx);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp2*(-4/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*Kp2*(16/(4*dy*dx))*dQy(Nx);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*Kp2*(-12/(4*dy*dx))*dQy(Nx);

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*Kp2*(3/(4*dy*dx))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*(-12/(4*dy*dx))*dQy(Nx); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*Kp2*(9/(4*dy*dx))*dQy(Nx);


%----------------- Kp2 dQ/dy d/dQ ( dQ/dx dU/dQ ) -----------------;

%--------------- 1: Kp2 dQ/dy dQ/dx d/dQ ( dU/dQ ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*Kp2*dQx(ii,jj)*(1/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*Kp2*dQx(ii,jj)*(-2/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp2*dQx(ii,jj)*(1/dy^2)*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+      alfa2*Kp2*dQx(ii,1)*(2/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp2*dQx(ii,1)*(-5/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+ alfa2*Kp2*dQx(ii,1)*(4/dy^2)*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+3*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+3*Nz+Nz)+alfa2*Kp2*dQx(ii,1)*(-1/dy^2)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)+alfa2*Kp2*dQx(ii,Ny)*(-1/dy^2)*dQy(ii);     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp2*dQx(ii,Ny)*(4/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*dQx(ii,Ny)*(-5/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp2*dQx(ii,Ny)*(2/dy^2)*dQy(ii);                                                                                          
end

%-------------- 2: Kp2 dQ/dy d^2Q/dxdq  dU/dQ  -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp2*d2Qxq(ii,jj)*(-1/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp2*d2Qxq(ii,jj)*(1/(2*dy))*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*Kp2*d2Qxq(ii,1)*(-3/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*Kp2*d2Qxq(ii,1)*(4/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*d2Qxq(ii,1)*(-1/(2*dy))*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp2*d2Qxq(ii,Ny)*(1/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*d2Qxq(ii,Ny)*(-4/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp2*d2Qxq(ii,Ny)*(3/(2*dy))*dQy(ii);                                                                                          
end


%----------------- Kp2 dQ/dy d/dQ ( dZ/dx dU/dz ) -----------------;

%--------------- 1: Kp2 dQ/dy dZ/dx d/dQ ( dU/dz ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*Kp2*dZx(ii,jj,Nz)*(-1/(4*dy*dz))*dQy(ii);     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*Kp2*dZx(ii,jj,Nz)*(4/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*Kp2*dZx(ii,jj,Nz)*(-3/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*Kp2*dZx(ii,jj,Nz)*(1/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*Kp2*dZx(ii,jj,Nz)*(-4/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp2*dZx(ii,jj,Nz)*(3/(4*dy*dz))*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)+     alfa2*Kp2*dZx(ii,1,Nz)*(-3/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*Kp2*dZx(ii,1,Nz)*(4/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*Kp2*dZx(ii,1,Nz)*(-1/(4*dy*dz))*dQy(ii);    
    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)+    alfa2*Kp2*dZx(ii,1,Nz)*(12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*Kp2*dZx(ii,1,Nz)*(-16/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*Kp2*dZx(ii,1,Nz)*(4/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*Kp2*dZx(ii,1,Nz)*(-9/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp2*dZx(ii,1,Nz)*(12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*dZx(ii,1,Nz)*(-3/(4*dy*dz))*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*Kp2*dZx(ii,Ny,Nz)*(1/(4*dy*dz))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*Kp2*dZx(ii,Ny,Nz)*(-4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*Kp2*dZx(ii,Ny,Nz)*(3/(4*dy*dz))*dQy(ii);     
    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*Kp2*dZx(ii,Ny,Nz)*(-4/(4*dy*dz))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*Kp2*dZx(ii,Ny,Nz)*(16/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*Kp2*dZx(ii,Ny,Nz)*(-12/(4*dy*dz))*dQy(ii); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*Kp2*dZx(ii,Ny,Nz)*(3/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*dZx(ii,Ny,Nz)*(-12/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*Kp2*dZx(ii,Ny,Nz)*(9/(4*dy*dz))*dQy(ii);                                                                                          
end

%--------------- 2: Kp2 dQ/dy d^2Z/dxdq  dU/dz  -----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*d2Zxq(ii,jj,Nz)*(1/(2*dz))*dQy(ii);     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*d2Zxq(ii,jj,Nz)*(-4/(2*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*d2Zxq(ii,jj,Nz)*(3/(2*dz))*dQy(ii);
end 
end


%------------------- Kp2 dZ/dy d/dz ( dU/dx ) -------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*Kp2*(-1/(4*dz*dx))*dZy(ii,jj,Nz);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*Kp2*(4/(4*dz*dx))*dZy(ii,jj,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+  alfa2*Kp2*(-3/(4*dz*dx))*dZy(ii,jj,Nz); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*(1/(4*dz*dx))*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*(-4/(4*dz*dx))*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*(3/(4*dz*dx))*dZy(ii,jj,Nz); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*(1/(4*dz*dx))*dZy(Nx,jj,Nz);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*(-4/(4*dz*dx))*dZy(Nx,jj,Nz);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*(3/(4*dz*dx))*dZy(Nx,jj,Nz); 
    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*Kp2*(-4/(4*dz*dx))*dZy(Nx,jj,Nz);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*(16/(4*dz*dx))*dZy(Nx,jj,Nz);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*Kp2*(-12/(4*dz*dx))*dZy(Nx,jj,Nz); 

MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+  alfa2*Kp2*(3/(4*dz*dx))*dZy(Nx,jj,Nz);
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*(-12/(4*dz*dx))*dZy(Nx,jj,Nz); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+    alfa2*Kp2*(9/(4*dz*dx))*dZy(Nx,jj,Nz);                                                                                          
end 


%----------------- Kp2 dZ/dy d/dz ( dQ/dx dU/dQ ) ------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*Kp2*dQx(ii,jj)*(-1/(4*dz*dy))*dZy(ii,jj,Nz);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*Kp2*dQx(ii,jj)*(4/(4*dz*dy))*dZy(ii,jj,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*Kp2*dQx(ii,jj)*(-3/(4*dz*dy))*dZy(ii,jj,Nz); 

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*Kp2*dQx(ii,jj)*(1/(4*dz*dy))*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*Kp2*dQx(ii,jj)*(-4/(4*dz*dy))*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp2*dQx(ii,jj)*(3/(4*dz*dy))*dZy(ii,jj,Nz); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)+     alfa2*Kp2*dQx(ii,1)*(-3/(4*dz*dy))*dZy(ii,1,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*Kp2*dQx(ii,1)*(4/(4*dz*dy))*dZy(ii,1,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*Kp2*dQx(ii,1)*(-1/(4*dz*dy))*dZy(ii,1,Nz);    
    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)+    alfa2*Kp2*dQx(ii,1)*(12/(4*dz*dy))*dZy(ii,1,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*Kp2*dQx(ii,1)*(-16/(4*dz*dy))*dZy(ii,1,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*Kp2*dQx(ii,1)*(4/(4*dz*dy))*dZy(ii,1,Nz);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+    alfa2*Kp2*dQx(ii,1)*(-9/(4*dz*dy))*dZy(ii,1,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+ alfa2*Kp2*dQx(ii,1)*(12/(4*dz*dy))*dZy(ii,1,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*dQx(ii,1)*(3/(4*dz*dy))*dZy(ii,1,Nz);

end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*Kp2*dQx(ii,Ny)*(1/(4*dz*dy))*dZy(ii,Ny,Nz);     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*Kp2*dQx(ii,Ny)*(-4/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*Kp2*dQx(ii,Ny)*(3/(4*dz*dy))*dZy(ii,Ny,Nz);    
    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*Kp2*dQx(ii,Ny)*(-4/(4*dz*dy))*dZy(ii,Ny,Nz);     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*Kp2*dQx(ii,Ny)*(16/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*Kp2*dQx(ii,Ny)*(-12/(4*dz*dy))*dZy(ii,Ny,Nz);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*Kp2*dQx(ii,Ny)*(3/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*dQx(ii,Ny)*(-12/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*Kp2*dQx(ii,Ny)*(9/(4*dz*dy))*dZy(ii,Ny,Nz);                                                                                          
end


%---------------- Kp2 dZ/dy d/dz ( dZ/dx dU/dz ) ------------------;

%-------------- 1: Kp2 dZ/dy dZ/dx d/dz ( dU/dz ) ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*Kp2*dZx(ii,jj,Nz)*(-1/dz^2)*dZy(ii,jj,Nz);     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*dZx(ii,jj,Nz)*(4/dz^2)*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*dZx(ii,jj,Nz)*(-5/dz^2)*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*dZx(ii,jj,Nz)*(2/dz^2)*dZy(ii,jj,Nz); 
end 
end

%-------------- 2: Kp2 dZ/dy d^2Z/dxdz  dU/dz ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*d2Zxz(ii,jj,Nz)*(1/(2*dz))*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*d2Zxz(ii,jj,Nz)*(-4/(2*dz))*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*d2Zxz(ii,jj,Nz)*(3/(2*dz))*dZy(ii,jj,Nz); 
end 
end


%---------------- Kp2 dQ/dy d/dQ ( dZ/dz dW/dz ) ------------------;

%-------------- 1: Kp2 dQ/dy dZ/dz d/dQ ( dW/dz ) ------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*Kp2*dZz(ii,jj)*(-1/(4*dy*dz))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*Kp2*dZz(ii,jj)*(4/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*Kp2*dZz(ii,jj)*(-3/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*Kp2*dZz(ii,jj)*(1/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*Kp2*dZz(ii,jj)*(-4/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*Kp2*dZz(ii,jj)*(3/(4*dy*dz))*dQy(ii);                                                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*Kp2*dZz(ii,1)*(-3/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*Kp2*dZz(ii,1)*(4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*Kp2*dZz(ii,1)*(-1/(4*dy*dz))*dQy(ii);    
    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+    alfa2*Kp2*dZz(ii,1)*(12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*Kp2*dZz(ii,1)*(-16/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*Kp2*dZz(ii,1)*(4/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*Kp2*dZz(ii,1)*(-9/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*Kp2*dZz(ii,1)*(12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*dZz(ii,1)*(-3/(4*dy*dz))*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*Kp2*dZz(ii,Ny)*(1/(4*dy*dz))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*Kp2*dZz(ii,Ny)*(-4/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*Kp2*dZz(ii,Ny)*(3/(4*dy*dz))*dQy(ii);    
    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*Kp2*dZz(ii,Ny)*(-4/(4*dy*dz))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*Kp2*dZz(ii,Ny)*(16/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*Kp2*dZz(ii,Ny)*(-12/(4*dy*dz))*dQy(ii);

MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*Kp2*dZz(ii,Ny)*(3/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*dZz(ii,Ny)*(-12/(4*dy*dz))*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*Kp2*dZz(ii,Ny)*(9/(4*dy*dz))*dQy(ii);                                                                                          
end

%-------------- 2: Kp2 dQ/dy d^2Z/dzdq  dW/dz  ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*d2Zzq(ii,jj)*(1/(2*dz))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*d2Zzq(ii,jj)*(-4/(2*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*d2Zzq(ii,jj)*(3/(2*dz))*dQy(ii);
end 
end


%---------------- Kp2 dZ/dy d/dz ( dZ/dz dW/dz ) ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*Kp2*dZz(ii,jj)*(-1/dz^2)*dZy(ii,jj,Nz);     
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*dZz(ii,jj)*(4/dz^2)*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*dZz(ii,jj)*(-5/dz^2)*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*dZz(ii,jj)*(2/dz^2)*dZy(ii,jj,Nz);                                                                              
end 
end



%++++++++++ KpE (-1/H) 3/(2 dz) ( Fyx dHs/dx + Fyy dHs/dy )^Nz +++++++++++;

%---------------- (- 1/H) 3/(2*dz) dHb/dx dQ/dy dU/dQ ----------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*(-1/(2*dy))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*(1/(2*dy))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*(-3/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHbx(ii,1)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*(4/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHbx(ii,1)*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*(-1/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHbx(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(1/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHbx(ii,Ny)*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-4/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHbx(ii,Ny)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(3/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHbx(ii,Ny)*dQy(ii); 
end


%------------ (- 1/H) 3/(2*dz) dHb/dx dZ/dy dU/dz ---------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(1/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZy(ii,jj,Nz);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-4/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(3/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZy(ii,jj,Nz); 
end 
end


%--------------- (- 1/H) 3/(2*dz) dHb/dx dV/dx -----------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(-1/(2*dx))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)+     alfa2*(1/(2*dx))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*(1/(2*dx))*(3/(2*dz))*dZz(Nx,jj)*dHbx(Nx,jj);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(-4/(2*dx))*(3/(2*dz))*dZz(Nx,jj)*dHbx(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*(3/(2*dx))*(3/(2*dz))*dZz(Nx,jj)*dHbx(Nx,jj); 
end 


%------------- (- 1/H) 3/(2*dz) dHb/dx dQ/dx dV/dQ --------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*(-1/(2*dy))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*(1/(2*dy))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*(-3/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHbx(ii,1)*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*(4/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHbx(ii,1)*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*(-1/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHbx(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(1/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHbx(ii,Ny)*dQx(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-4/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHbx(ii,Ny)*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(3/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHbx(ii,Ny)*dQx(ii,Ny); 
end


%-------------- (- 1/H) 3/(2*dz) dHb/dx dZ/dx dV/dz -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(1/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZx(ii,jj,Nz);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-4/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZx(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(3/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZx(ii,jj,Nz); 
end 
end


%------------- Kp1 (-1/H) 3/(2*dz) dHb/dy dQ/dy dV/dQ ---------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp1*(-1/(2*dy))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp1*(1/(2*dy))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*Kp1*(-3/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHby(ii,1)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*Kp1*(4/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHby(ii,1)*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp1*(-1/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHby(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp1*(1/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHby(ii,Ny)*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp1*(-4/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHby(ii,Ny)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp1*(3/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHby(ii,Ny)*dQy(ii); 
end


%------------- Kp1 (-1/H) 3/(2*dz) dHb/dy dZ/dy dV/dz ---------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp1*(1/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZy(ii,jj,Nz);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp1*(-4/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZy(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp1*(3/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZy(ii,jj,Nz); 
end 
end


%--------------- Kp2 (-1/H) 3/(2*dz) dHb/dy dU/dx -----------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*Kp2*(-1/(2*dx))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)+     alfa2*Kp2*(1/(2*dx))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*Kp2*(1/(2*dx))*(3/(2*dz))*dZz(Nx,jj)*dHby(Nx,jj);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*Kp2*(-4/(2*dx))*(3/(2*dz))*dZz(Nx,jj)*dHby(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*Kp2*(3/(2*dx))*(3/(2*dz))*dZz(Nx,jj)*dHby(Nx,jj); 
end 


%------------- Kp2 (-1/H) 3/(2*dz) dHb/dy dQ/dx dU/dQ ----------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*Kp2*(-1/(2*dy))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*Kp2*(1/(2*dy))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*Kp2*(-3/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHby(ii,1)*dQx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*Kp2*(4/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHby(ii,1)*dQx(ii,1);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*Kp2*(-1/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHby(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*Kp2*(1/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHby(ii,Ny)*dQx(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*Kp2*(-4/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHby(ii,Ny)*dQx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*Kp2*(3/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHby(ii,Ny)*dQx(ii,Ny); 
end


%------------- Kp2 (-1/H) 3/(2*dz) dHb/dy dZ/dx dU/dz ----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*(1/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZx(ii,jj,Nz);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*(-4/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZx(ii,jj,Nz); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*(3/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZx(ii,jj,Nz); 
end 
end


%--------------  Kp2 (-1/H) 3/(2*dz) dHb/dy dZ/dz dW/dz -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*(1/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZz(ii,jj);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*(-4/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZz(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*(3/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZz(ii,jj); 
end 
end


%+++++++++++++++ KpE (-1/H) (-4/(2 dz)) ( Fyz )^Nz-1 +++++++++++++++++;

%------------------ (-1/H) (-4/(2*dz)) dZ/dz dV/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*(-1/(2*dz))*(-4/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(1/(2*dz))*(-4/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
end 
end


%----------------- (-1/H) (-4/(2*dz)) dQ/dy dW/dQ -------------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+alfa2*(-1/(2*dy))*(-4/(2*dz))*dZz(ii,jj)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+     alfa2*(1/(2*dy))*(-4/(2*dz))*dZz(ii,jj)*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+     alfa2*(-3/(2*dy))*(-4/(2*dz))*dZz(ii,1)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+   alfa2*(4/(2*dy))*(-4/(2*dz))*dZz(ii,1)*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*(-1/(2*dy))*(-4/(2*dz))*dZz(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*(1/(2*dy))*(-4/(2*dz))*dZz(ii,Ny)*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+alfa2*(-4/(2*dy))*(-4/(2*dz))*dZz(ii,Ny)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+ alfa2*(3/(2*dy))*(-4/(2*dz))*dZz(ii,Ny)*dQy(ii); 
end


%----------------- (-1/H) (-4/(2*dz)) dZ/dy dW/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*(-1/(2*dz))*(-4/(2*dz))*dZz(ii,jj)*dZy(ii,jj,Nz-1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(1/(2*dz))*(-4/(2*dz))*dZz(ii,jj)*dZy(ii,jj,Nz-1); 
end 
end



%+++++++++++++++ KpE (-1/H) (1/(2 dz)) ( Fyz )^Nz-2 +++++++++++++++++;

%------------------ (-1/H) (1/(2*dz)) dZ/dz dV/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*(-1/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*(1/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
end 
end


%----------------- (-1/H) (1/(2*dz)) dQ/dy dW/dQ -------------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*(-1/(2*dy))*(1/(2*dz))*dZz(ii,jj)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+     alfa2*(1/(2*dy))*(1/(2*dz))*dZz(ii,jj)*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*(-3/(2*dy))*(1/(2*dz))*dZz(ii,1)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*(4/(2*dy))*(1/(2*dz))*dZz(ii,1)*dQy(ii);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*(-1/(2*dy))*(1/(2*dz))*dZz(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*(1/(2*dy))*(1/(2*dz))*dZz(ii,Ny)*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*(-4/(2*dy))*(1/(2*dz))*dZz(ii,Ny)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*(3/(2*dy))*(1/(2*dz))*dZz(ii,Ny)*dQy(ii); 
end


%----------------- (-1/H) (1/(2*dz)) dZ/dy dW/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*(-1/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dZy(ii,jj,Nz-2); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*(1/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dZy(ii,jj,Nz-2); 
end 
end



%------------------  KpE (-1/H) 3/(2*dz) dHb/dy P' -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj) + alfa2*KpE*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj);                                                          
end 
end


%--------------------- right-hand term --------------------; 

% for ii=2:1:Nx 
% for jj=1:1:Ny 
%     VB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=alfa2*KpE*dens*g*(3/(2*dz))*dHby(ii,jj);
% end 
% end


%+++++++++++++++++++ + KpE dens d^2/dt^2 V ++++++++++++++++++++++;

for ii=2:1:Nx 
for jj=1:1:Ny 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz) + alfa2*KpE*dens*Freq^2;
end 
end




%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++++++++ The third equation +++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%++++++++++++++++++++++++ KpE dFzx/dx +++++++++++++++++++++++++++;

%--------------------- d/dx ( dZ/dz dU/dz ) ---------------------;

%------------------ 1: dZ/dz d/dx ( dU/dz ) ---------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*dZz(ii,jj)*(-1/(4*dx*dz));     
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*dZz(ii,jj)*(4/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+  alfa2*dZz(ii,jj)*(-3/(4*dx*dz));

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*dZz(ii,jj)*(1/(4*dx*dz));
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZz(ii,jj)*(-4/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*dZz(ii,jj)*(3/(4*dx*dz));                                  
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*dZz(Nx,jj)*(1/(4*dx*dz));    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZz(Nx,jj)*(-4/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*dZz(Nx,jj)*(3/(4*dx*dz));
    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*dZz(Nx,jj)*(-4/(4*dx*dz));    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZz(Nx,jj)*(16/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*dZz(Nx,jj)*(-12/(4*dx*dz));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+  alfa2*dZz(Nx,jj)*(3/(4*dx*dz));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZz(Nx,jj)*(-12/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+    alfa2*dZz(Nx,jj)*(9/(4*dx*dz));                                  
end


%---------------- dQ/dx d/dQ ( dZ/dz dU/dz ) ------------------;

%-------------- 1: dQ/dx dZ/dz d/dQ ( dU/dz ) ------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*dZz(ii,jj)*(-1/(4*dy*dz))*dQx(ii,jj);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*dZz(ii,jj)*(4/(4*dy*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*dZz(ii,jj)*(-3/(4*dy*dz))*dQx(ii,jj);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*dZz(ii,jj)*(1/(4*dy*dz))*dQx(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*dZz(ii,jj)*(-4/(4*dy*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*dZz(ii,jj)*(3/(4*dy*dz))*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)+     alfa2*dZz(ii,1)*(-3/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*dZz(ii,1)*(4/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*dZz(ii,1)*(-1/(4*dy*dz))*dQx(ii,1);    
    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)+    alfa2*dZz(ii,1)*(12/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*dZz(ii,1)*(-16/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*dZz(ii,1)*(4/(4*dy*dz))*dQx(ii,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa2*dZz(ii,1)*(-9/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dZz(ii,1)*(12/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*dZz(ii,1)*(-3/(4*dy*dz))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*dZz(ii,Ny)*(1/(4*dy*dz))*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*dZz(ii,Ny)*(-4/(4*dy*dz))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*dZz(ii,Ny)*(3/(4*dy*dz))*dQx(ii,Ny);    
    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*dZz(ii,Ny)*(-4/(4*dy*dz))*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*dZz(ii,Ny)*(16/(4*dy*dz))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*dZz(ii,Ny)*(-12/(4*dy*dz))*dQx(ii,Ny);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dZz(ii,Ny)*(3/(4*dy*dz))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dZz(ii,Ny)*(-12/(4*dy*dz))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dZz(ii,Ny)*(9/(4*dy*dz))*dQx(ii,Ny);                                                                                          
end

%----------------- 2: dQ/dx d^2Z/dzdq  dU/dz  -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zzq(ii,jj)*(1/(2*dz))*dQx(ii,jj);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zzq(ii,jj)*(-4/(2*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zzq(ii,jj)*(3/(2*dz))*dQx(ii,jj);
end 
end


%----------------- dZ/dx d/dz ( dZ/dz dU/dz ) --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*dZz(ii,jj)*(-1/dz^2)*dZx(ii,jj,Nz);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*dZz(ii,jj)*(4/dz^2)*dZx(ii,jj,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZz(ii,jj)*(-5/dz^2)*dZx(ii,jj,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*dZz(ii,jj)*(2/dz^2)*dZx(ii,jj,Nz); 
end 
end


%---------------------- d/dx ( dW/dx ) -----------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(1/dx^2); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(-2/dx^2); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)+    alfa2*(1/dx^2); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-4)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-4)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(-1/dx^2);    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(4/dx^2); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(-5/dx^2); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(2/dx^2);                                        
end 


%--------------------- d/dx ( dQ/dx dW/dQ ) ---------------------;

%------------------- 1: dQ/dx d/dx ( dW/dQ ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*dQx(ii,jj)*(1/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+Nz)+   alfa2*dQx(ii,jj)*(-1/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+Nz)+   alfa2*dQx(ii,jj)*(-1/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+Nz)=        MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+Nz)+        alfa2*dQx(ii,jj)*(1/(4*dx*dy)); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz)+     alfa2*dQx(ii,1)*(3/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+Nz)+ alfa2*dQx(ii,1)*(-4/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+Nz)+alfa2*dQx(ii,1)*(1/(4*dx*dy)); 

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+Nz)+     alfa2*dQx(ii,1)*(-3/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+Nz+Nz)+   alfa2*dQx(ii,1)*(4/(4*dx*dy));
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+2*Nz+Nz)+alfa2*dQx(ii,1)*(-1/(4*dx*dy));
end

%----- j=Ny ----;

for ii=2:1:Nx-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)+alfa2*dQx(ii,Ny)*(-1/(4*dx*dy));    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*dQx(ii,Ny)*(4/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*dQx(ii,Ny)*(-3/(4*dx*dy));

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*dQx(ii,Ny)*(1/(4*dx*dy));
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQx(ii,Ny)*(-4/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*dQx(ii,Ny)*(3/(4*dx*dy));                                  
end

%---- i=Nx ----;

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz)+     alfa2*dQx(Nx,1)*(-3/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+Nz)+   alfa2*dQx(Nx,1)*(4/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+Nz)+alfa2*dQx(Nx,1)*(-1/(4*dx*dy));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz)+    alfa2*dQx(Nx,1)*(12/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+Nz)+alfa2*dQx(Nx,1)*(-16/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+Nz)+alfa2*dQx(Nx,1)*(4/(4*dx*dy));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz)+     alfa2*dQx(Nx,1)*(-9/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+Nz)+  alfa2*dQx(Nx,1)*(12/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+Nz)+alfa2*dQx(Nx,1)*(-3/(4*dx*dy));

for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*dQx(Nx,jj)*(-1/(4*dx*dy));    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*dQx(Nx,jj)*(4/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*dQx(Nx,jj)*(-3/(4*dx*dy));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+Nz)+ alfa2*dQx(Nx,jj)*(1/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+Nz)+alfa2*dQx(Nx,jj)*(-4/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+Nz)+ alfa2*dQx(Nx,jj)*(3/(4*dx*dy));                                  
end 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*dQx(Nx,Ny)*(1/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQx(Nx,Ny)*(-4/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*dQx(Nx,Ny)*(3/(4*dx*dy));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*dQx(Nx,Ny)*(-4/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*dQx(Nx,Ny)*(16/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*dQx(Nx,Ny)*(-12/(4*dx*dy));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dQx(Nx,Ny)*(3/(4*dx*dy));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQx(Nx,Ny)*(-12/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dQx(Nx,Ny)*(9/(4*dx*dy)); 

%------------------- 2: d^2Q/dxdx  dW/dQ  ---------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*d2Qxx(ii,jj)*(-1/(2*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*d2Qxx(ii,jj)*(1/(2*dy)); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*d2Qxx(ii,1)*(-3/(2*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*d2Qxx(ii,1)*(4/(2*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*d2Qxx(ii,1)*(-1/(2*dy)); 
end

%----- j=Ny ----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*d2Qxx(ii,Ny)*(1/(2*dy));    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*d2Qxx(ii,Ny)*(-4/(2*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*d2Qxx(ii,Ny)*(3/(2*dy));
end


%--------------------- d/dx ( dZ/dx dW/dz ) ---------------------;

%------------------- 1: dZ/dx d/dx ( dW/dz ) ---------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*dZx(ii,jj,Nz)*(-1/(4*dx*dz));     
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*dZx(ii,jj,Nz)*(4/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+  alfa2*dZx(ii,jj,Nz)*(-3/(4*dx*dz));

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*dZx(ii,jj,Nz)*(1/(4*dx*dz));
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZx(ii,jj,Nz)*(-4/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*dZx(ii,jj,Nz)*(3/(4*dx*dz)); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*dZx(Nx,jj,Nz)*(1/(4*dx*dz));    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZx(Nx,jj,Nz)*(-4/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*dZx(Nx,jj,Nz)*(3/(4*dx*dz));    
    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*dZx(Nx,jj,Nz)*(-4/(4*dx*dz));    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZx(Nx,jj,Nz)*(16/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*dZx(Nx,jj,Nz)*(-12/(4*dx*dz));

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+  alfa2*dZx(Nx,jj,Nz)*(3/(4*dx*dz));
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZx(Nx,jj,Nz)*(-12/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+    alfa2*dZx(Nx,jj,Nz)*(9/(4*dx*dz));                                  
end

%------------------- 2: d^2Z/dxdx  dW/dz  ---------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zxx(ii,jj,Nz)*(1/(2*dz));     
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zxx(ii,jj,Nz)*(-4/(2*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zxx(ii,jj,Nz)*(3/(2*dz));
end 
end


%-------------------- dQ/dx d/dQ ( dW/dx ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*(1/(4*dy*dx))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+Nz)+   alfa2*(-1/(4*dy*dx))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+Nz)+   alfa2*(-1/(4*dy*dx))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+Nz)=        MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+Nz)+        alfa2*(1/(4*dy*dx))*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz)+     alfa2*(3/(4*dy*dx))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+Nz+Nz)+ alfa2*(-4/(4*dy*dx))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+2*Nz+Nz)+alfa2*(1/(4*dy*dx))*dQx(ii,1);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+Nz)+     alfa2*(-3/(4*dy*dx))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+Nz+Nz)+   alfa2*(4/(4*dy*dx))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+2*Nz+Nz)+alfa2*(-1/(4*dy*dx))*dQx(ii,1);
end

%----- j=Ny ----;

for ii=2:1:Nx-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-3)*Nz+Nz)+alfa2*(-1/(4*dy*dx))*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*(4/(4*dy*dx))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*(-3/(4*dy*dx))*dQx(ii,Ny);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(1/(4*dy*dx))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-4/(4*dy*dx))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(3/(4*dy*dx))*dQx(ii,Ny);                                                                                          
end

%---- i=Nx ----;

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz)+     alfa2*(-3/(4*dy*dx))*dQx(Nx,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+Nz+Nz)+   alfa2*(4/(4*dy*dx))*dQx(Nx,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+2*Nz+Nz)+alfa2*(-1/(4*dy*dx))*dQx(Nx,1);

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz)+    alfa2*(12/(4*dy*dx))*dQx(Nx,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+Nz+Nz)+alfa2*(-16/(4*dy*dx))*dQx(Nx,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+2*Nz+Nz)+alfa2*(4/(4*dy*dx))*dQx(Nx,1);

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz)+     alfa2*(-9/(4*dy*dx))*dQx(Nx,1); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz+Nz)+  alfa2*(12/(4*dy*dx))*dQx(Nx,1);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+2*Nz+Nz)+alfa2*(-3/(4*dy*dx))*dQx(Nx,1);

for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*(-1/(4*dy*dx))*dQx(Nx,jj);    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*(4/(4*dy*dx))*dQx(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*(-3/(4*dy*dx))*dQx(Nx,jj);

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+jj*Nz+Nz)+ alfa2*(1/(4*dy*dx))*dQx(Nx,jj);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+jj*Nz+Nz)+alfa2*(-4/(4*dy*dx))*dQx(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+jj*Nz+Nz)+ alfa2*(3/(4*dy*dx))*dQx(Nx,jj);                                                                                          
end 

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(1/(4*dy*dx))*dQx(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(3/(4*dy*dx))*dQx(Nx,Ny);

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(-4/(4*dy*dx))*dQx(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-2)*Nz+Nz)+ alfa2*(16/(4*dy*dx))*dQx(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(Ny-1)*Nz+Nz)+alfa2*(-12/(4*dy*dx))*dQx(Nx,Ny);

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*(3/(4*dy*dx))*dQx(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-12/(4*dy*dx))*dQx(Nx,Ny);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*(9/(4*dy*dx))*dQx(Nx,Ny);


%------------------- dQ/dx d/dQ ( dQ/dx dW/dQ ) --------------------;

%----------------- 1: dQ/dx dQ/dx d/dQ ( dW/dQ ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*(1/dy^2)*dQx(ii,jj)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(-2/dy^2)*dQx(ii,jj)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*(1/dy^2)*dQx(ii,jj)^2; 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+      alfa2*(2/dy^2)*dQx(ii,1)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*(-5/dy^2)*dQx(ii,1)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+ alfa2*(4/dy^2)*dQx(ii,1)^2;
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+Nz)+alfa2*(-1/dy^2)*dQx(ii,1)^2;
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)+alfa2*(-1/dy^2)*dQx(ii,Ny)^2;    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(4/dy^2)*dQx(ii,Ny)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-5/dy^2)*dQx(ii,Ny)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(2/dy^2)*dQx(ii,Ny)^2;                                                                                          
end

%----------------- 2: dQ/dx d^2Q/dxdq  dW/dQ  --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*d2Qxq(ii,jj)*(-1/(2*dy))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*d2Qxq(ii,jj)*(1/(2*dy))*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*d2Qxq(ii,1)*(-3/(2*dy))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*d2Qxq(ii,1)*(4/(2*dy))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*d2Qxq(ii,1)*(-1/(2*dy))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*d2Qxq(ii,Ny)*(1/(2*dy))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*d2Qxq(ii,Ny)*(-4/(2*dy))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*d2Qxq(ii,Ny)*(3/(2*dy))*dQx(ii,Ny);                                                                                          
end


%-------------------- dQ/dx d/dQ ( dZ/dx dW/dz ) --------------------;

%------------------ 1: dQ/dx dZ/dx d/dQ ( dW/dz ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*dZx(ii,jj,Nz)*(-1/(4*dy*dz))*dQx(ii,jj);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*dZx(ii,jj,Nz)*(4/(4*dy*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*dZx(ii,jj,Nz)*(-3/(4*dy*dz))*dQx(ii,jj);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*dZx(ii,jj,Nz)*(1/(4*dy*dz))*dQx(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*dZx(ii,jj,Nz)*(-4/(4*dy*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*dZx(ii,jj,Nz)*(3/(4*dy*dz))*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*dZx(ii,1,Nz)*(-3/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*dZx(ii,1,Nz)*(4/(4*dy*dz))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*dZx(ii,1,Nz)*(-1/(4*dy*dz))*dQx(ii,1);
    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+    alfa2*dZx(ii,1,Nz)*(12/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*dZx(ii,1,Nz)*(-16/(4*dy*dz))*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*dZx(ii,1,Nz)*(4/(4*dy*dz))*dQx(ii,1); 

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*dZx(ii,1,Nz)*(-9/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dZx(ii,1,Nz)*(12/(4*dy*dz))*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*dZx(ii,1,Nz)*(-3/(4*dy*dz))*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*dZx(ii,Ny,Nz)*(1/(4*dy*dz))*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*dZx(ii,Ny,Nz)*(-4/(4*dy*dz))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*dZx(ii,Ny,Nz)*(3/(4*dy*dz))*dQx(ii,Ny);    
    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*dZx(ii,Ny,Nz)*(-4/(4*dy*dz))*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*dZx(ii,Ny,Nz)*(16/(4*dy*dz))*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*dZx(ii,Ny,Nz)*(-12/(4*dy*dz))*dQx(ii,Ny);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dZx(ii,Ny,Nz)*(3/(4*dy*dz))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dZx(ii,Ny,Nz)*(-12/(4*dy*dz))*dQx(ii,Ny);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dZx(ii,Ny,Nz)*(9/(4*dy*dz))*dQx(ii,Ny);                                                                                          
end

%---------------- 2: dQ/dx d^2Z/dxdq  dW/dz  ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zxq(ii,jj,Nz)*(1/(2*dz))*dQx(ii,jj);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zxq(ii,jj,Nz)*(-4/(2*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zxq(ii,jj,Nz)*(3/(2*dz))*dQx(ii,jj);
end 
end


%-------------------- dZ/dx d/dz ( dW/dx ) --------------------;

for ii=2:1:Nx-1 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*(-1/(4*dz*dx))*dZx(ii,jj,Nz);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*(4/(4*dz*dx))*dZx(ii,jj,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+  alfa2*(-3/(4*dz*dx))*dZx(ii,jj,Nz);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(1/(4*dz*dx))*dZx(ii,jj,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-4/(4*dz*dx))*dZx(ii,jj,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(3/(4*dz*dx))*dZx(ii,jj,Nz); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(1/(4*dz*dx))*dZx(Nx,jj,Nz);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-4/(4*dz*dx))*dZx(Nx,jj,Nz);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(3/(4*dz*dx))*dZx(Nx,jj,Nz);    
    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*(-4/(4*dz*dx))*dZx(Nx,jj,Nz);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(16/(4*dz*dx))*dZx(Nx,jj,Nz);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*(-12/(4*dz*dx))*dZx(Nx,jj,Nz);

MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+  alfa2*(3/(4*dz*dx))*dZx(Nx,jj,Nz);
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-12/(4*dz*dx))*dZx(Nx,jj,Nz); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+    alfa2*(9/(4*dz*dx))*dZx(Nx,jj,Nz);                                                                                          
end 


%-------------------- dZ/dx d/dz ( dQ/dx dW/dQ ) --------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*dQx(ii,jj)*(-1/(4*dz*dy))*dZx(ii,jj,Nz);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*dQx(ii,jj)*(4/(4*dz*dy))*dZx(ii,jj,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*dQx(ii,jj)*(-3/(4*dz*dy))*dZx(ii,jj,Nz);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*dQx(ii,jj)*(1/(4*dz*dy))*dZx(ii,jj,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*dQx(ii,jj)*(-4/(4*dz*dy))*dZx(ii,jj,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*dQx(ii,jj)*(3/(4*dz*dy))*dZx(ii,jj,Nz); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*dQx(ii,1)*(-3/(4*dz*dy))*dZx(ii,1,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*dQx(ii,1)*(4/(4*dz*dy))*dZx(ii,1,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*dQx(ii,1)*(-1/(4*dz*dy))*dZx(ii,1,Nz);    
    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+    alfa2*dQx(ii,1)*(12/(4*dz*dy))*dZx(ii,1,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*dQx(ii,1)*(-16/(4*dz*dy))*dZx(ii,1,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*dQx(ii,1)*(4/(4*dz*dy))*dZx(ii,1,Nz);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*dQx(ii,1)*(-9/(4*dz*dy))*dZx(ii,1,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dQx(ii,1)*(12/(4*dz*dy))*dZx(ii,1,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*dQx(ii,1)*(-3/(4*dz*dy))*dZx(ii,1,Nz); 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*dQx(ii,Ny)*(1/(4*dz*dy))*dZx(ii,Ny,Nz);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*dQx(ii,Ny)*(-4/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*dQx(ii,Ny)*(3/(4*dz*dy))*dZx(ii,Ny,Nz);    
    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*dQx(ii,Ny)*(-4/(4*dz*dy))*dZx(ii,Ny,Nz);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*dQx(ii,Ny)*(16/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*dQx(ii,Ny)*(-12/(4*dz*dy))*dZx(ii,Ny,Nz); 

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dQx(ii,Ny)*(3/(4*dz*dy))*dZx(ii,Ny,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQx(ii,Ny)*(-12/(4*dz*dy))*dZx(ii,Ny,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dQx(ii,Ny)*(9/(4*dz*dy))*dZx(ii,Ny,Nz);                                                                                          
end


%-------------------- dZ/dx d/dz ( dZ/dx dW/dz ) --------------------;

%------------------ 1: dZ/dx dZ/dx d/dz ( dW/dz ) --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*(-1/dz^2)*dZx(ii,jj,Nz)^2;    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(4/dz^2)*dZx(ii,jj,Nz)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-5/dz^2)*dZx(ii,jj,Nz)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(2/dz^2)*dZx(ii,jj,Nz)^2; 
end 
end

%------------------ 2: dZ/dx d^2Z/dxdz  dW/dz --------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zxz(ii,jj,Nz)*(1/(2*dz))*dZx(ii,jj,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zxz(ii,jj,Nz)*(-4/(2*dz))*dZx(ii,jj,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zxz(ii,jj,Nz)*(3/(2*dz))*dZx(ii,jj,Nz); 
end 
end



%+++++++++++++++++++++++++ KpE dFzy/dy +++++++++++++++++++++++++++;

%------------------ dQ/dy d/dQ ( dZ/dz dV/dz ) -------------------;

%----------------- 1: dQ/dy dZ/dz d/dQ ( dV/dz ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*dZz(ii,jj)*(-1/(4*dy*dz))*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*dZz(ii,jj)*(4/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*dZz(ii,jj)*(-3/(4*dy*dz))*dQy(ii); 

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*dZz(ii,jj)*(1/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*dZz(ii,jj)*(-4/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*dZz(ii,jj)*(3/(4*dy*dz))*dQy(ii);                                                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*dZz(ii,1)*(-3/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*dZz(ii,1)*(4/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*dZz(ii,1)*(-1/(4*dy*dz))*dQy(ii);    
    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+    alfa2*dZz(ii,1)*(12/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*dZz(ii,1)*(-16/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*dZz(ii,1)*(4/(4*dy*dz))*dQy(ii);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*dZz(ii,1)*(-9/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dZz(ii,1)*(12/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*dZz(ii,1)*(-3/(4*dy*dz))*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*dZz(ii,Ny)*(1/(4*dy*dz))*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*dZz(ii,Ny)*(-4/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*dZz(ii,Ny)*(3/(4*dy*dz))*dQy(ii);    
    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*dZz(ii,Ny)*(-4/(4*dy*dz))*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*dZz(ii,Ny)*(16/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*dZz(ii,Ny)*(-12/(4*dy*dz))*dQy(ii); 

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dZz(ii,Ny)*(3/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dZz(ii,Ny)*(-12/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dZz(ii,Ny)*(9/(4*dy*dz))*dQy(ii);                                                                                          
end

%---------------- 2: dQ/dy d^2Z/dzdq  dV/dz  ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zzq(ii,jj)*(1/(2*dz))*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zzq(ii,jj)*(-4/(2*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zzq(ii,jj)*(3/(2*dz))*dQy(ii); 
end 
end


%----------------- dZ/dy d/dz ( dZ/dz dV/dz ) ------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*dZz(ii,jj)*(-1/dz^2)*dZy(ii,jj,Nz);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*dZz(ii,jj)*(4/dz^2)*dZy(ii,jj,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*dZz(ii,jj)*(-5/dz^2)*dZy(ii,jj,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*dZz(ii,jj)*(2/dz^2)*dZy(ii,jj,Nz);                                                                              
end 
end


%---------------- dQ/dy d/dQ ( dQ/dy dW/dQ ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+ alfa2*(1/dy^2)*dQy(ii)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(-2/dy^2)*dQy(ii)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*(1/dy^2)*dQy(ii)^2; 
end 
end

%----- j=1 -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+      alfa2*(2/dy^2)*dQy(ii)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*(-5/dy^2)*dQy(ii)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+ alfa2*(4/dy^2)*dQy(ii)^2;
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+3*Nz+Nz)+alfa2*(-1/dy^2)*dQy(ii)^2;
end

%----- j=Ny -----;

for ii=2:1:Nx 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-4)*Nz+Nz)+alfa2*(-1/dy^2)*dQy(ii)^2;    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(4/dy^2)*dQy(ii)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-5/dy^2)*dQy(ii)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(2/dy^2)*dQy(ii)^2;                                                                                          
end


%--------------- dQ/dy d/dQ ( dZ/dy dW/dz ) -----------------;

%------------- 1: dQ/dy dZ/dy d/dQ ( dW/dz ) -----------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*dZy(ii,jj,Nz)*(-1/(4*dy*dz))*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*dZy(ii,jj,Nz)*(4/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*dZy(ii,jj,Nz)*(-3/(4*dy*dz))*dQy(ii);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*dZy(ii,jj,Nz)*(1/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*dZy(ii,jj,Nz)*(-4/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*dZy(ii,jj,Nz)*(3/(4*dy*dz))*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*dZy(ii,1,Nz)*(-3/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*dZy(ii,1,Nz)*(4/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*dZy(ii,1,Nz)*(-1/(4*dy*dz))*dQy(ii);    
    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+    alfa2*dZy(ii,1,Nz)*(12/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*dZy(ii,1,Nz)*(-16/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*dZy(ii,1,Nz)*(4/(4*dy*dz))*dQy(ii);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*dZy(ii,1,Nz)*(-9/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dZy(ii,1,Nz)*(12/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*dZy(ii,1,Nz)*(-3/(4*dy*dz))*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*dZy(ii,Ny,Nz)*(1/(4*dy*dz))*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*dZy(ii,Ny,Nz)*(-4/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*dZy(ii,Ny,Nz)*(3/(4*dy*dz))*dQy(ii);    
    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*dZy(ii,Ny,Nz)*(-4/(4*dy*dz))*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*dZy(ii,Ny,Nz)*(16/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*dZy(ii,Ny,Nz)*(-12/(4*dy*dz))*dQy(ii);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dZy(ii,Ny,Nz)*(3/(4*dy*dz))*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dZy(ii,Ny,Nz)*(-12/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dZy(ii,Ny,Nz)*(9/(4*dy*dz))*dQy(ii);                                                                                          
end

%-------------- 2: dQ/dy d^2Z/dydq  dW/dz -----------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zyq(ii,jj,Nz)*(1/(2*dz))*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zyq(ii,jj,Nz)*(-4/(2*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zyq(ii,jj,Nz)*(3/(2*dz))*dQy(ii);
end 
end


%--------------- dZ/dy d/dz ( dQ/dy dW/dQ ) -------------------;

for ii=2:1:Nx 
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZy(ii,jj,Nz);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+ alfa2*dQy(ii)*(4/(4*dz*dy))*dZy(ii,jj,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+  alfa2*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,jj,Nz);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+ alfa2*dQy(ii)*(1/(4*dz*dy))*dZy(ii,jj,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+alfa2*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,jj,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+   alfa2*dQy(ii)*(3/(4*dz*dy))*dZy(ii,jj,Nz); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,1,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*dQy(ii)*(4/(4*dz*dy))*dZy(ii,1,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*dQy(ii)*(-1/(4*dz*dy))*dZy(ii,1,Nz);    
    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+    alfa2*dQy(ii)*(12/(4*dz*dy))*dZy(ii,1,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+alfa2*dQy(ii)*(-16/(4*dz*dy))*dZy(ii,1,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*dQy(ii)*(4/(4*dz*dy))*dZy(ii,1,Nz);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*dQy(ii)*(-9/(4*dz*dy))*dZy(ii,1,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+  alfa2*dQy(ii)*(12/(4*dz*dy))*dZy(ii,1,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*dQy(ii)*(-3/(4*dz*dy))*dZy(ii,1,Nz);

end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*dQy(ii)*(1/(4*dz*dy))*dZy(ii,Ny,Nz);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*dQy(ii)*(3/(4*dz*dy))*dZy(ii,Ny,Nz);    
    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*dQy(ii)*(-4/(4*dz*dy))*dZy(ii,Ny,Nz);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+ alfa2*dQy(ii)*(16/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+alfa2*dQy(ii)*(-12/(4*dz*dy))*dZy(ii,Ny,Nz);

MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+  alfa2*dQy(ii)*(3/(4*dz*dy))*dZy(ii,Ny,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*dQy(ii)*(-12/(4*dz*dy))*dZy(ii,Ny,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+  alfa2*dQy(ii)*(9/(4*dz*dy))*dZy(ii,Ny,Nz);                                                                                          
end


%--------------- dZ/dy d/dz ( dZ/dy dW/dz ) -------------------;

%------------- 1: dZ/dy dZ/dy d/dz ( dW/dz ) -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*(-1/dz^2)*dZy(ii,jj,Nz)^2;    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(4/dz^2)*dZy(ii,jj,Nz)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-5/dz^2)*dZy(ii,jj,Nz)^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(2/dz^2)*dZy(ii,jj,Nz)^2; 
end 
end

%------------- 2: dZ/dy d^2Z/dydz  dW/dz -------------------;

for ii=2:1:Nx 
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*d2Zyz(ii,jj,Nz)*(1/(2*dz))*dZy(ii,jj,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*d2Zyz(ii,jj,Nz)*(-4/(2*dz))*dZy(ii,jj,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*d2Zyz(ii,jj,Nz)*(3/(2*dz))*dZy(ii,jj,Nz); 
end 
end



%++++++++++ KpE (-1/H) 3/(2 dz) ( Fzx dHb/dx + Fzy dHb/dy )^Nz ++++++++++;

%----------------- (- 1/H) 3/(2*dz) dHb/dx dZ/dz dU/dz -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(1/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZz(ii,jj);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-4/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(3/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZz(ii,jj); 
end 
end


%------------------- (- 1/H) 3/(2*dz) dHb/dx dW/dx --------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(-1/(2*dx))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)+     alfa2*(1/(2*dx))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*(1/(2*dx))*(3/(2*dz))*dZz(Nx,jj)*dHbx(Nx,jj);    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa2*(-4/(2*dx))*(3/(2*dz))*dZz(Nx,jj)*dHbx(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa2*(3/(2*dx))*(3/(2*dz))*dZz(Nx,jj)*dHbx(Nx,jj); 
end 


%----------------- (- 1/H) 3/(2*dz) dHb/dx dQ/dx dW/dQ -----------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*(-1/(2*dy))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*(1/(2*dy))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dQx(ii,jj); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*(-3/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHbx(ii,1)*dQx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*(4/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHbx(ii,1)*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*(-1/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHbx(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(1/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHbx(ii,Ny)*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-4/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHbx(ii,Ny)*dQx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(3/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHbx(ii,Ny)*dQx(ii,Ny); 
end


%-------------- (- 1/H) 3/(2*dz) dHb/dx dZ/dx dW/dz --------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(1/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZx(ii,jj,Nz);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-4/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZx(ii,jj,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(3/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj)*dZx(ii,jj,Nz); 
end 
end


%-------------- (- 1/H) 3/(2*dz) dHb/dy dZ/dz dV/dz --------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(1/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZz(ii,jj);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-4/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(3/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZz(ii,jj); 
end 
end


%-------------- (- 1/H) 3/(2*dz) dHb/dy dQ/dy dW/dQ ---------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa2*(-1/(2*dy))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa2*(1/(2*dy))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dQy(ii); 
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa2*(-3/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHby(ii,1)*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa2*(4/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHby(ii,1)*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa2*(-1/(2*dy))*(3/(2*dz))*dZz(ii,1)*dHby(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa2*(1/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHby(ii,Ny)*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa2*(-4/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHby(ii,Ny)*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa2*(3/(2*dy))*(3/(2*dz))*dZz(ii,Ny)*dHby(ii,Ny)*dQy(ii); 
end


%-------------- (- 1/H) 3/(2*dz) dHb/dy dZ/dy dW/dz ---------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*(1/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZy(ii,jj,Nz);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*(-4/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZy(ii,jj,Nz); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*(3/(2*dz))*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj)*dZy(ii,jj,Nz); 
end 
end



%+++++++++++++++ KpE (-1/H) (-4/(2 dz)) ( Fzz )^Nz-1 ++++++++++++++++++;

%---------------- Kp1 (-1/H) (-4/(2*dz)) dZ/dz dW/dz -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*Kp1*(-1/(2*dz))*(-4/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp1*(1/(2*dz))*(-4/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
end 
end


%----------------- Kp2 (-1/H) (-4/(2*dz)) dU/dx ------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*(-1/(2*dx))*(-4/(2*dz))*dZz(ii,jj);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-1)+     alfa2*Kp2*(1/(2*dx))*(-4/(2*dz))*dZz(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*Kp2*(1/(2*dx))*(-4/(2*dz))*dZz(Nx,jj);     
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa2*Kp2*(-4/(2*dx))*(-4/(2*dz))*dZz(Nx,jj);            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*Kp2*(3/(2*dx))*(-4/(2*dz))*dZz(Nx,jj);                                                          
end 


%--------------- Kp2 (-1/H) (-4/(2*dz)) dQ/dx dU/dQ ----------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+alfa2*Kp2*(-1/(2*dy))*(-4/(2*dz))*dZz(ii,jj)*dQx(ii,jj);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-1)+     alfa2*Kp2*(1/(2*dy))*(-4/(2*dz))*dZz(ii,jj)*dQx(ii,jj);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-1)+     alfa2*Kp2*(-3/(2*dy))*(-4/(2*dz))*dZz(ii,1)*dQx(ii,1);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-1)+   alfa2*Kp2*(4/(2*dy))*(-4/(2*dz))*dZz(ii,1)*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*Kp2*(-1/(2*dy))*(-4/(2*dz))*dZz(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*Kp2*(1/(2*dy))*(-4/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+alfa2*Kp2*(-4/(2*dy))*(-4/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+ alfa2*Kp2*(3/(2*dy))*(-4/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);                                                          
end


%---------------- Kp2 (-1/H) (-4/(2*dz)) dZ/dx dU/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*Kp2*(-1/(2*dz))*(-4/(2*dz))*dZz(ii,jj)*dZx(ii,jj,Nz-1);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*(1/(2*dz))*(-4/(2*dz))*dZz(ii,jj)*dZx(ii,jj,Nz-1);                                                          
end 
end


%---------------- Kp2 (-1/H) (-4/(2*dz)) dQ/dy dV/dQ ------------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-1)+alfa2*Kp2*(-1/(2*dy))*(-4/(2*dz))*dZz(ii,jj)*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-1)+     alfa2*Kp2*(1/(2*dy))*(-4/(2*dz))*dZz(ii,jj)*dQy(ii);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-1)+     alfa2*Kp2*(-3/(2*dy))*(-4/(2*dz))*dZz(ii,1)*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-1)+   alfa2*Kp2*(4/(2*dy))*(-4/(2*dz))*dZz(ii,1)*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-1)+alfa2*Kp2*(-1/(2*dy))*(-4/(2*dz))*dZz(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-1)+ alfa2*Kp2*(1/(2*dy))*(-4/(2*dz))*dZz(ii,Ny)*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-1)+alfa2*Kp2*(-4/(2*dy))*(-4/(2*dz))*dZz(ii,Ny)*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-1)+ alfa2*Kp2*(3/(2*dy))*(-4/(2*dz))*dZz(ii,Ny)*dQy(ii);                                                          
end


%----------------- Kp2 (-1/H) (-4/(2*dz)) dZ/dy dV/dz ---------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*Kp2*(-1/(2*dz))*(-4/(2*dz))*dZz(ii,jj)*dZy(ii,jj,Nz-1);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa2*Kp2*(1/(2*dz))*(-4/(2*dz))*dZz(ii,jj)*dZy(ii,jj,Nz-1);                                                          
end 
end



%+++++++++++++++ KpE (-1/H) 1/(2 dz) ( Fzz )^Nz-2 ++++++++++++++++++;

%---------------- Kp1 (-1/H) 1/(2*dz) dZ/dz dW/dz -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*Kp1*(-1/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*Kp1*(1/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dZz(ii,jj); 
end 
end


%----------------- Kp2 (-1/H) 1/(2*dz) dU/dx ------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*Kp2*(-1/(2*dx))*(1/(2*dz))*dZz(ii,jj);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-2)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz-2)+     alfa2*Kp2*(1/(2*dx))*(1/(2*dz))*dZz(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*(1/(2*dx))*(1/(2*dz))*dZz(Nx,jj);     
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz-2)+alfa2*Kp2*(-4/(2*dx))*(1/(2*dz))*dZz(Nx,jj);            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa2*Kp2*(3/(2*dx))*(1/(2*dz))*dZz(Nx,jj);                                                          
end 


%--------------- Kp2 (-1/H) 1/(2*dz) dQ/dx dU/dQ ----------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*Kp2*(-1/(2*dy))*(1/(2*dz))*dZz(ii,jj)*dQx(ii,jj);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz-2)+     alfa2*Kp2*(1/(2*dy))*(1/(2*dz))*dZz(ii,jj)*dQx(ii,jj);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz-2)+     alfa2*Kp2*(-3/(2*dy))*(1/(2*dz))*dZz(ii,1)*dQx(ii,1);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*Kp2*(4/(2*dy))*(1/(2*dz))*dZz(ii,1)*dQx(ii,1);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*Kp2*(-1/(2*dy))*(1/(2*dz))*dZz(ii,1)*dQx(ii,1);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*Kp2*(1/(2*dy))*(1/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*Kp2*(-4/(2*dy))*(1/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*Kp2*(3/(2*dy))*(1/(2*dz))*dZz(ii,Ny)*dQx(ii,Ny);                                                          
end


%---------------- Kp2 (-1/H) 1/(2*dz) dZ/dx dU/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*Kp2*(-1/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dZx(ii,jj,Nz-2);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*Kp2*(1/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dZx(ii,jj,Nz-2);                                                          
end 
end


%---------------- Kp2 (-1/H) 1/(2*dz) dQ/dy dV/dQ ------------------;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz-2)+alfa2*Kp2*(-1/(2*dy))*(1/(2*dz))*dZz(ii,jj)*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz-2)+     alfa2*Kp2*(1/(2*dy))*(1/(2*dz))*dZz(ii,jj)*dQy(ii);                                                          
end 
end

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz-2)+     alfa2*Kp2*(-3/(2*dy))*(1/(2*dz))*dZz(ii,1)*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz-2)+   alfa2*Kp2*(4/(2*dy))*(1/(2*dz))*dZz(ii,1)*dQy(ii);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz-2)+alfa2*Kp2*(-1/(2*dy))*(1/(2*dz))*dZz(ii,1)*dQy(ii);
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz-2)+ alfa2*Kp2*(1/(2*dy))*(1/(2*dz))*dZz(ii,Ny)*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz-2)+alfa2*Kp2*(-4/(2*dy))*(1/(2*dz))*dZz(ii,Ny)*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz-2)+ alfa2*Kp2*(3/(2*dy))*(1/(2*dz))*dZz(ii,Ny)*dQy(ii);                                                          
end


%----------------- Kp2 (-1/H) 1/(2*dz) dZ/dy dV/dz ---------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-3)+alfa2*Kp2*(-1/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dZy(ii,jj,Nz-2);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+ alfa2*Kp2*(1/(2*dz))*(1/(2*dz))*dZz(ii,jj)*dZy(ii,jj,Nz-2);                                                          
end 
end



%------------------ - KpE (-1/H) 3/(2 dz) P' -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj) - alfa2*KpE*(3/(2*dz))*dZz(ii,jj);                                                          
end 
end



%+++++++++++++++++++ + KpE dens d^2/dt^2 W ++++++++++++++++++++++;

for ii=2:1:Nx 
for jj=1:1:Ny 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz) + alfa2*KpE*dens*Freq^2;
end 
end




%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++ Basic Boundary Conditions at Ice Shelf Base ++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++ The first equation (basic equation #1) +++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

 
%++++++++++++++++++++++ KpE Fxx dHb/dx ++++++++++++++++++++++;

%--------------------- Kp1 dU/dx dHb/dx ---------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa1*Kp1*(-1/(2*dx))*dHbx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)+     alfa1*Kp1*(1/(2*dx))*dHbx(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa1*Kp1*(1/(2*dx))*dHbx(Nx,jj);       
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa1*Kp1*(-4/(2*dx))*dHbx(Nx,jj);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa1*Kp1*(3/(2*dx))*dHbx(Nx,jj);                                                          
end 


%---------------- Kp1 dQ/dx dU/dQ dHb/dx ----------------;

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa1*Kp1*(-3/(2*dy))*dQx(ii,1)*dHbx(ii,1);     
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+   alfa1*Kp1*(4/(2*dy))*dQx(ii,1)*dHbx(ii,1);            
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa1*Kp1*(-1/(2*dy))*dQx(ii,1)*dHbx(ii,1);                                                          
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa1*Kp1*(-1/(2*dy))*dQx(ii,jj)*dHbx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa1*Kp1*(1/(2*dy))*dQx(ii,jj)*dHbx(ii,jj);                                                          
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa1*Kp1*(1/(2*dy))*dQx(ii,Ny)*dHbx(ii,Ny);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa1*Kp1*(-4/(2*dy))*dQx(ii,Ny)*dHbx(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa1*Kp1*(3/(2*dy))*dQx(ii,Ny)*dHbx(ii,Ny);                                                          
end


%--------------- Kp1 dZ/dx dU/dz dHb/dx ----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa1*Kp1*(1/(2*dz))*dZx(ii,jj,Nz)*dHbx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa1*Kp1*(-4/(2*dz))*dZx(ii,jj,Nz)*dHbx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa1*Kp1*(3/(2*dz))*dZx(ii,jj,Nz)*dHbx(ii,jj);                                                          
end 
end


%--------------- Kp2 dQ/dy dV/dQ dHb/dx -----------------;

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa1*Kp2*(-3/(2*dy))*dQy(ii)*dHbx(ii,1);     
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa1*Kp2*(4/(2*dy))*dQy(ii)*dHbx(ii,1);            
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa1*Kp2*(-1/(2*dy))*dQy(ii)*dHbx(ii,1);                                                          
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa1*Kp2*(-1/(2*dy))*dQy(ii)*dHbx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa1*Kp2*(1/(2*dy))*dQy(ii)*dHbx(ii,jj);                                                          
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa1*Kp2*(1/(2*dy))*dQy(ii)*dHbx(ii,Ny);     
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa1*Kp2*(-4/(2*dy))*dQy(ii)*dHbx(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa1*Kp2*(3/(2*dy))*dQy(ii)*dHbx(ii,Ny);                                                          
end


%------------ Kp2 dZ/dy dV/dz dHb/dx ---------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa1*Kp2*(1/(2*dz))*dZy(ii,jj,Nz)*dHbx(ii,jj);    
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa1*Kp2*(-4/(2*dz))*dZy(ii,jj,Nz)*dHbx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa1*Kp2*(3/(2*dz))*dZy(ii,jj,Nz)*dHbx(ii,jj);
end 
end

%----------  Kp2 dZ/dz dW/dz dHb/dx -------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa1*Kp2*(1/(2*dz))*dZz(ii,jj)*dHbx(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa1*Kp2*(-4/(2*dz))*dZz(ii,jj)*dHbx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa1*Kp2*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj); 
end 
end


%++++++++++++++++++++++ KpE Fxy dHb/dy ++++++++++++++++++++++;


%------------------- dQ/dy dU/dQ dHb/dy ---------------------;

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa1*(-3/(2*dy))*dQy(ii)*dHby(ii,1);
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+   alfa1*(4/(2*dy))*dQy(ii)*dHby(ii,1);            
MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa1*(-1/(2*dy))*dQy(ii)*dHby(ii,1);                                                          
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa1*(-1/(2*dy))*dQy(ii)*dHby(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa1*(1/(2*dy))*dQy(ii)*dHby(ii,jj);                                                          
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa1*(1/(2*dy))*dQy(ii)*dHby(ii,Ny);
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa1*(-4/(2*dy))*dQy(ii)*dHby(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa1*(3/(2*dy))*dQy(ii)*dHby(ii,Ny);                                                          
end


%------------------- dZ/dy dU/dz dHb/dy ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa1*(1/(2*dz))*dZy(ii,jj,Nz)*dHby(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa1*(-4/(2*dz))*dZy(ii,jj,Nz)*dHby(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa1*(3/(2*dz))*dZy(ii,jj,Nz)*dHby(ii,jj);                                                          
end 
end


%--------------------- dV/dx dHb/dy ---------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa1*(-1/(2*dx))*dHby(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)+     alfa1*(1/(2*dx))*dHby(ii,jj);                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa1*(1/(2*dx))*dHby(Nx,jj);    
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa1*(-4/(2*dx))*dHby(Nx,jj);            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa1*(3/(2*dx))*dHby(Nx,jj);                                                          
end 


%------------ dQ/dx dV/dQ dHb/dy ---------------;

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa1*(-3/(2*dy))*dQx(ii,1)*dHby(ii,1);    
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa1*(4/(2*dy))*dQx(ii,1)*dHby(ii,1);            
MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa1*(-1/(2*dy))*dQx(ii,1)*dHby(ii,1);                                                          
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa1*(-1/(2*dy))*dQx(ii,jj)*dHby(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa1*(1/(2*dy))*dQx(ii,jj)*dHby(ii,jj);                                                          
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa1*(1/(2*dy))*dQx(ii,Ny)*dHby(ii,Ny);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa1*(-4/(2*dy))*dQx(ii,Ny)*dHby(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa1*(3/(2*dy))*dQx(ii,Ny)*dHby(ii,Ny);                                                          
end


%--------------- dZ/dx dV/dz dHb/dy ---------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa1*(1/(2*dz))*dZx(ii,jj,Nz)*dHby(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa1*(-4/(2*dz))*dZx(ii,jj,Nz)*dHby(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa1*(3/(2*dz))*dZx(ii,jj,Nz)*dHby(ii,jj);                                                          
end 
end


%+++++++++++++++++++++ - KpE  Fxz +++++++++++++++++++++++++;

%------------------  - dZ/dz dU/dz ---------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)- alfa1*(1/(2*dz))*dZz(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)-alfa1*(-4/(2*dz))*dZz(ii,jj);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)-   alfa1*(3/(2*dz))*dZz(ii,jj);                                                          
end 
end

%---------------------- - dW/dx ------------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)-alfa1*(-1/(2*dx));            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)-     alfa1*(1/(2*dx));                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)- alfa1*(1/(2*dx));
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)-alfa1*(-4/(2*dx));            
MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB((Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)- alfa1*(3/(2*dx));                                                          
end 


%------------------- - dQ/dx dW/dQ ---------------------;

%----- j=1 -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)-     alfa1*(-3/(2*dy))*dQx(ii,1);    
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)-   alfa1*(4/(2*dy))*dQx(ii,1);            
MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB((ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)-alfa1*(-1/(2*dy))*dQx(ii,1);                                                          
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)-alfa1*(-1/(2*dy))*dQx(ii,jj);            
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)-     alfa1*(1/(2*dy))*dQx(ii,jj);                                                          
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)- alfa1*(1/(2*dy))*dQx(ii,Ny);    
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)-alfa1*(-4/(2*dy))*dQx(ii,Ny);            
MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB((ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)- alfa1*(3/(2*dy))*dQx(ii,Ny);                                                          
end


%------------------ - dZ/dx dW/dz --------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)- alfa1*(1/(2*dz))*dZx(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)-alfa1*(-4/(2*dz))*dZx(ii,jj,Nz);
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)-   alfa1*(3/(2*dz))*dZx(ii,jj,Nz);                                                          
end 
end



%-----------------  KpE dHb/dx P' -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+Nz,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj) + alfa1*KpE*dHbx(ii,jj);                                                          
end 
end




%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++ The second equation (basic equation #2) +++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++++++ KpE Fyx dHs/dx ++++++++++++++++++++++++;

%--------------------- dQ/dy dU/dQ dHb/dx ---------------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa1*(-3/(2*dy))*dQy(ii)*dHbx(ii,1);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+   alfa1*(4/(2*dy))*dQy(ii)*dHbx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa1*(-1/(2*dy))*dQy(ii)*dHbx(ii,1); 
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa1*(-1/(2*dy))*dQy(ii)*dHbx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa1*(1/(2*dy))*dQy(ii)*dHbx(ii,jj); 
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa1*(1/(2*dy))*dQy(ii)*dHbx(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa1*(-4/(2*dy))*dQy(ii)*dHbx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa1*(3/(2*dy))*dQy(ii)*dHbx(ii,Ny); 
end


%----------------- dZ/dy dU/dz dHb/dx --------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa1*(1/(2*dz))*dZy(ii,jj,Nz)*dHbx(ii,jj);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa1*(-4/(2*dz))*dZy(ii,jj,Nz)*dHbx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa1*(3/(2*dz))*dZy(ii,jj,Nz)*dHbx(ii,jj); 
end 
end


%------------------ dV/dx dHb/dx -----------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa1*(-1/(2*dx))*dHbx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)+     alfa1*(1/(2*dx))*dHbx(ii,jj); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa1*(1/(2*dx))*dHbx(Nx,jj);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa1*(-4/(2*dx))*dHbx(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa1*(3/(2*dx))*dHbx(Nx,jj); 
end 


%------------- dQ/dx dV/dQ dHb/dx --------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa1*(-3/(2*dy))*dQx(ii,1)*dHbx(ii,1);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa1*(4/(2*dy))*dQx(ii,1)*dHbx(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa1*(-1/(2*dy))*dQx(ii,1)*dHbx(ii,1); 
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa1*(-1/(2*dy))*dQx(ii,jj)*dHbx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa1*(1/(2*dy))*dQx(ii,jj)*dHbx(ii,jj); 
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa1*(1/(2*dy))*dQx(ii,Ny)*dHbx(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa1*(-4/(2*dy))*dQx(ii,Ny)*dHbx(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa1*(3/(2*dy))*dQx(ii,Ny)*dHbx(ii,Ny); 
end


%-------------- dZ/dx dV/dz dHb/dx -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa1*(1/(2*dz))*dZx(ii,jj,Nz)*dHbx(ii,jj);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa1*(-4/(2*dz))*dZx(ii,jj,Nz)*dHbx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa1*(3/(2*dz))*dZx(ii,jj,Nz)*dHbx(ii,jj); 
end 
end


%+++++++++++++++++++++ KpE Fyy dHs/dy ++++++++++++++++++++++;


%----------------- Kp1 dQ/dy dV/dQ dHb/dy ------------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa1*Kp1*(-3/(2*dy))*dQy(ii)*dHby(ii,1);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa1*Kp1*(4/(2*dy))*dQy(ii)*dHby(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa1*Kp1*(-1/(2*dy))*dQy(ii)*dHby(ii,1); 
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa1*Kp1*(-1/(2*dy))*dQy(ii)*dHby(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa1*Kp1*(1/(2*dy))*dQy(ii)*dHby(ii,jj); 
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa1*Kp1*(1/(2*dy))*dQy(ii)*dHby(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa1*Kp1*(-4/(2*dy))*dQy(ii)*dHby(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa1*Kp1*(3/(2*dy))*dQy(ii)*dHby(ii,Ny); 
end


%---------------- Kp1 dZ/dy dV/dz dHb/dy ---------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa1*Kp1*(1/(2*dz))*dZy(ii,jj,Nz)*dHby(ii,jj);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa1*Kp1*(-4/(2*dz))*dZy(ii,jj,Nz)*dHby(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa1*Kp1*(3/(2*dz))*dZy(ii,jj,Nz)*dHby(ii,jj); 
end 
end


%--------------- Kp2 dU/dx dHb/dy -----------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa1*Kp2*(-1/(2*dx))*dHby(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)+     alfa1*Kp2*(1/(2*dx))*dHby(ii,jj); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa1*Kp2*(1/(2*dx))*dHby(Nx,jj);    
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa1*Kp2*(-4/(2*dx))*dHby(Nx,jj); 
MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa1*Kp2*(3/(2*dx))*dHby(Nx,jj); 
end 


%------------- Kp2 dQ/dx dU/dQ dHb/dy ---------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)+     alfa1*Kp2*(-3/(2*dy))*dQx(ii,1)*dHby(ii,1);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)+   alfa1*Kp2*(4/(2*dy))*dQx(ii,1)*dHby(ii,1); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)+alfa1*Kp2*(-1/(2*dy))*dQx(ii,1)*dHby(ii,1); 
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa1*Kp2*(-1/(2*dy))*dQx(ii,jj)*dHby(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa1*Kp2*(1/(2*dy))*dQx(ii,jj)*dHby(ii,jj); 
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa1*Kp2*(1/(2*dy))*dQx(ii,Ny)*dHby(ii,Ny);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa1*Kp2*(-4/(2*dy))*dQx(ii,Ny)*dHby(ii,Ny); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa1*Kp2*(3/(2*dy))*dQx(ii,Ny)*dHby(ii,Ny); 
end


%-------------- Kp2 dZ/dx dU/dz dHb/dy ----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa1*Kp2*(1/(2*dz))*dZx(ii,jj,Nz)*dHby(ii,jj);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa1*Kp2*(-4/(2*dz))*dZx(ii,jj,Nz)*dHby(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa1*Kp2*(3/(2*dz))*dZx(ii,jj,Nz)*dHby(ii,jj); 
end 
end


%--------------- Kp2 dZ/dz dW/dz dHb/dy -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa1*Kp2*(1/(2*dz))*dZz(ii,jj)*dHby(ii,jj);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa1*Kp2*(-4/(2*dz))*dZz(ii,jj)*dHby(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa1*Kp2*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj); 
end 
end


%+++++++++++++++++++ - KpE Fyz ++++++++++++++++++++++;

%----------------- - dZ/dz dV/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)- alfa1*(1/(2*dz))*dZz(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)-alfa1*(-4/(2*dz))*dZz(ii,jj);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)-   alfa1*(3/(2*dz))*dZz(ii,jj); 
end 
end


%----------------- - dQ/dy dW/dQ -------------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)-     alfa1*(-3/(2*dy))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)-   alfa1*(4/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)-alfa1*(-1/(2*dy))*dQy(ii); 
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)-alfa1*(-1/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)-     alfa1*(1/(2*dy))*dQy(ii); 
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)- alfa1*(1/(2*dy))*dQy(ii);    
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)-alfa1*(-4/(2*dy))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)- alfa1*(3/(2*dy))*dQy(ii); 
end


%----------------- - dZ/dy dW/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)- alfa1*(1/(2*dz))*dZy(ii,jj,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)-alfa1*(-4/(2*dz))*dZy(ii,jj,Nz);
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)-   alfa1*(3/(2*dz))*dZy(ii,jj,Nz); 
end 
end


%------------------  KpE dHb/dy P' -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj) + alfa1*KpE*dHby(ii,jj);                                                          
end 
end




%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++ The third equation (basic equation #3) +++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%++++++++++++++++++++ KpE Fzx dHb/dx ++++++++++++++++++++;

%------------------ dZ/dz dU/dz dHb/dx -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa1*(1/(2*dz))*dZz(ii,jj)*dHbx(ii,jj);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa1*(-4/(2*dz))*dZz(ii,jj)*dHbx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa1*(3/(2*dz))*dZz(ii,jj)*dHbx(ii,jj); 
end 
end


%------------------- dW/dx dHb/dx --------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa1*(-1/(2*dx))*dHbx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+Nz)+     alfa1*(1/(2*dx))*dHbx(ii,jj); 
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa1*(1/(2*dx))*dHbx(Nx,jj);    
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)+alfa1*(-4/(2*dx))*dHbx(Nx,jj); 
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)+ alfa1*(3/(2*dx))*dHbx(Nx,jj); 
end 


%----------------- dQ/dx dW/dQ dHb/dx -----------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa1*(-3/(2*dy))*dQx(ii,1)*dHbx(ii,1);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa1*(4/(2*dy))*dQx(ii,1)*dHbx(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa1*(-1/(2*dy))*dQx(ii,1)*dHbx(ii,1); 
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa1*(-1/(2*dy))*dQx(ii,jj)*dHbx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa1*(1/(2*dy))*dQx(ii,jj)*dHbx(ii,jj); 
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa1*(1/(2*dy))*dQx(ii,Ny)*dHbx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa1*(-4/(2*dy))*dQx(ii,Ny)*dHbx(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa1*(3/(2*dy))*dQx(ii,Ny)*dHbx(ii,Ny); 
end


%-------------- dZ/dx dW/dz dHb/dx --------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa1*(1/(2*dz))*dZx(ii,jj,Nz)*dHbx(ii,jj);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa1*(-4/(2*dz))*dZx(ii,jj,Nz)*dHbx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa1*(3/(2*dz))*dZx(ii,jj,Nz)*dHbx(ii,jj); 
end 
end



%++++++++++++++++ KpE Fzy dHb/dy +++++++++++++++++;


%-------------- dZ/dz dV/dz dHb/dy --------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa1*(1/(2*dz))*dZz(ii,jj)*dHby(ii,jj);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa1*(-4/(2*dz))*dZz(ii,jj)*dHby(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa1*(3/(2*dz))*dZz(ii,jj)*dHby(ii,jj); 
end 
end


%-------------- dQ/dy dW/dQ dHb/dy ---------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)+     alfa1*(-3/(2*dy))*dQy(ii)*dHby(ii,1);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)+   alfa1*(4/(2*dy))*dQy(ii)*dHby(ii,1); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)+alfa1*(-1/(2*dy))*dQy(ii)*dHby(ii,1); 
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)+alfa1*(-1/(2*dy))*dQy(ii)*dHby(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)+     alfa1*(1/(2*dy))*dQy(ii)*dHby(ii,jj); 
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)+ alfa1*(1/(2*dy))*dQy(ii)*dHby(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)+alfa1*(-4/(2*dy))*dQy(ii)*dHby(ii,Ny); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)+ alfa1*(3/(2*dy))*dQy(ii)*dHby(ii,Ny); 
end


%-------------- dZ/dy dW/dz dHb/dy ---------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)+ alfa1*(1/(2*dz))*dZy(ii,jj,Nz)*dHby(ii,jj);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)+alfa1*(-4/(2*dz))*dZy(ii,jj,Nz)*dHby(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)+   alfa1*(3/(2*dz))*dZy(ii,jj,Nz)*dHby(ii,jj); 
end 
end



%++++++++++++++++++++ - KpE Fzz ++++++++++++++++++++;

%---------------- - Kp1 dZ/dz dW/dz -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)- alfa1*Kp1*(1/(2*dz))*dZz(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)-alfa1*Kp1*(-4/(2*dz))*dZz(ii,jj);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)-   alfa1*Kp1*(3/(2*dz))*dZz(ii,jj); 
end 
end


%----------------- - Kp2 dU/dx ------------------;

for ii=2:1:Nx-1
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-2)*Ny*Nz+(jj-1)*Nz+Nz)-alfa1*Kp2*(-1/(2*dx));            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,ii*Ny*Nz+(jj-1)*Nz+Nz)-     alfa1*Kp2*(1/(2*dx));                                                          
end 
end

%---- i=Nx ----;

for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-3)*Ny*Nz+(jj-1)*Nz+Nz)- alfa1*Kp2*(1/(2*dx));     
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-2)*Ny*Nz+(jj-1)*Nz+Nz)-alfa1*Kp2*(-4/(2*dx));            
MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz,(Nx-1)*Ny*Nz+(jj-1)*Nz+Nz)- alfa1*Kp2*(3/(2*dx));                                                          
end 


%--------------- - Kp2 dQ/dx dU/dQ ----------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz)-     alfa1*Kp2*(-3/(2*dy))*dQx(ii,1);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+Nz+Nz)-   alfa1*Kp2*(4/(2*dy))*dQx(ii,1);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,(ii-1)*Ny*Nz+2*Nz+Nz)-alfa1*Kp2*(-1/(2*dy))*dQx(ii,1);                                                          
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)-alfa1*Kp2*(-1/(2*dy))*dQx(ii,jj);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+jj*Nz+Nz)-     alfa1*Kp2*(1/(2*dy))*dQx(ii,jj);                                                          
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)- alfa1*Kp2*(1/(2*dy))*dQx(ii,Ny);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)-alfa1*Kp2*(-4/(2*dy))*dQx(ii,Ny);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)- alfa1*Kp2*(3/(2*dy))*dQx(ii,Ny);                                                          
end


%---------------- - Kp2 dZ/dx dU/dz ------------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)- alfa1*Kp2*(1/(2*dz))*dZx(ii,jj,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)-alfa1*Kp2*(-4/(2*dz))*dZx(ii,jj,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)-   alfa1*Kp2*(3/(2*dz))*dZx(ii,jj,Nz);                                                          
end 
end


%---------------- - Kp2 dQ/dy dV/dQ ------------------;

%----- j=1 -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)=     MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz)-     alfa1*Kp2*(-3/(2*dy))*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz+Nz)-   alfa1*Kp2*(4/(2*dy))*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+2*Nz+Nz)-alfa1*Kp2*(-1/(2*dy))*dQy(ii);                                                          
end

%----- 1<j<Ny -----;

for ii=2:1:Nx
for jj=2:1:Ny-1
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+Nz)-alfa1*Kp2*(-1/(2*dy))*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+Nz)-     alfa1*Kp2*(1/(2*dy))*dQy(ii);                                                          
end 
end

%----- j=Ny -----;

for ii=2:1:Nx
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-3)*Nz+Nz)- alfa1*Kp2*(1/(2*dy))*dQy(ii);    
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-2)*Nz+Nz)-alfa1*Kp2*(-4/(2*dy))*dQy(ii);            
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(Ny-1)*Nz+Nz)- alfa1*Kp2*(3/(2*dy))*dQy(ii);                                                          
end


%----------------- - Kp2 dZ/dy dV/dz ---------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-2)- alfa1*Kp2*(1/(2*dz))*dZy(ii,jj,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz-1)-alfa1*Kp2*(-4/(2*dz))*dZy(ii,jj,Nz);
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)-   alfa1*Kp2*(3/(2*dz))*dZy(ii,jj,Nz);                                                          
end 
end


%------------------ - KpE P' -----------------;

for ii=2:1:Nx
for jj=1:1:Ny
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj) - alfa1*KpE;                                                          
end 
end




%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++ The momentum equations inside the ice shelf +++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++ 1<Ix<Nx; 1<Jy<Ny; 1<Kz<Nz +++++++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++++++++ The first equation +++++++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

%+++++++++++++++++++++++ KpE dFxx/dx ++++++++++++++++++++++++;

%--------------------- Kp1 d^2U/dx^2 ----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk)= alfa12*Kp1/dx^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=-alfa12*Kp1*2/dx^2; 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk)=     alfa12*Kp1/dx^2; 
end 
end 
end


%----------------- Kp1 d/dx (dQ/dx dU/dQ) --------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*Kp1*(dQx(ii-1,jj)/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+jj*Nz+kk)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+jj*Nz+kk)+   alfa12*Kp1*(-dQx(ii-1,jj)/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-2)*Nz+kk)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-2)*Nz+kk)+   alfa12*Kp1*(-dQx(ii+1,jj)/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+jj*Nz+kk)=        MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+jj*Nz+kk)+        alfa12*Kp1*(dQx(ii+1,jj)/(4*dx*dy));                                  
end 
end 
end


%--------------------- Kp1 d/dx ( dZ/dx dU/dz ) --------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa12*Kp1*(dZx(ii-1,jj,kk)/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*Kp1*(-dZx(ii-1,jj,kk)/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk-1)+    alfa12*Kp1*(-dZx(ii+1,jj,kk)/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk+1)+     alfa12*Kp1*(dZx(ii+1,jj,kk)/(4*dx*dz));                                  
end 
end 
end


%-------------------- Kp1 dQ/dx d/dQ ( dU/dx ) -----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*Kp1*(1/(4*dy*dx))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-2)*Nz+kk)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-2)*Nz+kk)+   alfa12*Kp1*(-1/(4*dy*dx))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+jj*Nz+kk)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+jj*Nz+kk)+   alfa12*Kp1*(-1/(4*dy*dx))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+jj*Nz+kk)=        MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+jj*Nz+kk)+        alfa12*Kp1*(1/(4*dy*dx))*dQx(ii,jj); 
end 
end 
end



%---------------- Kp1 dQ/dx d/dQ ( dQ/dx dU/dQ ) -----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*Kp1*(Gpy1_ijk(ii,jj)/dy^2)*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*Kp1*(-(Gpy1_ijk(ii,jj)+Gpy1_ijk(ii,jj+1))/dy^2)*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk)+    alfa12*Kp1*(Gpy1_ijk(ii,jj+1)/dy^2)*dQx(ii,jj); 
end 
end 
end


%-------------- Kp1 dQ/dx d/dQ ( dZ/dx dU/dz ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*Kp1*(dZx(ii,jj-1,kk)/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*Kp1*(-dZx(ii,jj-1,kk)/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*Kp1*(-dZx(ii,jj+1,kk)/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*Kp1*(dZx(ii,jj+1,kk)/(4*dy*dz))*dQx(ii,jj); 
end 
end 
end


%------------------- Kp1 dZ/dx d/dz ( dU/dx ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa12*Kp1*(1/(4*dz*dx))*dZx(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk-1)+    alfa12*Kp1*(-1/(4*dz*dx))*dZx(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*Kp1*(-1/(4*dz*dx))*dZx(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk+1)+     alfa12*Kp1*(1/(4*dz*dx))*dZx(ii,jj,kk); 
end 
end 
end


%------------------- Kp1 dZ/dx d/dz ( dQ/dx dU/dQ ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*Kp1*(dQx(ii,jj)/(4*dz*dy))*dZx(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*Kp1*(-dQx(ii,jj)/(4*dz*dy))*dZx(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*Kp1*(-dQx(ii,jj)/(4*dz*dy))*dZx(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*Kp1*(dQx(ii,jj)/(4*dz*dy))*dZx(ii,jj,kk); 
end 
end 
end


%------------------- Kp1 dZ/dx d/dz ( dZ/dx dU/dz ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*Kp1*(Gpz1_ijk(ii,jj,kk)/dz^2)*dZx(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+  alfa12*Kp1*(-(Gpz1_ijk(ii,jj,kk)+Gpz1_ijk(ii,jj,kk+1))/dz^2)*dZx(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*Kp1*(Gpz1_ijk(ii,jj,kk+1)/dz^2)*dZx(ii,jj,kk); 
end  
end 
end



%------------------- Kp2 d/dx ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+kk)=alfa12*Kp2*(dQy(ii-1)/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+kk)=   alfa12*Kp2*(-dQy(ii-1)/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+kk)=   alfa12*Kp2*(-dQy(ii+1)/(4*dx*dy)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+kk)=        alfa12*Kp2*(dQy(ii+1)/(4*dx*dy));                                  
end 
end 
end


%------------------- Kp2 d/dx ( dZ/dy dV/dz ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa12*Kp2*(dZy(ii-1,jj,kk)/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*Kp2*(-dZy(ii-1,jj,kk)/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk-1)+    alfa12*Kp2*(-dZy(ii+1,jj,kk)/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk+1)+     alfa12*Kp2*(dZy(ii+1,jj,kk)/(4*dx*dz)); 
end 
end 
end


%--------------- alfa12*Kp2 dQ/dx d/dQ ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*Kp2*(dQy(ii)/dy^2)*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*Kp2*(-2*dQy(ii)/dy^2)*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk)+    alfa12*Kp2*(dQy(ii)/dy^2)*dQx(ii,jj); 
end 
end 
end


%--------------- Kp2 dQ/dx d/dQ ( dZ/dy dV/dz ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*Kp2*(dZy(ii,jj-1,kk)/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*Kp2*(-dZy(ii,jj-1,kk)/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*Kp2*(-dZy(ii,jj+1,kk)/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*Kp2*(dZy(ii,jj+1,kk)/(4*dy*dz))*dQx(ii,jj); 
end 
end 
end


%--------------- Kp2 dZ/dx d/dz ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*Kp2*(dQy(ii)/(4*dz*dy))*dZx(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*Kp2*(-dQy(ii)/(4*dz*dy))*dZx(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*Kp2*(-dQy(ii)/(4*dz*dy))*dZx(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*Kp2*(dQy(ii)/(4*dz*dy))*dZx(ii,jj,kk); 
end 
end 
end


%------------- Kp2 dZ/dx d/dz ( dZ/dy dV/dz ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*Kp2*(Gpz2_ijk(ii,jj,kk)/dz^2)*dZx(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+  alfa12*Kp2*(-(Gpz2_ijk(ii,jj,kk)+Gpz2_ijk(ii,jj,kk+1))/dz^2)*dZx(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*Kp2*(Gpz2_ijk(ii,jj,kk+1)/dz^2)*dZx(ii,jj,kk); 
end  
end 
end



%----------------- Kp2 d/dx ( dZ/dz dW/dz ) -----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)= alfa12*Kp2*(dZz(ii-1,jj)/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)=alfa12*Kp2*(-dZz(ii-1,jj)/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk-1)=    alfa12*Kp2*(-dZz(ii+1,jj)/(4*dx*dz)); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk+1)=     alfa12*Kp2*(dZz(ii+1,jj)/(4*dx*dz));                                  
end 
end 
end

%--------------- Kp2 dQ/dx d/dQ ( dZ/dz dW/dz ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*Kp2*(dZz(ii,jj-1)/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*Kp2*(-dZz(ii,jj-1)/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*Kp2*(-dZz(ii,jj+1)/(4*dy*dz))*dQx(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*Kp2*(dZz(ii,jj+1)/(4*dy*dz))*dQx(ii,jj); 
end 
end 
end

%---------------- Kp2 dZ/dx d/dz ( dZ/dz dW/dz ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa12*Kp2*(dZz(ii,jj)/dz^2)*dZx(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+  alfa12*Kp2*(-(2*dZz(ii,jj))/dz^2)*dZx(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa12*Kp2*(dZz(ii,jj)/dz^2)*dZx(ii,jj,kk); 
end  
end 
end



%+++++++++++++++++++++++ KpE dFxy/dy +++++++++++++++++++++++++++;

%---------------- dQ/dy d/dQ ( dQ/dy dU/dQ ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*(dQy(ii)/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*(-(2*dQy(ii))/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk)+    alfa12*(dQy(ii)/dy^2)*dQy(ii); 
end 
end 
end


%----------------- dQ/dy d/dQ ( dZ/dy dU/dz ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*(dZy(ii,jj-1,kk)/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*(-dZy(ii,jj-1,kk)/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*(-dZy(ii,jj+1,kk)/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*(dZy(ii,jj+1,kk)/(4*dy*dz))*dQy(ii); 
end 
end 
end

%------------------- dZ/dy d/dz ( dQ/dy dU/dQ ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*(dQy(ii)/(4*dz*dy))*dZy(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*(-dQy(ii)/(4*dz*dy))*dZy(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*(-dQy(ii)/(4*dz*dy))*dZy(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*(dQy(ii)/(4*dz*dy))*dZy(ii,jj,kk); 
end 
end 
end

%------------------- dZ/dy d/dz ( dZ/dy dU/dz ) -----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+  alfa12*(Gpz2_ijk(ii,jj,kk)/dz^2)*dZy(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+  alfa12*(-(Gpz2_ijk(ii,jj,kk)+Gpz2_ijk(ii,jj,kk+1))/dz^2)*dZy(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+  alfa12*(Gpz2_ijk(ii,jj,kk+1)/dz^2)*dZy(ii,jj,kk); 
end  
end 
end

%----------------- dQ/dy d/dQ ( dV/dx ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*(1/(4*dy*dx))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+kk)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+kk)+   alfa12*(-1/(4*dy*dx))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+kk)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+kk)+   alfa12*(-1/(4*dy*dx))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+kk)=        MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+kk)+        alfa12*(1/(4*dy*dx))*dQy(ii); 
end 
end 
end


%------------------ dQ/dy d/dQ ( dQ/dx dV/dQ ) -----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*(Gpy1_ijk(ii,jj)/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*(-(Gpy1_ijk(ii,jj)+Gpy1_ijk(ii,jj+1))/dy^2)*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk)+    alfa12*(Gpy1_ijk(ii,jj+1)/dy^2)*dQy(ii); 
end 
end 
end


%----------------- dQ/dy d/dQ ( dZ/dx dV/dz ) -----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*(dZx(ii,jj-1,kk)/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*(-dZx(ii,jj-1,kk)/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*(-dZx(ii,jj+1,kk)/(4*dy*dz))*dQy(ii); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*(dZx(ii,jj+1,kk)/(4*dy*dz))*dQy(ii); 
end 
end 
end

%----------------- dZ/dy d/dz ( dV/dx ) -----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa12*(1/(4*dz*dx))*dZy(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk-1)+    alfa12*(-1/(4*dz*dx))*dZy(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(-1/(4*dz*dx))*dZy(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk+1)+     alfa12*(1/(4*dz*dx))*dZy(ii,jj,kk); 
end 
end 
end

%----------------- dZ/dy d/dz ( dQ/dx dV/dQ ) -----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*(dQx(ii,jj)/(4*dz*dy))*dZy(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*(-dQx(ii,jj)/(4*dz*dy))*dZy(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*(-dQx(ii,jj)/(4*dz*dy))*dZy(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*(dQx(ii,jj)/(4*dz*dy))*dZy(ii,jj,kk); 
end 
end 
end


%---------------- dZ/dy d/dz ( dZ/dx dV/dz ) -----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*(Gpz1_ijk(ii,jj,kk)/dz^2)*dZy(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*(-(Gpz1_ijk(ii,jj,kk)+Gpz1_ijk(ii,jj,kk+1))/dz^2)*dZy(ii,jj,kk); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(Gpz1_ijk(ii,jj,kk+1)/dz^2)*dZy(ii,jj,kk); 
end  
end 
end



%+++++++++++++++++++++++++ KpE dFxz/dz ++++++++++++++++++++++++++;

%------------------- dZ/dz d/dz ( dZ/dz dU/dz ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*(dZz(ii,jj)/dz^2)*dZz(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*(-2*dZz(ii,jj)/dz^2)*dZz(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(dZz(ii,jj)/dz^2)*dZz(ii,jj);                                          
end  
end 
end

%-------------------- dZ/dz d/dz ( dW/dx ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa12*(1/(4*dz*dx))*dZz(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk-1)+    alfa12*(-1/(4*dz*dx))*dZz(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(-1/(4*dz*dx))*dZz(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk+1)+     alfa12*(1/(4*dz*dx))*dZz(ii,jj); 
end 
end 
end


%------------------- dZ/dz d/dz ( dQ/dx dW/dQ ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*(dQx(ii,jj)/(4*dz*dy))*dZz(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*(-dQx(ii,jj)/(4*dz*dy))*dZz(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*(-dQx(ii,jj)/(4*dz*dy))*dZz(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*(dQx(ii,jj)/(4*dz*dy))*dZz(ii,jj); 
end 
end 
end


%------------------- dZ/dz d/dz ( dZ/dx dW/dz ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*(Gpz1_ijk(ii,jj,kk)/dz^2)*dZz(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*(-(Gpz1_ijk(ii,jj,kk)+Gpz1_ijk(ii,jj,kk+1))/dz^2)*dZz(ii,jj); 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(Gpz1_ijk(ii,jj,kk+1)/dz^2)*dZz(ii,jj); 
end  
end 
end

%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++++ + KpE dens Freq^2 U +++++++++++++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=MB((ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk) + alfa12*KpE*dens*Freq^2; 
end  
end 
end





%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++++++++ The second equation +++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

%+++++++++++++++++++++++++ KpE dFyx/dx ++++++++++++++++++++++++++;

%--------------------- d/dx ( dQ/dy dU/dQ ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-2)*Nz+kk)=alfa12*(dQy(ii-1)/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+jj*Nz+kk)=   alfa12*(-dQy(ii-1)/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-2)*Nz+kk)=   alfa12*(-dQy(ii+1)/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+jj*Nz+kk)=        alfa12*(dQy(ii+1)/(4*dx*dy));                                  
end 
end 
end

%--------------------- d/dx ( dZ/dy dU/dz ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa12*(dZy(ii-1,jj,kk)/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(-dZy(ii-1,jj,kk)/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk-1)+    alfa12*(-dZy(ii+1,jj,kk)/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk+1)+     alfa12*(dZy(ii+1,jj,kk)/(4*dx*dz));                                  
end 
end 
end

%-------------------- dQ/dx d/dQ ( dQ/dy dU/dQ ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*(dQy(ii)/dy^2)*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*(-2*dQy(ii)/dy^2)*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk)+    alfa12*(dQy(ii)/dy^2)*dQx(ii,jj); 
end 
end 
end

%-------------------- dQ/dx d/dQ ( dZ/dy dU/dz ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*(dZy(ii,jj-1,kk)/(4*dy*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*(-dZy(ii,jj-1,kk)/(4*dy*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*(-dZy(ii,jj+1,kk)/(4*dy*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*(dZy(ii,jj+1,kk)/(4*dy*dz))*dQx(ii,jj); 
end 
end 
end

%-------------------- dZ/dx d/dz ( dQ/dy dU/dQ ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*(dQy(ii)/(4*dz*dy))*dZx(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*(-dQy(ii)/(4*dz*dy))*dZx(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*(-dQy(ii)/(4*dz*dy))*dZx(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*(dQy(ii)/(4*dz*dy))*dZx(ii,jj,kk); 
end 
end 
end

%------------- dZ/dx d/dz ( dZ/dy dU/dz ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*(Gpz2_ijk(ii,jj,kk)/dz^2)*dZx(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*(-(Gpz2_ijk(ii,jj,kk)+Gpz2_ijk(ii,jj,kk+1))/dz^2)*dZx(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(Gpz2_ijk(ii,jj,kk+1)/dz^2)*dZx(ii,jj,kk); 
end  
end 
end

%------------------------ d/dx ( dV/dx ) -----------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk)= alfa12*1/dx^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=-alfa12*2/dx^2; 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk)=     alfa12*1/dx^2; 
end 
end 
end

%--------------------- d/dx ( dQ/dx dV/dQ ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*(dQx(ii-1,jj)/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+kk)+   alfa12*(-dQx(ii-1,jj)/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+kk)+   alfa12*(-dQx(ii+1,jj)/(4*dx*dy)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+kk)=        MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+kk)+        alfa12*(dQx(ii+1,jj)/(4*dx*dy)); 
end 
end 
end

%--------------------- d/dx ( dZ/dx dV/dz ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa12*(dZx(ii-1,jj,kk)/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(-dZx(ii-1,jj,kk)/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk-1)+    alfa12*(-dZx(ii+1,jj,kk)/(4*dx*dz)); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk+1)+     alfa12*(dZx(ii+1,jj,kk)/(4*dx*dz)); 
end 
end 
end

%-------------------- dQ/dx d/dQ ( dV/dx ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*(1/(4*dy*dx))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+kk)+   alfa12*(-1/(4*dy*dx))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+kk)+   alfa12*(-1/(4*dy*dx))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+kk)=        MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+kk)+        alfa12*(1/(4*dy*dx))*dQx(ii,jj); 
end 
end 
end

%------------------- dQ/dx d/dQ ( dQ/dx dV/dQ ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*(Gpy1_ijk(ii,jj)/dy^2)*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*(-(Gpy1_ijk(ii,jj)+Gpy1_ijk(ii,jj+1))/dy^2)*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk)+    alfa12*(Gpy1_ijk(ii,jj+1)/dy^2)*dQx(ii,jj);                                                                            
end 
end 
end

%-------------------- dQ/dx d/dQ ( dZ/dx dV/dz ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*(dZx(ii,jj-1,kk)/(4*dy*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*(-dZx(ii,jj-1,kk)/(4*dy*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*(-dZx(ii,jj+1,kk)/(4*dy*dz))*dQx(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*(dZx(ii,jj+1,kk)/(4*dy*dz))*dQx(ii,jj);                                                                                         
end 
end 
end

%-------------------- dZ/dx d/dz ( dV/dx ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa12*(1/(4*dz*dx))*dZx(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk-1)+    alfa12*(-1/(4*dz*dx))*dZx(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(-1/(4*dz*dx))*dZx(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk+1)+     alfa12*(1/(4*dz*dx))*dZx(ii,jj,kk); 
end 
end 
end

%-------------------- dZ/dx d/dz ( dQ/dx dV/dQ ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*(dQx(ii,jj)/(4*dz*dy))*dZx(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*(-dQx(ii,jj)/(4*dz*dy))*dZx(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*(-dQx(ii,jj)/(4*dz*dy))*dZx(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*(dQx(ii,jj)/(4*dz*dy))*dZx(ii,jj,kk); 
end 
end 
end

%-------------------- dZ/dx d/dz ( dZ/dx dV/dz ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*(Gpz1_ijk(ii,jj,kk)/dz^2)*dZx(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*(-(Gpz1_ijk(ii,jj,kk)+Gpz1_ijk(ii,jj,kk+1))/dz^2)*dZx(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(Gpz1_ijk(ii,jj,kk+1)/dz^2)*dZx(ii,jj,kk);                                                                            
end  
end 
end


%++++++++++++++++++++++++ KpE dFyy/dy ++++++++++++++++++++++++++++;

%-------------- Kp1 dQ/dy d/dQ ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*Kp1*(dQy(ii)/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*Kp1*(-2*dQy(ii)/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk)+    alfa12*Kp1*(dQy(ii)/dy^2)*dQy(ii);                                                                            
end 
end 
end

%-------------- Kp1 dQ/dy d/dQ ( dZ/dy dV/dz ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*Kp1*(dZy(ii,jj-1,kk)/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*Kp1*(-dZy(ii,jj-1,kk)/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*Kp1*(-dZy(ii,jj+1,kk)/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*Kp1*(dZy(ii,jj+1,kk)/(4*dy*dz))*dQy(ii); 
end 
end 
end

%-------------- Kp1 dZ/dy d/dz ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*Kp1*(dQy(ii)/(4*dz*dy))*dZy(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*Kp1*(-dQy(ii)/(4*dz*dy))*dZy(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*Kp1*(-dQy(ii)/(4*dz*dy))*dZy(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*Kp1*(dQy(ii)/(4*dz*dy))*dZy(ii,jj,kk); 
end 
end 
end

%--------------- Kp1 dZ/dy d/dz ( dZ/dy dV/dz ) ----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*Kp1*(Gpz2_ijk(ii,jj,kk)/dz^2)*dZy(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+  alfa12*Kp1*(-(Gpz2_ijk(ii,jj,kk)+Gpz2_ijk(ii,jj,kk+1))/dz^2)*dZy(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*Kp1*(Gpz2_ijk(ii,jj,kk+1)/dz^2)*dZy(ii,jj,kk); 
end  
end 
end

%------------------ Kp2 dQ/dy d/dQ ( dU/dx ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*Kp2*(1/(4*dy*dx))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-2)*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-2)*Nz+kk)+   alfa12*Kp2*(-1/(4*dy*dx))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+jj*Nz+kk)+   alfa12*Kp2*(-1/(4*dy*dx))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+jj*Nz+kk)=        MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+jj*Nz+kk)+        alfa12*Kp2*(1/(4*dy*dx))*dQy(ii); 
end 
end 
end

%-------------- Kp2 dQ/dy d/dQ ( dQ/dx dU/dQ ) -----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*Kp2*(Gpy1_ijk(ii,jj)/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*Kp2*(-(Gpy1_ijk(ii,jj)+Gpy1_ijk(ii,jj+1))/dy^2)*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk)+    alfa12*Kp2*(Gpy1_ijk(ii,jj+1)/dy^2)*dQy(ii); 
end 
end 
end

%----------------- Kp2 dQ/dy d/dQ ( dZ/dx dU/dz ) -----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*Kp2*(dZx(ii,jj-1,kk)/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*Kp2*(-dZx(ii,jj-1,kk)/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*Kp2*(-dZx(ii,jj+1,kk)/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*Kp2*(dZx(ii,jj+1,kk)/(4*dy*dz))*dQy(ii); 
end 
end 
end

%------------------- Kp2 dZ/dy d/dz ( dU/dx ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa12*Kp2*(1/(4*dz*dx))*dZy(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk-1)+    alfa12*Kp2*(-1/(4*dz*dx))*dZy(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*Kp2*(-1/(4*dz*dx))*dZy(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk+1)+     alfa12*Kp2*(1/(4*dz*dx))*dZy(ii,jj,kk); 
end 
end 
end

%----------------- Kp2 dZ/dy d/dz ( dQ/dx dU/dQ ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*Kp2*(dQx(ii,jj)/(4*dz*dy))*dZy(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*Kp2*(-dQx(ii,jj)/(4*dz*dy))*dZy(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*Kp2*(-dQx(ii,jj)/(4*dz*dy))*dZy(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*Kp2*(dQx(ii,jj)/(4*dz*dy))*dZy(ii,jj,kk); 
end 
end 
end

%----------------- Kp2 dZ/dy d/dz ( dZ/dx dU/dz ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*Kp2*(Gpz1_ijk(ii,jj,kk)/dz^2)*dZy(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+  alfa12*Kp2*(-(Gpz1_ijk(ii,jj,kk)+Gpz1_ijk(ii,jj,kk+1))/dz^2)*dZy(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*Kp2*(Gpz1_ijk(ii,jj,kk+1)/dz^2)*dZy(ii,jj,kk); 
end  
end 
end

%----------------- Kp2 dQ/dy d/dQ ( dZ/dz dW/dz ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)= alfa12*Kp2*(dZz(ii,jj-1)/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=alfa12*Kp2*(-dZz(ii,jj-1)/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    alfa12*Kp2*(-dZz(ii,jj+1)/(4*dy*dz))*dQy(ii); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=     alfa12*Kp2*(dZz(ii,jj+1)/(4*dy*dz))*dQy(ii); 
end 
end 
end

%----------- Kp2 dZ/dy d/dz ( dZ/dz dW/dz ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*Kp2*(dZz(ii,jj)/dz^2)*dZy(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+  alfa12*Kp2*(-2*dZz(ii,jj)/dz^2)*dZy(ii,jj,kk); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*Kp2*(dZz(ii,jj)/dz^2)*dZy(ii,jj,kk); 
end  
end 
end


%+++++++++++++++++++++++++ KpE dFyz/dz +++++++++++++++++++++++++++++;

%------------------- dZ/dz d/dz ( dZ/dz dV/dz ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*(dZz(ii,jj)/dz^2)*dZz(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*(-2*dZz(ii,jj)/dz^2)*dZz(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(dZz(ii,jj)/dz^2)*dZz(ii,jj);                                                                            
end  
end 
end

%------------------- dZ/dz d/dz ( dQ/dy dW/dQ ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*(dQy(ii)/(4*dz*dy))*dZz(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*(-dQy(ii)/(4*dz*dy))*dZz(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*(-dQy(ii)/(4*dz*dy))*dZz(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*(dQy(ii)/(4*dz*dy))*dZz(ii,jj); 
end 
end 
end

%--------------- dZ/dz d/dz ( dZ/dy dW/dz ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*(Gpz2_ijk(ii,jj,kk)/dz^2)*dZz(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*(-(Gpz2_ijk(ii,jj,kk)+Gpz2_ijk(ii,jj,kk+1))/dz^2)*dZz(ii,jj); 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(Gpz2_ijk(ii,jj,kk+1)/dz^2)*dZz(ii,jj); 
end  
end 
end


%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++ + KpE dens Freq^2 V +++++++++++++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk) + alfa12*KpE*dens*Freq^2; 
end  
end 
end




%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++++++++++ The third equation ++++++++++++++++++++++;
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%+++++++++++++++++++++++++ KpE dFzx/dx ++++++++++++++++++++++++++;

%--------------------- d/dx ( dZ/dz dU/dz ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)= alfa12*(dZz(ii-1,jj)/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)=alfa12*(-dZz(ii-1,jj)/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk-1)=    alfa12*(-dZz(ii+1,jj)/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk+1)=     alfa12*(dZz(ii+1,jj)/(4*dx*dz));                                  
end 
end 
end

%-------------------- dQ/dx d/dQ ( dZ/dz dU/dz ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*(dZz(ii,jj-1)/(4*dy*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*(-dZz(ii,jj-1)/(4*dy*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*(-dZz(ii,jj+1)/(4*dy*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*(dZz(ii,jj+1)/(4*dy*dz))*dQx(ii,jj); 
end 
end 
end

%------------------- dZ/dx d/dz ( dZ/dz dU/dz ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa12*(dZz(ii,jj)/dz^2)*dZx(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*(-2*dZz(ii,jj)/dz^2)*dZx(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+ alfa12*(dZz(ii,jj)/dz^2)*dZx(ii,jj,kk); 
end  
end 
end

%------------------------ d/dx ( dW/dx ) -----------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk)= alfa12*1/dx^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=-alfa12*2/dx^2; 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk)=     alfa12*1/dx^2; 
end 
end 
end

%--------------------- d/dx ( dQ/dx dW/dQ ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*(dQx(ii-1,jj)/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+kk)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+kk)+   alfa12*(-dQx(ii-1,jj)/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+kk)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+kk)+   alfa12*(-dQx(ii+1,jj)/(4*dx*dy)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+kk)=        MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+kk)+        alfa12*(dQx(ii+1,jj)/(4*dx*dy)); 
end 
end 
end

%--------------------- d/dx ( dZ/dx dW/dz ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa12*(dZx(ii-1,jj,kk)/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(-dZx(ii-1,jj,kk)/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk-1)+    alfa12*(-dZx(ii+1,jj,kk)/(4*dx*dz)); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk+1)+     alfa12*(dZx(ii+1,jj,kk)/(4*dx*dz)); 
end 
end 
end

%-------------------- dQ/dx d/dQ ( dW/dx ) ---------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*(1/(4*dy*dx))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+kk)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-2)*Nz+kk)+   alfa12*(-1/(4*dy*dx))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+kk)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+jj*Nz+kk)+   alfa12*(-1/(4*dy*dx))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+kk)=        MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+jj*Nz+kk)+        alfa12*(1/(4*dy*dx))*dQx(ii,jj); 
end 
end 
end

%------------------- dQ/dx d/dQ ( dQ/dx dW/dQ ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*(Gpy1_ijk(ii,jj)/dy^2)*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*(-(Gpy1_ijk(ii,jj)+Gpy1_ijk(ii,jj+1))/dy^2)*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk)+    alfa12*(Gpy1_ijk(ii,jj+1)/dy^2)*dQx(ii,jj);                                                                            
end 
end 
end

%-------------------- dQ/dx d/dQ ( dZ/dx dW/dz ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*(dZx(ii,jj-1,kk)/(4*dy*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*(-dZx(ii,jj-1,kk)/(4*dy*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*(-dZx(ii,jj+1,kk)/(4*dy*dz))*dQx(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*(dZx(ii,jj+1,kk)/(4*dy*dz))*dQx(ii,jj);                                                                                         
end 
end 
end

%-------------------- dZ/dx d/dz ( dW/dx ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa12*(1/(4*dz*dx))*dZx(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk-1)+    alfa12*(-1/(4*dz*dx))*dZx(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(-1/(4*dz*dx))*dZx(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+ii*Ny*Nz+(jj-1)*Nz+kk+1)+     alfa12*(1/(4*dz*dx))*dZx(ii,jj,kk); 
end 
end 
end

%-------------------- dZ/dx d/dz ( dQ/dx dW/dQ ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*(dQx(ii,jj)/(4*dz*dy))*dZx(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*(-dQx(ii,jj)/(4*dz*dy))*dZx(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*(-dQx(ii,jj)/(4*dz*dy))*dZx(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*(dQx(ii,jj)/(4*dz*dy))*dZx(ii,jj,kk); 
end 
end 
end

%-------------------- dZ/dx d/dz ( dZ/dx dW/dz ) --------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*(Gpz1_ijk(ii,jj,kk)/dz^2)*dZx(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*(-(Gpz1_ijk(ii,jj,kk)+Gpz1_ijk(ii,jj,kk+1))/dz^2)*dZx(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(Gpz1_ijk(ii,jj,kk+1)/dz^2)*dZx(ii,jj,kk);                                                                            
end  
end 
end



%+++++++++++++++++++++++++ KpE dFzy/dy ++++++++++++++++++++++++++;

%----------------- dQ/dy d/dQ ( dZ/dz dV/dz ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)= alfa12*(dZz(ii,jj-1)/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=alfa12*(-dZz(ii,jj-1)/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    alfa12*(-dZz(ii,jj+1)/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=     alfa12*(dZz(ii,jj+1)/(4*dy*dz))*dQy(ii); 
end 
end 
end

%----------------- dZ/dy d/dz ( dZ/dy dV/dz ) ----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*(dZz(ii,jj)/dz^2)*dZy(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+  alfa12*(-2*dZz(ii,jj)/dz^2)*dZy(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(dZz(ii,jj)/dz^2)*dZy(ii,jj,kk); 
end  
end 
end

%-------------- dQ/dy d/dQ ( dQ/dy dW/dQ ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk)+alfa12*(dQy(ii)/dy^2)*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+alfa12*(-2*dQy(ii)/dy^2)*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk)+    alfa12*(dQy(ii)/dy^2)*dQy(ii);                                                                            
end 
end 
end

%---------------- dQ/dy d/dQ ( dZ/dy dW/dz ) -----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*(dZy(ii,jj-1,kk)/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*(-dZy(ii,jj-1,kk)/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*(-dZy(ii,jj+1,kk)/(4*dy*dz))*dQy(ii); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*(dZy(ii,jj+1,kk)/(4*dy*dz))*dQy(ii); 
end 
end 
end

%----------------- dZ/dy d/dz ( dQ/dy dW/dQ ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*(dQy(ii)/(4*dz*dy))*dZy(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*(-dQy(ii)/(4*dz*dy))*dZy(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*(-dQy(ii)/(4*dz*dy))*dZy(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*(dQy(ii)/(4*dz*dy))*dZy(ii,jj,kk); 
end 
end 
end

%----------------- dZ/dy d/dz ( dZ/dy dW/dz ) -----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*(Gpz2_ijk(ii,jj,kk)/dz^2)*dZy(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+  alfa12*(-(Gpz2_ijk(ii,jj,kk)+Gpz2_ijk(ii,jj,kk+1))/dz^2)*dZy(ii,jj,kk); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*(Gpz2_ijk(ii,jj,kk+1)/dz^2)*dZy(ii,jj,kk); 
end  
end 
end



%+++++++++++++++++++++++++ KpE dFzz/dz ++++++++++++++++++++++++++;

%--------------- Kp1 dZ/dz d/dz ( dZ/dz dW/dz ) -----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*Kp1*(dZz(ii,jj)/dz^2)*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+  alfa12*Kp1*(-2*dZz(ii,jj)/dz^2)*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*Kp1*(dZz(ii,jj)/dz^2)*dZz(ii,jj); 
end  
end 
end

%------------------- Kp2 dZ/dz d/dz ( dU/dx ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk-1)+ alfa12*Kp2*(1/(4*dz*dx))*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk-1)+    alfa12*Kp2*(-1/(4*dz*dx))*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-2)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*Kp2*(-1/(4*dz*dx))*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,ii*Ny*Nz+(jj-1)*Nz+kk+1)+     alfa12*Kp2*(1/(4*dz*dx))*dZz(ii,jj); 
end 
end 
end

%----------------- Kp2 dZ/dz d/dz ( dQ/dx dU/dQ ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*Kp2*(dQx(ii,jj)/(4*dz*dy))*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*Kp2*(-dQx(ii,jj)/(4*dz*dy))*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*Kp2*(-dQx(ii,jj)/(4*dz*dy))*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*Kp2*(dQx(ii,jj)/(4*dz*dy))*dZz(ii,jj); 
end 
end 
end

%----------------- Kp2 dZ/dz d/dz ( dZ/dx dU/dz ) ------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*Kp2*(Gpz1_ijk(ii,jj,kk)/dz^2)*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+  alfa12*Kp2*(-(Gpz1_ijk(ii,jj,kk)+Gpz1_ijk(ii,jj,kk+1))/dz^2)*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*Kp2*(Gpz1_ijk(ii,jj,kk+1)/dz^2)*dZz(ii,jj); 
end  
end 
end

%-------------- Kp2 dZ/dz d/dz ( dQ/dy dV/dQ ) -------------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk-1)+ alfa12*Kp2*(dQy(ii)/(4*dz*dy))*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk-1)+    alfa12*Kp2*(-dQy(ii)/(4*dz*dy))*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-2)*Nz+kk+1)+alfa12*Kp2*(-dQy(ii)/(4*dz*dy))*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)=    MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+jj*Nz+kk+1)+     alfa12*Kp2*(dQy(ii)/(4*dz*dy))*dZz(ii,jj); 
end 
end 
end

%--------------- Kp2 dZ/dz d/dz ( dZ/dy dV/dz ) ----------------;

for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk-1)+alfa12*Kp2*(Gpz2_ijk(ii,jj,kk)/dz^2)*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=  MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)+  alfa12*Kp2*(-(Gpz2_ijk(ii,jj,kk)+Gpz2_ijk(ii,jj,kk+1))/dz^2)*dZz(ii,jj); 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk+1)+alfa12*Kp2*(Gpz2_ijk(ii,jj,kk+1)/dz^2)*dZz(ii,jj); 
end  
end 
end



%+++++++++++++++++++++ + KpE dens Freq^2 W +++++++++++++++++++++++++;


for ii=2:1:Nx-1 
for jj=2:1:Ny-1
for kk=2:1:Nz-1 
MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk)=MB(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk) + alfa12*KpE*dens*Freq^2; 
end  
end 
end


%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++++++ Sea Water under the Ice Shelf ++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
    
    
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%++++++++++++++++++++++++++++ W=W' +++++++++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
    
for ii=1:1:Nx
for jj=1:1:Ny
MB(3*Nx*Ny*Nz+(ii-1)*Ny+jj,2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+Nz)=-1;
MB(3*Nx*Ny*Nz+(ii-1)*Ny+jj,3*Nx*Ny*Nz+(ii-1)*Ny+jj)=1;
end
end
    

%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++ Wave equation & Boundary conditions +++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


%--------------- X=0 (Ix=1): dP'/dx=0 ---------------;


for jj=1:1:Ny
    MB(3*Nx*Ny*Nz+Nx*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+jj)=     -3/(2*dx);
    MB(3*Nx*Ny*Nz+Nx*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+Ny+jj)=   4/(2*dx);
    MB(3*Nx*Ny*Nz+Nx*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+2*Ny+jj)=-1/(2*dx);
end


%---------- X=L (Ix=Nx): P'= p g dz cos (wt) ----------;

for jj=1:1:Ny

    MB(3*Nx*Ny*Nz+Nx*Ny+(Nx-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(Nx-1)*Ny+jj)=1;
    
    
    VB(3*Nx*Ny*Nz+Nx*Ny+(Nx-1)*Ny+jj)=dens_w*g*AmSLC;
      
      
end


%------------- Y=Y1 (J=1): dP'/dn = dP'/dx dY1/dx + dQx dP'/dq dY1/dx - dQy dP'/dq = 0 ---------------;

for ii=2:1:Nx-1
    
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+1,3*Nx*Ny*Nz+Nx*Ny+(ii-2)*Ny+1)=(-1/(2*dx))*difY1(ii);     
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+1,3*Nx*Ny*Nz+Nx*Ny+ii*Ny+1)=     (1/(2*dx))*difY1(ii);
    
    
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+1,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+1)=(-3/(2*dy))*dQx(ii,1)*difY1(ii);     
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+1,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+2)= (4/(2*dy))*dQx(ii,1)*difY1(ii);
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+1,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+3)=(-1/(2*dy))*dQx(ii,1)*difY1(ii);
    
    
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+1,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+1)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+1,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+1)-(-3/(2*dy))*dQy(ii);     
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+1,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+2)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+1,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+2)- (4/(2*dy))*dQy(ii);
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+1,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+3)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+1,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+3)-(-1/(2*dy))*dQy(ii);
    
end



%------------- Y=Y2 (J=Ny): dP'/dn = - dP'/dx dY2/dx - dQx dP'/dq dY2/dx + dQy dP'/dq  = 0 --------------;

for ii=2:1:Nx-1
    
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny,3*Nx*Ny*Nz+Nx*Ny+(ii-2)*Ny+Ny)=-(-1/(2*dx))*difY2(ii);
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny,3*Nx*Ny*Nz+Nx*Ny+ii*Ny+Ny)=     -(1/(2*dx))*difY2(ii); 
    
    
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny-2)= -(1/(2*dy))*dQx(ii,Ny)*difY2(ii);
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny-1)=-(-4/(2*dy))*dQx(ii,Ny)*difY2(ii);     
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny)=   -(3/(2*dy))*dQx(ii,Ny)*difY2(ii);
    
    
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny-2)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny-2)+ (1/(2*dy))*dQy(ii);
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny-1)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny-1)+(-4/(2*dy))*dQy(ii);     
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny)=  MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+Ny)+   (3/(2*dy))*dQy(ii);
    
end



%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++ The wave equation for the water layer ++++++++++++;    
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;


for ii=2:1:Nx-1
for jj=2:1:Ny-1
    
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+(ii-1)*Ny+jj)=Freq^2;
   
    %------- (1) d/dx ( D dP'/dx ) ---------;
    
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-2)*Ny+jj)= (1/dens_w)*(WDx_ij(ii,jj)*WWx_i(ii)/Width(ii))*(1/dx^2);     
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj)=-(1/dens_w)*((WDx_ij(ii,jj)*WWx_i(ii)+WDx_ij(ii+1,jj)*WWx_i(ii+1))/Width(ii))*(1/dx^2);
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+ii*Ny+jj)=     (1/dens_w)*(WDx_ij(ii+1,jj)*WWx_i(ii+1)/Width(ii))*(1/dx^2);
    
    
    
    %------- (2) d/dx ( D dQx dP'/dq ) -------;
    
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj-1)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj-1) +(1/dens_w)*((WDQxx_ij(ii,jj)*WWx_i(ii)-WDQxx_ij(ii+1,jj)*WWx_i(ii+1))/Width(ii))*(1/(4*dx*dy)); 
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj+1)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj+1)+(1/dens_w)*((-WDQxx_ij(ii,jj)*WWx_i(ii)+WDQxx_ij(ii+1,jj)*WWx_i(ii+1))/Width(ii))*(1/(4*dx*dy)); 
     
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-2)*Ny+jj-1)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-2)*Ny+jj-1) +(1/dens_w)*(WDQxx_ij(ii,jj)*WWx_i(ii)/Width(ii))*(1/(4*dx*dy)); 
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-2)*Ny+jj+1)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-2)*Ny+jj+1)+(1/dens_w)*(-WDQxx_ij(ii,jj)*WWx_i(ii)/Width(ii))*(1/(4*dx*dy));
    
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+ii*Ny+jj-1)=    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+ii*Ny+jj-1)    +(1/dens_w)*(-WDQxx_ij(ii+1,jj)*WWx_i(ii+1)/Width(ii))*(1/(4*dx*dy)); 
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+ii*Ny+jj+1)=    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+ii*Ny+jj+1)     +(1/dens_w)*(WDQxx_ij(ii+1,jj)*WWx_i(ii+1)/Width(ii))*(1/(4*dx*dy));                                  
    
    
    
    %------ (3) dQy d/dq ( D dQy dP'/dq ) --------;
    
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj-1)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj-1)+(1/dens_w)*(WDy_ij(ii,jj)*dQy(ii)/Width(ii))*(1/dy^2);     
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj)=  MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj)  +(1/dens_w)*(-(WDy_ij(ii,jj)*dQy(ii)+WDy_ij(ii,jj+1)*dQy(ii))/Width(ii))*(1/dy^2);
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj+1)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj+1)+(1/dens_w)*(WDy_ij(ii,jj+1)*dQy(ii)/Width(ii))*(1/dy^2);
    
        
    
    %------ (4) dQx d/dq ( D dP'/dx ) --------;
    
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-2)*Ny+jj)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-2)*Ny+jj) +(1/dens_w)*((WDQxy_ij(ii,jj)-WDQxy_ij(ii,jj+1))/(4*dy*dx)); 
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+ii*Ny+jj)=    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+ii*Ny+jj)    +(1/dens_w)*((-WDQxy_ij(ii,jj)+WDQxy_ij(ii,jj+1))/(4*dy*dx)); 
    
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-2)*Ny+jj-1)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-2)*Ny+jj-1) +(1/dens_w)*(WDQxy_ij(ii,jj)/(4*dy*dx)); 
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+ii*Ny+jj-1)=    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+ii*Ny+jj-1)    +(1/dens_w)*(-WDQxy_ij(ii,jj)/(4*dy*dx));
    
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-2)*Ny+jj+1)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-2)*Ny+jj+1)+(1/dens_w)*(-WDQxy_ij(ii,jj+1)/(4*dy*dx)); 
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+ii*Ny+jj+1)=    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+ii*Ny+jj+1)     +(1/dens_w)*(WDQxy_ij(ii,jj+1)/(4*dy*dx));
    
    
    %------ (5) dQx d/dq ( D dQx dP'/dq ) --------;
    
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj-1)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj-1)+(1/dens_w)*(WDQx2y_ij(ii,jj)/dy^2);     
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj)=  MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj)  +(1/dens_w)*(-(WDQx2y_ij(ii,jj)+WDQx2y_ij(ii,jj+1))/dy^2);
    MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj+1)=MB(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj,3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj+1)+(1/dens_w)*(WDQx2y_ij(ii,jj+1)/dy^2);
            
        
end
end

     

%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
    
        
    SolveUVW=linsolve(MB,VB);
        
    
%-----------------------------------------------------------------;

for ii=1:1:Nx
for jj=1:1:Ny
for kk=1:1:Nz 
    masU(ii,jj,kk)=SolveUVW((ii-1)*Ny*Nz+(jj-1)*Nz+kk); 
end 
end
end
                                                                           
for ii=1:1:Nx 
for jj=1:1:Ny
for kk=1:1:Nz 
    masV(ii,jj,kk)=SolveUVW(Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk); 
end 
end
end

for ii=1:1:Nx
for jj=1:1:Ny
for kk=1:1:Nz 
    masW(ii,jj,kk)=SolveUVW(2*Nx*Ny*Nz+(ii-1)*Ny*Nz+(jj-1)*Nz+kk); 
end 
end
end


for ii=1:1:Nx
for jj=1:1:Ny 
    Delta_H(ii,jj,JFreq)=SolveUVW(3*Nx*Ny*Nz+(ii-1)*Ny+jj); 
end
end


for ii=1:1:Nx
for jj=1:1:Ny 
    Delta_P(ii,jj,JFreq)=SolveUVW(3*Nx*Ny*Nz+Nx*Ny+(ii-1)*Ny+jj); 
end
end
    

for ii=1:1:Nx
for jj=1:1:Ny 
    Delta_Ws(ii,jj,JFreq)=masW(ii,jj,1); 
end
end




%+++++++++++++++++++++++++++++++++++++++++++++++++++++++;

max_Delta_H(JFreq)=0;

for ii=1:1:Nx 
    for jj=1:1:Ny
        if max_Delta_H(JFreq)<abs(Delta_H(ii,jj,JFreq)), max_Delta_H(JFreq)=abs(Delta_H(ii,jj,JFreq)); end 
    end
end



%+++++++++++++++++++++++++++++++++++++++++++++++++++++++;

    disp(JFreq);
    
    datestr(clock)


end


%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;
%+++++++++++++++++++++++ THE MAIN CYCLE END ++++++++++++++++++++++++++++;
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++;

%-----------------------------------------------------------------------;
%------------------ Results: the amplitude spectra ---------------------;
%--------- ( the maximal displacement vs periodicity/frequency )--------;
%-----------------------------------------------------------------------;


L_Freq1=zeros(NFreq,1); L_Freq2=zeros(NFreq,1); L_PT=zeros(NFreq,1);


for JFreq=1:1:NFreq, L_Freq1(JFreq)=2*pi/(PT0+JFreq*D_PT); end

for JFreq=1:1:NFreq, L_Freq2(JFreq)=1/(PT0+JFreq*D_PT); end

for JFreq=1:1:NFreq, L_PT(JFreq)=PT0+JFreq*D_PT; end



figure(6);

plot(L_Freq1,max_Delta_H,'-r.'), title('Amplitude spectrum'), xlabel('Frequency (Hz)'), ylabel('Displacement (m)'), grid;


figure(7);

plot(L_PT,max_Delta_H,'-r.'), title('Amplitude spectrum'), xlabel('Periodicity (s)'), ylabel('Displacement (m)'), grid;



save D:\Programs_and_Files\Results\Freq1.txt -ascii L_Freq1

save D:\Programs_and_Files\Results\Freq2.txt -ascii L_Freq2

save D:\Programs_and_Files\Results\Periodicity.txt -ascii L_PT


save D:\Programs_and_Files\Results\Amplitude.txt -ascii max_Delta_H


%------------------------------------------------------------------;



